/*     */ package com.google.code.yanf4j.nio.impl;
/*     */ 
/*     */ import com.google.code.yanf4j.config.Configuration;
/*     */ import com.google.code.yanf4j.core.CodecFactory;
/*     */ import com.google.code.yanf4j.core.Dispatcher;
/*     */ import com.google.code.yanf4j.core.Handler;
/*     */ import com.google.code.yanf4j.core.Session;
/*     */ import com.google.code.yanf4j.core.WriteMessage;
/*     */ import com.google.code.yanf4j.core.impl.AbstractController;
/*     */ import com.google.code.yanf4j.nio.NioSessionConfig;
/*     */ import com.google.code.yanf4j.nio.SelectionKeyHandler;
/*     */ import com.google.code.yanf4j.util.SystemUtils;
/*     */ import java.io.IOException;
/*     */ import java.net.InetSocketAddress;
/*     */ import java.nio.channels.SelectableChannel;
/*     */ import java.nio.channels.SelectionKey;
/*     */ import java.util.Queue;
/*     */ import org.slf4j.Logger;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class NioController
/*     */   extends AbstractController
/*     */   implements SelectionKeyHandler
/*     */ {
/*     */   protected SelectorManager selectorManager;
/*  44 */   protected int selectorPoolSize = SystemUtils.getSystemThreadCount();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public int getSelectorPoolSize()
/*     */   {
/*  51 */     return this.selectorPoolSize;
/*     */   }
/*     */   
/*     */   public void setSelectorPoolSize(int selectorPoolSize) {
/*  55 */     if (isStarted()) {
/*  56 */       throw new IllegalStateException("Controller has been started");
/*     */     }
/*  58 */     this.selectorPoolSize = selectorPoolSize;
/*     */   }
/*     */   
/*     */ 
/*     */   public NioController() {}
/*     */   
/*     */   public NioController(Configuration configuration, CodecFactory codecFactory)
/*     */   {
/*  66 */     super(configuration, codecFactory);
/*     */   }
/*     */   
/*     */   public NioController(Configuration configuration, Handler handler, CodecFactory codecFactory)
/*     */   {
/*  71 */     super(configuration, handler, codecFactory);
/*     */   }
/*     */   
/*     */   public NioController(Configuration configuration) {
/*  75 */     super(configuration);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private final class WriteTask
/*     */     implements Runnable
/*     */   {
/*     */     private final SelectionKey key;
/*     */     
/*     */ 
/*     */     private WriteTask(SelectionKey key)
/*     */     {
/*  88 */       this.key = key;
/*     */     }
/*     */     
/*     */     public final void run() {
/*  92 */       NioController.this.dispatchWriteEvent(this.key);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private final class ReadTask
/*     */     implements Runnable
/*     */   {
/*     */     private final SelectionKey key;
/*     */     
/*     */ 
/*     */     private ReadTask(SelectionKey key)
/*     */     {
/* 106 */       this.key = key;
/*     */     }
/*     */     
/*     */     public final void run() {
/* 110 */       NioController.this.dispatchReadEvent(this.key);
/*     */     }
/*     */   }
/*     */   
/*     */   public final SelectorManager getSelectorManager() {
/* 115 */     return this.selectorManager;
/*     */   }
/*     */   
/*     */   protected void start0() throws IOException
/*     */   {
/*     */     try {
/* 121 */       initialSelectorManager();
/* 122 */       doStart();
/*     */     } catch (IOException e) {
/* 124 */       log.error("Start server error", e);
/* 125 */       notifyException(e);
/* 126 */       stop();
/* 127 */       throw e;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void initialSelectorManager()
/*     */     throws IOException
/*     */   {
/* 138 */     if (this.selectorManager == null) {
/* 139 */       this.selectorManager = new SelectorManager(this.selectorPoolSize, this, this.configuration);
/*     */       
/* 141 */       this.selectorManager.start();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected abstract void doStart()
/*     */     throws IOException;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void onRead(SelectionKey key)
/*     */   {
/* 156 */     if (this.readEventDispatcher == null) {
/* 157 */       dispatchReadEvent(key);
/*     */     } else {
/* 159 */       this.readEventDispatcher.dispatch(new ReadTask(key, null));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void onWrite(SelectionKey key)
/*     */   {
/* 167 */     if (this.writeEventDispatcher == null) {
/* 168 */       dispatchWriteEvent(key);
/*     */     } else {
/* 170 */       this.writeEventDispatcher.dispatch(new WriteTask(key, null));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void closeSelectionKey(SelectionKey key)
/*     */   {
/* 178 */     if ((key.attachment() instanceof Session)) {
/* 179 */       Session session = (Session)key.attachment();
/* 180 */       if (session != null) {
/* 181 */         session.close();
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected abstract void dispatchReadEvent(SelectionKey paramSelectionKey);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected abstract void dispatchWriteEvent(SelectionKey paramSelectionKey);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void stop0()
/*     */     throws IOException
/*     */   {
/* 204 */     if ((this.selectorManager == null) || (!this.selectorManager.isStarted())) {
/* 205 */       return;
/*     */     }
/* 207 */     this.selectorManager.stop();
/*     */   }
/*     */   
/*     */   public synchronized void bind(int port) throws IOException {
/* 211 */     if (isStarted()) {
/* 212 */       throw new IllegalStateException("Server has been bind to " + getLocalSocketAddress());
/*     */     }
/*     */     
/* 215 */     bind(new InetSocketAddress(port));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final NioSessionConfig buildSessionConfig(SelectableChannel sc, Queue<WriteMessage> queue)
/*     */   {
/* 227 */     NioSessionConfig sessionConfig = new NioSessionConfig(sc, getHandler(), this.selectorManager, getCodecFactory(), getStatistics(), queue, this.dispatchMessageDispatcher, isHandleReadWriteConcurrently(), this.sessionTimeout, this.configuration.getSessionIdleTimeout());
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 232 */     return sessionConfig;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\nio\impl\NioController.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */