/*     */ package com.google.code.yanf4j.nio.impl;
/*     */ 
/*     */ import com.google.code.yanf4j.buffer.IoBuffer;
/*     */ import com.google.code.yanf4j.config.Configuration;
/*     */ import com.google.code.yanf4j.core.CodecFactory.Decoder;
/*     */ import com.google.code.yanf4j.core.CodecFactory.Encoder;
/*     */ import com.google.code.yanf4j.core.EventType;
/*     */ import com.google.code.yanf4j.core.WriteMessage;
/*     */ import com.google.code.yanf4j.core.impl.FutureImpl;
/*     */ import com.google.code.yanf4j.core.impl.WriteMessageImpl;
/*     */ import com.google.code.yanf4j.nio.NioSessionConfig;
/*     */ import com.google.code.yanf4j.statistics.Statistics;
/*     */ import com.google.code.yanf4j.util.ByteBufferUtils;
/*     */ import com.google.code.yanf4j.util.SelectorFactory;
/*     */ import java.io.IOException;
/*     */ import java.net.InetAddress;
/*     */ import java.net.InetSocketAddress;
/*     */ import java.net.Socket;
/*     */ import java.nio.channels.ClosedChannelException;
/*     */ import java.nio.channels.ReadableByteChannel;
/*     */ import java.nio.channels.SelectableChannel;
/*     */ import java.nio.channels.SelectionKey;
/*     */ import java.nio.channels.Selector;
/*     */ import java.nio.channels.SocketChannel;
/*     */ import java.util.concurrent.Future;
/*     */ import java.util.concurrent.atomic.AtomicLong;
/*     */ import org.slf4j.Logger;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class NioTCPSession
/*     */   extends AbstractNioSession
/*     */ {
/*     */   private InetSocketAddress remoteAddress;
/*     */   
/*     */   public final boolean isExpired()
/*     */   {
/*  46 */     if (log.isDebugEnabled()) {
/*  47 */       log.debug("sessionTimeout=" + this.sessionTimeout + ",this.timestamp=" + this.lastOperationTimeStamp.get() + ",current=" + System.currentTimeMillis());
/*     */     }
/*     */     
/*     */ 
/*  51 */     return this.sessionTimeout > 0L;
/*     */   }
/*     */   
/*     */   public NioTCPSession(NioSessionConfig sessionConfig, int readRecvBufferSize)
/*     */   {
/*  56 */     super(sessionConfig);
/*  57 */     if ((this.selectableChannel != null) && (getRemoteSocketAddress() != null))
/*     */     {
/*  59 */       this.loopback = getRemoteSocketAddress().getAddress().isLoopbackAddress();
/*     */     }
/*     */     
/*  62 */     setReadBuffer(IoBuffer.allocate(readRecvBufferSize));
/*  63 */     onCreated();
/*     */   }
/*     */   
/*     */   protected Object writeToChannel(WriteMessage message) throws IOException
/*     */   {
/*  68 */     if ((message.getWriteFuture() != null) && (!message.isWriting()) && (message.getWriteFuture().isCancelled()))
/*     */     {
/*  70 */       return message.getMessage();
/*     */     }
/*  72 */     if (message.getWriteBuffer() == null) {
/*  73 */       if (message.getWriteFuture() != null) {
/*  74 */         message.getWriteFuture().setResult(Boolean.TRUE);
/*     */       }
/*  76 */       return message.getMessage();
/*     */     }
/*  78 */     IoBuffer writeBuffer = message.getWriteBuffer();
/*     */     
/*  80 */     message.writing();
/*  81 */     if (this.useBlockingWrite) {
/*  82 */       return blockingWrite(this.selectableChannel, message, writeBuffer);
/*     */     }
/*     */     for (;;)
/*     */     {
/*  86 */       long n = doRealWrite(this.selectableChannel, writeBuffer);
/*  87 */       if (n > 0L) {
/*  88 */         this.statistics.statisticsWrite(n);
/*  89 */         this.scheduleWritenBytes.addAndGet(0L - n);
/*     */       }
/*  91 */       if ((writeBuffer == null) || (!writeBuffer.hasRemaining())) {
/*  92 */         if (message.getWriteFuture() != null) {
/*  93 */           message.getWriteFuture().setResult(Boolean.TRUE);
/*     */         }
/*  95 */         return message.getMessage(); }
/*  96 */       if (n == 0L)
/*     */       {
/*     */ 
/*  99 */         return null;
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public InetSocketAddress getRemoteSocketAddress()
/*     */   {
/* 107 */     if (this.remoteAddress == null) {
/* 108 */       this.remoteAddress = ((InetSocketAddress)((SocketChannel)this.selectableChannel).socket().getRemoteSocketAddress());
/*     */     }
/*     */     
/* 111 */     return this.remoteAddress;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final Object blockingWrite(SelectableChannel channel, WriteMessage message, IoBuffer writeBuffer)
/*     */     throws IOException, ClosedChannelException
/*     */   {
/* 127 */     SelectionKey tmpKey = null;
/* 128 */     Selector writeSelector = null;
/* 129 */     int attempts = 0;
/* 130 */     int bytesProduced = 0;
/*     */     try {
/* 132 */       while (writeBuffer.hasRemaining()) {
/* 133 */         long len = doRealWrite(channel, writeBuffer);
/* 134 */         if (len > 0L) {
/* 135 */           attempts = 0;
/* 136 */           bytesProduced = (int)(bytesProduced + len);
/* 137 */           this.statistics.statisticsWrite(len);
/*     */         } else {
/* 139 */           attempts++;
/* 140 */           if (writeSelector == null) {
/* 141 */             writeSelector = SelectorFactory.getSelector();
/* 142 */             if (writeSelector != null)
/*     */             {
/*     */ 
/*     */ 
/* 146 */               tmpKey = channel.register(writeSelector, 4);
/*     */             }
/*     */           }
/* 149 */           else if ((writeSelector.select(1000L) == 0) && 
/* 150 */             (attempts > 2)) {
/* 151 */             throw new IOException("Client disconnected");
/*     */           }
/*     */         }
/*     */       }
/*     */       
/* 156 */       if ((!writeBuffer.hasRemaining()) && (message.getWriteFuture() != null)) {
/* 157 */         message.getWriteFuture().setResult(Boolean.TRUE);
/*     */       }
/*     */     } finally {
/* 160 */       if (tmpKey != null) {
/* 161 */         tmpKey.cancel();
/* 162 */         tmpKey = null;
/*     */       }
/* 164 */       if (writeSelector != null)
/*     */       {
/* 166 */         writeSelector.selectNow();
/* 167 */         SelectorFactory.returnSelector(writeSelector);
/*     */       }
/*     */     }
/* 170 */     this.scheduleWritenBytes.addAndGet(0 - bytesProduced);
/* 171 */     return message.getMessage();
/*     */   }
/*     */   
/*     */   protected WriteMessage wrapMessage(Object msg, Future<Boolean> writeFuture)
/*     */   {
/* 176 */     WriteMessage message = new WriteMessageImpl(msg, (FutureImpl)writeFuture);
/*     */     
/* 178 */     if (message.getWriteBuffer() == null) {
/* 179 */       message.setWriteBuffer(this.encoder.encode(message.getMessage(), this));
/*     */     }
/*     */     
/* 182 */     return message;
/*     */   }
/*     */   
/*     */   protected void readFromBuffer()
/*     */   {
/* 187 */     if (!this.readBuffer.hasRemaining()) {
/* 188 */       if (this.readBuffer.capacity() < Configuration.MAX_READ_BUFFER_SIZE) {
/* 189 */         this.readBuffer = IoBuffer.wrap(ByteBufferUtils.increaseBufferCapatity(this.readBuffer.buf()));
/*     */       }
/*     */       else
/*     */       {
/* 193 */         return;
/*     */       }
/*     */     }
/* 196 */     if (this.closed) {
/* 197 */       return;
/*     */     }
/* 199 */     int n = -1;
/* 200 */     int readCount = 0;
/*     */     try
/*     */     {
/* 203 */       while ((n = ((ReadableByteChannel)this.selectableChannel).read(this.readBuffer.buf())) > 0) {
/* 204 */         readCount += n;
/*     */       }
/* 206 */       if (readCount > 0) {
/* 207 */         decodeAndDispatch();
/* 208 */       } else if ((readCount == 0) && (!((SocketChannel)this.selectableChannel).socket().isInputShutdown()) && (this.useBlockingRead))
/*     */       {
/*     */ 
/* 211 */         n = blockingRead();
/* 212 */         if (n > 0) {
/* 213 */           readCount += n;
/*     */         }
/*     */       }
/* 216 */       if (n < 0) {
/* 217 */         close();
/*     */       } else {
/* 219 */         this.selectorManager.registerSession(this, EventType.ENABLE_READ);
/*     */       }
/*     */       
/* 222 */       if (log.isDebugEnabled()) {
/* 223 */         log.debug("read " + readCount + " bytes from channel");
/*     */       }
/*     */     }
/*     */     catch (ClosedChannelException e) {
/* 227 */       close();
/*     */     } catch (Throwable e) {
/* 229 */       onException(e);
/* 230 */       close();
/*     */     }
/*     */   }
/*     */   
/*     */   private void decodeAndDispatch() {
/* 235 */     updateTimeStamp();
/* 236 */     this.readBuffer.flip();
/* 237 */     decode();
/* 238 */     this.readBuffer.compact();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final int blockingRead()
/*     */     throws ClosedChannelException, IOException
/*     */   {
/* 250 */     int n = 0;
/* 251 */     int readCount = 0;
/* 252 */     Selector readSelector = SelectorFactory.getSelector();
/* 253 */     SelectionKey tmpKey = null;
/*     */     try {
/* 255 */       if (this.selectableChannel.isOpen()) {
/* 256 */         tmpKey = this.selectableChannel.register(readSelector, 0);
/* 257 */         tmpKey.interestOps(tmpKey.interestOps() | 0x1);
/* 258 */         int code = readSelector.select(500L);
/* 259 */         tmpKey.interestOps(tmpKey.interestOps() & 0xFFFFFFFE);
/* 260 */         if (code > 0) {
/*     */           do {
/* 262 */             n = ((ReadableByteChannel)this.selectableChannel).read(this.readBuffer.buf());
/*     */             
/* 264 */             readCount += n;
/* 265 */             if (log.isDebugEnabled()) {
/* 266 */               log.debug("use temp selector read " + n + " bytes");
/*     */             }
/* 268 */           } while ((n > 0) && (this.readBuffer.hasRemaining()));
/* 269 */           if (readCount > 0) {
/* 270 */             decodeAndDispatch();
/*     */           }
/*     */         }
/*     */       }
/*     */     } finally {
/* 275 */       if (tmpKey != null) {
/* 276 */         tmpKey.cancel();
/* 277 */         tmpKey = null;
/*     */       }
/* 279 */       if (readSelector != null)
/*     */       {
/* 281 */         readSelector.selectNow();
/* 282 */         SelectorFactory.returnSelector(readSelector);
/*     */       }
/*     */     }
/* 285 */     return readCount;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void decode()
/*     */   {
/* 294 */     int size = this.readBuffer.remaining();
/* 295 */     for (;;) { if (this.readBuffer.hasRemaining()) {
/*     */         try {
/* 297 */           Object message = this.decoder.decode(this.readBuffer, this);
/* 298 */           if (message != null)
/*     */           {
/*     */ 
/* 301 */             if (this.statistics.isStatistics()) {
/* 302 */               this.statistics.statisticsRead(size - this.readBuffer.remaining());
/*     */               
/* 304 */               size = this.readBuffer.remaining();
/*     */             }
/*     */             
/* 307 */             dispatchReceivedMessage(message);
/*     */           }
/* 309 */         } catch (Exception e) { onException(e);
/* 310 */           log.error("Decode error", e);
/* 311 */           super.close();
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public Socket socket() {
/* 318 */     return ((SocketChannel)this.selectableChannel).socket();
/*     */   }
/*     */   
/*     */   protected final void closeChannel() throws IOException
/*     */   {
/* 323 */     flush0();
/*     */     
/* 325 */     Socket socket = ((SocketChannel)this.selectableChannel).socket();
/*     */     try {
/* 327 */       if ((!socket.isClosed()) && (!socket.isOutputShutdown())) {
/* 328 */         socket.shutdownOutput();
/*     */       }
/* 330 */       if ((!socket.isClosed()) && (!socket.isInputShutdown())) {
/* 331 */         socket.shutdownInput();
/*     */       }
/*     */     }
/*     */     catch (Exception e) {}
/*     */     try {
/* 336 */       socket.close();
/*     */     }
/*     */     catch (Exception e) {}
/*     */     
/* 340 */     unregisterSession();
/* 341 */     unregisterChannel();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\nio\impl\NioTCPSession.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */