/*     */ package com.netapp.asup.data.objects;
/*     */ 
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.data.schema.SchemaHelper;
/*     */ import com.netapp.asup.data.store.BaseContainer;
/*     */ import com.netapp.asup.data.store.StoreContainer;
/*     */ import com.netapp.asup.filter.InstanceKey;
/*     */ import com.netapp.asup.metamodel.MergeType;
/*     */ import com.netapp.asup.metamodel.NodeObject;
/*     */ import java.io.ByteArrayOutputStream;
/*     */ import java.io.IOException;
/*     */ import java.io.ObjectInputStream;
/*     */ import java.io.ObjectOutputStream;
/*     */ import java.io.Serializable;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collection;
/*     */ import java.util.HashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Set;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.generic.GenericData.Array;
/*     */ import org.apache.avro.generic.GenericData.Record;
/*     */ import org.apache.avro.generic.GenericDatumReader;
/*     */ import org.apache.avro.generic.GenericDatumWriter;
/*     */ import org.apache.avro.generic.GenericRecord;
/*     */ import org.apache.avro.io.BinaryDecoder;
/*     */ import org.apache.avro.io.BinaryEncoder;
/*     */ import org.apache.avro.io.DecoderFactory;
/*     */ import org.apache.avro.io.EncoderFactory;
/*     */ import org.apache.avro.util.Utf8;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ObjectData
/*     */   extends ByteWrappedGenericRecord<Map<Utf8, GenericRecord>, GenericRecord>
/*     */   implements Serializable
/*     */ {
/*     */   private static final long serialVersionUID = 5270009714008702182L;
/*     */   private static final String INDEX_MAP = "indexMap";
/*     */   private static final String OBJECT_MAP = "objectMap";
/*     */   private static final String OPTION_GROUP_ID = "option_group_id";
/*     */   private static final String KEY_INDEX = "keyIndex";
/*     */   private String objectName;
/*     */   private static final String DERIVED_ATTRIBUTES = "derivedAttributes";
/*     */   private Map<Utf8, Map<Utf8, GenericRecord>> systemToGenericRecord;
/*  57 */   private Map<Utf8, Map<Utf8, Map<Utf8, GenericData.Array<Utf8>>>> indexMap = new HashMap();
/*     */   
/*  59 */   private Map<Utf8, Map<Utf8, Utf8>> instanceVsDerAttrs = new HashMap();
/*     */   
/*     */   private Map<Utf8, List<Utf8>> keyIndex;
/*     */   
/*     */   private Schema writerSchema;
/*     */   
/*     */   private Schema readerSchema;
/*     */   
/*     */ 
/*     */   ObjectData(Utf8 objectName, Schema writerSchema, Schema readerSchema, String id)
/*     */   {
/*  70 */     super(ARMASchema.OBJECTDATA, writerSchema, readerSchema, id);
/*  71 */     this.object = new GenericData.Record(ARMASchema.OBJECTDATA.getSchema());
/*  72 */     this.writerSchema = writerSchema;
/*  73 */     this.readerSchema = readerSchema;
/*  74 */     AvroGenricRecordUtility.putStringValue((GenericRecord)this.object, "name", objectName.toString());
/*     */     
/*  76 */     this.objectName = objectName.toString();
/*     */     
/*  78 */     setSystemToGenericRecord(new HashMap());
/*     */     
/*     */ 
/*  81 */     Map<Utf8, Map<Utf8, Map<Utf8, GenericData.Array<Utf8>>>> emptyIndex = new HashMap();
/*  82 */     setIndex(emptyIndex);
/*  83 */     Map<Utf8, List<Utf8>> emptyKeyIndex = new HashMap();
/*  84 */     setKeyIndex(emptyKeyIndex);
/*  85 */     Map<Utf8, Map<Utf8, Utf8>> derAttr = new HashMap();
/*  86 */     setDerivedAttributes(derAttr);
/*     */   }
/*     */   
/*     */   ObjectData(GenericRecord object, Schema writerSchema, Schema readerSchema, String id)
/*     */     throws IOException
/*     */   {
/*  92 */     super(ARMASchema.OBJECTDATA, writerSchema, readerSchema, id);
/*  93 */     initObjectData(object, writerSchema, readerSchema);
/*     */   }
/*     */   
/*     */   private void initObjectData(GenericRecord object, Schema writerSchema, Schema readerSchema) throws IOException
/*     */   {
/*  98 */     this.object = object;
/*  99 */     this.writerSchema = writerSchema;
/* 100 */     this.readerSchema = readerSchema;
/* 101 */     this.objectName = AvroGenricRecordUtility.getStringValue((GenericRecord)this.object, "name");
/* 102 */     Map<Utf8, GenericRecord> emptyObjectValue = new HashMap();
/* 103 */     setSystemToGenericRecord(deSerializeMap((Map)object.get("objectMap"), emptyObjectValue));
/*     */     
/* 105 */     this.instanceVsDerAttrs = ((Map)object.get("derivedAttributes"));
/* 106 */     this.keyIndex = ((Map)object.get("keyIndex"));
/* 107 */     this.indexMap = ((Map)object.get("indexMap"));
/*     */   }
/*     */   
/*     */   public void addSystemData(StatusHeader header, String entityName, Utf8 systemId, Schema writerSchema, Schema readerSchema, byte[] data)
/*     */     throws IOException
/*     */   {
/* 113 */     if (null == data) {
/* 114 */       header.addErrorMessage("Object data not found for the request entity :" + entityName);
/*     */       
/* 116 */       return;
/*     */     }
/* 118 */     this.systemToGenericRecord.put(systemId, deserializeData(writerSchema, readerSchema, data));
/*     */   }
/*     */   
/*     */   public void addSystemData(StatusHeader header, String entityName, Utf8 systemId, byte[] data)
/*     */     throws IOException
/*     */   {
/* 124 */     if (null == data) {
/* 125 */       header.addErrorMessage("Object data not found for the request entity :" + entityName);
/*     */       
/* 127 */       return;
/*     */     }
/* 129 */     this.systemToGenericRecord.put(systemId, deserializeData(data));
/*     */   }
/*     */   
/*     */   public GenericRecord getObject() throws IOException {
/* 133 */     Map<Utf8, ByteBuffer> serializeMap = serializeMap(this.systemToGenericRecord);
/*     */     
/*     */ 
/* 136 */     Map<Utf8, ByteBuffer> existingObjectMap = (Map)((GenericRecord)this.object).get("objectMap");
/*     */     
/* 138 */     if (existingObjectMap != null) {
/* 139 */       existingObjectMap.putAll(serializeMap);
/* 140 */       ((GenericRecord)this.object).put("objectMap", existingObjectMap);
/*     */     } else {
/* 142 */       ((GenericRecord)this.object).put("objectMap", serializeMap);
/*     */     }
/* 144 */     ((GenericRecord)this.object).put("derivedAttributes", this.instanceVsDerAttrs);
/*     */     
/* 146 */     return (GenericRecord)this.object;
/*     */   }
/*     */   
/*     */   public Utf8 getName() {
/* 150 */     return AvroGenricRecordUtility.getUtf8Value((GenericRecord)this.object, "name");
/*     */   }
/*     */   
/*     */   public Map<Utf8, Map<Utf8, Map<Utf8, GenericData.Array<Utf8>>>> getIndex()
/*     */   {
/* 155 */     return (Map)((GenericRecord)this.object).get("indexMap");
/*     */   }
/*     */   
/*     */ 
/*     */   public void setIndex(Map<Utf8, Map<Utf8, Map<Utf8, GenericData.Array<Utf8>>>> index)
/*     */   {
/* 161 */     ((GenericRecord)this.object).put("indexMap", index);
/*     */   }
/*     */   
/*     */   public Map<Utf8, Map<Utf8, GenericRecord>> getSystemToGenericRecord()
/*     */   {
/* 166 */     if (null == this.systemToGenericRecord) {
/* 167 */       Map<Utf8, GenericRecord> emptyObjectValue = new HashMap();
/*     */       try {
/* 169 */         this.systemToGenericRecord = deSerializeMap((Map)((GenericRecord)this.object).get("objectMap"), emptyObjectValue);
/*     */       }
/*     */       catch (IOException e)
/*     */       {
/* 173 */         logger.debug(null, "error de-serializing objectdata map from bytes to generic record :" + e.getMessage());
/*     */       }
/*     */       
/*     */ 
/* 177 */       if (null == this.systemToGenericRecord)
/* 178 */         this.systemToGenericRecord = new HashMap();
/*     */     }
/* 180 */     return this.systemToGenericRecord;
/*     */   }
/*     */   
/*     */   public Map<Utf8, Map<Utf8, Map<Utf8, GenericData.Array<Utf8>>>> getIndexMap()
/*     */   {
/* 185 */     if (this.indexMap == null) {
/* 186 */       this.indexMap = ((Map)((GenericRecord)this.object).get("indexMap"));
/*     */       
/*     */ 
/* 189 */       if (this.indexMap == null)
/* 190 */         this.indexMap = new HashMap();
/*     */     }
/* 192 */     return this.indexMap;
/*     */   }
/*     */   
/*     */ 
/*     */   public void setSystemToGenericRecord(Map<Utf8, Map<Utf8, GenericRecord>> systemToGenericRecord)
/*     */   {
/* 198 */     this.systemToGenericRecord = systemToGenericRecord;
/*     */   }
/*     */   
/*     */ 
/*     */   public Map<Utf8, Map<Utf8, Map<Utf8, GenericData.Array<Utf8>>>> populateIndexesForParent(NodeObject parentNodeObject)
/*     */   {
/* 204 */     return IndexBuilder.populateIndexesForParent(parentNodeObject, this.objectName, this.systemToGenericRecord, this.indexMap);
/*     */   }
/*     */   
/*     */   public Map<Utf8, List<Utf8>> getKeyIndex() {
/*     */     Map<Utf8, GenericData.Array<Utf8>> kIndex;
/* 209 */     if (null == this.keyIndex) {
/* 210 */       kIndex = (Map)((GenericRecord)this.object).get("keyIndex");
/*     */       
/* 212 */       if (null == kIndex) {
/* 213 */         this.keyIndex = new HashMap();
/*     */       } else {
/* 215 */         this.keyIndex = new HashMap(kIndex.size());
/* 216 */         for (Utf8 key : kIndex.keySet()) {
/* 217 */           this.keyIndex.put(key, new ArrayList((Collection)kIndex.get(key)));
/*     */         }
/*     */       }
/*     */     }
/* 221 */     return this.keyIndex;
/*     */   }
/*     */   
/*     */   public void setDerivedAttributes(Map<Utf8, Map<Utf8, Utf8>> dervAttrs) {
/* 225 */     ((GenericRecord)this.object).put("derivedAttributes", dervAttrs);
/*     */   }
/*     */   
/*     */ 
/*     */   public Map<Utf8, Map<Utf8, Utf8>> getDerivedAttributes()
/*     */   {
/* 231 */     if (null == this.instanceVsDerAttrs) {
/* 232 */       this.instanceVsDerAttrs = ((Map)((GenericRecord)this.object).get("derivedAttributes"));
/* 233 */       if (this.instanceVsDerAttrs == null) {
/* 234 */         this.instanceVsDerAttrs = new HashMap();
/*     */       }
/*     */     }
/* 237 */     return this.instanceVsDerAttrs;
/*     */   }
/*     */   
/*     */   public void setKeyIndex(Map<Utf8, List<Utf8>> keyIndex) {
/* 241 */     this.keyIndex = keyIndex;
/*     */     
/* 243 */     Map<Utf8, GenericData.Array<Utf8>> kIndex = new HashMap();
/*     */     
/*     */ 
/*     */ 
/* 247 */     for (Utf8 key : keyIndex.keySet()) {
/* 248 */       List<Utf8> list = (List)keyIndex.get(key);
/* 249 */       GenericData.Array<Utf8> indexes = new GenericData.Array(list.size(), ARMASchema.KEY_INDEX.getSchema());
/*     */       
/* 251 */       indexes.addAll(list);
/* 252 */       kIndex.put(key, indexes);
/*     */     }
/*     */     
/* 255 */     ((GenericRecord)this.object).put("keyIndex", kIndex);
/*     */   }
/*     */   
/*     */   public Utf8 getOptionGroupIdVal(GenericRecord record) {
/* 259 */     GenericRecord configRecord = (GenericRecord)record.get("config");
/*     */     
/* 261 */     if (null != configRecord) {
/* 262 */       return configRecord.get("option_group_id") == null ? null : (Utf8)configRecord.get("option_group_id");
/*     */     }
/*     */     
/* 265 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */   public void addDerivedAttribute(Utf8 uniqueKey, Utf8 derAttrName, Utf8 derAttrValue)
/*     */   {
/* 271 */     Map<Utf8, Utf8> derivedAttributes = (Map)this.instanceVsDerAttrs.get(uniqueKey);
/* 272 */     if (null == derivedAttributes) {
/* 273 */       derivedAttributes = new HashMap();
/* 274 */       this.instanceVsDerAttrs.put(uniqueKey, derivedAttributes);
/*     */     }
/* 276 */     derivedAttributes.put(derAttrName, derAttrValue);
/*     */   }
/*     */   
/*     */   public ObjectData retainInstances(Set<InstanceKey> instanceKeys)
/*     */   {
/* 281 */     ObjectData filteredData = new ObjectData(new Utf8(this.objectName), this.writerSchema, this.readerSchema, this.id);
/*     */     
/* 283 */     if ((null == instanceKeys) || (instanceKeys.isEmpty()))
/*     */     {
/* 285 */       return filteredData;
/*     */     }
/*     */     
/* 288 */     Map<Utf8, Map<Utf8, GenericRecord>> retainedSystemToGenericRecord = new HashMap();
/* 289 */     Map<Utf8, List<Utf8>> retainedkeyIndex = new HashMap();
/* 290 */     Map<Utf8, Map<Utf8, Utf8>> retainedDerAttrs = new HashMap();
/*     */     
/* 292 */     Utf8 sysKey = null;
/* 293 */     Utf8 insKey = null;
/* 294 */     for (InstanceKey instanceKey : instanceKeys) {
/* 295 */       insKey = new Utf8(instanceKey.getInstanceId());
/*     */       
/* 297 */       if (MergeType.INDIRECT.equals(instanceKey.getType())) {
/* 298 */         sysKey = new Utf8(instanceKey.getSysId());
/* 299 */         retainInstance(retainedSystemToGenericRecord, retainedDerAttrs, sysKey, insKey);
/*     */       }
/*     */       else {
/* 302 */         retainkeyIndex(retainedSystemToGenericRecord, retainedkeyIndex, retainedDerAttrs, insKey, (List)this.keyIndex.get(insKey));
/*     */       }
/*     */     }
/*     */     
/*     */ 
/* 307 */     filteredData.setKeyIndex(retainedkeyIndex);
/* 308 */     filteredData.setSystemToGenericRecord(retainedSystemToGenericRecord);
/* 309 */     filteredData.setDerivedAttributes(retainedDerAttrs);
/* 310 */     return filteredData;
/*     */   }
/*     */   
/*     */   private void retainkeyIndex(Map<Utf8, Map<Utf8, GenericRecord>> retainedSystemToGenericRecord, Map<Utf8, List<Utf8>> retainedkeyIndex, Map<Utf8, Map<Utf8, Utf8>> retainedDerAttrs, Utf8 insKey, List<Utf8> sysIds)
/*     */   {
/* 315 */     List<Utf8> sysKeys = new ArrayList(sysIds.size());
/* 316 */     for (Utf8 sysIdUtf : sysIds) {
/* 317 */       sysKeys.add(sysIdUtf);
/* 318 */       retainInstance(retainedSystemToGenericRecord, retainedDerAttrs, sysIdUtf, insKey);
/*     */     }
/* 320 */     retainedkeyIndex.put(insKey, sysKeys);
/*     */   }
/*     */   
/*     */ 
/*     */   private void retainInstance(Map<Utf8, Map<Utf8, GenericRecord>> retainedSystemToGenericRecord, Map<Utf8, Map<Utf8, Utf8>> retainedDerAttrs, Utf8 sysKey, Utf8 insKey)
/*     */   {
/* 326 */     Map<Utf8, GenericRecord> sysMap = (Map)retainedSystemToGenericRecord.get(sysKey);
/* 327 */     if (sysMap == null) {
/* 328 */       sysMap = new HashMap();
/* 329 */       retainedSystemToGenericRecord.put(sysKey, sysMap);
/*     */     }
/* 331 */     sysMap.put(insKey, ((Map)this.systemToGenericRecord.get(sysKey)).get(insKey));
/*     */     
/* 333 */     Map<Utf8, Utf8> derMap = (Map)this.instanceVsDerAttrs.get(insKey);
/* 334 */     if (null != derMap) {
/* 335 */       Map<Utf8, Utf8> retDerMap = new HashMap();
/* 336 */       retDerMap.putAll(derMap);
/* 337 */       retainedDerAttrs.put(insKey, retDerMap);
/*     */     }
/*     */   }
/*     */   
/*     */   private void writeObject(ObjectOutputStream oos)
/*     */     throws IOException
/*     */   {
/* 344 */     logger.info(null, "entered custom writeObject..");
/* 345 */     if (this.schema == null) {
/* 346 */       this.schema = ARMASchema.OBJECTDATA;
/*     */     }
/* 348 */     SchemaHelper schemaHelper = BaseContainer.getInstance().getStoreContainer().getSchemaHelper();
/* 349 */     Schema writerSchema = schemaHelper.getLatestSchema(this.objectName);
/* 350 */     if (this.reader == null) {
/* 351 */       this.reader = new GenericDatumReader(writerSchema, schemaHelper.getActiveSchema(this.objectName));
/*     */     }
/* 353 */     if (this.writer == null) {
/* 354 */       this.writer = new GenericDatumWriter(writerSchema);
/*     */     }
/*     */     
/* 357 */     BinaryEncoder binEncoder = encoderFactory.binaryEncoder(oos, EncoderFactory.get().binaryEncoder(oos, null));
/*     */     try {
/* 359 */       oos.writeUTF(this.id);
/* 360 */       this.schema.getWriter().write(getObject(), binEncoder);
/* 361 */       binEncoder.flush();
/* 362 */       logger.info(null, "written objectdata using custom writeObject..");
/*     */     } catch (IOException e) {
/* 364 */       logger.info(null, "unable to serialize objectdata for saving it in memcache.." + e.getMessage());
/*     */     }
/*     */     finally
/*     */     {
/* 368 */       if (null != oos) {
/* 369 */         oos.close();
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private void readObject(ObjectInputStream ois) throws IOException, ClassNotFoundException {
/* 375 */     logger.info(null, "entered custom readobject..");
/* 376 */     logger.info(null, "reading bytes data: " + ois.available());
/*     */     
/* 378 */     this.id = ois.readUTF();
/* 379 */     BinaryDecoder binaryDecoder = DecoderFactory.get().binaryDecoder(ois, null);
/* 380 */     GenericDatumReader<GenericRecord> reader = new GenericDatumReader(ARMASchema.OBJECTDATA.getSchema());
/* 381 */     this.object = new GenericData.Record(reader.getSchema());
/* 382 */     reader.read(this.object, binaryDecoder);
/*     */     
/* 384 */     this.objectName = AvroGenricRecordUtility.getStringValue((GenericRecord)this.object, "name");
/* 385 */     SchemaHelper schemaHelper = BaseContainer.getInstance().getStoreContainer().getSchemaHelper();
/* 386 */     Schema writerSchema = schemaHelper.getLatestSchema(this.objectName);
/* 387 */     Schema readerSchema = schemaHelper.getActiveSchema(this.objectName);
/* 388 */     this.reader = new GenericDatumReader(writerSchema, readerSchema);
/* 389 */     this.writer = new GenericDatumWriter(writerSchema);
/* 390 */     initObjectData((GenericRecord)this.object, writerSchema, readerSchema);
/*     */   }
/*     */   
/*     */   public String toString()
/*     */   {
/* 395 */     return "ObjectData [OPTION_GROUP_ID=option_group_id, systemToGenericRecord=" + this.systemToGenericRecord + ", indexMap=" + this.indexMap + ", instanceVsDerAttrs=" + this.instanceVsDerAttrs + ", keyIndex=" + this.keyIndex + "]";
/*     */   }
/*     */   
/*     */   public byte[] getBytes() throws IOException
/*     */   {
/* 400 */     ByteArrayOutputStream baos = new ByteArrayOutputStream();
/* 401 */     ObjectOutputStream os = new ObjectOutputStream(baos);
/* 402 */     writeObject(os);
/* 403 */     return baos.toByteArray();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\objects\ObjectData.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */