/*     */ package com.netapp.asup.data.store;
/*     */ 
/*     */ import com.netapp.asup.adapter.Adapter;
/*     */ import com.netapp.asup.avro.AvroReader;
/*     */ import com.netapp.asup.cache.ARMAGenericCacheImpl;
/*     */ import com.netapp.asup.cache.server.AsupGenericCache;
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.config.ARMAConfiguration;
/*     */ import com.netapp.asup.data.adapter.RESTAdapter;
/*     */ import com.netapp.asup.data.function.FunctionRegistry;
/*     */ import com.netapp.asup.data.pojo.ARMAPOJORegistry;
/*     */ import com.netapp.asup.data.schema.ClientSchemaHelper;
/*     */ import com.netapp.asup.data.schema.SchemaHelper;
/*     */ import com.netapp.asup.data.util.HttpConnectionManager;
/*     */ import com.netapp.asup.exception.ARMAConfigurationException;
/*     */ import com.netapp.asup.metamodel.Connector;
/*     */ import com.netapp.asup.metamodel.MetaModelFactory;
/*     */ import com.netapp.asup.metamodel.ServletConnector;
/*     */ import com.netapp.asupdata.client.ARMA.ResultInitializer;
/*     */ import java.io.IOException;
/*     */ import org.apache.avro.Schema;
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ClientStoreContainerImpl
/*     */   implements StoreContainer
/*     */ {
/*  28 */   private DataStore store = null;
/*     */   
/*  30 */   private Adapter adapter = null;
/*     */   
/*  32 */   private AsupGenericCache<String, Object> cache = null;
/*     */   
/*  34 */   private MetaModelFactory metaModelprovider = null;
/*     */   
/*     */   private ClientSchemaHelper schemaHelper;
/*     */   
/*     */   private static final String GRAPH_ML_PROPERTIES = "GraphMl.properties";
/*     */   
/*     */   private static final String CACHE_CONFIG_PROPERTIES = "conf/cache.properties";
/*     */   
/*     */   private static final String SERV_DETAILS_PROPERTIES = "conf/arma_serv_details.properties";
/*     */   
/*  44 */   private static final AsupLogger LOGGER = AsupLogger.getLogger(ClientStoreContainerImpl.class);
/*     */   
/*     */   private BaseContainer baseContainer;
/*     */   
/*     */   private ARMAConfiguration config;
/*     */   
/*     */ 
/*     */   public ClientStoreContainerImpl(BaseContainer baseContainer, ARMAConfiguration config)
/*     */   {
/*  53 */     this.baseContainer = baseContainer;
/*  54 */     this.config = config;
/*     */   }
/*     */   
/*     */   public StoreContainer initializeContainer()
/*     */     throws ARMAConfigurationException
/*     */   {
/*     */     try
/*     */     {
/*  62 */       this.config.loadDefaultResource("conf/cache.properties");
/*  63 */       this.config.loadDefaultResource("conf/arma_serv_details.properties");
/*     */       
/*  65 */       HttpConnectionManager.initialize(this.config);
/*  66 */       this.cache = new ARMAGenericCacheImpl(this.config);
/*  67 */       this.schemaHelper = new ClientSchemaHelper(this.config);
/*  68 */       Connector connector = new ServletConnector("GraphMl.properties");
/*  69 */       this.metaModelprovider = new MetaModelFactory(connector);
/*  70 */       this.adapter = new RESTAdapter(this.schemaHelper);
/*  71 */       this.store = new DataStoreImpl(this.adapter, this.cache, this.metaModelprovider, this.schemaHelper);
/*     */     } catch (Exception e) {
/*  73 */       LOGGER.info("-9999", "Fatal Error: Container initialization failed.");
/*     */       
/*  75 */       throw new ARMAConfigurationException(LOGGER, "Fatal Error: Container initialization failed.", e);
/*     */     }
/*     */     
/*     */ 
/*  79 */     return this;
/*     */   }
/*     */   
/*     */   public DataStore getStore() {
/*  83 */     return this.store;
/*     */   }
/*     */   
/*     */   public Adapter getAdapter() {
/*  87 */     return this.adapter;
/*     */   }
/*     */   
/*     */   public AsupGenericCache<String, Object> getCache() {
/*  91 */     return this.cache;
/*     */   }
/*     */   
/*     */   public MetaModelFactory getMetaModelprovider() {
/*  95 */     return this.metaModelprovider;
/*     */   }
/*     */   
/*     */   public AvroReader getAvroReader() {
/*  99 */     return this.baseContainer.getAvroReader();
/*     */   }
/*     */   
/*     */   public void destroyContainer() {
/* 103 */     this.cache.shutdown();
/*     */   }
/*     */   
/*     */   public void clearCache() {
/* 107 */     this.cache.clear();
/*     */   }
/*     */   
/*     */   public SchemaHelper getSchemaHelper() {
/* 111 */     return this.schemaHelper;
/*     */   }
/*     */   
/*     */   public FunctionRegistry getFunctionRegistry()
/*     */   {
/* 116 */     return this.baseContainer.getFunctionRegistry();
/*     */   }
/*     */   
/*     */   public ARMAPOJORegistry getPOJORegistry()
/*     */   {
/* 121 */     return this.baseContainer.getPOJORegistry();
/*     */   }
/*     */   
/*     */ 
/*     */   public String getCacheStats()
/*     */   {
/* 127 */     return this.cache.getCacheStats();
/*     */   }
/*     */   
/*     */   public void removeServer(String serverConnection) throws Exception
/*     */   {
/* 132 */     this.cache.removeServer(serverConnection);
/*     */   }
/*     */   
/*     */ 
/*     */   public ResultInitializer getResultInitializer()
/*     */   {
/* 138 */     return null;
/*     */   }
/*     */   
/*     */   public String getMode()
/*     */   {
/* 143 */     return "CLIENT";
/*     */   }
/*     */   
/*     */   public Schema getSchemaFromHDFS(String absolutePath) throws IOException
/*     */   {
/* 148 */     throw new UnsupportedOperationException("this operation not supported from client");
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\store\ClientStoreContainerImpl.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */