/*      */ package net.rubyeye.xmemcached;
/*      */ 
/*      */ import com.google.code.yanf4j.buffer.IoBuffer;
/*      */ import com.google.code.yanf4j.config.Configuration;
/*      */ import com.google.code.yanf4j.core.Session;
/*      */ import com.google.code.yanf4j.core.SocketOption;
/*      */ import com.google.code.yanf4j.util.SystemUtils;
/*      */ import java.io.IOException;
/*      */ import java.io.UnsupportedEncodingException;
/*      */ import java.net.InetSocketAddress;
/*      */ import java.net.URLDecoder;
/*      */ import java.net.URLEncoder;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Collection;
/*      */ import java.util.Collections;
/*      */ import java.util.HashMap;
/*      */ import java.util.HashSet;
/*      */ import java.util.Iterator;
/*      */ import java.util.LinkedList;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.Map.Entry;
/*      */ import java.util.Queue;
/*      */ import java.util.Set;
/*      */ import java.util.concurrent.CopyOnWriteArrayList;
/*      */ import java.util.concurrent.CountDownLatch;
/*      */ import java.util.concurrent.ExecutionException;
/*      */ import java.util.concurrent.Future;
/*      */ import java.util.concurrent.TimeUnit;
/*      */ import java.util.concurrent.TimeoutException;
/*      */ import java.util.concurrent.atomic.AtomicInteger;
/*      */ import net.rubyeye.xmemcached.auth.AuthInfo;
/*      */ import net.rubyeye.xmemcached.buffer.BufferAllocator;
/*      */ import net.rubyeye.xmemcached.buffer.SimpleBufferAllocator;
/*      */ import net.rubyeye.xmemcached.codec.MemcachedCodecFactory;
/*      */ import net.rubyeye.xmemcached.command.Command;
/*      */ import net.rubyeye.xmemcached.command.CommandType;
/*      */ import net.rubyeye.xmemcached.command.ServerAddressAware;
/*      */ import net.rubyeye.xmemcached.command.TextCommandFactory;
/*      */ import net.rubyeye.xmemcached.command.binary.BinaryGetMultiCommand;
/*      */ import net.rubyeye.xmemcached.exception.MemcachedException;
/*      */ import net.rubyeye.xmemcached.impl.ArrayMemcachedSessionLocator;
/*      */ import net.rubyeye.xmemcached.impl.ClosedMemcachedTCPSession;
/*      */ import net.rubyeye.xmemcached.impl.DefaultKeyProvider;
/*      */ import net.rubyeye.xmemcached.impl.KeyIteratorImpl;
/*      */ import net.rubyeye.xmemcached.impl.MemcachedClientStateListenerAdapter;
/*      */ import net.rubyeye.xmemcached.impl.MemcachedConnector;
/*      */ import net.rubyeye.xmemcached.impl.MemcachedHandler;
/*      */ import net.rubyeye.xmemcached.impl.MemcachedTCPSession;
/*      */ import net.rubyeye.xmemcached.impl.ReconnectRequest;
/*      */ import net.rubyeye.xmemcached.monitor.Constants;
/*      */ import net.rubyeye.xmemcached.monitor.MemcachedClientNameHolder;
/*      */ import net.rubyeye.xmemcached.monitor.XMemcachedMbeanServer;
/*      */ import net.rubyeye.xmemcached.networking.Connector;
/*      */ import net.rubyeye.xmemcached.networking.MemcachedSession;
/*      */ import net.rubyeye.xmemcached.transcoders.CachedData;
/*      */ import net.rubyeye.xmemcached.transcoders.SerializingTranscoder;
/*      */ import net.rubyeye.xmemcached.transcoders.Transcoder;
/*      */ import net.rubyeye.xmemcached.utils.AddrUtil;
/*      */ import net.rubyeye.xmemcached.utils.ByteUtils;
/*      */ import net.rubyeye.xmemcached.utils.InetSocketAddressWrapper;
/*      */ import net.rubyeye.xmemcached.utils.Protocol;
/*      */ import org.slf4j.Logger;
/*      */ import org.slf4j.LoggerFactory;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class XMemcachedClient
/*      */   implements XMemcachedClientMBean, MemcachedClient
/*      */ {
/*   88 */   private static final Logger log = LoggerFactory.getLogger(XMemcachedClient.class);
/*      */   
/*      */   protected MemcachedSessionLocator sessionLocator;
/*      */   
/*      */   private volatile boolean shutdown;
/*      */   protected MemcachedConnector connector;
/*      */   private Transcoder transcoder;
/*      */   private boolean sanitizeKeys;
/*      */   private MemcachedHandler memcachedHandler;
/*      */   protected CommandFactory commandFactory;
/*   98 */   private long opTimeout = 5000L;
/*   99 */   private long connectTimeout = 60000L;
/*  100 */   protected int connectionPoolSize = 1;
/*  101 */   protected int maxQueuedNoReplyOperations = DEFAULT_MAX_QUEUED_NOPS;
/*      */   
/*  103 */   protected final AtomicInteger serverOrderCount = new AtomicInteger();
/*      */   
/*  105 */   private Map<InetSocketAddress, AuthInfo> authInfoMap = new HashMap();
/*      */   
/*      */   private String name;
/*      */   
/*      */   private volatile boolean failureMode;
/*      */   
/*  111 */   private final CopyOnWriteArrayList<MemcachedClientStateListenerAdapter> stateListenerAdapters = new CopyOnWriteArrayList();
/*      */   private Thread shutdownHookThread;
/*  113 */   private volatile boolean isHutdownHookCalled = false;
/*      */   
/*      */ 
/*  116 */   private KeyProvider keyProvider = DefaultKeyProvider.INSTANCE;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void setMergeFactor(int mergeFactor)
/*      */   {
/*  124 */     if (mergeFactor < 0) {
/*  125 */       throw new IllegalArgumentException("mergeFactor<0");
/*      */     }
/*  127 */     this.connector.setMergeFactor(mergeFactor);
/*      */   }
/*      */   
/*      */   public KeyProvider getKeyProvider() {
/*  131 */     return this.keyProvider;
/*      */   }
/*      */   
/*      */   public void setKeyProvider(KeyProvider keyProvider) {
/*  135 */     if (keyProvider == null)
/*  136 */       throw new IllegalArgumentException("Null key provider");
/*  137 */     this.keyProvider = keyProvider;
/*      */   }
/*      */   
/*      */   public final MemcachedSessionLocator getSessionLocator() {
/*  141 */     return this.sessionLocator;
/*      */   }
/*      */   
/*      */   public final CommandFactory getCommandFactory() {
/*  145 */     return this.commandFactory;
/*      */   }
/*      */   
/*      */   public String getName() {
/*  149 */     return this.name;
/*      */   }
/*      */   
/*      */   public void setName(String name) {
/*  153 */     this.name = name;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public long getConnectTimeout()
/*      */   {
/*  162 */     return this.connectTimeout;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setConnectTimeout(long connectTimeout)
/*      */   {
/*  171 */     if (connectTimeout < 0L) {
/*  172 */       throw new IllegalArgumentException("connectTimeout<0");
/*      */     }
/*  174 */     this.connectTimeout = connectTimeout;
/*      */   }
/*      */   
/*      */   public void setEnableHeartBeat(boolean enableHeartBeat) {
/*  178 */     this.memcachedHandler.setEnableHeartBeat(enableHeartBeat);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final long getOpTimeout()
/*      */   {
/*  187 */     return this.opTimeout;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void setOpTimeout(long opTimeout)
/*      */   {
/*  196 */     if (opTimeout < 0L) {
/*  197 */       throw new IllegalArgumentException("opTimeout<0");
/*      */     }
/*  199 */     this.opTimeout = opTimeout;
/*      */   }
/*      */   
/*      */   public void setHealSessionInterval(long healConnectionInterval) {
/*  203 */     if (null != this.connector) {
/*  204 */       this.connector.setHealSessionInterval(healConnectionInterval);
/*      */     }
/*      */   }
/*      */   
/*      */   public long getHealSessionInterval()
/*      */   {
/*  210 */     if (null != this.connector) {
/*  211 */       return this.connector.getHealSessionInterval();
/*      */     }
/*  213 */     return -1L;
/*      */   }
/*      */   
/*      */   public Map<InetSocketAddress, AuthInfo> getAuthInfoMap() {
/*  217 */     return this.authInfoMap;
/*      */   }
/*      */   
/*      */   public void setAuthInfoMap(Map<InetSocketAddress, AuthInfo> map) {
/*  221 */     this.authInfoMap = map;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final Connector getConnector()
/*      */   {
/*  230 */     return this.connector;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void setOptimizeMergeBuffer(boolean optimizeMergeBuffer)
/*      */   {
/*  240 */     this.connector.setOptimizeMergeBuffer(optimizeMergeBuffer);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean isShutdown()
/*      */   {
/*  249 */     return this.shutdown;
/*      */   }
/*      */   
/*      */ 
/*      */   private final <T> GetsResponse<T> gets0(String key, byte[] keyBytes, Transcoder<T> transcoder)
/*      */     throws MemcachedException, TimeoutException, InterruptedException
/*      */   {
/*  256 */     GetsResponse<T> result = (GetsResponse)fetch0(key, keyBytes, CommandType.GETS_ONE, this.opTimeout, transcoder);
/*      */     
/*  258 */     return result;
/*      */   }
/*      */   
/*      */   private final void sendCommand(Command cmd) throws MemcachedException {
/*  262 */     if (this.shutdown) {
/*  263 */       throw new MemcachedException("Xmemcached is stopped");
/*      */     }
/*  265 */     this.connector.send(cmd);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public XMemcachedClient(String server, int port)
/*      */     throws IOException
/*      */   {
/*  279 */     this(server, port, 1);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public XMemcachedClient(String host, int port, int weight)
/*      */     throws IOException
/*      */   {
/*  296 */     if (weight <= 0) {
/*  297 */       throw new IllegalArgumentException("weight<=0");
/*      */     }
/*  299 */     checkServerPort(host, port);
/*  300 */     buildConnector(new ArrayMemcachedSessionLocator(), new SimpleBufferAllocator(), XMemcachedClientBuilder.getDefaultConfiguration(), XMemcachedClientBuilder.getDefaultSocketOptions(), new TextCommandFactory(), new SerializingTranscoder());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  305 */     start0();
/*  306 */     connect(new InetSocketAddressWrapper(newSocketAddress(host, port), this.serverOrderCount.incrementAndGet(), weight, null));
/*      */   }
/*      */   
/*      */ 
/*      */   protected InetSocketAddress newSocketAddress(String server, int port)
/*      */   {
/*  312 */     return new InetSocketAddress(server, port);
/*      */   }
/*      */   
/*      */   private void checkServerPort(String server, int port) {
/*  316 */     if ((server == null) || (server.length() == 0)) {
/*  317 */       throw new IllegalArgumentException();
/*      */     }
/*  319 */     if (port <= 0) {
/*  320 */       throw new IllegalArgumentException();
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void addServer(String server, int port)
/*      */     throws IOException
/*      */   {
/*  332 */     addServer(server, port, 1);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void addServer(String server, int port, int weight)
/*      */     throws IOException
/*      */   {
/*  345 */     if (weight <= 0) {
/*  346 */       throw new IllegalArgumentException("weight<=0");
/*      */     }
/*  348 */     checkServerPort(server, port);
/*  349 */     connect(new InetSocketAddressWrapper(newSocketAddress(server, port), this.serverOrderCount.incrementAndGet(), weight, null));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void addServer(InetSocketAddress inetSocketAddress)
/*      */     throws IOException
/*      */   {
/*  362 */     addServer(inetSocketAddress, 1);
/*      */   }
/*      */   
/*      */   public final void addServer(InetSocketAddress inetSocketAddress, int weight) throws IOException
/*      */   {
/*  367 */     if (inetSocketAddress == null) {
/*  368 */       throw new IllegalArgumentException("Null InetSocketAddress");
/*      */     }
/*  370 */     if (weight <= 0) {
/*  371 */       throw new IllegalArgumentException("weight<=0");
/*      */     }
/*  373 */     connect(new InetSocketAddressWrapper(inetSocketAddress, this.serverOrderCount.incrementAndGet(), weight, null));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void addServer(String hostList)
/*      */     throws IOException
/*      */   {
/*  383 */     Map<InetSocketAddress, InetSocketAddress> addresses = AddrUtil.getAddressMap(hostList);
/*      */     
/*  385 */     if ((addresses != null) && (addresses.size() > 0)) {
/*  386 */       for (Map.Entry<InetSocketAddress, InetSocketAddress> entry : addresses.entrySet())
/*      */       {
/*  388 */         InetSocketAddress mainNodeAddr = (InetSocketAddress)entry.getKey();
/*  389 */         InetSocketAddress standbyNodeAddr = (InetSocketAddress)entry.getValue();
/*  390 */         connect(new InetSocketAddressWrapper(mainNodeAddr, this.serverOrderCount.incrementAndGet(), 1, null));
/*      */         
/*  392 */         if (standbyNodeAddr != null) {
/*  393 */           connect(new InetSocketAddressWrapper(standbyNodeAddr, this.serverOrderCount.incrementAndGet(), 1, mainNodeAddr));
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   public void addOneServerWithWeight(String server, int weight)
/*      */     throws IOException
/*      */   {
/*  403 */     Map<InetSocketAddress, InetSocketAddress> addresses = AddrUtil.getAddressMap(server);
/*      */     
/*  405 */     if (addresses == null) {
/*  406 */       throw new IllegalArgumentException("Null Server");
/*      */     }
/*  408 */     if (addresses.size() != 1) {
/*  409 */       throw new IllegalArgumentException("Please add one server at one time");
/*      */     }
/*      */     
/*  412 */     if (weight <= 0) {
/*  413 */       throw new IllegalArgumentException("weight<=0");
/*      */     }
/*  415 */     if ((addresses != null) && (addresses.size() > 0)) {
/*  416 */       for (Map.Entry<InetSocketAddress, InetSocketAddress> entry : addresses.entrySet())
/*      */       {
/*  418 */         InetSocketAddress mainNodeAddr = (InetSocketAddress)entry.getKey();
/*  419 */         InetSocketAddress standbyNodeAddr = (InetSocketAddress)entry.getValue();
/*  420 */         connect(new InetSocketAddressWrapper(mainNodeAddr, this.serverOrderCount.incrementAndGet(), 1, null));
/*      */         
/*  422 */         if (standbyNodeAddr != null) {
/*  423 */           connect(new InetSocketAddressWrapper(standbyNodeAddr, this.serverOrderCount.incrementAndGet(), 1, mainNodeAddr));
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final List<String> getServersDescription()
/*      */   {
/*  438 */     List<String> result = new ArrayList();
/*  439 */     for (Session session : this.connector.getSessionSet()) {
/*  440 */       InetSocketAddress socketAddress = session.getRemoteSocketAddress();
/*  441 */       int weight = ((MemcachedSession)session).getInetSocketAddressWrapper().getWeight();
/*      */       
/*  443 */       result.add(SystemUtils.getRawAddress(socketAddress) + ":" + socketAddress.getPort() + "(weight=" + weight + ")");
/*      */     }
/*      */     
/*  446 */     return result;
/*      */   }
/*      */   
/*      */   public final void setServerWeight(String server, int weight) {
/*  450 */     InetSocketAddress socketAddress = AddrUtil.getOneAddress(server);
/*  451 */     Queue<Session> sessionQueue = this.connector.getSessionByAddress(socketAddress);
/*      */     
/*  453 */     if (sessionQueue == null) {
/*  454 */       throw new IllegalArgumentException("There is no server " + server);
/*      */     }
/*  456 */     for (Session session : sessionQueue) {
/*  457 */       if (session != null) {
/*  458 */         ((MemcachedTCPSession)session).getInetSocketAddressWrapper().setWeight(weight);
/*      */       }
/*      */     }
/*      */     
/*  462 */     this.connector.updateSessions();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void removeServer(String hostList)
/*      */   {
/*  472 */     List<InetSocketAddress> addresses = AddrUtil.getAddresses(hostList);
/*  473 */     if ((addresses != null) && (addresses.size() > 0)) {
/*  474 */       for (InetSocketAddress address : addresses)
/*      */       {
/*  476 */         Queue<Session> sessionQueue = this.connector.getSessionByAddress(address);
/*      */         
/*  478 */         if (sessionQueue != null) {
/*  479 */           for (Session session : sessionQueue) {
/*  480 */             if (session != null)
/*      */             {
/*  482 */               ((MemcachedSession)session).setAllowReconnect(false);
/*      */               
/*      */ 
/*  485 */               ((MemcachedSession)session).quit();
/*      */             }
/*      */           }
/*      */         }
/*      */         
/*  490 */         List<Session> standBySession = this.connector.getStandbySessionListByMainNodeAddr(address);
/*      */         
/*  492 */         if (standBySession != null) {
/*  493 */           for (Session session : standBySession) {
/*  494 */             this.connector.removeReconnectRequest(session.getRemoteSocketAddress());
/*      */             
/*  496 */             if (session != null)
/*      */             {
/*  498 */               ((MemcachedSession)session).setAllowReconnect(false);
/*      */               
/*      */ 
/*  501 */               ((MemcachedSession)session).quit();
/*      */             }
/*      */           }
/*      */         }
/*  505 */         this.connector.removeReconnectRequest(address);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   protected void checkSocketAddress(InetSocketAddress address) {}
/*      */   
/*      */ 
/*      */ 
/*      */   private void connect(InetSocketAddressWrapper inetSocketAddressWrapper)
/*      */     throws IOException
/*      */   {
/*  519 */     InetSocketAddress inetSocketAddress = inetSocketAddressWrapper.getInetSocketAddress();
/*      */     
/*  521 */     checkSocketAddress(inetSocketAddress);
/*  522 */     for (int i = 0; i < this.connectionPoolSize; i++) {
/*  523 */       Future<Boolean> future = null;
/*  524 */       boolean connected = false;
/*  525 */       Throwable throwable = null;
/*      */       try {
/*  527 */         future = this.connector.connect(inetSocketAddressWrapper);
/*      */         
/*  529 */         if (!((Boolean)future.get(this.connectTimeout, TimeUnit.MILLISECONDS)).booleanValue()) {
/*  530 */           log.error("connect to " + SystemUtils.getRawAddress(inetSocketAddress) + ":" + inetSocketAddress.getPort() + " fail");
/*      */         }
/*      */         else
/*      */         {
/*  534 */           connected = true;
/*      */         }
/*      */       } catch (InterruptedException e) {
/*  537 */         Thread.currentThread().interrupt();
/*      */       } catch (ExecutionException e) {
/*  539 */         throwable = e;
/*  540 */         log.error("connect to " + SystemUtils.getRawAddress(inetSocketAddress) + ":" + inetSocketAddress.getPort() + " error", e);
/*      */ 
/*      */       }
/*      */       catch (TimeoutException e)
/*      */       {
/*      */ 
/*  546 */         throwable = e;
/*  547 */         log.error("connect to " + SystemUtils.getRawAddress(inetSocketAddress) + ":" + inetSocketAddress.getPort() + " timeout", e);
/*      */ 
/*      */       }
/*      */       catch (Exception e)
/*      */       {
/*      */ 
/*  553 */         throwable = e;
/*  554 */         log.error("connect to " + SystemUtils.getRawAddress(inetSocketAddress) + ":" + inetSocketAddress.getPort() + " error", e);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  562 */       if (!connected) {
/*  563 */         if (future != null) {
/*  564 */           future.cancel(true);
/*      */         }
/*      */         
/*  567 */         if (this.failureMode) {
/*  568 */           this.connector.addSession(new ClosedMemcachedTCPSession(inetSocketAddressWrapper));
/*      */         }
/*      */         
/*  571 */         this.connector.addToWatingQueue(new ReconnectRequest(inetSocketAddressWrapper, 0, getHealSessionInterval()));
/*      */         
/*      */ 
/*  574 */         log.error("Connect to " + SystemUtils.getRawAddress(inetSocketAddress) + ":" + inetSocketAddress.getPort() + " fail", throwable);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final <T> Object fetch0(String key, byte[] keyBytes, CommandType cmdType, long timeout, Transcoder<T> transcoder)
/*      */     throws InterruptedException, TimeoutException, MemcachedException, MemcachedException
/*      */   {
/*  589 */     Command command = this.commandFactory.createGetCommand(key, keyBytes, cmdType, this.transcoder);
/*      */     
/*  591 */     sendCommand(command);
/*  592 */     latchWait(command, timeout);
/*  593 */     command.getIoBuffer().free();
/*  594 */     checkException(command);
/*  595 */     CachedData data = (CachedData)command.getResult();
/*  596 */     if (data == null) {
/*  597 */       return null;
/*      */     }
/*  599 */     if (transcoder == null) {
/*  600 */       transcoder = this.transcoder;
/*      */     }
/*  602 */     if (cmdType == CommandType.GETS_ONE) {
/*  603 */       return new GetsResponse(data.getCas(), transcoder.decode(data));
/*      */     }
/*  605 */     return transcoder.decode(data);
/*      */   }
/*      */   
/*      */   private final void start0() throws IOException
/*      */   {
/*  610 */     registerMBean();
/*  611 */     startConnector();
/*  612 */     MemcachedClientNameHolder.clear();
/*      */   }
/*      */   
/*      */   private final void startConnector() throws IOException {
/*  616 */     if (this.shutdown) {
/*  617 */       this.shutdown = false;
/*  618 */       this.connector.start();
/*  619 */       this.memcachedHandler.start();
/*  620 */       this.shutdownHookThread = new Thread()
/*      */       {
/*      */         public void run() {
/*      */           try {
/*  624 */             XMemcachedClient.this.isHutdownHookCalled = true;
/*  625 */             XMemcachedClient.this.shutdown();
/*      */           } catch (IOException e) {
/*  627 */             XMemcachedClient.log.error("Shutdown XMemcachedClient error", e);
/*      */           }
/*      */         }
/*  630 */       };
/*  631 */       Runtime.getRuntime().addShutdownHook(this.shutdownHookThread);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   void setMaxQueuedNoReplyOperations(int maxQueuedNoReplyOperations)
/*      */   {
/*  641 */     if (maxQueuedNoReplyOperations <= 1)
/*  642 */       throw new IllegalArgumentException("maxQueuedNoReplyOperations<=1");
/*  643 */     this.maxQueuedNoReplyOperations = maxQueuedNoReplyOperations;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void buildConnector(MemcachedSessionLocator locator, BufferAllocator bufferAllocator, Configuration configuration, Map<SocketOption, Object> socketOptions, CommandFactory commandFactory, Transcoder transcoder)
/*      */   {
/*  651 */     if (locator == null) {
/*  652 */       locator = new ArrayMemcachedSessionLocator();
/*      */     }
/*      */     
/*  655 */     if (bufferAllocator == null) {
/*  656 */       bufferAllocator = new SimpleBufferAllocator();
/*      */     }
/*  658 */     if (configuration == null) {
/*  659 */       configuration = XMemcachedClientBuilder.getDefaultConfiguration();
/*      */     }
/*  661 */     if (transcoder == null) {
/*  662 */       transcoder = new SerializingTranscoder();
/*      */     }
/*  664 */     if (commandFactory == null) {
/*  665 */       commandFactory = new TextCommandFactory();
/*      */     }
/*  667 */     if (this.name == null) {
/*  668 */       this.name = ("MemcachedClient-" + Constants.MEMCACHED_CLIENT_COUNTER.getAndIncrement());
/*      */       
/*  670 */       MemcachedClientNameHolder.setName(this.name);
/*      */     }
/*  672 */     this.commandFactory = commandFactory;
/*  673 */     ByteUtils.setProtocol(this.commandFactory.getProtocol());
/*  674 */     log.warn("XMemcachedClient use " + this.commandFactory.getProtocol().name() + " protocol");
/*      */     
/*  676 */     this.commandFactory.setBufferAllocator(bufferAllocator);
/*  677 */     this.shutdown = true;
/*  678 */     this.transcoder = transcoder;
/*  679 */     this.sessionLocator = locator;
/*  680 */     this.connector = newConnector(bufferAllocator, configuration, this.sessionLocator, this.commandFactory, this.connectionPoolSize, this.maxQueuedNoReplyOperations);
/*      */     
/*      */ 
/*  683 */     this.memcachedHandler = new MemcachedHandler(this);
/*  684 */     this.connector.setHandler(this.memcachedHandler);
/*  685 */     this.connector.setCodecFactory(new MemcachedCodecFactory());
/*  686 */     this.connector.setSessionTimeout(-1L);
/*  687 */     this.connector.setSocketOptions(socketOptions);
/*  688 */     if (isFailureMode()) {
/*  689 */       log.warn("XMemcachedClient in failure mode.");
/*      */     }
/*  691 */     this.connector.setFailureMode(this.failureMode);
/*  692 */     this.sessionLocator.setFailureMode(this.failureMode);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected MemcachedConnector newConnector(BufferAllocator bufferAllocator, Configuration configuration, MemcachedSessionLocator memcachedSessionLocator, CommandFactory commandFactory, int poolSize, int maxQueuedNoReplyOperations)
/*      */   {
/*  701 */     configuration.setDispatchMessageThreadCount(0);
/*  702 */     return new MemcachedConnector(configuration, memcachedSessionLocator, bufferAllocator, commandFactory, poolSize, maxQueuedNoReplyOperations);
/*      */   }
/*      */   
/*      */ 
/*      */   private final void registerMBean()
/*      */   {
/*  708 */     if (this.shutdown) {
/*  709 */       XMemcachedMbeanServer.getInstance().registMBean(this, getClass().getPackage().getName() + ":type=" + getClass().getSimpleName() + "-" + MemcachedClientNameHolder.getName());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setOptimizeGet(boolean optimizeGet)
/*      */   {
/*  718 */     this.connector.setOptimizeGet(optimizeGet);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void setBufferAllocator(BufferAllocator bufferAllocator)
/*      */   {
/*  729 */     this.connector.setBufferAllocator(bufferAllocator);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public XMemcachedClient(InetSocketAddress inetSocketAddress, int weight)
/*      */     throws IOException
/*      */   {
/*  742 */     if (inetSocketAddress == null) {
/*  743 */       throw new IllegalArgumentException("Null InetSocketAddress");
/*      */     }
/*      */     
/*  746 */     if (weight <= 0) {
/*  747 */       throw new IllegalArgumentException("weight<=0");
/*      */     }
/*  749 */     buildConnector(new ArrayMemcachedSessionLocator(), new SimpleBufferAllocator(), XMemcachedClientBuilder.getDefaultConfiguration(), XMemcachedClientBuilder.getDefaultSocketOptions(), new TextCommandFactory(), new SerializingTranscoder());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  754 */     start0();
/*  755 */     connect(new InetSocketAddressWrapper(inetSocketAddress, this.serverOrderCount.incrementAndGet(), weight, null));
/*      */   }
/*      */   
/*      */   public XMemcachedClient(InetSocketAddress inetSocketAddress)
/*      */     throws IOException
/*      */   {
/*  761 */     this(inetSocketAddress, 1);
/*      */   }
/*      */   
/*      */   public XMemcachedClient() throws IOException
/*      */   {
/*  766 */     buildConnector(new ArrayMemcachedSessionLocator(), new SimpleBufferAllocator(), XMemcachedClientBuilder.getDefaultConfiguration(), XMemcachedClientBuilder.getDefaultSocketOptions(), new TextCommandFactory(), new SerializingTranscoder());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  771 */     start0();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   XMemcachedClient(MemcachedSessionLocator locator, BufferAllocator allocator, Configuration conf, Map<SocketOption, Object> socketOptions, CommandFactory commandFactory, Transcoder transcoder, Map<InetSocketAddress, InetSocketAddress> addressMap, List<MemcachedClientStateListener> stateListeners, Map<InetSocketAddress, AuthInfo> map, int poolSize, long connectTimeout, String name, boolean failureMode)
/*      */     throws IOException
/*      */   {
/*  797 */     setConnectTimeout(connectTimeout);
/*  798 */     setFailureMode(failureMode);
/*  799 */     setName(name);
/*  800 */     optimiezeSetReadThreadCount(conf, addressMap == null ? 0 : addressMap.size());
/*      */     
/*  802 */     buildConnector(locator, allocator, conf, socketOptions, commandFactory, transcoder);
/*      */     
/*  804 */     if (stateListeners != null) {
/*  805 */       for (MemcachedClientStateListener stateListener : stateListeners) {
/*  806 */         addStateListener(stateListener);
/*      */       }
/*      */     }
/*  809 */     setAuthInfoMap(map);
/*  810 */     setConnectionPoolSize(poolSize);
/*  811 */     start0();
/*  812 */     if (addressMap != null) {
/*  813 */       for (Map.Entry<InetSocketAddress, InetSocketAddress> entry : addressMap.entrySet())
/*      */       {
/*  815 */         InetSocketAddress mainNodeAddr = (InetSocketAddress)entry.getKey();
/*  816 */         InetSocketAddress standbyNodeAddr = (InetSocketAddress)entry.getValue();
/*  817 */         connect(new InetSocketAddressWrapper(mainNodeAddr, this.serverOrderCount.incrementAndGet(), 1, null));
/*      */         
/*  819 */         if (standbyNodeAddr != null) {
/*  820 */           connect(new InetSocketAddressWrapper(standbyNodeAddr, this.serverOrderCount.incrementAndGet(), 1, mainNodeAddr));
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   XMemcachedClient(MemcachedSessionLocator locator, BufferAllocator allocator, Configuration conf, Map<SocketOption, Object> socketOptions, CommandFactory commandFactory, Transcoder transcoder, Map<InetSocketAddress, InetSocketAddress> addressMap, int[] weights, List<MemcachedClientStateListener> stateListeners, Map<InetSocketAddress, AuthInfo> infoMap, int poolSize, long connectTimeout, String name, boolean failureMode)
/*      */     throws IOException
/*      */   {
/*  853 */     setConnectTimeout(connectTimeout);
/*  854 */     setFailureMode(failureMode);
/*  855 */     setName(name);
/*  856 */     if ((weights == null) && (addressMap != null)) {
/*  857 */       throw new IllegalArgumentException("Null weights");
/*      */     }
/*  859 */     if ((weights != null) && (addressMap == null)) {
/*  860 */       throw new IllegalArgumentException("Null addressList");
/*      */     }
/*      */     
/*  863 */     if (weights != null) {
/*  864 */       for (int weight : weights) {
/*  865 */         if (weight <= 0) {
/*  866 */           throw new IllegalArgumentException("Some weights<=0");
/*      */         }
/*      */       }
/*      */     }
/*  870 */     if ((weights != null) && (addressMap != null) && (weights.length < addressMap.size()))
/*      */     {
/*  872 */       throw new IllegalArgumentException("weights.length is less than addressList.size()");
/*      */     }
/*      */     
/*  875 */     optimiezeSetReadThreadCount(conf, addressMap == null ? 0 : addressMap.size());
/*      */     
/*  877 */     buildConnector(locator, allocator, conf, socketOptions, commandFactory, transcoder);
/*      */     
/*  879 */     if (stateListeners != null) {
/*  880 */       for (MemcachedClientStateListener stateListener : stateListeners) {
/*  881 */         addStateListener(stateListener);
/*      */       }
/*      */     }
/*  884 */     setAuthInfoMap(infoMap);
/*  885 */     setConnectionPoolSize(poolSize);
/*  886 */     start0();
/*  887 */     int i; if ((addressMap != null) && (weights != null)) {
/*  888 */       i = 0;
/*  889 */       for (Map.Entry<InetSocketAddress, InetSocketAddress> entry : addressMap.entrySet())
/*      */       {
/*  891 */         InetSocketAddress mainNodeAddr = (InetSocketAddress)entry.getKey();
/*  892 */         InetSocketAddress standbyNodeAddr = (InetSocketAddress)entry.getValue();
/*  893 */         connect(new InetSocketAddressWrapper(mainNodeAddr, this.serverOrderCount.incrementAndGet(), weights[i], null));
/*      */         
/*      */ 
/*  896 */         if (standbyNodeAddr != null) {
/*  897 */           connect(new InetSocketAddressWrapper(standbyNodeAddr, this.serverOrderCount.incrementAndGet(), weights[i], mainNodeAddr));
/*      */         }
/*      */         
/*      */ 
/*  901 */         i++;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private final void optimiezeSetReadThreadCount(Configuration conf, int addressCount)
/*      */   {
/*  908 */     if ((conf != null) && (addressCount > 1) && 
/*  909 */       (isLinuxPlatform()) && (conf.getReadThreadCount() == 0))
/*      */     {
/*  911 */       int threadCount = 2 * SystemUtils.getSystemThreadCount();
/*  912 */       conf.setReadThreadCount(addressCount > threadCount ? threadCount : addressCount);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private final boolean isLinuxPlatform()
/*      */   {
/*  919 */     String osName = System.getProperty("os.name");
/*  920 */     if ((osName != null) && (osName.toLowerCase().indexOf("linux") >= 0)) {
/*  921 */       return true;
/*      */     }
/*  923 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public XMemcachedClient(List<InetSocketAddress> addressList)
/*      */     throws IOException
/*      */   {
/*  936 */     if ((addressList == null) || (addressList.isEmpty())) {
/*  937 */       throw new IllegalArgumentException("Empty address list");
/*      */     }
/*  939 */     BufferAllocator simpleBufferAllocator = new SimpleBufferAllocator();
/*  940 */     buildConnector(new ArrayMemcachedSessionLocator(), simpleBufferAllocator, XMemcachedClientBuilder.getDefaultConfiguration(), XMemcachedClientBuilder.getDefaultSocketOptions(), new TextCommandFactory(), new SerializingTranscoder());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  945 */     start0();
/*  946 */     for (InetSocketAddress inetSocketAddress : addressList) {
/*  947 */       connect(new InetSocketAddressWrapper(inetSocketAddress, this.serverOrderCount.incrementAndGet(), 1, null));
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> T get(String key, long timeout, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/*  963 */     return (T)get0(key, timeout, CommandType.GET_ONE, transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> T get(String key, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/*  974 */     return (T)get(key, timeout, this.transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> T get(String key, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/*  985 */     return (T)get(key, this.opTimeout, transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> T get(String key)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/*  996 */     return (T)get(key, this.opTimeout);
/*      */   }
/*      */   
/*      */   private <T> Object get0(String key, long timeout, CommandType cmdType, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1002 */     key = preProcessKey(key);
/* 1003 */     byte[] keyBytes = ByteUtils.getBytes(key);
/* 1004 */     ByteUtils.checkKey(keyBytes);
/* 1005 */     return fetch0(key, keyBytes, cmdType, timeout, transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> GetsResponse<T> gets(String key, long timeout, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1018 */     return (GetsResponse)get0(key, timeout, CommandType.GETS_ONE, transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> GetsResponse<T> gets(String key)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1029 */     return gets(key, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> GetsResponse<T> gets(String key, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1040 */     return gets(key, timeout, this.transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> GetsResponse<T> gets(String key, Transcoder transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1053 */     return gets(key, this.opTimeout, transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> Map<String, T> get(Collection<String> keyCollections, long timeout, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1066 */     return getMulti0(keyCollections, timeout, CommandType.GET_MANY, transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> Map<String, T> get(Collection<String> keyCollections, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1080 */     return getMulti0(keyCollections, this.opTimeout, CommandType.GET_MANY, transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> Map<String, T> get(Collection<String> keyCollections)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1091 */     return get(keyCollections, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> Map<String, T> get(Collection<String> keyCollections, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1104 */     return get(keyCollections, timeout, this.transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> Map<String, GetsResponse<T>> gets(Collection<String> keyCollections, long timeout, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1118 */     return getMulti0(keyCollections, timeout, CommandType.GETS_MANY, transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> Map<String, GetsResponse<T>> gets(Collection<String> keyCollections)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1130 */     return gets(keyCollections, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> Map<String, GetsResponse<T>> gets(Collection<String> keyCollections, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1143 */     return gets(keyCollections, timeout, this.transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> Map<String, GetsResponse<T>> gets(Collection<String> keyCollections, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1156 */     return gets(keyCollections, this.opTimeout, transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */   private final <T> Map<String, T> getMulti0(Collection<String> keys, long timeout, CommandType cmdType, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1163 */     if ((keys == null) || (keys.size() == 0)) {
/* 1164 */       return null;
/*      */     }
/* 1166 */     Collection<String> keyCollections = keys;
/* 1167 */     if (this.sanitizeKeys) {
/* 1168 */       keyCollections = new ArrayList(keys.size());
/* 1169 */       for (String key : keys) {
/* 1170 */         keyCollections.add(preProcessKey(key));
/*      */       }
/*      */     }
/*      */     List<Command> commands;
/*      */     CountDownLatch latch;
/* 1175 */     if (this.connector.getSessionSet().size() <= 1) {
/* 1176 */       List<Command> commands = new ArrayList(1);
/* 1177 */       CountDownLatch latch = new CountDownLatch(1);
/* 1178 */       commands.add(sendGetMultiCommand(keyCollections, latch, cmdType, transcoder));
/*      */     }
/*      */     else
/*      */     {
/* 1182 */       Collection<List<String>> catalogKeys = catalogKeys(keyCollections);
/*      */       
/* 1184 */       commands = new ArrayList(catalogKeys.size());
/* 1185 */       latch = new CountDownLatch(catalogKeys.size());
/* 1186 */       for (List<String> catalogKeyCollection : catalogKeys) {
/* 1187 */         commands.add(sendGetMultiCommand(catalogKeyCollection, latch, cmdType, transcoder));
/*      */       }
/*      */     }
/*      */     
/* 1191 */     if (!latch.await(timeout, TimeUnit.MILLISECONDS)) {
/* 1192 */       for (Command getCmd : commands) {
/* 1193 */         getCmd.cancel();
/*      */       }
/* 1195 */       throw new TimeoutException("Timed out waiting for operation");
/*      */     }
/* 1197 */     return reduceResult(cmdType, transcoder, commands);
/*      */   }
/*      */   
/*      */ 
/*      */   private <T> Map<String, T> reduceResult(CommandType cmdType, Transcoder<T> transcoder, List<Command> commands)
/*      */     throws MemcachedException
/*      */   {
/* 1204 */     Map<String, T> result = new HashMap(commands.size());
/* 1205 */     for (Command getCmd : commands) {
/* 1206 */       getCmd.getIoBuffer().free();
/* 1207 */       checkException(getCmd);
/* 1208 */       Map<String, CachedData> map = (Map)getCmd.getResult();
/*      */       
/* 1210 */       if (cmdType == CommandType.GET_MANY) {
/* 1211 */         Iterator<Map.Entry<String, CachedData>> it = map.entrySet().iterator();
/*      */         
/* 1213 */         while (it.hasNext()) {
/* 1214 */           Map.Entry<String, CachedData> entry = (Map.Entry)it.next();
/* 1215 */           if (this.sanitizeKeys) {
/* 1216 */             result.put(decodeKey((String)entry.getKey()), transcoder.decode((CachedData)entry.getValue()));
/*      */           }
/*      */           else {
/* 1219 */             result.put(entry.getKey(), transcoder.decode((CachedData)entry.getValue()));
/*      */           }
/*      */         }
/*      */       }
/*      */       else
/*      */       {
/* 1225 */         Iterator<Map.Entry<String, CachedData>> it = map.entrySet().iterator();
/*      */         
/* 1227 */         while (it.hasNext()) {
/* 1228 */           Map.Entry<String, CachedData> entry = (Map.Entry)it.next();
/* 1229 */           GetsResponse getsResponse = new GetsResponse(((CachedData)entry.getValue()).getCas(), transcoder.decode((CachedData)entry.getValue()));
/*      */           
/*      */ 
/* 1232 */           result.put(entry.getKey(), getsResponse);
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/* 1238 */     return result;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final Collection<List<String>> catalogKeys(Collection<String> keyCollections)
/*      */   {
/* 1249 */     Map<Session, List<String>> catalogMap = new HashMap();
/*      */     
/* 1251 */     for (String key : keyCollections) {
/* 1252 */       Session index = this.sessionLocator.getSessionByKey(key);
/* 1253 */       if (!catalogMap.containsKey(index)) {
/* 1254 */         List<String> tmpKeys = new ArrayList(100);
/* 1255 */         tmpKeys.add(key);
/* 1256 */         catalogMap.put(index, tmpKeys);
/*      */       } else {
/* 1258 */         ((List)catalogMap.get(index)).add(key);
/*      */       }
/*      */     }
/*      */     
/* 1262 */     Collection<List<String>> catalogKeys = catalogMap.values();
/* 1263 */     return catalogKeys;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final Collection<Map<String, Integer>> catalogKeys(Map<String, Integer> keyExpMap)
/*      */   {
/* 1274 */     Map<Session, Map<String, Integer>> catalogMap = new HashMap();
/* 1275 */     for (Map.Entry<String, Integer> entry : keyExpMap.entrySet()) {
/* 1276 */       String key = (String)entry.getKey();
/* 1277 */       Integer expTime = (Integer)entry.getValue();
/* 1278 */       Session index = this.sessionLocator.getSessionByKey(key);
/* 1279 */       if (!catalogMap.containsKey(index)) {
/* 1280 */         Map<String, Integer> tmpKeyExpMap = new HashMap(100);
/*      */         
/* 1282 */         tmpKeyExpMap.put(key, expTime);
/* 1283 */         catalogMap.put(index, tmpKeyExpMap);
/*      */       } else {
/* 1285 */         ((Map)catalogMap.get(index)).put(key, expTime);
/*      */       }
/*      */     }
/* 1288 */     return catalogMap.values();
/*      */   }
/*      */   
/*      */ 
/*      */   private final <T> Command sendGetMultiCommand(Collection<String> keys, CountDownLatch latch, CommandType cmdType, Transcoder<T> transcoder)
/*      */     throws InterruptedException, TimeoutException, MemcachedException
/*      */   {
/* 1295 */     Command command = this.commandFactory.createGetMultiCommand(keys, latch, cmdType, transcoder);
/*      */     
/* 1297 */     sendCommand(command);
/* 1298 */     return command;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean set(String key, int exp, T value, Transcoder<T> transcoder, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1310 */     key = preProcessKey(key);
/* 1311 */     byte[] keyBytes = checkStoreArguments(key, exp, value);
/* 1312 */     return sendStoreCommand(this.commandFactory.createSetCommand(key, keyBytes, exp, value, false, transcoder), timeout);
/*      */   }
/*      */   
/*      */ 
/*      */   public void setWithNoReply(String key, int exp, Object value)
/*      */     throws InterruptedException, MemcachedException
/*      */   {
/* 1319 */     setWithNoReply(key, exp, value, this.transcoder);
/*      */   }
/*      */   
/*      */   public <T> void setWithNoReply(String key, int exp, T value, Transcoder<T> transcoder)
/*      */     throws InterruptedException, MemcachedException
/*      */   {
/* 1325 */     key = preProcessKey(key);
/* 1326 */     byte[] keyBytes = checkStoreArguments(key, exp, value);
/*      */     try {
/* 1328 */       sendStoreCommand(this.commandFactory.createSetCommand(key, keyBytes, exp, value, true, transcoder), this.opTimeout);
/*      */     }
/*      */     catch (TimeoutException e) {
/* 1331 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */   private final <T> byte[] checkStoreArguments(String key, int exp, T value)
/*      */   {
/* 1337 */     byte[] keyBytes = ByteUtils.getBytes(key);
/* 1338 */     ByteUtils.checkKey(keyBytes);
/* 1339 */     if (value == null) {
/* 1340 */       throw new IllegalArgumentException("value could not be null");
/*      */     }
/* 1342 */     if (exp < 0) {
/* 1343 */       throw new IllegalArgumentException("Expire time must be greater than 0");
/*      */     }
/*      */     
/* 1346 */     return keyBytes;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean set(String key, int exp, Object value)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1357 */     return set(key, exp, value, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean set(String key, int exp, Object value, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1370 */     return set(key, exp, value, this.transcoder, timeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean set(String key, int exp, T value, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1382 */     return set(key, exp, value, transcoder, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean add(String key, int exp, T value, Transcoder<T> transcoder, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1394 */     key = preProcessKey(key);
/* 1395 */     byte[] keyBytes = checkStoreArguments(key, exp, value);
/* 1396 */     return sendStoreCommand(this.commandFactory.createAddCommand(key, keyBytes, exp, value, false, transcoder), timeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean add(String key, int exp, Object value)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1408 */     return add(key, exp, value, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean add(String key, int exp, Object value, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1421 */     return add(key, exp, value, this.transcoder, timeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean add(String key, int exp, T value, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1433 */     return add(key, exp, value, transcoder, this.opTimeout);
/*      */   }
/*      */   
/*      */   public void addWithNoReply(String key, int exp, Object value)
/*      */     throws InterruptedException, MemcachedException
/*      */   {
/* 1439 */     addWithNoReply(key, exp, value, this.transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> void addWithNoReply(String key, int exp, T value, Transcoder<T> transcoder)
/*      */     throws InterruptedException, MemcachedException
/*      */   {
/* 1446 */     key = preProcessKey(key);
/* 1447 */     byte[] keyBytes = checkStoreArguments(key, exp, value);
/*      */     try {
/* 1449 */       sendStoreCommand(this.commandFactory.createAddCommand(key, keyBytes, exp, value, true, transcoder), this.opTimeout);
/*      */     }
/*      */     catch (TimeoutException e) {
/* 1452 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   public void replaceWithNoReply(String key, int exp, Object value)
/*      */     throws InterruptedException, MemcachedException
/*      */   {
/* 1460 */     replaceWithNoReply(key, exp, value, this.transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> void replaceWithNoReply(String key, int exp, T value, Transcoder<T> transcoder)
/*      */     throws InterruptedException, MemcachedException
/*      */   {
/* 1467 */     key = preProcessKey(key);
/* 1468 */     byte[] keyBytes = checkStoreArguments(key, exp, value);
/*      */     try {
/* 1470 */       sendStoreCommand(this.commandFactory.createReplaceCommand(key, keyBytes, exp, value, true, transcoder), this.opTimeout);
/*      */     }
/*      */     catch (TimeoutException e) {
/* 1473 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean replace(String key, int exp, T value, Transcoder<T> transcoder, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1487 */     key = preProcessKey(key);
/* 1488 */     byte[] keyBytes = checkStoreArguments(key, exp, value);
/* 1489 */     return sendStoreCommand(this.commandFactory.createReplaceCommand(key, keyBytes, exp, value, false, transcoder), timeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean replace(String key, int exp, Object value)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1502 */     return replace(key, exp, value, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean replace(String key, int exp, Object value, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1515 */     return replace(key, exp, value, this.transcoder, timeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean replace(String key, int exp, T value, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1527 */     return replace(key, exp, value, transcoder, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean append(String key, Object value)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1538 */     return append(key, value, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean append(String key, Object value, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1550 */     key = preProcessKey(key);
/* 1551 */     byte[] keyBytes = checkStoreArguments(key, 0, value);
/* 1552 */     return sendStoreCommand(this.commandFactory.createAppendCommand(key, keyBytes, value, false, this.transcoder), timeout);
/*      */   }
/*      */   
/*      */   public void appendWithNoReply(String key, Object value)
/*      */     throws InterruptedException, MemcachedException
/*      */   {
/* 1558 */     key = preProcessKey(key);
/* 1559 */     byte[] keyBytes = checkStoreArguments(key, 0, value);
/*      */     try {
/* 1561 */       sendStoreCommand(this.commandFactory.createAppendCommand(key, keyBytes, value, true, this.transcoder), this.opTimeout);
/*      */     }
/*      */     catch (TimeoutException e) {
/* 1564 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean prepend(String key, Object value)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1577 */     return prepend(key, value, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean prepend(String key, Object value, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1589 */     key = preProcessKey(key);
/* 1590 */     byte[] keyBytes = checkStoreArguments(key, 0, value);
/* 1591 */     return sendStoreCommand(this.commandFactory.createPrependCommand(key, keyBytes, value, false, this.transcoder), timeout);
/*      */   }
/*      */   
/*      */   public void prependWithNoReply(String key, Object value)
/*      */     throws InterruptedException, MemcachedException
/*      */   {
/* 1597 */     key = preProcessKey(key);
/* 1598 */     byte[] keyBytes = checkStoreArguments(key, 0, value);
/*      */     try {
/* 1600 */       sendStoreCommand(this.commandFactory.createPrependCommand(key, keyBytes, value, true, this.transcoder), this.opTimeout);
/*      */     }
/*      */     catch (TimeoutException e) {
/* 1603 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean cas(String key, int exp, Object value, long cas)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1616 */     return cas(key, exp, value, this.opTimeout, cas);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean cas(String key, int exp, T value, Transcoder<T> transcoder, long timeout, long cas)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1628 */     key = preProcessKey(key);
/* 1629 */     byte[] keyBytes = checkStoreArguments(key, 0, value);
/* 1630 */     return sendStoreCommand(this.commandFactory.createCASCommand(key, keyBytes, exp, value, cas, false, transcoder), timeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean cas(String key, int exp, Object value, long timeout, long cas)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1644 */     return cas(key, exp, value, this.transcoder, timeout, cas);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean cas(String key, int exp, T value, Transcoder<T> transcoder, long cas)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1656 */     return cas(key, exp, value, transcoder, this.opTimeout, cas);
/*      */   }
/*      */   
/*      */ 
/*      */   private final <T> boolean cas0(String key, int exp, GetsResponse<T> getsResponse, CASOperation<T> operation, Transcoder<T> transcoder, byte[] keyBytes, boolean noreply)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1663 */     if (operation == null) {
/* 1664 */       throw new IllegalArgumentException("CASOperation could not be null");
/*      */     }
/* 1666 */     if (operation.getMaxTries() < 0) {
/* 1667 */       throw new IllegalArgumentException("max tries must be greater than 0");
/*      */     }
/*      */     
/* 1670 */     int tryCount = 0;
/* 1671 */     GetsResponse<T> result = getsResponse;
/* 1672 */     if (result == null) {
/* 1673 */       throw new MemcachedException("Null GetsResponse");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1683 */     while ((tryCount <= operation.getMaxTries()) && (result != null) && (!sendStoreCommand(this.commandFactory.createCASCommand(key, keyBytes, exp, operation.getNewValue(result.getCas(), result.getValue()), result.getCas(), noreply, transcoder), this.opTimeout)) && (!noreply)) {
/* 1684 */       tryCount++;
/* 1685 */       result = gets0(key, keyBytes, transcoder);
/* 1686 */       if (result == null) {
/* 1687 */         throw new MemcachedException("could not gets the value for Key=" + key + " for cas");
/*      */       }
/*      */       
/* 1690 */       if (tryCount > operation.getMaxTries()) {
/* 1691 */         throw new TimeoutException("CAS try times is greater than max");
/*      */       }
/*      */     }
/* 1694 */     return true;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean cas(String key, int exp, CASOperation<T> operation, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1707 */     key = preProcessKey(key);
/* 1708 */     byte[] keyBytes = ByteUtils.getBytes(key);
/* 1709 */     ByteUtils.checkKey(keyBytes);
/* 1710 */     GetsResponse<T> result = gets0(key, keyBytes, transcoder);
/* 1711 */     return cas0(key, exp, result, operation, transcoder, keyBytes, false);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean cas(String key, int exp, GetsResponse<T> getsReponse, CASOperation<T> operation, Transcoder<T> transcoder)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1726 */     key = preProcessKey(key);
/* 1727 */     byte[] keyBytes = ByteUtils.getBytes(key);
/* 1728 */     ByteUtils.checkKey(keyBytes);
/* 1729 */     return cas0(key, exp, getsReponse, operation, transcoder, keyBytes, false);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean cas(String key, int exp, GetsResponse<T> getsReponse, CASOperation<T> operation)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1744 */     return cas(key, exp, getsReponse, operation, this.transcoder);
/*      */   }
/*      */   
/*      */   public <T> void casWithNoReply(String key, CASOperation<T> operation) throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1749 */     casWithNoReply(key, 0, operation);
/*      */   }
/*      */   
/*      */   public <T> void casWithNoReply(String key, GetsResponse<T> getsResponse, CASOperation<T> operation)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1755 */     casWithNoReply(key, 0, getsResponse, operation);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public <T> void casWithNoReply(String key, int exp, CASOperation<T> operation)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1763 */     key = preProcessKey(key);
/* 1764 */     byte[] keyBytes = ByteUtils.getBytes(key);
/* 1765 */     GetsResponse<T> result = gets0(key, keyBytes, this.transcoder);
/* 1766 */     casWithNoReply(key, exp, result, operation);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public <T> void casWithNoReply(String key, int exp, GetsResponse<T> getsReponse, CASOperation<T> operation)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1774 */     key = preProcessKey(key);
/* 1775 */     byte[] keyBytes = ByteUtils.getBytes(key);
/* 1776 */     ByteUtils.checkKey(keyBytes);
/* 1777 */     cas0(key, exp, getsReponse, operation, this.transcoder, keyBytes, true);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean cas(String key, GetsResponse<T> getsReponse, CASOperation<T> operation)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1791 */     return cas(key, 0, getsReponse, operation);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean cas(String key, int exp, CASOperation<T> operation)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1804 */     return cas(key, exp, operation, this.transcoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final <T> boolean cas(String key, CASOperation<T> operation)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1816 */     return cas(key, 0, operation);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean delete(String key, int time)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1826 */     return delete0(key, time, false, this.opTimeout);
/*      */   }
/*      */   
/*      */   public boolean delete(String key, long opTimeout) throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1831 */     return delete0(key, 0, false, opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void deleteWithNoReply(String key, int time)
/*      */     throws InterruptedException, MemcachedException
/*      */   {
/*      */     try
/*      */     {
/* 1845 */       delete0(key, time, true, this.opTimeout);
/*      */     } catch (TimeoutException e) {
/* 1847 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */   public final void deleteWithNoReply(String key) throws InterruptedException, MemcachedException
/*      */   {
/* 1853 */     deleteWithNoReply(key, 0);
/*      */   }
/*      */   
/*      */   private boolean delete0(String key, int time, boolean noreply, long opTimeout)
/*      */     throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 1859 */     key = preProcessKey(key);
/* 1860 */     byte[] keyBytes = ByteUtils.getBytes(key);
/* 1861 */     ByteUtils.checkKey(keyBytes);
/* 1862 */     Command command = this.commandFactory.createDeleteCommand(key, keyBytes, time, noreply);
/*      */     
/* 1864 */     sendCommand(command);
/* 1865 */     if (!command.isNoreply()) {
/* 1866 */       latchWait(command, opTimeout);
/* 1867 */       command.getIoBuffer().free();
/* 1868 */       checkException(command);
/* 1869 */       if (command.getResult() == null) {
/* 1870 */         throw new MemcachedException("Operation fail,may be caused by networking or timeout");
/*      */       }
/*      */     }
/*      */     else {
/* 1874 */       return false;
/*      */     }
/* 1876 */     return ((Boolean)command.getResult()).booleanValue();
/*      */   }
/*      */   
/*      */   void checkException(Command command) throws MemcachedException {
/* 1880 */     if (command.getException() != null) {
/* 1881 */       if ((command.getException() instanceof MemcachedException)) {
/* 1882 */         throw ((MemcachedException)command.getException());
/*      */       }
/* 1884 */       throw new MemcachedException(command.getException());
/*      */     }
/*      */   }
/*      */   
/*      */   public boolean touch(String key, int exp, long opTimeout) throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1890 */     key = preProcessKey(key);
/* 1891 */     byte[] keyBytes = ByteUtils.getBytes(key);
/* 1892 */     ByteUtils.checkKey(keyBytes);
/* 1893 */     CountDownLatch latch = new CountDownLatch(1);
/* 1894 */     Command command = this.commandFactory.createTouchCommand(key, keyBytes, latch, exp, false);
/*      */     
/* 1896 */     sendCommand(command);
/* 1897 */     latchWait(command, opTimeout);
/* 1898 */     command.getIoBuffer().free();
/* 1899 */     checkException(command);
/* 1900 */     if (command.getResult() == null) {
/* 1901 */       throw new MemcachedException("Operation fail,may be caused by networking or timeout");
/*      */     }
/*      */     
/* 1904 */     return ((Boolean)command.getResult()).booleanValue();
/*      */   }
/*      */   
/*      */   public boolean touch(String key, int exp) throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1909 */     return touch(key, exp, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private <T> Map<String, T> getAndTouch(Map<String, Integer> keyExpMap, long opTimeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 1926 */     if ((keyExpMap == null) || (keyExpMap.isEmpty())) {
/* 1927 */       return null;
/*      */     }
/* 1929 */     Map<String, Integer> innerKeyExpMap = keyExpMap;
/* 1930 */     if (this.sanitizeKeys) {
/* 1931 */       innerKeyExpMap = new HashMap(keyExpMap.size());
/* 1932 */       for (Map.Entry<String, Integer> entry : keyExpMap.entrySet()) {
/* 1933 */         innerKeyExpMap.put(preProcessKey((String)entry.getKey()), entry.getValue());
/*      */       }
/*      */     }
/*      */     
/*      */     List<Command> commands;
/*      */     CountDownLatch latch;
/* 1939 */     if (this.connector.getSessionSet().size() <= 1) {
/* 1940 */       List<Command> commands = new ArrayList(1);
/* 1941 */       CountDownLatch latch = new CountDownLatch(1);
/* 1942 */       commands.add(sendMultiGATCommand(innerKeyExpMap, latch));
/*      */     }
/*      */     else {
/* 1945 */       Collection<Map<String, Integer>> catalogKeyExps = catalogKeys(innerKeyExpMap);
/*      */       
/* 1947 */       commands = new ArrayList(catalogKeyExps.size());
/* 1948 */       latch = new CountDownLatch(catalogKeyExps.size());
/* 1949 */       for (Map<String, Integer> catalogKeyExpMap : catalogKeyExps) {
/* 1950 */         commands.add(sendMultiGATCommand(catalogKeyExpMap, latch));
/*      */       }
/*      */     }
/* 1953 */     if (!latch.await(opTimeout, TimeUnit.MILLISECONDS)) {
/* 1954 */       for (Command getCmd : commands) {
/* 1955 */         getCmd.cancel();
/*      */       }
/* 1957 */       throw new TimeoutException("Timed out waiting for operation");
/*      */     }
/* 1959 */     return reduceResult(CommandType.GET_MANY, this.transcoder, commands);
/*      */   }
/*      */   
/*      */   private Command sendMultiGATCommand(Map<String, Integer> innerKeyExpMap, CountDownLatch latch)
/*      */     throws MemcachedException
/*      */   {
/* 1965 */     Iterator<String> it = innerKeyExpMap.keySet().iterator();
/* 1966 */     String key = null;
/* 1967 */     List<IoBuffer> bufferList = new ArrayList();
/* 1968 */     int totalLength = 0;
/* 1969 */     while (it.hasNext()) {
/* 1970 */       key = (String)it.next();
/* 1971 */       if (it.hasNext())
/*      */       {
/* 1973 */         Command command = this.commandFactory.createGetAndTouchCommand(key, ByteUtils.getBytes(key), null, ((Integer)innerKeyExpMap.get(key)).intValue(), true);
/*      */         
/*      */ 
/* 1976 */         command.encode();
/* 1977 */         totalLength += command.getIoBuffer().remaining();
/* 1978 */         bufferList.add(command.getIoBuffer());
/*      */       }
/*      */     }
/*      */     
/* 1982 */     Command lastCommand = this.commandFactory.createGetAndTouchCommand(key, ByteUtils.getBytes(key), null, ((Integer)innerKeyExpMap.get(key)).intValue(), false);
/*      */     
/* 1984 */     lastCommand.encode();
/* 1985 */     bufferList.add(lastCommand.getIoBuffer());
/* 1986 */     totalLength += lastCommand.getIoBuffer().remaining();
/*      */     
/* 1988 */     IoBuffer mergedBuffer = IoBuffer.allocate(totalLength);
/* 1989 */     for (IoBuffer buffer : bufferList) {
/* 1990 */       mergedBuffer.put(buffer.buf());
/*      */     }
/* 1992 */     mergedBuffer.flip();
/*      */     
/* 1994 */     Command resultCommand = new BinaryGetMultiCommand(key, CommandType.GET_MANY, latch);
/*      */     
/* 1996 */     resultCommand.setIoBuffer(mergedBuffer);
/* 1997 */     sendCommand(resultCommand);
/* 1998 */     return resultCommand;
/*      */   }
/*      */   
/*      */   private <T> Map<String, T> getAndTouch(Map<String, Integer> keyExpMap) throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2003 */     return getAndTouch(keyExpMap, this.opTimeout);
/*      */   }
/*      */   
/*      */   public <T> T getAndTouch(String key, int newExp, long opTimeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2009 */     key = preProcessKey(key);
/* 2010 */     byte[] keyBytes = ByteUtils.getBytes(key);
/* 2011 */     ByteUtils.checkKey(keyBytes);
/* 2012 */     CountDownLatch latch = new CountDownLatch(1);
/* 2013 */     Command command = this.commandFactory.createGetAndTouchCommand(key, keyBytes, latch, newExp, false);
/*      */     
/* 2015 */     sendCommand(command);
/* 2016 */     latchWait(command, opTimeout);
/* 2017 */     command.getIoBuffer().free();
/* 2018 */     checkException(command);
/* 2019 */     CachedData data = (CachedData)command.getResult();
/* 2020 */     if (data == null) {
/* 2021 */       return null;
/*      */     }
/* 2023 */     return (T)this.transcoder.decode(data);
/*      */   }
/*      */   
/*      */   public <T> T getAndTouch(String key, int newExp)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2029 */     return (T)getAndTouch(key, newExp, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final long incr(String key, long delta)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2039 */     key = preProcessKey(key);
/* 2040 */     return sendIncrOrDecrCommand(key, delta, 0L, CommandType.INCR, false, this.opTimeout, 0);
/*      */   }
/*      */   
/*      */   public long incr(String key, long delta, long initValue)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2046 */     key = preProcessKey(key);
/* 2047 */     return sendIncrOrDecrCommand(key, delta, initValue, CommandType.INCR, false, this.opTimeout, 0);
/*      */   }
/*      */   
/*      */   public long incr(String key, long delta, long initValue, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2053 */     key = preProcessKey(key);
/* 2054 */     return sendIncrOrDecrCommand(key, delta, initValue, CommandType.INCR, false, timeout, 0);
/*      */   }
/*      */   
/*      */ 
/*      */   public long incr(String key, long delta, long initValue, long timeout, int exp)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2061 */     key = preProcessKey(key);
/* 2062 */     return sendIncrOrDecrCommand(key, delta, initValue, CommandType.INCR, false, timeout, exp);
/*      */   }
/*      */   
/*      */   public final void incrWithNoReply(String key, long delta)
/*      */     throws InterruptedException, MemcachedException
/*      */   {
/* 2068 */     key = preProcessKey(key);
/*      */     try {
/* 2070 */       sendIncrOrDecrCommand(key, delta, 0L, CommandType.INCR, true, this.opTimeout, 0);
/*      */     }
/*      */     catch (TimeoutException e) {
/* 2073 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */   public final void decrWithNoReply(String key, long delta) throws InterruptedException, MemcachedException
/*      */   {
/* 2079 */     key = preProcessKey(key);
/*      */     try {
/* 2081 */       sendIncrOrDecrCommand(key, delta, 0L, CommandType.DECR, true, this.opTimeout, 0);
/*      */     }
/*      */     catch (TimeoutException e) {
/* 2084 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final long decr(String key, long delta)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2095 */     key = preProcessKey(key);
/* 2096 */     return sendIncrOrDecrCommand(key, delta, 0L, CommandType.DECR, false, this.opTimeout, 0);
/*      */   }
/*      */   
/*      */   public long decr(String key, long delta, long initValue)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2102 */     key = preProcessKey(key);
/* 2103 */     return sendIncrOrDecrCommand(key, delta, initValue, CommandType.DECR, false, this.opTimeout, 0);
/*      */   }
/*      */   
/*      */   public long decr(String key, long delta, long initValue, long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2109 */     key = preProcessKey(key);
/* 2110 */     return sendIncrOrDecrCommand(key, delta, initValue, CommandType.DECR, false, timeout, 0);
/*      */   }
/*      */   
/*      */ 
/*      */   public long decr(String key, long delta, long initValue, long timeout, int exp)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2117 */     key = preProcessKey(key);
/* 2118 */     return sendIncrOrDecrCommand(key, delta, initValue, CommandType.DECR, false, timeout, exp);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void flushAll()
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2129 */     flushAll(this.opTimeout);
/*      */   }
/*      */   
/*      */   public void flushAllWithNoReply() throws InterruptedException, MemcachedException
/*      */   {
/*      */     try {
/* 2135 */       flushAllMemcachedServers(this.opTimeout, true, 0);
/*      */     } catch (TimeoutException e) {
/* 2137 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */   public void flushAllWithNoReply(int exptime) throws InterruptedException, MemcachedException
/*      */   {
/*      */     try {
/* 2144 */       flushAllMemcachedServers(this.opTimeout, true, exptime);
/*      */     } catch (TimeoutException e) {
/* 2146 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */   public void flushAllWithNoReply(InetSocketAddress address) throws MemcachedException, InterruptedException
/*      */   {
/*      */     try {
/* 2153 */       flushSpecialMemcachedServer(address, this.opTimeout, true, 0);
/*      */     } catch (TimeoutException e) {
/* 2155 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */   public void flushAllWithNoReply(InetSocketAddress address, int exptime) throws MemcachedException, InterruptedException
/*      */   {
/*      */     try {
/* 2162 */       flushSpecialMemcachedServer(address, this.opTimeout, true, exptime);
/*      */     }
/*      */     catch (TimeoutException e) {
/* 2165 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */   public final void flushAll(int exptime, long timeout) throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2171 */     flushAllMemcachedServers(timeout, false, exptime);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void flushAll(long timeout)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2181 */     flushAllMemcachedServers(timeout, false, 0);
/*      */   }
/*      */   
/*      */   private void flushAllMemcachedServers(long timeout, boolean noreply, int exptime)
/*      */     throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2187 */     Collection<Session> sessions = this.connector.getSessionSet();
/* 2188 */     CountDownLatch latch = new CountDownLatch(sessions.size());
/* 2189 */     List<Command> commands = new ArrayList(sessions.size());
/* 2190 */     for (Session session : sessions) {
/* 2191 */       if ((session != null) && (!session.isClosed())) {
/* 2192 */         Command command = this.commandFactory.createFlushAllCommand(latch, exptime, noreply);
/*      */         
/*      */ 
/* 2195 */         session.write(command);
/*      */       } else {
/* 2197 */         latch.countDown();
/*      */       }
/*      */     }
/* 2200 */     if ((!noreply) && 
/* 2201 */       (!latch.await(timeout, TimeUnit.MILLISECONDS))) {
/* 2202 */       for (Command cmd : commands) {
/* 2203 */         cmd.cancel();
/*      */       }
/* 2205 */       throw new TimeoutException("Timed out waiting for operation");
/*      */     }
/*      */   }
/*      */   
/*      */   public void setLoggingLevelVerbosity(InetSocketAddress address, int level)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2212 */     setMemcachedLoggingLevel(address, level, false);
/*      */   }
/*      */   
/*      */ 
/*      */   private void setMemcachedLoggingLevel(InetSocketAddress address, int level, boolean noreply)
/*      */     throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2219 */     if (address == null) {
/* 2220 */       throw new IllegalArgumentException("Null adderss");
/*      */     }
/* 2222 */     CountDownLatch latch = new CountDownLatch(1);
/*      */     
/* 2224 */     Queue<Session> sessionQueue = this.connector.getSessionByAddress(address);
/*      */     
/* 2226 */     if ((sessionQueue == null) || (sessionQueue.peek() == null)) {
/* 2227 */       throw new MemcachedException("could not find session for " + SystemUtils.getRawAddress(address) + ":" + address.getPort() + ",maybe it have not been connected");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 2232 */     Command command = this.commandFactory.createVerbosityCommand(latch, level, noreply);
/*      */     
/* 2234 */     ((Session)sessionQueue.peek()).write(command);
/* 2235 */     if (!noreply) {
/* 2236 */       latchWait(command, this.opTimeout);
/*      */     }
/*      */   }
/*      */   
/*      */   public void setLoggingLevelVerbosityWithNoReply(InetSocketAddress address, int level) throws InterruptedException, MemcachedException
/*      */   {
/*      */     try {
/* 2243 */       setMemcachedLoggingLevel(address, level, true);
/*      */     } catch (TimeoutException e) {
/* 2245 */       throw new MemcachedException(e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void flushAll(InetSocketAddress address)
/*      */     throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2259 */     flushAll(address, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void flushAll(InetSocketAddress address, long timeout)
/*      */     throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2271 */     flushSpecialMemcachedServer(address, timeout, false, 0);
/*      */   }
/*      */   
/*      */   public final void flushAll(InetSocketAddress address, long timeout, int exptime)
/*      */     throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2277 */     flushSpecialMemcachedServer(address, timeout, false, exptime);
/*      */   }
/*      */   
/*      */   private void flushSpecialMemcachedServer(InetSocketAddress address, long timeout, boolean noreply, int exptime)
/*      */     throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2283 */     if (address == null) {
/* 2284 */       throw new IllegalArgumentException("Null adderss");
/*      */     }
/* 2286 */     CountDownLatch latch = new CountDownLatch(1);
/*      */     
/* 2288 */     Queue<Session> sessionQueue = this.connector.getSessionByAddress(address);
/*      */     
/* 2290 */     if ((sessionQueue == null) || (sessionQueue.peek() == null)) {
/* 2291 */       throw new MemcachedException("could not find session for " + SystemUtils.getRawAddress(address) + ":" + address.getPort() + ",maybe it have not been connected");
/*      */     }
/*      */     
/*      */ 
/* 2295 */     Command command = this.commandFactory.createFlushAllCommand(latch, exptime, noreply);
/*      */     
/* 2297 */     ((Session)sessionQueue.peek()).write(command);
/* 2298 */     if (!noreply) {
/* 2299 */       latchWait(command, timeout);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void flushAll(String host)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2310 */     flushAll(AddrUtil.getOneAddress(host), this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final Map<String, String> stats(InetSocketAddress address)
/*      */     throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2321 */     return stats(address, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final Map<String, String> stats(InetSocketAddress address, long timeout)
/*      */     throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2335 */     if (address == null) {
/* 2336 */       throw new IllegalArgumentException("Null inetSocketAddress");
/*      */     }
/* 2338 */     CountDownLatch latch = new CountDownLatch(1);
/*      */     
/* 2340 */     Queue<Session> sessionQueue = this.connector.getSessionByAddress(address);
/*      */     
/* 2342 */     if ((sessionQueue == null) || (sessionQueue.peek() == null)) {
/* 2343 */       throw new MemcachedException("could not find session for " + SystemUtils.getRawAddress(address) + ":" + address.getPort() + ",maybe it have not been connected");
/*      */     }
/*      */     
/*      */ 
/* 2347 */     Command command = this.commandFactory.createStatsCommand(address, latch, null);
/*      */     
/* 2349 */     ((Session)sessionQueue.peek()).write(command);
/* 2350 */     latchWait(command, timeout);
/* 2351 */     return (Map)command.getResult();
/*      */   }
/*      */   
/*      */   public final Map<InetSocketAddress, Map<String, String>> getStats() throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2356 */     return getStats(this.opTimeout);
/*      */   }
/*      */   
/*      */   public final Map<InetSocketAddress, Map<String, String>> getStatsByItem(String itemName)
/*      */     throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2362 */     return getStatsByItem(itemName, this.opTimeout);
/*      */   }
/*      */   
/*      */ 
/*      */   public final Map<InetSocketAddress, Map<String, String>> getStatsByItem(String itemName, long timeout)
/*      */     throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2369 */     Set<Session> sessionSet = this.connector.getSessionSet();
/* 2370 */     Map<InetSocketAddress, Map<String, String>> collectResult = new HashMap();
/* 2371 */     if (sessionSet.size() == 0) {
/* 2372 */       return collectResult;
/*      */     }
/* 2374 */     CountDownLatch latch = new CountDownLatch(sessionSet.size());
/* 2375 */     List<Command> commands = new ArrayList(sessionSet.size());
/* 2376 */     for (Session session : sessionSet) {
/* 2377 */       Command command = this.commandFactory.createStatsCommand(session.getRemoteSocketAddress(), latch, itemName);
/*      */       
/*      */ 
/* 2380 */       session.write(command);
/* 2381 */       commands.add(command);
/*      */     }
/*      */     
/* 2384 */     if (!latch.await(timeout, TimeUnit.MILLISECONDS)) {
/* 2385 */       for (Command command : commands) {
/* 2386 */         command.cancel();
/*      */       }
/* 2388 */       throw new TimeoutException("Timed out waiting for operation");
/*      */     }
/* 2390 */     for (Command command : commands) {
/* 2391 */       checkException(command);
/* 2392 */       collectResult.put(((ServerAddressAware)command).getServer(), (Map)command.getResult());
/*      */     }
/*      */     
/* 2395 */     return collectResult;
/*      */   }
/*      */   
/*      */   public final Map<InetSocketAddress, String> getVersions() throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2400 */     return getVersions(this.opTimeout);
/*      */   }
/*      */   
/*      */   public final Map<InetSocketAddress, String> getVersions(long timeout) throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2405 */     Set<Session> sessionSet = this.connector.getSessionSet();
/* 2406 */     Map<InetSocketAddress, String> collectResult = new HashMap();
/* 2407 */     if (sessionSet.size() == 0) {
/* 2408 */       return collectResult;
/*      */     }
/* 2410 */     CountDownLatch latch = new CountDownLatch(sessionSet.size());
/* 2411 */     List<Command> commands = new ArrayList(sessionSet.size());
/* 2412 */     for (Session session : sessionSet) {
/* 2413 */       Command command = this.commandFactory.createVersionCommand(latch, session.getRemoteSocketAddress());
/*      */       
/* 2415 */       session.write(command);
/* 2416 */       commands.add(command);
/*      */     }
/*      */     
/*      */ 
/* 2420 */     if (!latch.await(timeout, TimeUnit.MILLISECONDS)) {
/* 2421 */       for (Command command : commands) {
/* 2422 */         command.cancel();
/*      */       }
/* 2424 */       throw new TimeoutException("Timed out waiting for operation");
/*      */     }
/* 2426 */     for (Command command : commands) {
/* 2427 */       checkException(command);
/* 2428 */       collectResult.put(((ServerAddressAware)command).getServer(), (String)command.getResult());
/*      */     }
/*      */     
/* 2431 */     return collectResult;
/*      */   }
/*      */   
/*      */   public Map<InetSocketAddress, Map<String, String>> getStats(long timeout) throws MemcachedException, InterruptedException, TimeoutException
/*      */   {
/* 2436 */     return getStatsByItem(null, timeout);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void shutdown()
/*      */     throws IOException
/*      */   {
/* 2445 */     if (this.shutdown) {
/* 2446 */       return;
/*      */     }
/* 2448 */     this.shutdown = true;
/* 2449 */     this.connector.quitAllSessions();
/* 2450 */     this.connector.stop();
/* 2451 */     this.memcachedHandler.stop();
/* 2452 */     XMemcachedMbeanServer.getInstance().shutdown();
/* 2453 */     if (!this.isHutdownHookCalled) {
/* 2454 */       Runtime.getRuntime().removeShutdownHook(this.shutdownHookThread);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private long sendIncrOrDecrCommand(String key, long delta, long initValue, CommandType cmdType, boolean noreply, long operationTimeout, int exp)
/*      */     throws InterruptedException, TimeoutException, MemcachedException
/*      */   {
/* 2462 */     byte[] keyBytes = ByteUtils.getBytes(key);
/* 2463 */     ByteUtils.checkKey(keyBytes);
/* 2464 */     Command command = this.commandFactory.createIncrDecrCommand(key, keyBytes, delta, initValue, exp, cmdType, noreply);
/*      */     
/* 2466 */     sendCommand(command);
/* 2467 */     if (!command.isNoreply()) {
/* 2468 */       latchWait(command, operationTimeout);
/* 2469 */       command.getIoBuffer().free();
/* 2470 */       checkException(command);
/* 2471 */       if (command.getResult() == null) {
/* 2472 */         throw new MemcachedException("Operation fail,may be caused by networking or timeout");
/*      */       }
/*      */       
/* 2475 */       Object result = command.getResult();
/* 2476 */       if ((result instanceof String)) {
/* 2477 */         if (((String)result).equals("NOT_FOUND")) {
/* 2478 */           if (add(key, exp, String.valueOf(initValue), this.opTimeout))
/*      */           {
/* 2480 */             return initValue;
/*      */           }
/* 2482 */           return sendIncrOrDecrCommand(key, delta, initValue, cmdType, noreply, operationTimeout, exp);
/*      */         }
/*      */         
/*      */ 
/*      */ 
/* 2487 */         throw new MemcachedException("Unknown result type for incr/decr:" + result.getClass() + ",result=" + result);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 2492 */       return ((Long)command.getResult()).longValue();
/*      */     }
/*      */     
/* 2495 */     return -1L;
/*      */   }
/*      */   
/*      */   public void setConnectionPoolSize(int poolSize)
/*      */   {
/* 2500 */     if ((!this.shutdown) && (getAvaliableServers().size() > 0)) {
/* 2501 */       throw new IllegalStateException("Xmemcached client has been started");
/*      */     }
/*      */     
/* 2504 */     if (poolSize <= 0) {
/* 2505 */       throw new IllegalArgumentException("poolSize<=0");
/*      */     }
/* 2507 */     this.connectionPoolSize = poolSize;
/* 2508 */     this.connector.setConnectionPoolSize(poolSize);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean delete(String key)
/*      */     throws TimeoutException, InterruptedException, MemcachedException
/*      */   {
/* 2518 */     return delete(key, 0);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final Transcoder getTranscoder()
/*      */   {
/* 2528 */     return this.transcoder;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final void setTranscoder(Transcoder transcoder)
/*      */   {
/* 2540 */     this.transcoder = transcoder;
/*      */   }
/*      */   
/*      */   private final <T> boolean sendStoreCommand(Command command, long timeout)
/*      */     throws InterruptedException, TimeoutException, MemcachedException
/*      */   {
/* 2546 */     sendCommand(command);
/* 2547 */     if (!command.isNoreply()) {
/* 2548 */       latchWait(command, timeout);
/* 2549 */       command.getIoBuffer().free();
/* 2550 */       checkException(command);
/* 2551 */       if (command.getResult() == null) {
/* 2552 */         throw new MemcachedException("Operation fail,may be caused by networking or timeout");
/*      */       }
/*      */     }
/*      */     else {
/* 2556 */       return false;
/*      */     }
/* 2558 */     return ((Boolean)command.getResult()).booleanValue();
/*      */   }
/*      */   
/*      */   private void latchWait(Command cmd, long timeout) throws InterruptedException, TimeoutException
/*      */   {
/* 2563 */     if (!cmd.getLatch().await(timeout, TimeUnit.MILLISECONDS)) {
/* 2564 */       cmd.cancel();
/* 2565 */       throw new TimeoutException("Timed out(" + timeout + ") waiting for operation");
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   /**
/*      */    * @deprecated
/*      */    */
/*      */   public final Collection<InetSocketAddress> getAvaliableServers()
/*      */   {
/* 2577 */     return getAvailableServers();
/*      */   }
/*      */   
/*      */   public Collection<InetSocketAddress> getAvailableServers() {
/* 2581 */     Set<Session> sessionSet = this.connector.getSessionSet();
/* 2582 */     Set<InetSocketAddress> result = new HashSet();
/* 2583 */     for (Session session : sessionSet) {
/* 2584 */       result.add(session.getRemoteSocketAddress());
/*      */     }
/* 2586 */     return Collections.unmodifiableSet(result);
/*      */   }
/*      */   
/*      */   public final int getConnectionSizeBySocketAddress(InetSocketAddress address) {
/* 2590 */     Queue<Session> sessionList = this.connector.getSessionByAddress(address);
/*      */     
/* 2592 */     return sessionList == null ? 0 : sessionList.size();
/*      */   }
/*      */   
/*      */   public void addStateListener(MemcachedClientStateListener listener) {
/* 2596 */     MemcachedClientStateListenerAdapter adapter = new MemcachedClientStateListenerAdapter(listener, this);
/*      */     
/* 2598 */     this.stateListenerAdapters.add(adapter);
/* 2599 */     this.connector.addStateListener(adapter);
/*      */   }
/*      */   
/*      */   public Collection<MemcachedClientStateListener> getStateListeners() {
/* 2603 */     List<MemcachedClientStateListener> result = new ArrayList(this.stateListenerAdapters.size());
/*      */     
/* 2605 */     for (MemcachedClientStateListenerAdapter adapter : this.stateListenerAdapters) {
/* 2606 */       result.add(adapter.getMemcachedClientStateListener());
/*      */     }
/* 2608 */     return result;
/*      */   }
/*      */   
/*      */   public void setPrimitiveAsString(boolean primitiveAsString) {
/* 2612 */     this.transcoder.setPrimitiveAsString(primitiveAsString);
/*      */   }
/*      */   
/*      */   public void removeStateListener(MemcachedClientStateListener listener) {
/* 2616 */     for (MemcachedClientStateListenerAdapter adapter : this.stateListenerAdapters) {
/* 2617 */       if (adapter.getMemcachedClientStateListener().equals(listener)) {
/* 2618 */         this.stateListenerAdapters.remove(adapter);
/* 2619 */         this.connector.removeStateListener(adapter);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public Protocol getProtocol() {
/* 2625 */     return this.commandFactory.getProtocol();
/*      */   }
/*      */   
/*      */   public boolean isSanitizeKeys() {
/* 2629 */     return this.sanitizeKeys;
/*      */   }
/*      */   
/*      */   public void setSanitizeKeys(boolean sanitizeKeys) {
/* 2633 */     this.sanitizeKeys = sanitizeKeys;
/*      */   }
/*      */   
/*      */   private String decodeKey(String key) throws MemcachedException {
/*      */     try {
/* 2638 */       return this.sanitizeKeys ? URLDecoder.decode(key, "UTF-8") : key;
/*      */     } catch (UnsupportedEncodingException e) {
/* 2640 */       throw new MemcachedException("Unsupport encoding utf-8 when decodeKey", e);
/*      */     }
/*      */   }
/*      */   
/*      */   private String preProcessKey(String key) throws MemcachedException
/*      */   {
/* 2646 */     key = this.keyProvider.process(key);
/*      */     try {
/* 2648 */       return this.sanitizeKeys ? URLEncoder.encode(key, "UTF-8") : key;
/*      */     } catch (UnsupportedEncodingException e) {
/* 2650 */       throw new MemcachedException("Unsupport encoding utf-8 when sanitize key", e);
/*      */     }
/*      */   }
/*      */   
/*      */   public Counter getCounter(String key, long initialValue)
/*      */   {
/* 2656 */     return new Counter(this, key, initialValue);
/*      */   }
/*      */   
/*      */   public Counter getCounter(String key) {
/* 2660 */     return new Counter(this, key, 0L);
/*      */   }
/*      */   
/*      */ 
/*      */   /**
/*      */    * @deprecated
/*      */    */
/*      */   public KeyIterator getKeyIterator(InetSocketAddress address)
/*      */     throws MemcachedException, TimeoutException, InterruptedException
/*      */   {
/* 2670 */     if (address == null) {
/* 2671 */       throw new IllegalArgumentException("null address");
/*      */     }
/* 2673 */     Queue<Session> sessions = this.connector.getSessionByAddress(address);
/* 2674 */     if ((sessions == null) || (sessions.size() == 0)) {
/* 2675 */       throw new MemcachedException("The special memcached server has not been connected," + address);
/*      */     }
/*      */     
/*      */ 
/* 2679 */     Session session = (Session)sessions.peek();
/* 2680 */     CountDownLatch latch = new CountDownLatch(1);
/* 2681 */     Command command = this.commandFactory.createStatsCommand(session.getRemoteSocketAddress(), latch, "items");
/*      */     
/* 2683 */     session.write(command);
/* 2684 */     if (!latch.await(5000L, TimeUnit.MILLISECONDS)) {
/* 2685 */       throw new TimeoutException("Operation timeout");
/*      */     }
/* 2687 */     if (command.getException() != null) {
/* 2688 */       if ((command.getException() instanceof MemcachedException)) {
/* 2689 */         throw ((MemcachedException)command.getException());
/*      */       }
/* 2691 */       throw new MemcachedException("stats items failed", command.getException());
/*      */     }
/*      */     
/* 2694 */     Map<String, String> result = (Map)command.getResult();
/* 2695 */     LinkedList<Integer> itemNumberList = new LinkedList();
/* 2696 */     for (Map.Entry<String, String> entry : result.entrySet()) {
/* 2697 */       String key = (String)entry.getKey();
/* 2698 */       String[] keys = key.split(":");
/* 2699 */       if ((keys.length == 3) && (keys[2].equals("number")) && (keys[0].equals("items")))
/*      */       {
/*      */ 
/* 2702 */         if (Integer.parseInt((String)entry.getValue()) > 0) {
/* 2703 */           itemNumberList.add(Integer.valueOf(Integer.parseInt(keys[1])));
/*      */         }
/*      */       }
/*      */     }
/* 2707 */     return new KeyIteratorImpl(itemNumberList, this, address);
/*      */   }
/*      */   
/*      */   public void setFailureMode(boolean failureMode) {
/* 2711 */     this.failureMode = failureMode;
/* 2712 */     if (this.sessionLocator != null) {
/* 2713 */       this.sessionLocator.setFailureMode(failureMode);
/*      */     }
/* 2715 */     if (this.connector != null) {
/* 2716 */       this.connector.setFailureMode(failureMode);
/*      */     }
/*      */   }
/*      */   
/*      */   public boolean isFailureMode() {
/* 2721 */     return this.failureMode;
/*      */   }
/*      */   
/*      */   public Queue<ReconnectRequest> getReconnectRequestQueue() {
/* 2725 */     return this.connector != null ? this.connector.getReconnectRequestQueue() : null;
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\XMemcachedClient.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */