/*     */ package org.apache.avro.io;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.nio.ByteBuffer;
/*     */ import org.apache.avro.AvroTypeException;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.io.parsing.Parser.ActionHandler;
/*     */ import org.apache.avro.io.parsing.SkipParser;
/*     */ import org.apache.avro.io.parsing.Symbol;
/*     */ import org.apache.avro.io.parsing.Symbol.Alternative;
/*     */ import org.apache.avro.io.parsing.Symbol.IntCheckAction;
/*     */ import org.apache.avro.io.parsing.ValidatingGrammarGenerator;
/*     */ import org.apache.avro.util.Utf8;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ValidatingDecoder
/*     */   extends ParsingDecoder
/*     */   implements Parser.ActionHandler
/*     */ {
/*     */   protected Decoder in;
/*     */   
/*     */   ValidatingDecoder(Symbol root, Decoder in)
/*     */     throws IOException
/*     */   {
/*  46 */     super(root);
/*  47 */     configure(in);
/*     */   }
/*     */   
/*     */   ValidatingDecoder(Schema schema, Decoder in) throws IOException {
/*  51 */     this(getSymbol(schema), in);
/*     */   }
/*     */   
/*     */   private static Symbol getSymbol(Schema schema) {
/*  55 */     if (null == schema) {
/*  56 */       throw new NullPointerException("Schema cannot be null");
/*     */     }
/*  58 */     return new ValidatingGrammarGenerator().generate(schema);
/*     */   }
/*     */   
/*     */   public ValidatingDecoder configure(Decoder in) throws IOException
/*     */   {
/*  63 */     this.parser.reset();
/*  64 */     this.in = in;
/*  65 */     return this;
/*     */   }
/*     */   
/*     */   public void readNull() throws IOException
/*     */   {
/*  70 */     this.parser.advance(Symbol.NULL);
/*  71 */     this.in.readNull();
/*     */   }
/*     */   
/*     */   public boolean readBoolean() throws IOException
/*     */   {
/*  76 */     this.parser.advance(Symbol.BOOLEAN);
/*  77 */     return this.in.readBoolean();
/*     */   }
/*     */   
/*     */   public int readInt() throws IOException
/*     */   {
/*  82 */     this.parser.advance(Symbol.INT);
/*  83 */     return this.in.readInt();
/*     */   }
/*     */   
/*     */   public long readLong() throws IOException
/*     */   {
/*  88 */     this.parser.advance(Symbol.LONG);
/*  89 */     return this.in.readLong();
/*     */   }
/*     */   
/*     */   public float readFloat() throws IOException
/*     */   {
/*  94 */     this.parser.advance(Symbol.FLOAT);
/*  95 */     return this.in.readFloat();
/*     */   }
/*     */   
/*     */   public double readDouble() throws IOException
/*     */   {
/* 100 */     this.parser.advance(Symbol.DOUBLE);
/* 101 */     return this.in.readDouble();
/*     */   }
/*     */   
/*     */   public Utf8 readString(Utf8 old) throws IOException
/*     */   {
/* 106 */     this.parser.advance(Symbol.STRING);
/* 107 */     return this.in.readString(old);
/*     */   }
/*     */   
/*     */   public String readString() throws IOException
/*     */   {
/* 112 */     this.parser.advance(Symbol.STRING);
/* 113 */     return this.in.readString();
/*     */   }
/*     */   
/*     */   public void skipString() throws IOException
/*     */   {
/* 118 */     this.parser.advance(Symbol.STRING);
/* 119 */     this.in.skipString();
/*     */   }
/*     */   
/*     */   public ByteBuffer readBytes(ByteBuffer old) throws IOException
/*     */   {
/* 124 */     this.parser.advance(Symbol.BYTES);
/* 125 */     return this.in.readBytes(old);
/*     */   }
/*     */   
/*     */   public void skipBytes() throws IOException
/*     */   {
/* 130 */     this.parser.advance(Symbol.BYTES);
/* 131 */     this.in.skipBytes();
/*     */   }
/*     */   
/*     */   private void checkFixed(int size) throws IOException {
/* 135 */     this.parser.advance(Symbol.FIXED);
/* 136 */     Symbol.IntCheckAction top = (Symbol.IntCheckAction)this.parser.popSymbol();
/* 137 */     if (size != top.size) {
/* 138 */       throw new AvroTypeException("Incorrect length for fixed binary: expected " + top.size + " but received " + size + " bytes.");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public void readFixed(byte[] bytes, int start, int len)
/*     */     throws IOException
/*     */   {
/* 146 */     checkFixed(len);
/* 147 */     this.in.readFixed(bytes, start, len);
/*     */   }
/*     */   
/*     */   public void skipFixed(int length) throws IOException
/*     */   {
/* 152 */     checkFixed(length);
/* 153 */     this.in.skipFixed(length);
/*     */   }
/*     */   
/*     */   protected void skipFixed() throws IOException
/*     */   {
/* 158 */     this.parser.advance(Symbol.FIXED);
/* 159 */     Symbol.IntCheckAction top = (Symbol.IntCheckAction)this.parser.popSymbol();
/* 160 */     this.in.skipFixed(top.size);
/*     */   }
/*     */   
/*     */   public int readEnum() throws IOException
/*     */   {
/* 165 */     this.parser.advance(Symbol.ENUM);
/* 166 */     Symbol.IntCheckAction top = (Symbol.IntCheckAction)this.parser.popSymbol();
/* 167 */     int result = this.in.readEnum();
/* 168 */     if ((result < 0) || (result >= top.size)) {
/* 169 */       throw new AvroTypeException("Enumeration out of range: max is " + top.size + " but received " + result);
/*     */     }
/*     */     
/*     */ 
/* 173 */     return result;
/*     */   }
/*     */   
/*     */   public long readArrayStart() throws IOException
/*     */   {
/* 178 */     this.parser.advance(Symbol.ARRAY_START);
/* 179 */     long result = this.in.readArrayStart();
/* 180 */     if (result == 0L) {
/* 181 */       this.parser.advance(Symbol.ARRAY_END);
/*     */     }
/* 183 */     return result;
/*     */   }
/*     */   
/*     */   public long arrayNext() throws IOException
/*     */   {
/* 188 */     this.parser.processTrailingImplicitActions();
/* 189 */     long result = this.in.arrayNext();
/* 190 */     if (result == 0L) {
/* 191 */       this.parser.advance(Symbol.ARRAY_END);
/*     */     }
/* 193 */     return result;
/*     */   }
/*     */   
/*     */   public long skipArray() throws IOException
/*     */   {
/* 198 */     this.parser.advance(Symbol.ARRAY_START);
/* 199 */     for (long c = this.in.skipArray(); c != 0L; c = this.in.skipArray()) {
/* 200 */       while (c-- > 0L) {
/* 201 */         this.parser.skipRepeater();
/*     */       }
/*     */     }
/* 204 */     this.parser.advance(Symbol.ARRAY_END);
/* 205 */     return 0L;
/*     */   }
/*     */   
/*     */   public long readMapStart() throws IOException
/*     */   {
/* 210 */     this.parser.advance(Symbol.MAP_START);
/* 211 */     long result = this.in.readMapStart();
/* 212 */     if (result == 0L) {
/* 213 */       this.parser.advance(Symbol.MAP_END);
/*     */     }
/* 215 */     return result;
/*     */   }
/*     */   
/*     */   public long mapNext() throws IOException
/*     */   {
/* 220 */     this.parser.processTrailingImplicitActions();
/* 221 */     long result = this.in.mapNext();
/* 222 */     if (result == 0L) {
/* 223 */       this.parser.advance(Symbol.MAP_END);
/*     */     }
/* 225 */     return result;
/*     */   }
/*     */   
/*     */   public long skipMap() throws IOException
/*     */   {
/* 230 */     this.parser.advance(Symbol.MAP_START);
/* 231 */     for (long c = this.in.skipMap(); c != 0L; c = this.in.skipMap()) {
/* 232 */       while (c-- > 0L) {
/* 233 */         this.parser.skipRepeater();
/*     */       }
/*     */     }
/* 236 */     this.parser.advance(Symbol.MAP_END);
/* 237 */     return 0L;
/*     */   }
/*     */   
/*     */   public int readIndex() throws IOException
/*     */   {
/* 242 */     this.parser.advance(Symbol.UNION);
/* 243 */     Symbol.Alternative top = (Symbol.Alternative)this.parser.popSymbol();
/* 244 */     int result = this.in.readIndex();
/* 245 */     this.parser.pushSymbol(top.getSymbol(result));
/* 246 */     return result;
/*     */   }
/*     */   
/*     */   public Symbol doAction(Symbol input, Symbol top) throws IOException
/*     */   {
/* 251 */     return null;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\io\ValidatingDecoder.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */