/*      */ package org.codehaus.jackson.impl;
/*      */ 
/*      */ import java.io.IOException;
/*      */ import org.codehaus.jackson.Base64Variant;
/*      */ import org.codehaus.jackson.JsonParseException;
/*      */ import org.codehaus.jackson.JsonParser.Feature;
/*      */ import org.codehaus.jackson.JsonToken;
/*      */ import org.codehaus.jackson.ObjectCodec;
/*      */ import org.codehaus.jackson.io.IOContext;
/*      */ import org.codehaus.jackson.sym.BytesToNameCanonicalizer;
/*      */ import org.codehaus.jackson.sym.Name;
/*      */ import org.codehaus.jackson.util.ByteArrayBuilder;
/*      */ import org.codehaus.jackson.util.CharTypes;
/*      */ import org.codehaus.jackson.util.TextBuffer;
/*      */ 
/*      */ public final class Utf8StreamParser extends StreamBasedParserBase
/*      */ {
/*      */   static final byte BYTE_LF = 10;
/*   19 */   private static final int[] sInputCodesUtf8 = ;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*   25 */   private static final int[] sInputCodesLatin1 = CharTypes.getInputCodeLatin1();
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected ObjectCodec _objectCodec;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final BytesToNameCanonicalizer _symbols;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*   54 */   protected int[] _quadBuffer = new int[16];
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*   61 */   protected boolean _tokenIncomplete = false;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private int _quad1;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Utf8StreamParser(IOContext ctxt, int features, java.io.InputStream in, ObjectCodec codec, BytesToNameCanonicalizer sym, byte[] inputBuffer, int start, int end, boolean bufferRecyclable)
/*      */   {
/*   79 */     super(ctxt, features, in, inputBuffer, start, end, bufferRecyclable);
/*   80 */     this._objectCodec = codec;
/*   81 */     this._symbols = sym;
/*      */     
/*   83 */     if (!JsonParser.Feature.CANONICALIZE_FIELD_NAMES.enabledIn(features))
/*      */     {
/*   85 */       _throwInternal();
/*      */     }
/*      */   }
/*      */   
/*      */   public ObjectCodec getCodec()
/*      */   {
/*   91 */     return this._objectCodec;
/*      */   }
/*      */   
/*      */   public void setCodec(ObjectCodec c)
/*      */   {
/*   96 */     this._objectCodec = c;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public String getText()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  109 */     JsonToken t = this._currToken;
/*  110 */     if (t == JsonToken.VALUE_STRING) {
/*  111 */       if (this._tokenIncomplete) {
/*  112 */         this._tokenIncomplete = false;
/*  113 */         _finishString();
/*      */       }
/*  115 */       return this._textBuffer.contentsAsString();
/*      */     }
/*  117 */     return _getText2(t);
/*      */   }
/*      */   
/*      */   protected final String _getText2(JsonToken t)
/*      */   {
/*  122 */     if (t == null) {
/*  123 */       return null;
/*      */     }
/*  125 */     switch (t) {
/*      */     case FIELD_NAME: 
/*  127 */       return this._parsingContext.getCurrentName();
/*      */     
/*      */ 
/*      */     case VALUE_STRING: 
/*      */     case VALUE_NUMBER_INT: 
/*      */     case VALUE_NUMBER_FLOAT: 
/*  133 */       return this._textBuffer.contentsAsString();
/*      */     }
/*  135 */     return t.asString();
/*      */   }
/*      */   
/*      */ 
/*      */   public char[] getTextCharacters()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  142 */     if (this._currToken != null) {
/*  143 */       switch (this._currToken)
/*      */       {
/*      */       case FIELD_NAME: 
/*  146 */         if (!this._nameCopied) {
/*  147 */           String name = this._parsingContext.getCurrentName();
/*  148 */           int nameLen = name.length();
/*  149 */           if (this._nameCopyBuffer == null) {
/*  150 */             this._nameCopyBuffer = this._ioContext.allocNameCopyBuffer(nameLen);
/*  151 */           } else if (this._nameCopyBuffer.length < nameLen) {
/*  152 */             this._nameCopyBuffer = new char[nameLen];
/*      */           }
/*  154 */           name.getChars(0, nameLen, this._nameCopyBuffer, 0);
/*  155 */           this._nameCopied = true;
/*      */         }
/*  157 */         return this._nameCopyBuffer;
/*      */       
/*      */       case VALUE_STRING: 
/*  160 */         if (this._tokenIncomplete) {
/*  161 */           this._tokenIncomplete = false;
/*  162 */           _finishString();
/*      */         }
/*      */       
/*      */       case VALUE_NUMBER_INT: 
/*      */       case VALUE_NUMBER_FLOAT: 
/*  167 */         return this._textBuffer.getTextBuffer();
/*      */       }
/*      */       
/*  170 */       return this._currToken.asCharArray();
/*      */     }
/*      */     
/*  173 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */   public int getTextLength()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  180 */     if (this._currToken != null) {
/*  181 */       switch (this._currToken)
/*      */       {
/*      */       case FIELD_NAME: 
/*  184 */         return this._parsingContext.getCurrentName().length();
/*      */       case VALUE_STRING: 
/*  186 */         if (this._tokenIncomplete) {
/*  187 */           this._tokenIncomplete = false;
/*  188 */           _finishString();
/*      */         }
/*      */       
/*      */       case VALUE_NUMBER_INT: 
/*      */       case VALUE_NUMBER_FLOAT: 
/*  193 */         return this._textBuffer.size();
/*      */       }
/*      */       
/*  196 */       return this._currToken.asCharArray().length;
/*      */     }
/*      */     
/*  199 */     return 0;
/*      */   }
/*      */   
/*      */ 
/*      */   public int getTextOffset()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  206 */     if (this._currToken != null) {
/*  207 */       switch (this._currToken) {
/*      */       case FIELD_NAME: 
/*  209 */         return 0;
/*      */       case VALUE_STRING: 
/*  211 */         if (this._tokenIncomplete) {
/*  212 */           this._tokenIncomplete = false;
/*  213 */           _finishString();
/*      */         }
/*      */       
/*      */       case VALUE_NUMBER_INT: 
/*      */       case VALUE_NUMBER_FLOAT: 
/*  218 */         return this._textBuffer.getTextOffset();
/*      */       }
/*      */     }
/*  221 */     return 0;
/*      */   }
/*      */   
/*      */ 
/*      */   public byte[] getBinaryValue(Base64Variant b64variant)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  228 */     if ((this._currToken != JsonToken.VALUE_STRING) && ((this._currToken != JsonToken.VALUE_EMBEDDED_OBJECT) || (this._binaryValue == null)))
/*      */     {
/*  230 */       _reportError("Current token (" + this._currToken + ") not VALUE_STRING or VALUE_EMBEDDED_OBJECT, can not access as binary");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  235 */     if (this._tokenIncomplete) {
/*      */       try {
/*  237 */         this._binaryValue = _decodeBase64(b64variant);
/*      */       } catch (IllegalArgumentException iae) {
/*  239 */         throw _constructError("Failed to decode VALUE_STRING as base64 (" + b64variant + "): " + iae.getMessage());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*  244 */       this._tokenIncomplete = false;
/*      */     }
/*  246 */     return this._binaryValue;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonToken nextToken()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  263 */     this._numTypesValid = 0;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  268 */     if (this._currToken == JsonToken.FIELD_NAME) {
/*  269 */       return _nextAfterName();
/*      */     }
/*  271 */     if (this._tokenIncomplete) {
/*  272 */       _skipString();
/*      */     }
/*      */     
/*  275 */     int i = _skipWSOrEnd();
/*  276 */     if (i < 0)
/*      */     {
/*      */ 
/*      */ 
/*  280 */       close();
/*  281 */       return this._currToken = null;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  287 */     this._tokenInputTotal = (this._currInputProcessed + this._inputPtr - 1L);
/*  288 */     this._tokenInputRow = this._currInputRow;
/*  289 */     this._tokenInputCol = (this._inputPtr - this._currInputRowStart - 1);
/*      */     
/*      */ 
/*  292 */     this._binaryValue = null;
/*      */     
/*      */ 
/*  295 */     if (i == 93) {
/*  296 */       if (!this._parsingContext.inArray()) {
/*  297 */         _reportMismatchedEndMarker(i, '}');
/*      */       }
/*  299 */       this._parsingContext = this._parsingContext.getParent();
/*  300 */       return this._currToken = JsonToken.END_ARRAY;
/*      */     }
/*  302 */     if (i == 125) {
/*  303 */       if (!this._parsingContext.inObject()) {
/*  304 */         _reportMismatchedEndMarker(i, ']');
/*      */       }
/*  306 */       this._parsingContext = this._parsingContext.getParent();
/*  307 */       return this._currToken = JsonToken.END_OBJECT;
/*      */     }
/*      */     
/*      */ 
/*  311 */     if (this._parsingContext.expectComma()) {
/*  312 */       if (i != 44) {
/*  313 */         _reportUnexpectedChar(i, "was expecting comma to separate " + this._parsingContext.getTypeDesc() + " entries");
/*      */       }
/*  315 */       i = _skipWS();
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  322 */     if (!this._parsingContext.inObject()) {
/*  323 */       return _nextTokenNotInObject(i);
/*      */     }
/*      */     
/*  326 */     Name n = _parseFieldName(i);
/*  327 */     this._parsingContext.setCurrentName(n.getName());
/*  328 */     this._currToken = JsonToken.FIELD_NAME;
/*  329 */     i = _skipWS();
/*  330 */     if (i != 58) {
/*  331 */       _reportUnexpectedChar(i, "was expecting a colon to separate field name and value");
/*      */     }
/*  333 */     i = _skipWS();
/*      */     
/*      */ 
/*  336 */     if (i == 34) {
/*  337 */       this._tokenIncomplete = true;
/*  338 */       this._nextToken = JsonToken.VALUE_STRING;
/*  339 */       return this._currToken;
/*      */     }
/*      */     
/*      */     JsonToken t;
/*  343 */     switch (i) {
/*      */     case 91: 
/*  345 */       t = JsonToken.START_ARRAY;
/*  346 */       break;
/*      */     case 123: 
/*  348 */       t = JsonToken.START_OBJECT;
/*  349 */       break;
/*      */     
/*      */ 
/*      */     case 93: 
/*      */     case 125: 
/*  354 */       _reportUnexpectedChar(i, "expected a value");
/*      */     case 116: 
/*  356 */       _matchToken(JsonToken.VALUE_TRUE);
/*  357 */       t = JsonToken.VALUE_TRUE;
/*  358 */       break;
/*      */     case 102: 
/*  360 */       _matchToken(JsonToken.VALUE_FALSE);
/*  361 */       t = JsonToken.VALUE_FALSE;
/*  362 */       break;
/*      */     case 110: 
/*  364 */       _matchToken(JsonToken.VALUE_NULL);
/*  365 */       t = JsonToken.VALUE_NULL;
/*  366 */       break;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     case 45: 
/*      */     case 48: 
/*      */     case 49: 
/*      */     case 50: 
/*      */     case 51: 
/*      */     case 52: 
/*      */     case 53: 
/*      */     case 54: 
/*      */     case 55: 
/*      */     case 56: 
/*      */     case 57: 
/*  383 */       t = parseNumberText(i);
/*  384 */       break;
/*      */     default: 
/*  386 */       t = _handleUnexpectedValue(i);
/*      */     }
/*  388 */     this._nextToken = t;
/*  389 */     return this._currToken;
/*      */   }
/*      */   
/*      */   private final JsonToken _nextTokenNotInObject(int i)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  395 */     if (i == 34) {
/*  396 */       this._tokenIncomplete = true;
/*  397 */       return this._currToken = JsonToken.VALUE_STRING;
/*      */     }
/*  399 */     switch (i) {
/*      */     case 91: 
/*  401 */       this._parsingContext = this._parsingContext.createChildArrayContext(this._tokenInputRow, this._tokenInputCol);
/*  402 */       return this._currToken = JsonToken.START_ARRAY;
/*      */     case 123: 
/*  404 */       this._parsingContext = this._parsingContext.createChildObjectContext(this._tokenInputRow, this._tokenInputCol);
/*  405 */       return this._currToken = JsonToken.START_OBJECT;
/*      */     
/*      */ 
/*      */     case 93: 
/*      */     case 125: 
/*  410 */       _reportUnexpectedChar(i, "expected a value");
/*      */     case 116: 
/*  412 */       _matchToken(JsonToken.VALUE_TRUE);
/*  413 */       return this._currToken = JsonToken.VALUE_TRUE;
/*      */     case 102: 
/*  415 */       _matchToken(JsonToken.VALUE_FALSE);
/*  416 */       return this._currToken = JsonToken.VALUE_FALSE;
/*      */     case 110: 
/*  418 */       _matchToken(JsonToken.VALUE_NULL);
/*  419 */       return this._currToken = JsonToken.VALUE_NULL;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     case 45: 
/*      */     case 48: 
/*      */     case 49: 
/*      */     case 50: 
/*      */     case 51: 
/*      */     case 52: 
/*      */     case 53: 
/*      */     case 54: 
/*      */     case 55: 
/*      */     case 56: 
/*      */     case 57: 
/*  435 */       return this._currToken = parseNumberText(i);
/*      */     }
/*  437 */     return this._currToken = _handleUnexpectedValue(i);
/*      */   }
/*      */   
/*      */   private final JsonToken _nextAfterName()
/*      */   {
/*  442 */     this._nameCopied = false;
/*  443 */     JsonToken t = this._nextToken;
/*  444 */     this._nextToken = null;
/*      */     
/*  446 */     if (t == JsonToken.START_ARRAY) {
/*  447 */       this._parsingContext = this._parsingContext.createChildArrayContext(this._tokenInputRow, this._tokenInputCol);
/*  448 */     } else if (t == JsonToken.START_OBJECT) {
/*  449 */       this._parsingContext = this._parsingContext.createChildObjectContext(this._tokenInputRow, this._tokenInputCol);
/*      */     }
/*  451 */     return this._currToken = t;
/*      */   }
/*      */   
/*      */   public void close()
/*      */     throws IOException
/*      */   {
/*  457 */     super.close();
/*      */     
/*  459 */     this._symbols.release();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final JsonToken parseNumberText(int c)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  487 */     char[] outBuf = this._textBuffer.emptyAndGetCurrentSegment();
/*  488 */     int outPtr = 0;
/*  489 */     boolean negative = c == 45;
/*      */     
/*      */ 
/*  492 */     if (negative) {
/*  493 */       outBuf[(outPtr++)] = '-';
/*      */       
/*  495 */       if (this._inputPtr >= this._inputEnd) {
/*  496 */         loadMoreGuaranteed();
/*      */       }
/*  498 */       c = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*      */       
/*  500 */       if ((c < 48) || (c > 57)) {
/*  501 */         return _handleInvalidNumberStart(c, true);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  506 */     if (c == 48) {
/*  507 */       c = _verifyNoLeadingZeroes();
/*      */     }
/*      */     
/*      */ 
/*  511 */     outBuf[(outPtr++)] = ((char)c);
/*  512 */     int intLen = 1;
/*      */     
/*      */ 
/*  515 */     int end = this._inputPtr + outBuf.length;
/*  516 */     if (end > this._inputEnd) {
/*  517 */       end = this._inputEnd;
/*      */     }
/*      */     
/*      */     for (;;)
/*      */     {
/*  522 */       if (this._inputPtr >= end)
/*      */       {
/*  524 */         return _parserNumber2(outBuf, outPtr, negative, intLen);
/*      */       }
/*  526 */       c = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  527 */       if ((c < 48) || (c > 57)) {
/*      */         break;
/*      */       }
/*  530 */       intLen++;
/*  531 */       outBuf[(outPtr++)] = ((char)c);
/*      */     }
/*  533 */     if ((c == 46) || (c == 101) || (c == 69)) {
/*  534 */       return _parseFloatText(outBuf, outPtr, c, negative, intLen);
/*      */     }
/*      */     
/*  537 */     this._inputPtr -= 1;
/*  538 */     this._textBuffer.setCurrentLength(outPtr);
/*      */     
/*      */ 
/*  541 */     return resetInt(negative, intLen);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final JsonToken _parserNumber2(char[] outBuf, int outPtr, boolean negative, int intPartLength)
/*      */     throws IOException, JsonParseException
/*      */   {
/*      */     for (;;)
/*      */     {
/*  554 */       if ((this._inputPtr >= this._inputEnd) && (!loadMore())) {
/*  555 */         this._textBuffer.setCurrentLength(outPtr);
/*  556 */         return resetInt(negative, intPartLength);
/*      */       }
/*  558 */       int c = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  559 */       if ((c > 57) || (c < 48)) {
/*  560 */         if ((c != 46) && (c != 101) && (c != 69)) break;
/*  561 */         return _parseFloatText(outBuf, outPtr, c, negative, intPartLength);
/*      */       }
/*      */       
/*      */ 
/*  565 */       if (outPtr >= outBuf.length) {
/*  566 */         outBuf = this._textBuffer.finishCurrentSegment();
/*  567 */         outPtr = 0;
/*      */       }
/*  569 */       outBuf[(outPtr++)] = ((char)c);
/*  570 */       intPartLength++;
/*      */     }
/*  572 */     this._inputPtr -= 1;
/*  573 */     this._textBuffer.setCurrentLength(outPtr);
/*      */     
/*      */ 
/*  576 */     return resetInt(negative, intPartLength);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final int _verifyNoLeadingZeroes()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  588 */     if ((this._inputPtr >= this._inputEnd) && (!loadMore())) {
/*  589 */       return 48;
/*      */     }
/*  591 */     int ch = this._inputBuffer[this._inputPtr] & 0xFF;
/*      */     
/*  593 */     if ((ch < 48) || (ch > 57)) {
/*  594 */       return 48;
/*      */     }
/*      */     
/*  597 */     if (!isEnabled(JsonParser.Feature.ALLOW_NUMERIC_LEADING_ZEROS)) {
/*  598 */       reportInvalidNumber("Leading zeroes not allowed");
/*      */     }
/*      */     
/*  601 */     this._inputPtr += 1;
/*  602 */     if (ch == 48) {
/*  603 */       while ((this._inputPtr < this._inputEnd) || (loadMore())) {
/*  604 */         ch = this._inputBuffer[this._inputPtr] & 0xFF;
/*  605 */         if ((ch < 48) || (ch > 57)) {
/*  606 */           return 48;
/*      */         }
/*  608 */         this._inputPtr += 1;
/*  609 */         if (ch != 48) {
/*      */           break;
/*      */         }
/*      */       }
/*      */     }
/*  614 */     return ch;
/*      */   }
/*      */   
/*      */ 
/*      */   private final JsonToken _parseFloatText(char[] outBuf, int outPtr, int c, boolean negative, int integerPartLength)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  621 */     int fractLen = 0;
/*  622 */     boolean eof = false;
/*      */     
/*      */ 
/*  625 */     if (c == 46) {
/*  626 */       outBuf[(outPtr++)] = ((char)c);
/*      */       
/*      */       for (;;)
/*      */       {
/*  630 */         if ((this._inputPtr >= this._inputEnd) && (!loadMore())) {
/*  631 */           eof = true;
/*  632 */           break;
/*      */         }
/*  634 */         c = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  635 */         if ((c < 48) || (c > 57)) {
/*      */           break;
/*      */         }
/*  638 */         fractLen++;
/*  639 */         if (outPtr >= outBuf.length) {
/*  640 */           outBuf = this._textBuffer.finishCurrentSegment();
/*  641 */           outPtr = 0;
/*      */         }
/*  643 */         outBuf[(outPtr++)] = ((char)c);
/*      */       }
/*      */       
/*  646 */       if (fractLen == 0) {
/*  647 */         reportUnexpectedNumberChar(c, "Decimal point not followed by a digit");
/*      */       }
/*      */     }
/*      */     
/*  651 */     int expLen = 0;
/*  652 */     if ((c == 101) || (c == 69)) {
/*  653 */       if (outPtr >= outBuf.length) {
/*  654 */         outBuf = this._textBuffer.finishCurrentSegment();
/*  655 */         outPtr = 0;
/*      */       }
/*  657 */       outBuf[(outPtr++)] = ((char)c);
/*      */       
/*  659 */       if (this._inputPtr >= this._inputEnd) {
/*  660 */         loadMoreGuaranteed();
/*      */       }
/*  662 */       c = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*      */       
/*  664 */       if ((c == 45) || (c == 43)) {
/*  665 */         if (outPtr >= outBuf.length) {
/*  666 */           outBuf = this._textBuffer.finishCurrentSegment();
/*  667 */           outPtr = 0;
/*      */         }
/*  669 */         outBuf[(outPtr++)] = ((char)c);
/*      */         
/*  671 */         if (this._inputPtr >= this._inputEnd) {
/*  672 */           loadMoreGuaranteed();
/*      */         }
/*  674 */         c = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*      */       }
/*      */       
/*      */ 
/*  678 */       while ((c <= 57) && (c >= 48)) {
/*  679 */         expLen++;
/*  680 */         if (outPtr >= outBuf.length) {
/*  681 */           outBuf = this._textBuffer.finishCurrentSegment();
/*  682 */           outPtr = 0;
/*      */         }
/*  684 */         outBuf[(outPtr++)] = ((char)c);
/*  685 */         if ((this._inputPtr >= this._inputEnd) && (!loadMore())) {
/*  686 */           eof = true;
/*  687 */           break;
/*      */         }
/*  689 */         c = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*      */       }
/*      */       
/*  692 */       if (expLen == 0) {
/*  693 */         reportUnexpectedNumberChar(c, "Exponent indicator not followed by a digit");
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  698 */     if (!eof) {
/*  699 */       this._inputPtr -= 1;
/*      */     }
/*  701 */     this._textBuffer.setCurrentLength(outPtr);
/*      */     
/*      */ 
/*  704 */     return resetFloat(negative, integerPartLength, fractLen, expLen);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final Name _parseFieldName(int i)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  716 */     if (i != 34) {
/*  717 */       return _handleUnusualFieldName(i);
/*      */     }
/*      */     
/*  720 */     if (this._inputPtr + 9 > this._inputEnd) {
/*  721 */       return slowParseFieldName();
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  730 */     byte[] input = this._inputBuffer;
/*  731 */     int[] codes = sInputCodesLatin1;
/*      */     
/*  733 */     int q = input[(this._inputPtr++)] & 0xFF;
/*      */     
/*  735 */     if (codes[q] == 0) {
/*  736 */       i = input[(this._inputPtr++)] & 0xFF;
/*  737 */       if (codes[i] == 0) {
/*  738 */         q = q << 8 | i;
/*  739 */         i = input[(this._inputPtr++)] & 0xFF;
/*  740 */         if (codes[i] == 0) {
/*  741 */           q = q << 8 | i;
/*  742 */           i = input[(this._inputPtr++)] & 0xFF;
/*  743 */           if (codes[i] == 0) {
/*  744 */             q = q << 8 | i;
/*  745 */             i = input[(this._inputPtr++)] & 0xFF;
/*  746 */             if (codes[i] == 0) {
/*  747 */               this._quad1 = q;
/*  748 */               return parseMediumFieldName(i, codes);
/*      */             }
/*  750 */             if (i == 34) {
/*  751 */               return findName(q, 4);
/*      */             }
/*  753 */             return parseFieldName(q, i, 4);
/*      */           }
/*  755 */           if (i == 34) {
/*  756 */             return findName(q, 3);
/*      */           }
/*  758 */           return parseFieldName(q, i, 3);
/*      */         }
/*  760 */         if (i == 34) {
/*  761 */           return findName(q, 2);
/*      */         }
/*  763 */         return parseFieldName(q, i, 2);
/*      */       }
/*  765 */       if (i == 34) {
/*  766 */         return findName(q, 1);
/*      */       }
/*  768 */       return parseFieldName(q, i, 1);
/*      */     }
/*  770 */     if (q == 34) {
/*  771 */       return BytesToNameCanonicalizer.getEmptyName();
/*      */     }
/*  773 */     return parseFieldName(0, q, 0);
/*      */   }
/*      */   
/*      */ 
/*      */   protected final Name parseMediumFieldName(int q2, int[] codes)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  780 */     int i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  781 */     if (codes[i] != 0) {
/*  782 */       if (i == 34) {
/*  783 */         return findName(this._quad1, q2, 1);
/*      */       }
/*  785 */       return parseFieldName(this._quad1, q2, i, 1);
/*      */     }
/*  787 */     q2 = q2 << 8 | i;
/*  788 */     i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  789 */     if (codes[i] != 0) {
/*  790 */       if (i == 34) {
/*  791 */         return findName(this._quad1, q2, 2);
/*      */       }
/*  793 */       return parseFieldName(this._quad1, q2, i, 2);
/*      */     }
/*  795 */     q2 = q2 << 8 | i;
/*  796 */     i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  797 */     if (codes[i] != 0) {
/*  798 */       if (i == 34) {
/*  799 */         return findName(this._quad1, q2, 3);
/*      */       }
/*  801 */       return parseFieldName(this._quad1, q2, i, 3);
/*      */     }
/*  803 */     q2 = q2 << 8 | i;
/*  804 */     i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  805 */     if (codes[i] != 0) {
/*  806 */       if (i == 34) {
/*  807 */         return findName(this._quad1, q2, 4);
/*      */       }
/*  809 */       return parseFieldName(this._quad1, q2, i, 4);
/*      */     }
/*  811 */     this._quadBuffer[0] = this._quad1;
/*  812 */     this._quadBuffer[1] = q2;
/*  813 */     return parseLongFieldName(i);
/*      */   }
/*      */   
/*      */ 
/*      */   protected Name parseLongFieldName(int q)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  820 */     int[] codes = sInputCodesLatin1;
/*  821 */     int qlen = 2;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     for (;;)
/*      */     {
/*  828 */       if (this._inputEnd - this._inputPtr < 4) {
/*  829 */         return parseEscapedFieldName(this._quadBuffer, qlen, 0, q, 0);
/*      */       }
/*      */       
/*      */ 
/*  833 */       int i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  834 */       if (codes[i] != 0) {
/*  835 */         if (i == 34) {
/*  836 */           return findName(this._quadBuffer, qlen, q, 1);
/*      */         }
/*  838 */         return parseEscapedFieldName(this._quadBuffer, qlen, q, i, 1);
/*      */       }
/*      */       
/*  841 */       q = q << 8 | i;
/*  842 */       i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  843 */       if (codes[i] != 0) {
/*  844 */         if (i == 34) {
/*  845 */           return findName(this._quadBuffer, qlen, q, 2);
/*      */         }
/*  847 */         return parseEscapedFieldName(this._quadBuffer, qlen, q, i, 2);
/*      */       }
/*      */       
/*  850 */       q = q << 8 | i;
/*  851 */       i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  852 */       if (codes[i] != 0) {
/*  853 */         if (i == 34) {
/*  854 */           return findName(this._quadBuffer, qlen, q, 3);
/*      */         }
/*  856 */         return parseEscapedFieldName(this._quadBuffer, qlen, q, i, 3);
/*      */       }
/*      */       
/*  859 */       q = q << 8 | i;
/*  860 */       i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  861 */       if (codes[i] != 0) {
/*  862 */         if (i == 34) {
/*  863 */           return findName(this._quadBuffer, qlen, q, 4);
/*      */         }
/*  865 */         return parseEscapedFieldName(this._quadBuffer, qlen, q, i, 4);
/*      */       }
/*      */       
/*      */ 
/*  869 */       if (qlen >= this._quadBuffer.length) {
/*  870 */         this._quadBuffer = growArrayBy(this._quadBuffer, qlen);
/*      */       }
/*  872 */       this._quadBuffer[(qlen++)] = q;
/*  873 */       q = i;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected Name slowParseFieldName()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  885 */     if ((this._inputPtr >= this._inputEnd) && 
/*  886 */       (!loadMore())) {
/*  887 */       _reportInvalidEOF(": was expecting closing '\"' for name");
/*      */     }
/*      */     
/*  890 */     int i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*  891 */     if (i == 34) {
/*  892 */       return BytesToNameCanonicalizer.getEmptyName();
/*      */     }
/*  894 */     return parseEscapedFieldName(this._quadBuffer, 0, 0, i, 0);
/*      */   }
/*      */   
/*      */   private final Name parseFieldName(int q1, int ch, int lastQuadBytes)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  900 */     return parseEscapedFieldName(this._quadBuffer, 0, q1, ch, lastQuadBytes);
/*      */   }
/*      */   
/*      */   private final Name parseFieldName(int q1, int q2, int ch, int lastQuadBytes)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  906 */     this._quadBuffer[0] = q1;
/*  907 */     return parseEscapedFieldName(this._quadBuffer, 1, q2, ch, lastQuadBytes);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected Name parseEscapedFieldName(int[] quads, int qlen, int currQuad, int ch, int currQuadBytes)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  926 */     int[] codes = sInputCodesLatin1;
/*      */     for (;;)
/*      */     {
/*  929 */       if (codes[ch] != 0) {
/*  930 */         if (ch == 34) {
/*      */           break;
/*      */         }
/*      */         
/*  934 */         if (ch != 92)
/*      */         {
/*  936 */           _throwUnquotedSpace(ch, "name");
/*      */         }
/*      */         else {
/*  939 */           ch = _decodeEscaped();
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  946 */         if (ch > 127)
/*      */         {
/*  948 */           if (currQuadBytes >= 4) {
/*  949 */             if (qlen >= quads.length) {
/*  950 */               this._quadBuffer = (quads = growArrayBy(quads, quads.length));
/*      */             }
/*  952 */             quads[(qlen++)] = currQuad;
/*  953 */             currQuad = 0;
/*  954 */             currQuadBytes = 0;
/*      */           }
/*  956 */           if (ch < 2048) {
/*  957 */             currQuad = currQuad << 8 | 0xC0 | ch >> 6;
/*  958 */             currQuadBytes++;
/*      */           }
/*      */           else {
/*  961 */             currQuad = currQuad << 8 | 0xE0 | ch >> 12;
/*  962 */             currQuadBytes++;
/*      */             
/*  964 */             if (currQuadBytes >= 4) {
/*  965 */               if (qlen >= quads.length) {
/*  966 */                 this._quadBuffer = (quads = growArrayBy(quads, quads.length));
/*      */               }
/*  968 */               quads[(qlen++)] = currQuad;
/*  969 */               currQuad = 0;
/*  970 */               currQuadBytes = 0;
/*      */             }
/*  972 */             currQuad = currQuad << 8 | 0x80 | ch >> 6 & 0x3F;
/*  973 */             currQuadBytes++;
/*      */           }
/*      */           
/*  976 */           ch = 0x80 | ch & 0x3F;
/*      */         }
/*      */       }
/*      */       
/*  980 */       if (currQuadBytes < 4) {
/*  981 */         currQuadBytes++;
/*  982 */         currQuad = currQuad << 8 | ch;
/*      */       } else {
/*  984 */         if (qlen >= quads.length) {
/*  985 */           this._quadBuffer = (quads = growArrayBy(quads, quads.length));
/*      */         }
/*  987 */         quads[(qlen++)] = currQuad;
/*  988 */         currQuad = ch;
/*  989 */         currQuadBytes = 1;
/*      */       }
/*  991 */       if ((this._inputPtr >= this._inputEnd) && 
/*  992 */         (!loadMore())) {
/*  993 */         _reportInvalidEOF(" in field name");
/*      */       }
/*      */       
/*  996 */       ch = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*      */     }
/*      */     
/*  999 */     if (currQuadBytes > 0) {
/* 1000 */       if (qlen >= quads.length) {
/* 1001 */         this._quadBuffer = (quads = growArrayBy(quads, quads.length));
/*      */       }
/* 1003 */       quads[(qlen++)] = currQuad;
/*      */     }
/* 1005 */     Name name = this._symbols.findName(quads, qlen);
/* 1006 */     if (name == null) {
/* 1007 */       name = addName(quads, qlen, currQuadBytes);
/*      */     }
/* 1009 */     return name;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final Name _handleUnusualFieldName(int ch)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1022 */     if ((ch == 39) && (isEnabled(JsonParser.Feature.ALLOW_SINGLE_QUOTES))) {
/* 1023 */       return _parseApostropheFieldName();
/*      */     }
/*      */     
/* 1026 */     if (!isEnabled(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES)) {
/* 1027 */       _reportUnexpectedChar(ch, "was expecting double-quote to start field name");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1033 */     int[] codes = CharTypes.getInputCodeUtf8JsNames();
/*      */     
/* 1035 */     if (codes[ch] != 0) {
/* 1036 */       _reportUnexpectedChar(ch, "was expecting either valid name character (for unquoted name) or double-quote (for quoted) to start field name");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1043 */     int[] quads = this._quadBuffer;
/* 1044 */     int qlen = 0;
/* 1045 */     int currQuad = 0;
/* 1046 */     int currQuadBytes = 0;
/*      */     
/*      */     for (;;)
/*      */     {
/* 1050 */       if (currQuadBytes < 4) {
/* 1051 */         currQuadBytes++;
/* 1052 */         currQuad = currQuad << 8 | ch;
/*      */       } else {
/* 1054 */         if (qlen >= quads.length) {
/* 1055 */           this._quadBuffer = (quads = growArrayBy(quads, quads.length));
/*      */         }
/* 1057 */         quads[(qlen++)] = currQuad;
/* 1058 */         currQuad = ch;
/* 1059 */         currQuadBytes = 1;
/*      */       }
/* 1061 */       if ((this._inputPtr >= this._inputEnd) && 
/* 1062 */         (!loadMore())) {
/* 1063 */         _reportInvalidEOF(" in field name");
/*      */       }
/*      */       
/* 1066 */       ch = this._inputBuffer[this._inputPtr] & 0xFF;
/* 1067 */       if (codes[ch] != 0) {
/*      */         break;
/*      */       }
/* 1070 */       this._inputPtr += 1;
/*      */     }
/*      */     
/* 1073 */     if (currQuadBytes > 0) {
/* 1074 */       if (qlen >= quads.length) {
/* 1075 */         this._quadBuffer = (quads = growArrayBy(quads, quads.length));
/*      */       }
/* 1077 */       quads[(qlen++)] = currQuad;
/*      */     }
/* 1079 */     Name name = this._symbols.findName(quads, qlen);
/* 1080 */     if (name == null) {
/* 1081 */       name = addName(quads, qlen, currQuadBytes);
/*      */     }
/* 1083 */     return name;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final Name _parseApostropheFieldName()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1094 */     if ((this._inputPtr >= this._inputEnd) && 
/* 1095 */       (!loadMore())) {
/* 1096 */       _reportInvalidEOF(": was expecting closing ''' for name");
/*      */     }
/*      */     
/* 1099 */     int ch = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/* 1100 */     if (ch == 39) {
/* 1101 */       return BytesToNameCanonicalizer.getEmptyName();
/*      */     }
/* 1103 */     int[] quads = this._quadBuffer;
/* 1104 */     int qlen = 0;
/* 1105 */     int currQuad = 0;
/* 1106 */     int currQuadBytes = 0;
/*      */     
/*      */ 
/*      */ 
/* 1110 */     int[] codes = sInputCodesLatin1;
/*      */     
/*      */ 
/* 1113 */     while (ch != 39)
/*      */     {
/*      */ 
/*      */ 
/* 1117 */       if ((ch != 34) && (codes[ch] != 0)) {
/* 1118 */         if (ch != 92)
/*      */         {
/*      */ 
/* 1121 */           _throwUnquotedSpace(ch, "name");
/*      */         }
/*      */         else {
/* 1124 */           ch = _decodeEscaped();
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1131 */         if (ch > 127)
/*      */         {
/* 1133 */           if (currQuadBytes >= 4) {
/* 1134 */             if (qlen >= quads.length) {
/* 1135 */               this._quadBuffer = (quads = growArrayBy(quads, quads.length));
/*      */             }
/* 1137 */             quads[(qlen++)] = currQuad;
/* 1138 */             currQuad = 0;
/* 1139 */             currQuadBytes = 0;
/*      */           }
/* 1141 */           if (ch < 2048) {
/* 1142 */             currQuad = currQuad << 8 | 0xC0 | ch >> 6;
/* 1143 */             currQuadBytes++;
/*      */           }
/*      */           else {
/* 1146 */             currQuad = currQuad << 8 | 0xE0 | ch >> 12;
/* 1147 */             currQuadBytes++;
/*      */             
/* 1149 */             if (currQuadBytes >= 4) {
/* 1150 */               if (qlen >= quads.length) {
/* 1151 */                 this._quadBuffer = (quads = growArrayBy(quads, quads.length));
/*      */               }
/* 1153 */               quads[(qlen++)] = currQuad;
/* 1154 */               currQuad = 0;
/* 1155 */               currQuadBytes = 0;
/*      */             }
/* 1157 */             currQuad = currQuad << 8 | 0x80 | ch >> 6 & 0x3F;
/* 1158 */             currQuadBytes++;
/*      */           }
/*      */           
/* 1161 */           ch = 0x80 | ch & 0x3F;
/*      */         }
/*      */       }
/*      */       
/* 1165 */       if (currQuadBytes < 4) {
/* 1166 */         currQuadBytes++;
/* 1167 */         currQuad = currQuad << 8 | ch;
/*      */       } else {
/* 1169 */         if (qlen >= quads.length) {
/* 1170 */           this._quadBuffer = (quads = growArrayBy(quads, quads.length));
/*      */         }
/* 1172 */         quads[(qlen++)] = currQuad;
/* 1173 */         currQuad = ch;
/* 1174 */         currQuadBytes = 1;
/*      */       }
/* 1176 */       if ((this._inputPtr >= this._inputEnd) && 
/* 1177 */         (!loadMore())) {
/* 1178 */         _reportInvalidEOF(" in field name");
/*      */       }
/*      */       
/* 1181 */       ch = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*      */     }
/*      */     
/* 1184 */     if (currQuadBytes > 0) {
/* 1185 */       if (qlen >= quads.length) {
/* 1186 */         this._quadBuffer = (quads = growArrayBy(quads, quads.length));
/*      */       }
/* 1188 */       quads[(qlen++)] = currQuad;
/*      */     }
/* 1190 */     Name name = this._symbols.findName(quads, qlen);
/* 1191 */     if (name == null) {
/* 1192 */       name = addName(quads, qlen, currQuadBytes);
/*      */     }
/* 1194 */     return name;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final Name findName(int q1, int lastQuadBytes)
/*      */     throws JsonParseException
/*      */   {
/* 1207 */     Name name = this._symbols.findName(q1);
/* 1208 */     if (name != null) {
/* 1209 */       return name;
/*      */     }
/*      */     
/* 1212 */     this._quadBuffer[0] = q1;
/* 1213 */     return addName(this._quadBuffer, 1, lastQuadBytes);
/*      */   }
/*      */   
/*      */ 
/*      */   private final Name findName(int q1, int q2, int lastQuadBytes)
/*      */     throws JsonParseException
/*      */   {
/* 1220 */     Name name = this._symbols.findName(q1, q2);
/* 1221 */     if (name != null) {
/* 1222 */       return name;
/*      */     }
/*      */     
/* 1225 */     this._quadBuffer[0] = q1;
/* 1226 */     this._quadBuffer[1] = q2;
/* 1227 */     return addName(this._quadBuffer, 2, lastQuadBytes);
/*      */   }
/*      */   
/*      */   private final Name findName(int[] quads, int qlen, int lastQuad, int lastQuadBytes)
/*      */     throws JsonParseException
/*      */   {
/* 1233 */     if (qlen >= quads.length) {
/* 1234 */       this._quadBuffer = (quads = growArrayBy(quads, quads.length));
/*      */     }
/* 1236 */     quads[(qlen++)] = lastQuad;
/* 1237 */     Name name = this._symbols.findName(quads, qlen);
/* 1238 */     if (name == null) {
/* 1239 */       return addName(quads, qlen, lastQuadBytes);
/*      */     }
/* 1241 */     return name;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final Name addName(int[] quads, int qlen, int lastQuadBytes)
/*      */     throws JsonParseException
/*      */   {
/* 1258 */     int byteLen = (qlen << 2) - 4 + lastQuadBytes;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     int lastQuad;
/*      */     
/*      */ 
/*      */ 
/* 1267 */     if (lastQuadBytes < 4) {
/* 1268 */       int lastQuad = quads[(qlen - 1)];
/*      */       
/* 1270 */       quads[(qlen - 1)] = (lastQuad << (4 - lastQuadBytes << 3));
/*      */     } else {
/* 1272 */       lastQuad = 0;
/*      */     }
/*      */     
/*      */ 
/* 1276 */     char[] cbuf = this._textBuffer.emptyAndGetCurrentSegment();
/* 1277 */     int cix = 0;
/*      */     
/* 1279 */     for (int ix = 0; ix < byteLen;) {
/* 1280 */       int ch = quads[(ix >> 2)];
/* 1281 */       int byteIx = ix & 0x3;
/* 1282 */       ch = ch >> (3 - byteIx << 3) & 0xFF;
/* 1283 */       ix++;
/*      */       
/* 1285 */       if (ch > 127) { int needed;
/*      */         int needed;
/* 1287 */         if ((ch & 0xE0) == 192) {
/* 1288 */           ch &= 0x1F;
/* 1289 */           needed = 1; } else { int needed;
/* 1290 */           if ((ch & 0xF0) == 224) {
/* 1291 */             ch &= 0xF;
/* 1292 */             needed = 2; } else { int needed;
/* 1293 */             if ((ch & 0xF8) == 240) {
/* 1294 */               ch &= 0x7;
/* 1295 */               needed = 3;
/*      */             } else {
/* 1297 */               _reportInvalidInitial(ch);
/* 1298 */               needed = ch = 1;
/*      */             } } }
/* 1300 */         if (ix + needed > byteLen) {
/* 1301 */           _reportInvalidEOF(" in field name");
/*      */         }
/*      */         
/*      */ 
/* 1305 */         int ch2 = quads[(ix >> 2)];
/* 1306 */         byteIx = ix & 0x3;
/* 1307 */         ch2 >>= 3 - byteIx << 3;
/* 1308 */         ix++;
/*      */         
/* 1310 */         if ((ch2 & 0xC0) != 128) {
/* 1311 */           _reportInvalidOther(ch2);
/*      */         }
/* 1313 */         ch = ch << 6 | ch2 & 0x3F;
/* 1314 */         if (needed > 1) {
/* 1315 */           ch2 = quads[(ix >> 2)];
/* 1316 */           byteIx = ix & 0x3;
/* 1317 */           ch2 >>= 3 - byteIx << 3;
/* 1318 */           ix++;
/*      */           
/* 1320 */           if ((ch2 & 0xC0) != 128) {
/* 1321 */             _reportInvalidOther(ch2);
/*      */           }
/* 1323 */           ch = ch << 6 | ch2 & 0x3F;
/* 1324 */           if (needed > 2) {
/* 1325 */             ch2 = quads[(ix >> 2)];
/* 1326 */             byteIx = ix & 0x3;
/* 1327 */             ch2 >>= 3 - byteIx << 3;
/* 1328 */             ix++;
/* 1329 */             if ((ch2 & 0xC0) != 128) {
/* 1330 */               _reportInvalidOther(ch2 & 0xFF);
/*      */             }
/* 1332 */             ch = ch << 6 | ch2 & 0x3F;
/*      */           }
/*      */         }
/* 1335 */         if (needed > 2) {
/* 1336 */           ch -= 65536;
/* 1337 */           if (cix >= cbuf.length) {
/* 1338 */             cbuf = this._textBuffer.expandCurrentSegment();
/*      */           }
/* 1340 */           cbuf[(cix++)] = ((char)(55296 + (ch >> 10)));
/* 1341 */           ch = 0xDC00 | ch & 0x3FF;
/*      */         }
/*      */       }
/* 1344 */       if (cix >= cbuf.length) {
/* 1345 */         cbuf = this._textBuffer.expandCurrentSegment();
/*      */       }
/* 1347 */       cbuf[(cix++)] = ((char)ch);
/*      */     }
/*      */     
/*      */ 
/* 1351 */     String baseName = new String(cbuf, 0, cix);
/*      */     
/* 1353 */     if (lastQuadBytes < 4) {
/* 1354 */       quads[(qlen - 1)] = lastQuad;
/*      */     }
/* 1356 */     return this._symbols.addName(baseName, quads, qlen);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void _finishString()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1370 */     int ptr = this._inputPtr;
/* 1371 */     if (ptr >= this._inputEnd) {
/* 1372 */       loadMoreGuaranteed();
/* 1373 */       ptr = this._inputPtr;
/*      */     }
/* 1375 */     int outPtr = 0;
/* 1376 */     char[] outBuf = this._textBuffer.emptyAndGetCurrentSegment();
/* 1377 */     int[] codes = sInputCodesUtf8;
/*      */     
/* 1379 */     int max = Math.min(this._inputEnd, ptr + outBuf.length);
/* 1380 */     byte[] inputBuffer = this._inputBuffer;
/* 1381 */     while (ptr < max) {
/* 1382 */       int c = inputBuffer[ptr] & 0xFF;
/* 1383 */       if (codes[c] != 0) {
/* 1384 */         if (c != 34) break;
/* 1385 */         this._inputPtr = (ptr + 1);
/* 1386 */         this._textBuffer.setCurrentLength(outPtr);
/* 1387 */         return;
/*      */       }
/*      */       
/*      */ 
/* 1391 */       ptr++;
/* 1392 */       outBuf[(outPtr++)] = ((char)c);
/*      */     }
/* 1394 */     this._inputPtr = ptr;
/* 1395 */     _finishString2(outBuf, outPtr);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private final void _finishString2(char[] outBuf, int outPtr)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1404 */     int[] codes = sInputCodesUtf8;
/* 1405 */     byte[] inputBuffer = this._inputBuffer;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     for (;;)
/*      */     {
/* 1412 */       int ptr = this._inputPtr;
/* 1413 */       if (ptr >= this._inputEnd) {
/* 1414 */         loadMoreGuaranteed();
/* 1415 */         ptr = this._inputPtr;
/*      */       }
/* 1417 */       if (outPtr >= outBuf.length) {
/* 1418 */         outBuf = this._textBuffer.finishCurrentSegment();
/* 1419 */         outPtr = 0;
/*      */       }
/* 1421 */       int max = Math.min(this._inputEnd, ptr + (outBuf.length - outPtr));
/* 1422 */       while (ptr < max) {
/* 1423 */         int c = inputBuffer[(ptr++)] & 0xFF;
/* 1424 */         if (codes[c] != 0) {
/* 1425 */           this._inputPtr = ptr;
/*      */           break label124;
/*      */         }
/* 1428 */         outBuf[(outPtr++)] = ((char)c);
/*      */       }
/* 1430 */       this._inputPtr = ptr;
/* 1431 */       continue;
/*      */       label124:
/* 1433 */       int c; if (c == 34) {
/*      */         break;
/*      */       }
/*      */       
/* 1437 */       switch (codes[c]) {
/*      */       case 1: 
/* 1439 */         c = _decodeEscaped();
/* 1440 */         break;
/*      */       case 2: 
/* 1442 */         c = _decodeUtf8_2(c);
/* 1443 */         break;
/*      */       case 3: 
/* 1445 */         if (this._inputEnd - this._inputPtr >= 2) {
/* 1446 */           c = _decodeUtf8_3fast(c);
/*      */         } else {
/* 1448 */           c = _decodeUtf8_3(c);
/*      */         }
/* 1450 */         break;
/*      */       case 4: 
/* 1452 */         c = _decodeUtf8_4(c);
/*      */         
/* 1454 */         outBuf[(outPtr++)] = ((char)(0xD800 | c >> 10));
/* 1455 */         if (outPtr >= outBuf.length) {
/* 1456 */           outBuf = this._textBuffer.finishCurrentSegment();
/* 1457 */           outPtr = 0;
/*      */         }
/* 1459 */         c = 0xDC00 | c & 0x3FF;
/*      */         
/* 1461 */         break;
/*      */       default: 
/* 1463 */         if (c < 32)
/*      */         {
/* 1465 */           _throwUnquotedSpace(c, "string value");
/*      */         }
/*      */         else {
/* 1468 */           _reportInvalidChar(c);
/*      */         }
/*      */         break;
/*      */       }
/* 1472 */       if (outPtr >= outBuf.length) {
/* 1473 */         outBuf = this._textBuffer.finishCurrentSegment();
/* 1474 */         outPtr = 0;
/*      */       }
/*      */       
/* 1477 */       outBuf[(outPtr++)] = ((char)c);
/*      */     }
/* 1479 */     this._textBuffer.setCurrentLength(outPtr);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void _skipString()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1490 */     this._tokenIncomplete = false;
/*      */     
/*      */ 
/* 1493 */     int[] codes = sInputCodesUtf8;
/* 1494 */     byte[] inputBuffer = this._inputBuffer;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     for (;;)
/*      */     {
/* 1502 */       int ptr = this._inputPtr;
/* 1503 */       int max = this._inputEnd;
/* 1504 */       if (ptr >= max) {
/* 1505 */         loadMoreGuaranteed();
/* 1506 */         ptr = this._inputPtr;
/* 1507 */         max = this._inputEnd;
/*      */       }
/* 1509 */       while (ptr < max) {
/* 1510 */         int c = inputBuffer[(ptr++)] & 0xFF;
/* 1511 */         if (codes[c] != 0) {
/* 1512 */           this._inputPtr = ptr;
/*      */           break label92;
/*      */         }
/*      */       }
/* 1516 */       this._inputPtr = ptr;
/* 1517 */       continue;
/*      */       label92:
/* 1519 */       int c; if (c == 34) {
/*      */         break;
/*      */       }
/*      */       
/* 1523 */       switch (codes[c]) {
/*      */       case 1: 
/* 1525 */         _decodeEscaped();
/* 1526 */         break;
/*      */       case 2: 
/* 1528 */         _skipUtf8_2(c);
/* 1529 */         break;
/*      */       case 3: 
/* 1531 */         _skipUtf8_3(c);
/* 1532 */         break;
/*      */       case 4: 
/* 1534 */         _skipUtf8_4(c);
/* 1535 */         break;
/*      */       default: 
/* 1537 */         if (c < 32)
/*      */         {
/* 1539 */           _throwUnquotedSpace(c, "string value");
/*      */         }
/*      */         else {
/* 1542 */           _reportInvalidChar(c);
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */         break;
/*      */       }
/*      */       
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   protected JsonToken _handleUnexpectedValue(int c)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1558 */     switch (c) {
/*      */     case 39: 
/* 1560 */       if (isEnabled(JsonParser.Feature.ALLOW_SINGLE_QUOTES)) {
/* 1561 */         return _handleApostropheValue();
/*      */       }
/*      */       break;
/*      */     case 78: 
/* 1565 */       if (_matchToken("NaN", 1)) {
/* 1566 */         if (isEnabled(JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS)) {
/* 1567 */           return resetAsNaN("NaN", NaN.0D);
/*      */         }
/* 1569 */         _reportError("Non-standard token 'NaN': enable JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS to allow");
/*      */       }
/* 1571 */       _reportUnexpectedChar(this._inputBuffer[(this._inputPtr++)] & 0xFF, "expected 'NaN' or a valid value");
/* 1572 */       break;
/*      */     case 43: 
/* 1574 */       if ((this._inputPtr >= this._inputEnd) && 
/* 1575 */         (!loadMore())) {
/* 1576 */         _reportInvalidEOFInValue();
/*      */       }
/*      */       
/* 1579 */       return _handleInvalidNumberStart(this._inputBuffer[(this._inputPtr++)] & 0xFF, false);
/*      */     }
/*      */     
/* 1582 */     _reportUnexpectedChar(c, "expected a valid value (number, String, array, object, 'true', 'false' or 'null')");
/* 1583 */     return null;
/*      */   }
/*      */   
/*      */   protected JsonToken _handleApostropheValue()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1589 */     int c = 0;
/*      */     
/* 1591 */     int outPtr = 0;
/* 1592 */     char[] outBuf = this._textBuffer.emptyAndGetCurrentSegment();
/*      */     
/*      */ 
/* 1595 */     int[] codes = sInputCodesUtf8;
/* 1596 */     byte[] inputBuffer = this._inputBuffer;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     for (;;)
/*      */     {
/* 1603 */       if (this._inputPtr >= this._inputEnd) {
/* 1604 */         loadMoreGuaranteed();
/*      */       }
/* 1606 */       if (outPtr >= outBuf.length) {
/* 1607 */         outBuf = this._textBuffer.finishCurrentSegment();
/* 1608 */         outPtr = 0;
/*      */       }
/* 1610 */       int max = this._inputEnd;
/*      */       
/* 1612 */       int max2 = this._inputPtr + (outBuf.length - outPtr);
/* 1613 */       if (max2 < max) {
/* 1614 */         max = max2;
/*      */       }
/*      */       
/* 1617 */       while (this._inputPtr < max) {
/* 1618 */         c = inputBuffer[(this._inputPtr++)] & 0xFF;
/* 1619 */         if ((c == 39) || (codes[c] != 0)) {
/*      */           break label140;
/*      */         }
/* 1622 */         outBuf[(outPtr++)] = ((char)c);
/*      */       }
/* 1624 */       continue;
/*      */       
/*      */       label140:
/* 1627 */       if (c == 39) {
/*      */         break;
/*      */       }
/*      */       
/* 1631 */       switch (codes[c]) {
/*      */       case 1: 
/* 1633 */         if (c != 34) {
/* 1634 */           c = _decodeEscaped();
/*      */         }
/*      */         break;
/*      */       case 2: 
/* 1638 */         c = _decodeUtf8_2(c);
/* 1639 */         break;
/*      */       case 3: 
/* 1641 */         if (this._inputEnd - this._inputPtr >= 2) {
/* 1642 */           c = _decodeUtf8_3fast(c);
/*      */         } else {
/* 1644 */           c = _decodeUtf8_3(c);
/*      */         }
/* 1646 */         break;
/*      */       case 4: 
/* 1648 */         c = _decodeUtf8_4(c);
/*      */         
/* 1650 */         outBuf[(outPtr++)] = ((char)(0xD800 | c >> 10));
/* 1651 */         if (outPtr >= outBuf.length) {
/* 1652 */           outBuf = this._textBuffer.finishCurrentSegment();
/* 1653 */           outPtr = 0;
/*      */         }
/* 1655 */         c = 0xDC00 | c & 0x3FF;
/*      */         
/* 1657 */         break;
/*      */       default: 
/* 1659 */         if (c < 32) {
/* 1660 */           _throwUnquotedSpace(c, "string value");
/*      */         }
/*      */         
/* 1663 */         _reportInvalidChar(c);
/*      */       }
/*      */       
/* 1666 */       if (outPtr >= outBuf.length) {
/* 1667 */         outBuf = this._textBuffer.finishCurrentSegment();
/* 1668 */         outPtr = 0;
/*      */       }
/*      */       
/* 1671 */       outBuf[(outPtr++)] = ((char)c);
/*      */     }
/* 1673 */     this._textBuffer.setCurrentLength(outPtr);
/*      */     
/* 1675 */     return JsonToken.VALUE_STRING;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JsonToken _handleInvalidNumberStart(int ch, boolean negative)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1685 */     if (ch == 73) {
/* 1686 */       if ((this._inputPtr >= this._inputEnd) && 
/* 1687 */         (!loadMore())) {
/* 1688 */         _reportInvalidEOFInValue();
/*      */       }
/*      */       
/* 1691 */       ch = this._inputBuffer[(this._inputPtr++)];
/* 1692 */       if (ch == 78) {
/* 1693 */         String match = negative ? "-INF" : "+INF";
/* 1694 */         if (_matchToken(match, 3)) {
/* 1695 */           if (isEnabled(JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS)) {
/* 1696 */             return resetAsNaN(match, negative ? Double.NEGATIVE_INFINITY : Double.POSITIVE_INFINITY);
/*      */           }
/* 1698 */           _reportError("Non-standard token '" + match + "': enable JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS to allow");
/*      */         }
/* 1700 */       } else if (ch == 110) {
/* 1701 */         String match = negative ? "-Infinity" : "+Infinity";
/* 1702 */         if (_matchToken(match, 3)) {
/* 1703 */           if (isEnabled(JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS)) {
/* 1704 */             return resetAsNaN(match, negative ? Double.NEGATIVE_INFINITY : Double.POSITIVE_INFINITY);
/*      */           }
/* 1706 */           _reportError("Non-standard token '" + match + "': enable JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS to allow");
/*      */         }
/*      */       }
/*      */     }
/* 1710 */     reportUnexpectedNumberChar(ch, "expected digit (0-9) to follow minus sign, for valid numeric value");
/* 1711 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */   protected void _matchToken(JsonToken token)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1718 */     byte[] matchBytes = token.asByteArray();
/* 1719 */     int i = 1;
/*      */     
/* 1721 */     for (int len = matchBytes.length; i < len; i++) {
/* 1722 */       if (this._inputPtr >= this._inputEnd) {
/* 1723 */         loadMoreGuaranteed();
/*      */       }
/* 1725 */       if (matchBytes[i] != this._inputBuffer[this._inputPtr]) {
/* 1726 */         _reportInvalidToken(token.asString().substring(0, i), "'null', 'true' or 'false'");
/*      */       }
/* 1728 */       this._inputPtr += 1;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final boolean _matchToken(String matchStr, int i)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1739 */     int len = matchStr.length();
/*      */     do
/*      */     {
/* 1742 */       if ((this._inputPtr >= this._inputEnd) && 
/* 1743 */         (!loadMore())) {
/* 1744 */         _reportInvalidEOF(" in a value");
/*      */       }
/*      */       
/* 1747 */       if (this._inputBuffer[this._inputPtr] != matchStr.charAt(i)) {
/* 1748 */         _reportInvalidToken(matchStr.substring(0, i), "'null', 'true', 'false' or NaN");
/*      */       }
/* 1750 */       this._inputPtr += 1;
/* 1751 */       i++; } while (i < len);
/*      */     
/*      */ 
/* 1754 */     if ((this._inputPtr >= this._inputEnd) && 
/* 1755 */       (!loadMore())) {
/* 1756 */       return true;
/*      */     }
/*      */     
/* 1759 */     char c = (char)_decodeCharForError(this._inputBuffer[this._inputPtr] & 0xFF);
/*      */     
/* 1761 */     if (Character.isJavaIdentifierPart(c)) {
/* 1762 */       this._inputPtr += 1;
/* 1763 */       _reportInvalidToken(matchStr.substring(0, i), "'null', 'true', 'false' or NaN");
/*      */     }
/* 1765 */     return true;
/*      */   }
/*      */   
/*      */   protected void _reportInvalidToken(String matchedPart, String msg)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1771 */     StringBuilder sb = new StringBuilder(matchedPart);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1777 */     while ((this._inputPtr < this._inputEnd) || (loadMore()))
/*      */     {
/*      */ 
/* 1780 */       int i = this._inputBuffer[(this._inputPtr++)];
/* 1781 */       char c = (char)_decodeCharForError(i);
/* 1782 */       if (!Character.isJavaIdentifierPart(c)) {
/*      */         break;
/*      */       }
/* 1785 */       this._inputPtr += 1;
/* 1786 */       sb.append(c);
/*      */     }
/* 1788 */     _reportError("Unrecognized token '" + sb.toString() + "': was expecting " + msg);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final int _skipWS()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1800 */     while ((this._inputPtr < this._inputEnd) || (loadMore())) {
/* 1801 */       int i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/* 1802 */       if (i > 32) {
/* 1803 */         if (i != 47) {
/* 1804 */           return i;
/*      */         }
/* 1806 */         _skipComment();
/* 1807 */       } else if (i != 32) {
/* 1808 */         if (i == 10) {
/* 1809 */           _skipLF();
/* 1810 */         } else if (i == 13) {
/* 1811 */           _skipCR();
/* 1812 */         } else if (i != 9) {
/* 1813 */           _throwInvalidSpace(i);
/*      */         }
/*      */       }
/*      */     }
/* 1817 */     throw _constructError("Unexpected end-of-input within/between " + this._parsingContext.getTypeDesc() + " entries");
/*      */   }
/*      */   
/*      */   private final int _skipWSOrEnd()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1823 */     while ((this._inputPtr < this._inputEnd) || (loadMore())) {
/* 1824 */       int i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/* 1825 */       if (i > 32) {
/* 1826 */         if (i != 47) {
/* 1827 */           return i;
/*      */         }
/* 1829 */         _skipComment();
/* 1830 */       } else if (i != 32) {
/* 1831 */         if (i == 10) {
/* 1832 */           _skipLF();
/* 1833 */         } else if (i == 13) {
/* 1834 */           _skipCR();
/* 1835 */         } else if (i != 9) {
/* 1836 */           _throwInvalidSpace(i);
/*      */         }
/*      */       }
/*      */     }
/*      */     
/* 1841 */     _handleEOF();
/* 1842 */     return -1;
/*      */   }
/*      */   
/*      */   private final void _skipComment()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1848 */     if (!isEnabled(JsonParser.Feature.ALLOW_COMMENTS)) {
/* 1849 */       _reportUnexpectedChar(47, "maybe a (non-standard) comment? (not recognized as one since Feature 'ALLOW_COMMENTS' not enabled for parser)");
/*      */     }
/*      */     
/* 1852 */     if ((this._inputPtr >= this._inputEnd) && (!loadMore())) {
/* 1853 */       _reportInvalidEOF(" in a comment");
/*      */     }
/* 1855 */     int c = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/* 1856 */     if (c == 47) {
/* 1857 */       _skipCppComment();
/* 1858 */     } else if (c == 42) {
/* 1859 */       _skipCComment();
/*      */     } else {
/* 1861 */       _reportUnexpectedChar(c, "was expecting either '*' or '/' for a comment");
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private final void _skipCComment()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1869 */     int[] codes = CharTypes.getInputCodeComment();
/*      */     
/*      */ 
/* 1872 */     while ((this._inputPtr < this._inputEnd) || (loadMore())) {
/* 1873 */       int i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/* 1874 */       int code = codes[i];
/* 1875 */       if (code != 0) {
/* 1876 */         switch (code) {
/*      */         case 42: 
/* 1878 */           if (this._inputBuffer[this._inputPtr] == 47) {
/* 1879 */             this._inputPtr += 1; return;
/*      */           }
/*      */           
/*      */           break;
/*      */         case 10: 
/* 1884 */           _skipLF();
/* 1885 */           break;
/*      */         case 13: 
/* 1887 */           _skipCR();
/* 1888 */           break;
/*      */         
/*      */         default: 
/* 1891 */           _reportInvalidChar(i);
/*      */         }
/*      */       }
/*      */     }
/* 1895 */     _reportInvalidEOF(" in a comment");
/*      */   }
/*      */   
/*      */ 
/*      */   private final void _skipCppComment()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1902 */     int[] codes = CharTypes.getInputCodeComment();
/* 1903 */     while ((this._inputPtr < this._inputEnd) || (loadMore())) {
/* 1904 */       int i = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/* 1905 */       int code = codes[i];
/* 1906 */       if (code != 0) {
/* 1907 */         switch (code) {
/*      */         case 10: 
/* 1909 */           _skipLF();
/* 1910 */           return;
/*      */         case 13: 
/* 1912 */           _skipCR(); return;
/*      */         
/*      */         case 42: 
/*      */           break;
/*      */         
/*      */         default: 
/* 1918 */           _reportInvalidChar(i);
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   protected final char _decodeEscaped()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1927 */     if ((this._inputPtr >= this._inputEnd) && 
/* 1928 */       (!loadMore())) {
/* 1929 */       _reportInvalidEOF(" in character escape sequence");
/*      */     }
/*      */     
/* 1932 */     int c = this._inputBuffer[(this._inputPtr++)];
/*      */     
/* 1934 */     switch (c)
/*      */     {
/*      */     case 98: 
/* 1937 */       return '\b';
/*      */     case 116: 
/* 1939 */       return '\t';
/*      */     case 110: 
/* 1941 */       return '\n';
/*      */     case 102: 
/* 1943 */       return '\f';
/*      */     case 114: 
/* 1945 */       return '\r';
/*      */     
/*      */ 
/*      */     case 34: 
/*      */     case 47: 
/*      */     case 92: 
/* 1951 */       return (char)c;
/*      */     
/*      */     case 117: 
/*      */       break;
/*      */     
/*      */     default: 
/* 1957 */       return _handleUnrecognizedCharacterEscape((char)_decodeCharForError(c));
/*      */     }
/*      */     
/*      */     
/* 1961 */     int value = 0;
/* 1962 */     for (int i = 0; i < 4; i++) {
/* 1963 */       if ((this._inputPtr >= this._inputEnd) && 
/* 1964 */         (!loadMore())) {
/* 1965 */         _reportInvalidEOF(" in character escape sequence");
/*      */       }
/*      */       
/* 1968 */       int ch = this._inputBuffer[(this._inputPtr++)];
/* 1969 */       int digit = CharTypes.charToHex(ch);
/* 1970 */       if (digit < 0) {
/* 1971 */         _reportUnexpectedChar(ch, "expected a hex-digit for character escape sequence");
/*      */       }
/* 1973 */       value = value << 4 | digit;
/*      */     }
/* 1975 */     return (char)value;
/*      */   }
/*      */   
/*      */   protected int _decodeCharForError(int firstByte)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1981 */     int c = firstByte;
/* 1982 */     if (c < 0)
/*      */     {
/*      */       int needed;
/*      */       int needed;
/* 1986 */       if ((c & 0xE0) == 192) {
/* 1987 */         c &= 0x1F;
/* 1988 */         needed = 1; } else { int needed;
/* 1989 */         if ((c & 0xF0) == 224) {
/* 1990 */           c &= 0xF;
/* 1991 */           needed = 2; } else { int needed;
/* 1992 */           if ((c & 0xF8) == 240)
/*      */           {
/* 1994 */             c &= 0x7;
/* 1995 */             needed = 3;
/*      */           } else {
/* 1997 */             _reportInvalidInitial(c & 0xFF);
/* 1998 */             needed = 1;
/*      */           }
/*      */         } }
/* 2001 */       int d = nextByte();
/* 2002 */       if ((d & 0xC0) != 128) {
/* 2003 */         _reportInvalidOther(d & 0xFF);
/*      */       }
/* 2005 */       c = c << 6 | d & 0x3F;
/*      */       
/* 2007 */       if (needed > 1) {
/* 2008 */         d = nextByte();
/* 2009 */         if ((d & 0xC0) != 128) {
/* 2010 */           _reportInvalidOther(d & 0xFF);
/*      */         }
/* 2012 */         c = c << 6 | d & 0x3F;
/* 2013 */         if (needed > 2) {
/* 2014 */           d = nextByte();
/* 2015 */           if ((d & 0xC0) != 128) {
/* 2016 */             _reportInvalidOther(d & 0xFF);
/*      */           }
/* 2018 */           c = c << 6 | d & 0x3F;
/*      */         }
/*      */       }
/*      */     }
/* 2022 */     return c;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final int _decodeUtf8_2(int c)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 2034 */     if (this._inputPtr >= this._inputEnd) {
/* 2035 */       loadMoreGuaranteed();
/*      */     }
/* 2037 */     int d = this._inputBuffer[(this._inputPtr++)];
/* 2038 */     if ((d & 0xC0) != 128) {
/* 2039 */       _reportInvalidOther(d & 0xFF, this._inputPtr);
/*      */     }
/* 2041 */     return (c & 0x1F) << 6 | d & 0x3F;
/*      */   }
/*      */   
/*      */   private final int _decodeUtf8_3(int c1)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 2047 */     if (this._inputPtr >= this._inputEnd) {
/* 2048 */       loadMoreGuaranteed();
/*      */     }
/* 2050 */     c1 &= 0xF;
/* 2051 */     int d = this._inputBuffer[(this._inputPtr++)];
/* 2052 */     if ((d & 0xC0) != 128) {
/* 2053 */       _reportInvalidOther(d & 0xFF, this._inputPtr);
/*      */     }
/* 2055 */     int c = c1 << 6 | d & 0x3F;
/* 2056 */     if (this._inputPtr >= this._inputEnd) {
/* 2057 */       loadMoreGuaranteed();
/*      */     }
/* 2059 */     d = this._inputBuffer[(this._inputPtr++)];
/* 2060 */     if ((d & 0xC0) != 128) {
/* 2061 */       _reportInvalidOther(d & 0xFF, this._inputPtr);
/*      */     }
/* 2063 */     c = c << 6 | d & 0x3F;
/* 2064 */     return c;
/*      */   }
/*      */   
/*      */   private final int _decodeUtf8_3fast(int c1)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 2070 */     c1 &= 0xF;
/* 2071 */     int d = this._inputBuffer[(this._inputPtr++)];
/* 2072 */     if ((d & 0xC0) != 128) {
/* 2073 */       _reportInvalidOther(d & 0xFF, this._inputPtr);
/*      */     }
/* 2075 */     int c = c1 << 6 | d & 0x3F;
/* 2076 */     d = this._inputBuffer[(this._inputPtr++)];
/* 2077 */     if ((d & 0xC0) != 128) {
/* 2078 */       _reportInvalidOther(d & 0xFF, this._inputPtr);
/*      */     }
/* 2080 */     c = c << 6 | d & 0x3F;
/* 2081 */     return c;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final int _decodeUtf8_4(int c)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 2091 */     if (this._inputPtr >= this._inputEnd) {
/* 2092 */       loadMoreGuaranteed();
/*      */     }
/* 2094 */     int d = this._inputBuffer[(this._inputPtr++)];
/* 2095 */     if ((d & 0xC0) != 128) {
/* 2096 */       _reportInvalidOther(d & 0xFF, this._inputPtr);
/*      */     }
/* 2098 */     c = (c & 0x7) << 6 | d & 0x3F;
/*      */     
/* 2100 */     if (this._inputPtr >= this._inputEnd) {
/* 2101 */       loadMoreGuaranteed();
/*      */     }
/* 2103 */     d = this._inputBuffer[(this._inputPtr++)];
/* 2104 */     if ((d & 0xC0) != 128) {
/* 2105 */       _reportInvalidOther(d & 0xFF, this._inputPtr);
/*      */     }
/* 2107 */     c = c << 6 | d & 0x3F;
/* 2108 */     if (this._inputPtr >= this._inputEnd) {
/* 2109 */       loadMoreGuaranteed();
/*      */     }
/* 2111 */     d = this._inputBuffer[(this._inputPtr++)];
/* 2112 */     if ((d & 0xC0) != 128) {
/* 2113 */       _reportInvalidOther(d & 0xFF, this._inputPtr);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 2119 */     return (c << 6 | d & 0x3F) - 65536;
/*      */   }
/*      */   
/*      */   private final void _skipUtf8_2(int c)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 2125 */     if (this._inputPtr >= this._inputEnd) {
/* 2126 */       loadMoreGuaranteed();
/*      */     }
/* 2128 */     c = this._inputBuffer[(this._inputPtr++)];
/* 2129 */     if ((c & 0xC0) != 128) {
/* 2130 */       _reportInvalidOther(c & 0xFF, this._inputPtr);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private final void _skipUtf8_3(int c)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 2140 */     if (this._inputPtr >= this._inputEnd) {
/* 2141 */       loadMoreGuaranteed();
/*      */     }
/*      */     
/* 2144 */     c = this._inputBuffer[(this._inputPtr++)];
/* 2145 */     if ((c & 0xC0) != 128) {
/* 2146 */       _reportInvalidOther(c & 0xFF, this._inputPtr);
/*      */     }
/* 2148 */     if (this._inputPtr >= this._inputEnd) {
/* 2149 */       loadMoreGuaranteed();
/*      */     }
/* 2151 */     c = this._inputBuffer[(this._inputPtr++)];
/* 2152 */     if ((c & 0xC0) != 128) {
/* 2153 */       _reportInvalidOther(c & 0xFF, this._inputPtr);
/*      */     }
/*      */   }
/*      */   
/*      */   private final void _skipUtf8_4(int c)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 2160 */     if (this._inputPtr >= this._inputEnd) {
/* 2161 */       loadMoreGuaranteed();
/*      */     }
/* 2163 */     int d = this._inputBuffer[(this._inputPtr++)];
/* 2164 */     if ((d & 0xC0) != 128) {
/* 2165 */       _reportInvalidOther(d & 0xFF, this._inputPtr);
/*      */     }
/* 2167 */     if (this._inputPtr >= this._inputEnd) {
/* 2168 */       loadMoreGuaranteed();
/*      */     }
/* 2170 */     d = this._inputBuffer[(this._inputPtr++)];
/* 2171 */     if ((d & 0xC0) != 128) {
/* 2172 */       _reportInvalidOther(d & 0xFF, this._inputPtr);
/*      */     }
/* 2174 */     if (this._inputPtr >= this._inputEnd) {
/* 2175 */       loadMoreGuaranteed();
/*      */     }
/* 2177 */     d = this._inputBuffer[(this._inputPtr++)];
/* 2178 */     if ((d & 0xC0) != 128) {
/* 2179 */       _reportInvalidOther(d & 0xFF, this._inputPtr);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final void _skipCR()
/*      */     throws IOException
/*      */   {
/* 2195 */     if (((this._inputPtr < this._inputEnd) || (loadMore())) && 
/* 2196 */       (this._inputBuffer[this._inputPtr] == 10)) {
/* 2197 */       this._inputPtr += 1;
/*      */     }
/*      */     
/* 2200 */     this._currInputRow += 1;
/* 2201 */     this._currInputRowStart = this._inputPtr;
/*      */   }
/*      */   
/*      */   protected final void _skipLF() throws IOException
/*      */   {
/* 2206 */     this._currInputRow += 1;
/* 2207 */     this._currInputRowStart = this._inputPtr;
/*      */   }
/*      */   
/*      */   private int nextByte()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 2213 */     if (this._inputPtr >= this._inputEnd) {
/* 2214 */       loadMoreGuaranteed();
/*      */     }
/* 2216 */     return this._inputBuffer[(this._inputPtr++)] & 0xFF;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void _reportInvalidChar(int c)
/*      */     throws JsonParseException
/*      */   {
/* 2229 */     if (c < 32) {
/* 2230 */       _throwInvalidSpace(c);
/*      */     }
/* 2232 */     _reportInvalidInitial(c);
/*      */   }
/*      */   
/*      */   protected void _reportInvalidInitial(int mask)
/*      */     throws JsonParseException
/*      */   {
/* 2238 */     _reportError("Invalid UTF-8 start byte 0x" + Integer.toHexString(mask));
/*      */   }
/*      */   
/*      */   protected void _reportInvalidOther(int mask)
/*      */     throws JsonParseException
/*      */   {
/* 2244 */     _reportError("Invalid UTF-8 middle byte 0x" + Integer.toHexString(mask));
/*      */   }
/*      */   
/*      */   protected void _reportInvalidOther(int mask, int ptr)
/*      */     throws JsonParseException
/*      */   {
/* 2250 */     this._inputPtr = ptr;
/* 2251 */     _reportInvalidOther(mask);
/*      */   }
/*      */   
/*      */   public static int[] growArrayBy(int[] arr, int more)
/*      */   {
/* 2256 */     if (arr == null) {
/* 2257 */       return new int[more];
/*      */     }
/* 2259 */     int[] old = arr;
/* 2260 */     int len = arr.length;
/* 2261 */     arr = new int[len + more];
/* 2262 */     System.arraycopy(old, 0, arr, 0, len);
/* 2263 */     return arr;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected byte[] _decodeBase64(Base64Variant b64variant)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 2276 */     ByteArrayBuilder builder = _getByteArrayBuilder();
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     for (;;)
/*      */     {
/* 2283 */       if (this._inputPtr >= this._inputEnd) {
/* 2284 */         loadMoreGuaranteed();
/*      */       }
/* 2286 */       int ch = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/* 2287 */       if (ch > 32) {
/* 2288 */         int bits = b64variant.decodeBase64Char(ch);
/* 2289 */         if (bits < 0) {
/* 2290 */           if (ch == 34) {
/* 2291 */             return builder.toByteArray();
/*      */           }
/* 2293 */           bits = _decodeBase64Escape(b64variant, ch, 0);
/* 2294 */           if (bits < 0) {}
/*      */         }
/*      */         else
/*      */         {
/* 2298 */           int decodedData = bits;
/*      */           
/*      */ 
/*      */ 
/* 2302 */           if (this._inputPtr >= this._inputEnd) {
/* 2303 */             loadMoreGuaranteed();
/*      */           }
/* 2305 */           ch = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/* 2306 */           bits = b64variant.decodeBase64Char(ch);
/* 2307 */           if (bits < 0) {
/* 2308 */             bits = _decodeBase64Escape(b64variant, ch, 1);
/*      */           }
/* 2310 */           decodedData = decodedData << 6 | bits;
/*      */           
/*      */ 
/* 2313 */           if (this._inputPtr >= this._inputEnd) {
/* 2314 */             loadMoreGuaranteed();
/*      */           }
/* 2316 */           ch = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/* 2317 */           bits = b64variant.decodeBase64Char(ch);
/*      */           
/*      */ 
/* 2320 */           if (bits < 0) {
/* 2321 */             if (bits != -2)
/*      */             {
/* 2323 */               if ((ch == 34) && (!b64variant.usesPadding())) {
/* 2324 */                 decodedData >>= 4;
/* 2325 */                 builder.append(decodedData);
/* 2326 */                 return builder.toByteArray();
/*      */               }
/* 2328 */               bits = _decodeBase64Escape(b64variant, ch, 2);
/*      */             }
/* 2330 */             if (bits == -2)
/*      */             {
/* 2332 */               if (this._inputPtr >= this._inputEnd) {
/* 2333 */                 loadMoreGuaranteed();
/*      */               }
/* 2335 */               ch = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/* 2336 */               if (!b64variant.usesPaddingChar(ch)) {
/* 2337 */                 throw reportInvalidChar(b64variant, ch, 3, "expected padding character '" + b64variant.getPaddingChar() + "'");
/*      */               }
/*      */               
/* 2340 */               decodedData >>= 4;
/* 2341 */               builder.append(decodedData);
/* 2342 */               continue;
/*      */             }
/*      */           }
/*      */           
/* 2346 */           decodedData = decodedData << 6 | bits;
/*      */           
/* 2348 */           if (this._inputPtr >= this._inputEnd) {
/* 2349 */             loadMoreGuaranteed();
/*      */           }
/* 2351 */           ch = this._inputBuffer[(this._inputPtr++)] & 0xFF;
/* 2352 */           bits = b64variant.decodeBase64Char(ch);
/* 2353 */           if (bits < 0) {
/* 2354 */             if (bits != -2)
/*      */             {
/* 2356 */               if ((ch == 34) && (!b64variant.usesPadding())) {
/* 2357 */                 decodedData >>= 2;
/* 2358 */                 builder.appendTwoBytes(decodedData);
/* 2359 */                 return builder.toByteArray();
/*      */               }
/* 2361 */               bits = _decodeBase64Escape(b64variant, ch, 3);
/*      */             }
/* 2363 */             if (bits == -2)
/*      */             {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2370 */               decodedData >>= 2;
/* 2371 */               builder.appendTwoBytes(decodedData);
/* 2372 */               continue;
/*      */             }
/*      */           }
/*      */           
/* 2376 */           decodedData = decodedData << 6 | bits;
/* 2377 */           builder.appendThreeBytes(decodedData);
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private final int _decodeBase64Escape(Base64Variant b64variant, int ch, int index) throws IOException, JsonParseException
/*      */   {
/* 2385 */     if (ch != 92) {
/* 2386 */       throw reportInvalidChar(b64variant, ch, index);
/*      */     }
/* 2388 */     int unescaped = _decodeEscaped();
/*      */     
/* 2390 */     if ((unescaped <= 32) && 
/* 2391 */       (index == 0)) {
/* 2392 */       return -1;
/*      */     }
/*      */     
/*      */ 
/* 2396 */     int bits = b64variant.decodeBase64Char(unescaped);
/* 2397 */     if (bits < 0) {
/* 2398 */       throw reportInvalidChar(b64variant, unescaped, index);
/*      */     }
/* 2400 */     return bits;
/*      */   }
/*      */   
/*      */   protected IllegalArgumentException reportInvalidChar(Base64Variant b64variant, int ch, int bindex)
/*      */     throws IllegalArgumentException
/*      */   {
/* 2406 */     return reportInvalidChar(b64variant, ch, bindex, null);
/*      */   }
/*      */   
/*      */ 
/*      */   protected IllegalArgumentException reportInvalidChar(Base64Variant b64variant, int ch, int bindex, String msg)
/*      */     throws IllegalArgumentException
/*      */   {
/*      */     String base;
/*      */     
/*      */     String base;
/*      */     
/* 2417 */     if (ch <= 32) {
/* 2418 */       base = "Illegal white space character (code 0x" + Integer.toHexString(ch) + ") as character #" + (bindex + 1) + " of 4-char base64 unit: can only used between units"; } else { String base;
/* 2419 */       if (b64variant.usesPaddingChar(ch)) {
/* 2420 */         base = "Unexpected padding character ('" + b64variant.getPaddingChar() + "') as character #" + (bindex + 1) + " of 4-char base64 unit: padding only legal as 3rd or 4th character"; } else { String base;
/* 2421 */         if ((!Character.isDefined(ch)) || (Character.isISOControl(ch)))
/*      */         {
/* 2423 */           base = "Illegal character (code 0x" + Integer.toHexString(ch) + ") in base64 content";
/*      */         } else
/* 2425 */           base = "Illegal character '" + (char)ch + "' (code 0x" + Integer.toHexString(ch) + ") in base64 content";
/*      */       } }
/* 2427 */     if (msg != null) {
/* 2428 */       base = base + ": " + msg;
/*      */     }
/* 2430 */     return new IllegalArgumentException(base);
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\impl\Utf8StreamParser.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */