/*     */ package org.codehaus.jackson.map.deser;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.reflect.Array;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.JsonToken;
/*     */ import org.codehaus.jackson.map.DeserializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.DeserializationContext;
/*     */ import org.codehaus.jackson.map.JsonDeserializer;
/*     */ import org.codehaus.jackson.map.TypeDeserializer;
/*     */ import org.codehaus.jackson.map.annotate.JacksonStdImpl;
/*     */ import org.codehaus.jackson.map.type.ArrayType;
/*     */ import org.codehaus.jackson.map.util.ObjectBuffer;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ @JacksonStdImpl
/*     */ public class ArrayDeserializer
/*     */   extends ContainerDeserializer<Object[]>
/*     */ {
/*     */   protected final JavaType _arrayType;
/*     */   protected final boolean _untyped;
/*     */   protected final Class<?> _elementClass;
/*     */   protected final JsonDeserializer<Object> _elementDeserializer;
/*     */   final TypeDeserializer _elementTypeDeserializer;
/*     */   
/*     */   @Deprecated
/*     */   public ArrayDeserializer(ArrayType arrayType, JsonDeserializer<Object> elemDeser)
/*     */   {
/*  55 */     this(arrayType, elemDeser, null);
/*     */   }
/*     */   
/*     */ 
/*     */   public ArrayDeserializer(ArrayType arrayType, JsonDeserializer<Object> elemDeser, TypeDeserializer elemTypeDeser)
/*     */   {
/*  61 */     super(Object[].class);
/*  62 */     this._arrayType = arrayType;
/*  63 */     this._elementClass = arrayType.getContentType().getRawClass();
/*  64 */     this._untyped = (this._elementClass == Object.class);
/*  65 */     this._elementDeserializer = elemDeser;
/*  66 */     this._elementTypeDeserializer = elemTypeDeser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JavaType getContentType()
/*     */   {
/*  77 */     return this._arrayType.getContentType();
/*     */   }
/*     */   
/*     */   public JsonDeserializer<Object> getContentDeserializer()
/*     */   {
/*  82 */     return this._elementDeserializer;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object[] deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/*  96 */     if (!jp.isExpectedStartArrayToken()) {
/*  97 */       return handleNonArray(jp, ctxt);
/*     */     }
/*     */     
/* 100 */     ObjectBuffer buffer = ctxt.leaseObjectBuffer();
/* 101 */     Object[] chunk = buffer.resetAndStart();
/* 102 */     int ix = 0;
/*     */     
/* 104 */     TypeDeserializer typeDeser = this._elementTypeDeserializer;
/*     */     JsonToken t;
/* 106 */     while ((t = jp.nextToken()) != JsonToken.END_ARRAY)
/*     */     {
/*     */       Object value;
/*     */       Object value;
/* 110 */       if (t == JsonToken.VALUE_NULL) {
/* 111 */         value = null; } else { Object value;
/* 112 */         if (typeDeser == null) {
/* 113 */           value = this._elementDeserializer.deserialize(jp, ctxt);
/*     */         } else
/* 115 */           value = this._elementDeserializer.deserializeWithType(jp, ctxt, typeDeser);
/*     */       }
/* 117 */       if (ix >= chunk.length) {
/* 118 */         chunk = buffer.appendCompletedChunk(chunk);
/* 119 */         ix = 0;
/*     */       }
/* 121 */       chunk[(ix++)] = value;
/*     */     }
/*     */     
/*     */     Object[] result;
/*     */     Object[] result;
/* 126 */     if (this._untyped) {
/* 127 */       result = buffer.completeAndClearBuffer(chunk, ix);
/*     */     } else {
/* 129 */       result = buffer.completeAndClearBuffer(chunk, ix, this._elementClass);
/*     */     }
/* 131 */     ctxt.returnObjectBuffer(buffer);
/* 132 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object[] deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 143 */     return (Object[])typeDeserializer.deserializeTypedFromArray(jp, ctxt);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Byte[] deserializeFromBase64(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 156 */     byte[] b = jp.getBinaryValue(ctxt.getBase64Variant());
/*     */     
/* 158 */     Byte[] result = new Byte[b.length];
/* 159 */     int i = 0; for (int len = b.length; i < len; i++) {
/* 160 */       result[i] = Byte.valueOf(b[i]);
/*     */     }
/* 162 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */   private final Object[] handleNonArray(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 169 */     if (!ctxt.isEnabled(DeserializationConfig.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY))
/*     */     {
/*     */ 
/*     */ 
/* 173 */       if ((jp.getCurrentToken() == JsonToken.VALUE_STRING) && (this._elementClass == Byte.class))
/*     */       {
/* 175 */         return deserializeFromBase64(jp, ctxt);
/*     */       }
/* 177 */       throw ctxt.mappingException(this._arrayType.getRawClass());
/*     */     }
/* 179 */     JsonToken t = jp.getCurrentToken();
/*     */     Object value;
/*     */     Object value;
/* 182 */     if (t == JsonToken.VALUE_NULL) {
/* 183 */       value = null; } else { Object value;
/* 184 */       if (this._elementTypeDeserializer == null) {
/* 185 */         value = this._elementDeserializer.deserialize(jp, ctxt);
/*     */       } else {
/* 187 */         value = this._elementDeserializer.deserializeWithType(jp, ctxt, this._elementTypeDeserializer);
/*     */       }
/*     */     }
/*     */     Object[] result;
/*     */     Object[] result;
/* 192 */     if (this._untyped) {
/* 193 */       result = new Object[1];
/*     */     } else {
/* 195 */       result = (Object[])Array.newInstance(this._elementClass, 1);
/*     */     }
/* 197 */     result[0] = value;
/* 198 */     return result;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\ArrayDeserializer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */