/*     */ package org.codehaus.jackson.map.deser;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.reflect.Constructor;
/*     */ import java.util.Collection;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.JsonToken;
/*     */ import org.codehaus.jackson.map.DeserializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.DeserializationContext;
/*     */ import org.codehaus.jackson.map.JsonDeserializer;
/*     */ import org.codehaus.jackson.map.TypeDeserializer;
/*     */ import org.codehaus.jackson.map.annotate.JacksonStdImpl;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ @JacksonStdImpl
/*     */ public class CollectionDeserializer
/*     */   extends ContainerDeserializer<Collection<Object>>
/*     */ {
/*     */   protected final JavaType _collectionType;
/*     */   final JsonDeserializer<Object> _valueDeserializer;
/*     */   final TypeDeserializer _valueTypeDeserializer;
/*     */   final Constructor<Collection<Object>> _defaultCtor;
/*     */   
/*     */   public CollectionDeserializer(JavaType collectionType, JsonDeserializer<Object> valueDeser, TypeDeserializer valueTypeDeser, Constructor<Collection<Object>> ctor)
/*     */   {
/*  55 */     super(collectionType.getRawClass());
/*  56 */     this._collectionType = collectionType;
/*  57 */     this._valueDeserializer = valueDeser;
/*  58 */     this._valueTypeDeserializer = valueTypeDeser;
/*  59 */     if (ctor == null) {
/*  60 */       throw new IllegalArgumentException("No default constructor found for container class " + collectionType.getRawClass().getName());
/*     */     }
/*  62 */     this._defaultCtor = ctor;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JavaType getContentType()
/*     */   {
/*  73 */     return this._collectionType.getContentType();
/*     */   }
/*     */   
/*     */   public JsonDeserializer<Object> getContentDeserializer()
/*     */   {
/*  78 */     return this._valueDeserializer;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public Collection<Object> deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/*     */     Collection<Object> result;
/*     */     
/*     */ 
/*     */ 
/*     */     try
/*     */     {
/*  93 */       result = (Collection)this._defaultCtor.newInstance(new Object[0]);
/*     */     } catch (Exception e) {
/*  95 */       throw ctxt.instantiationException(this._collectionType.getRawClass(), e);
/*     */     }
/*  97 */     return deserialize(jp, ctxt, result);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public Collection<Object> deserialize(JsonParser jp, DeserializationContext ctxt, Collection<Object> result)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 106 */     if (!jp.isExpectedStartArrayToken()) {
/* 107 */       return handleNonArray(jp, ctxt, result);
/*     */     }
/*     */     
/* 110 */     JsonDeserializer<Object> valueDes = this._valueDeserializer;
/*     */     
/* 112 */     TypeDeserializer typeDeser = this._valueTypeDeserializer;
/*     */     JsonToken t;
/* 114 */     while ((t = jp.nextToken()) != JsonToken.END_ARRAY) {
/*     */       Object value;
/*     */       Object value;
/* 117 */       if (t == JsonToken.VALUE_NULL) {
/* 118 */         value = null; } else { Object value;
/* 119 */         if (typeDeser == null) {
/* 120 */           value = valueDes.deserialize(jp, ctxt);
/*     */         } else
/* 122 */           value = valueDes.deserializeWithType(jp, ctxt, typeDeser);
/*     */       }
/* 124 */       result.add(value);
/*     */     }
/* 126 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 135 */     return typeDeserializer.deserializeTypedFromArray(jp, ctxt);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private final Collection<Object> handleNonArray(JsonParser jp, DeserializationContext ctxt, Collection<Object> result)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 148 */     if (!ctxt.isEnabled(DeserializationConfig.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)) {
/* 149 */       throw ctxt.mappingException(this._collectionType.getRawClass());
/*     */     }
/* 151 */     JsonDeserializer<Object> valueDes = this._valueDeserializer;
/* 152 */     TypeDeserializer typeDeser = this._valueTypeDeserializer;
/* 153 */     JsonToken t = jp.getCurrentToken();
/*     */     
/*     */     Object value;
/*     */     Object value;
/* 157 */     if (t == JsonToken.VALUE_NULL) {
/* 158 */       value = null; } else { Object value;
/* 159 */       if (typeDeser == null) {
/* 160 */         value = valueDes.deserialize(jp, ctxt);
/*     */       } else
/* 162 */         value = valueDes.deserializeWithType(jp, ctxt, typeDeser);
/*     */     }
/* 164 */     result.add(value);
/* 165 */     return result;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\CollectionDeserializer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */