/*     */ package org.codehaus.jackson.map.introspect;
/*     */ 
/*     */ import java.lang.reflect.AnnotatedElement;
/*     */ import java.lang.reflect.Constructor;
/*     */ import java.lang.reflect.Method;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collection;
/*     */ import java.util.Collections;
/*     */ import java.util.HashMap;
/*     */ import java.util.LinkedHashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import org.codehaus.jackson.map.AnnotationIntrospector;
/*     */ import org.codehaus.jackson.map.AnnotationIntrospector.ReferenceProperty;
/*     */ import org.codehaus.jackson.map.BeanDescription;
/*     */ import org.codehaus.jackson.map.MapperConfig;
/*     */ import org.codehaus.jackson.map.PropertyNamingStrategy;
/*     */ import org.codehaus.jackson.map.annotate.JsonSerialize.Inclusion;
/*     */ import org.codehaus.jackson.map.type.TypeBindings;
/*     */ import org.codehaus.jackson.map.util.Annotations;
/*     */ import org.codehaus.jackson.map.util.ClassUtil;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BasicBeanDescription
/*     */   extends BeanDescription
/*     */ {
/*     */   protected final MapperConfig<?> _config;
/*     */   protected final AnnotationIntrospector _annotationIntrospector;
/*     */   protected final AnnotatedClass _classInfo;
/*     */   protected TypeBindings _bindings;
/*     */   
/*     */   public BasicBeanDescription(MapperConfig<?> config, JavaType type, AnnotatedClass ac)
/*     */   {
/*  56 */     super(type);
/*  57 */     this._config = config;
/*  58 */     this._annotationIntrospector = config.getAnnotationIntrospector();
/*  59 */     this._classInfo = ac;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean hasKnownClassAnnotations()
/*     */   {
/*  74 */     return this._classInfo.hasAnnotations();
/*     */   }
/*     */   
/*     */   public Annotations getClassAnnotations()
/*     */   {
/*  79 */     return this._classInfo.getAnnotations();
/*     */   }
/*     */   
/*     */ 
/*     */   public TypeBindings bindingsForBeanType()
/*     */   {
/*  85 */     if (this._bindings == null) {
/*  86 */       this._bindings = new TypeBindings(this._config.getTypeFactory(), this._type);
/*     */     }
/*  88 */     return this._bindings;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public AnnotatedClass getClassInfo()
/*     */   {
/*  97 */     return this._classInfo;
/*     */   }
/*     */   
/*     */   public AnnotatedMethod findMethod(String name, Class<?>[] paramTypes) {
/* 101 */     return this._classInfo.findMethod(name, paramTypes);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object instantiateBean(boolean fixAccess)
/*     */   {
/* 114 */     AnnotatedConstructor ac = this._classInfo.getDefaultConstructor();
/* 115 */     if (ac == null) {
/* 116 */       return null;
/*     */     }
/* 118 */     if (fixAccess) {
/* 119 */       ac.fixAccess();
/*     */     }
/*     */     try {
/* 122 */       return ac.getAnnotated().newInstance(new Object[0]);
/*     */     } catch (Exception e) {
/* 124 */       Throwable t = e;
/* 125 */       while (t.getCause() != null) {
/* 126 */         t = t.getCause();
/*     */       }
/* 128 */       if ((t instanceof Error)) throw ((Error)t);
/* 129 */       if ((t instanceof RuntimeException)) throw ((RuntimeException)t);
/* 130 */       throw new IllegalArgumentException("Failed to instantiate bean of type " + this._classInfo.getAnnotated().getName() + ": (" + t.getClass().getName() + ") " + t.getMessage(), t);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public LinkedHashMap<String, AnnotatedMethod> findGetters(VisibilityChecker<?> visibilityChecker, Collection<String> ignoredProperties)
/*     */   {
/* 150 */     LinkedHashMap<String, AnnotatedMethod> results = new LinkedHashMap();
/* 151 */     PropertyNamingStrategy naming = this._config.getPropertyNamingStrategy();
/* 152 */     for (AnnotatedMethod am : this._classInfo.memberMethods())
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/* 157 */       if (am.getParameterCount() == 0)
/*     */       {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 164 */         String propName = this._annotationIntrospector.findGettablePropertyName(am);
/* 165 */         if (propName != null)
/*     */         {
/*     */ 
/*     */ 
/* 169 */           if (propName.length() == 0) {
/* 170 */             propName = okNameForAnyGetter(am, am.getName());
/* 171 */             if (propName == null) {
/* 172 */               propName = am.getName();
/*     */             }
/*     */             
/* 175 */             if (naming != null) {
/* 176 */               propName = naming.nameForGetterMethod(this._config, am, propName);
/*     */             }
/*     */           }
/*     */         } else {
/* 180 */           propName = am.getName();
/*     */           
/* 182 */           if (propName.startsWith("get")) {
/* 183 */             if (!visibilityChecker.isGetterVisible(am)) {
/*     */               continue;
/*     */             }
/* 186 */             propName = okNameForGetter(am, propName);
/*     */           } else {
/* 188 */             if (!visibilityChecker.isIsGetterVisible(am)) {
/*     */               continue;
/*     */             }
/* 191 */             propName = okNameForIsGetter(am, propName);
/*     */           }
/*     */           
/* 194 */           if ((propName == null) || 
/*     */           
/* 196 */             (this._annotationIntrospector.hasAnyGetterAnnotation(am))) {
/*     */             continue;
/*     */           }
/* 199 */           if (naming != null) {
/* 200 */             propName = naming.nameForGetterMethod(this._config, am, propName);
/*     */           }
/*     */         }
/*     */         
/* 204 */         if ((ignoredProperties == null) || 
/* 205 */           (!ignoredProperties.contains(propName)))
/*     */         {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 213 */           AnnotatedMethod old = (AnnotatedMethod)results.put(propName, am);
/* 214 */           if (old != null) {
/* 215 */             String oldDesc = old.getFullName();
/* 216 */             String newDesc = am.getFullName();
/* 217 */             throw new IllegalArgumentException("Conflicting getter definitions for property \"" + propName + "\": " + oldDesc + " vs " + newDesc);
/*     */           }
/*     */         } } }
/* 220 */     return results;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public AnnotatedMethod findJsonValueMethod()
/*     */   {
/* 231 */     AnnotatedMethod found = null;
/* 232 */     for (AnnotatedMethod am : this._classInfo.memberMethods())
/*     */     {
/* 234 */       if (this._annotationIntrospector.hasAsValueAnnotation(am))
/*     */       {
/*     */ 
/* 237 */         if (found != null) {
/* 238 */           throw new IllegalArgumentException("Multiple methods with active 'as-value' annotation (" + found.getName() + "(), " + am.getName() + ")");
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 245 */         if (!ClassUtil.hasGetterSignature(am.getAnnotated())) {
/* 246 */           throw new IllegalArgumentException("Method " + am.getName() + "() marked with an 'as-value' annotation, but does not have valid getter signature (non-static, takes no args, returns a value)");
/*     */         }
/* 248 */         found = am;
/*     */       } }
/* 250 */     return found;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Constructor<?> findDefaultConstructor()
/*     */   {
/* 267 */     AnnotatedConstructor ac = this._classInfo.getDefaultConstructor();
/* 268 */     if (ac == null) {
/* 269 */       return null;
/*     */     }
/* 271 */     return ac.getAnnotated();
/*     */   }
/*     */   
/*     */   public List<AnnotatedConstructor> getConstructors()
/*     */   {
/* 276 */     return this._classInfo.getConstructors();
/*     */   }
/*     */   
/*     */ 
/*     */   public List<AnnotatedMethod> getFactoryMethods()
/*     */   {
/* 282 */     List<AnnotatedMethod> candidates = this._classInfo.getStaticMethods();
/* 283 */     if (candidates.isEmpty()) {
/* 284 */       return candidates;
/*     */     }
/* 286 */     ArrayList<AnnotatedMethod> result = new ArrayList();
/* 287 */     for (AnnotatedMethod am : candidates) {
/* 288 */       if (isFactoryMethod(am)) {
/* 289 */         result.add(am);
/*     */       }
/*     */     }
/* 292 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Constructor<?> findSingleArgConstructor(Class<?>... argTypes)
/*     */   {
/* 303 */     for (AnnotatedConstructor ac : this._classInfo.getConstructors())
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/* 308 */       if (ac.getParameterCount() == 1) {
/* 309 */         Class<?> actArg = ac.getParameterClass(0);
/* 310 */         for (Class<?> expArg : argTypes) {
/* 311 */           if (expArg == actArg) {
/* 312 */             return ac.getAnnotated();
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 317 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Method findFactoryMethod(Class<?>... expArgTypes)
/*     */   {
/* 332 */     for (AnnotatedMethod am : this._classInfo.getStaticMethods()) {
/* 333 */       if (isFactoryMethod(am))
/*     */       {
/* 335 */         Class<?> actualArgType = am.getParameterClass(0);
/* 336 */         for (Class<?> expArgType : expArgTypes)
/*     */         {
/* 338 */           if (actualArgType.isAssignableFrom(expArgType)) {
/* 339 */             return am.getAnnotated();
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 344 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected boolean isFactoryMethod(AnnotatedMethod am)
/*     */   {
/* 353 */     Class<?> rt = am.getRawType();
/* 354 */     if (!getBeanClass().isAssignableFrom(rt)) {
/* 355 */       return false;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 362 */     if (this._annotationIntrospector.hasCreatorAnnotation(am)) {
/* 363 */       return true;
/*     */     }
/* 365 */     if ("valueOf".equals(am.getName())) {
/* 366 */       return true;
/*     */     }
/* 368 */     return false;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public List<String> findCreatorPropertyNames()
/*     */   {
/* 382 */     List<String> names = null;
/*     */     
/* 384 */     for (int i = 0; i < 2; i++) {
/* 385 */       List<? extends AnnotatedWithParams> l = i == 0 ? getConstructors() : getFactoryMethods();
/*     */       
/* 387 */       for (AnnotatedWithParams creator : l) {
/* 388 */         int argCount = creator.getParameterCount();
/* 389 */         if (argCount >= 1) {
/* 390 */           String name = this._annotationIntrospector.findPropertyNameForParam(creator.getParameter(0));
/* 391 */           if (name != null) {
/* 392 */             if (names == null) {
/* 393 */               names = new ArrayList();
/*     */             }
/* 395 */             names.add(name);
/* 396 */             for (int p = 1; p < argCount; p++)
/* 397 */               names.add(this._annotationIntrospector.findPropertyNameForParam(creator.getParameter(p)));
/*     */           }
/*     */         }
/*     */       } }
/* 401 */     if (names == null) {
/* 402 */       return Collections.emptyList();
/*     */     }
/* 404 */     return names;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public LinkedHashMap<String, AnnotatedField> findSerializableFields(VisibilityChecker<?> vchecker, Collection<String> ignoredProperties)
/*     */   {
/* 416 */     return _findPropertyFields(vchecker, ignoredProperties, true);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonSerialize.Inclusion findSerializationInclusion(JsonSerialize.Inclusion defValue)
/*     */   {
/* 433 */     return this._annotationIntrospector.findSerializationInclusion(this._classInfo, defValue);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public LinkedHashMap<String, AnnotatedMethod> findSetters(VisibilityChecker<?> vchecker)
/*     */   {
/* 445 */     LinkedHashMap<String, AnnotatedMethod> results = new LinkedHashMap();
/* 446 */     PropertyNamingStrategy naming = this._config.getPropertyNamingStrategy();
/* 447 */     for (AnnotatedMethod am : this._classInfo.memberMethods())
/*     */     {
/*     */ 
/*     */ 
/* 451 */       if (am.getParameterCount() == 1)
/*     */       {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 460 */         String propName = this._annotationIntrospector.findSettablePropertyName(am);
/* 461 */         if (propName != null)
/*     */         {
/*     */ 
/*     */ 
/* 465 */           if (propName.length() == 0) {
/* 466 */             propName = okNameForSetter(am);
/*     */             
/* 468 */             if (propName == null) {
/* 469 */               propName = am.getName();
/*     */             }
/*     */             
/* 472 */             if (naming != null) {
/* 473 */               propName = naming.nameForSetterMethod(this._config, am, propName);
/*     */             }
/*     */           }
/*     */         } else {
/* 477 */           if (!vchecker.isSetterVisible(am)) {
/*     */             continue;
/*     */           }
/* 480 */           propName = okNameForSetter(am);
/* 481 */           if (propName == null) {
/*     */             continue;
/*     */           }
/*     */           
/* 485 */           if (naming != null) {
/* 486 */             propName = naming.nameForSetterMethod(this._config, am, propName);
/*     */           }
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */ 
/* 493 */         AnnotatedMethod old = (AnnotatedMethod)results.put(propName, am);
/* 494 */         if (old != null)
/*     */         {
/*     */ 
/*     */ 
/*     */ 
/* 499 */           if (old.getDeclaringClass() == am.getDeclaringClass()) {
/* 500 */             String oldDesc = old.getFullName();
/* 501 */             String newDesc = am.getFullName();
/* 502 */             throw new IllegalArgumentException("Conflicting setter definitions for property \"" + propName + "\": " + oldDesc + " vs " + newDesc);
/*     */           }
/*     */           
/* 505 */           results.put(propName, old);
/*     */         }
/*     */       }
/*     */     }
/* 509 */     return results;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public AnnotatedMethod findAnySetter()
/*     */     throws IllegalArgumentException
/*     */   {
/* 525 */     AnnotatedMethod found = null;
/* 526 */     for (AnnotatedMethod am : this._classInfo.memberMethods())
/* 527 */       if (this._annotationIntrospector.hasAnySetterAnnotation(am))
/*     */       {
/*     */ 
/* 530 */         if (found != null) {
/* 531 */           throw new IllegalArgumentException("Multiple methods with 'any-setter' annotation (" + found.getName() + "(), " + am.getName() + ")");
/*     */         }
/* 533 */         int pcount = am.getParameterCount();
/* 534 */         if (pcount != 2) {
/* 535 */           throw new IllegalArgumentException("Invalid 'any-setter' annotation on method " + am.getName() + "(): takes " + pcount + " parameters, should take 2");
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 545 */         Class<?> type = am.getParameterClass(0);
/* 546 */         if ((type != String.class) && (type != Object.class)) {
/* 547 */           throw new IllegalArgumentException("Invalid 'any-setter' annotation on method " + am.getName() + "(): first argument not of type String or Object, but " + type.getName());
/*     */         }
/* 549 */         found = am;
/*     */       }
/* 551 */     return found;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public AnnotatedMethod findAnyGetter()
/*     */     throws IllegalArgumentException
/*     */   {
/* 564 */     AnnotatedMethod found = null;
/* 565 */     for (AnnotatedMethod am : this._classInfo.memberMethods())
/* 566 */       if (this._annotationIntrospector.hasAnyGetterAnnotation(am))
/*     */       {
/*     */ 
/* 569 */         if (found != null) {
/* 570 */           throw new IllegalArgumentException("Multiple methods with 'any-getter' annotation (" + found.getName() + "(), " + am.getName() + ")");
/*     */         }
/*     */         
/*     */ 
/*     */ 
/* 575 */         Class<?> type = am.getRawType();
/* 576 */         if (!Map.class.isAssignableFrom(type)) {
/* 577 */           throw new IllegalArgumentException("Invalid 'any-getter' annotation on method " + am.getName() + "(): return type is not instance of java.util.Map");
/*     */         }
/* 579 */         found = am;
/*     */       }
/* 581 */     return found;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Map<String, AnnotatedMember> findBackReferenceProperties()
/*     */   {
/* 591 */     HashMap<String, AnnotatedMember> result = null;
/*     */     
/* 593 */     for (AnnotatedMethod am : this._classInfo.memberMethods()) {
/* 594 */       if (am.getParameterCount() == 1) {
/* 595 */         AnnotationIntrospector.ReferenceProperty prop = this._annotationIntrospector.findReferenceType(am);
/* 596 */         if ((prop != null) && (prop.isBackReference())) {
/* 597 */           if (result == null) {
/* 598 */             result = new HashMap();
/*     */           }
/* 600 */           if (result.put(prop.getName(), am) != null) {
/* 601 */             throw new IllegalArgumentException("Multiple back-reference properties with name '" + prop.getName() + "'");
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */     
/* 607 */     for (AnnotatedField af : this._classInfo.fields()) {
/* 608 */       AnnotationIntrospector.ReferenceProperty prop = this._annotationIntrospector.findReferenceType(af);
/* 609 */       if ((prop != null) && (prop.isBackReference())) {
/* 610 */         if (result == null) {
/* 611 */           result = new HashMap();
/*     */         }
/* 613 */         if (result.put(prop.getName(), af) != null) {
/* 614 */           throw new IllegalArgumentException("Multiple back-reference properties with name '" + prop.getName() + "'");
/*     */         }
/*     */       }
/*     */     }
/* 618 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public LinkedHashMap<String, AnnotatedField> findDeserializableFields(VisibilityChecker<?> vchecker, Collection<String> ignoredProperties)
/*     */   {
/* 630 */     return _findPropertyFields(vchecker, ignoredProperties, false);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public String okNameForAnyGetter(AnnotatedMethod am, String name)
/*     */   {
/* 641 */     String str = okNameForIsGetter(am, name);
/* 642 */     if (str == null) {
/* 643 */       str = okNameForGetter(am, name);
/*     */     }
/* 645 */     return str;
/*     */   }
/*     */   
/*     */   public String okNameForGetter(AnnotatedMethod am, String name)
/*     */   {
/* 650 */     if (name.startsWith("get"))
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 658 */       if ("getCallbacks".equals(name)) {
/* 659 */         if (isCglibGetCallbacks(am)) {
/* 660 */           return null;
/*     */         }
/* 662 */       } else if ("getMetaClass".equals(name))
/*     */       {
/*     */ 
/*     */ 
/* 666 */         if (isGroovyMetaClassGetter(am)) {
/* 667 */           return null;
/*     */         }
/*     */       }
/* 670 */       return mangleGetterName(am, name.substring(3));
/*     */     }
/* 672 */     return null;
/*     */   }
/*     */   
/*     */   public String okNameForIsGetter(AnnotatedMethod am, String name)
/*     */   {
/* 677 */     if (name.startsWith("is"))
/*     */     {
/* 679 */       Class<?> rt = am.getRawType();
/* 680 */       if ((rt != Boolean.class) && (rt != Boolean.TYPE)) {
/* 681 */         return null;
/*     */       }
/* 683 */       return mangleGetterName(am, name.substring(2));
/*     */     }
/*     */     
/* 686 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected String mangleGetterName(Annotated a, String basename)
/*     */   {
/* 695 */     return manglePropertyName(basename);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected boolean isCglibGetCallbacks(AnnotatedMethod am)
/*     */   {
/* 711 */     Class<?> rt = am.getRawType();
/*     */     
/* 713 */     if ((rt == null) || (!rt.isArray())) {
/* 714 */       return false;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 720 */     Class<?> compType = rt.getComponentType();
/*     */     
/* 722 */     Package pkg = compType.getPackage();
/* 723 */     if (pkg != null) {
/* 724 */       String pname = pkg.getName();
/* 725 */       if ((pname.startsWith("net.sf.cglib")) || (pname.startsWith("org.hibernate.repackage.cglib")))
/*     */       {
/*     */ 
/* 728 */         return true;
/*     */       }
/*     */     }
/* 731 */     return false;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected boolean isGroovyMetaClassSetter(AnnotatedMethod am)
/*     */   {
/* 740 */     Class<?> argType = am.getParameterClass(0);
/* 741 */     Package pkg = argType.getPackage();
/* 742 */     if ((pkg != null) && (pkg.getName().startsWith("groovy.lang"))) {
/* 743 */       return true;
/*     */     }
/* 745 */     return false;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected boolean isGroovyMetaClassGetter(AnnotatedMethod am)
/*     */   {
/* 753 */     Class<?> rt = am.getRawType();
/* 754 */     if ((rt == null) || (rt.isArray())) {
/* 755 */       return false;
/*     */     }
/* 757 */     Package pkg = rt.getPackage();
/* 758 */     if ((pkg != null) && (pkg.getName().startsWith("groovy.lang"))) {
/* 759 */       return true;
/*     */     }
/* 761 */     return false;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public String okNameForSetter(AnnotatedMethod am)
/*     */   {
/* 772 */     String name = am.getName();
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 778 */     if (name.startsWith("set")) {
/* 779 */       name = mangleSetterName(am, name.substring(3));
/* 780 */       if (name == null) {
/* 781 */         return null;
/*     */       }
/* 783 */       if ("metaClass".equals(name))
/*     */       {
/* 785 */         if (isGroovyMetaClassSetter(am)) {
/* 786 */           return null;
/*     */         }
/*     */       }
/* 789 */       return name;
/*     */     }
/* 791 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected String mangleSetterName(Annotated a, String basename)
/*     */   {
/* 800 */     return manglePropertyName(basename);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public LinkedHashMap<String, AnnotatedField> _findPropertyFields(VisibilityChecker<?> vchecker, Collection<String> ignoredProperties, boolean forSerialization)
/*     */   {
/* 825 */     LinkedHashMap<String, AnnotatedField> results = new LinkedHashMap();
/* 826 */     PropertyNamingStrategy naming = this._config.getPropertyNamingStrategy();
/* 827 */     for (AnnotatedField af : this._classInfo.fields())
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 838 */       String propName = forSerialization ? this._annotationIntrospector.findSerializablePropertyName(af) : this._annotationIntrospector.findDeserializablePropertyName(af);
/*     */       
/*     */ 
/*     */ 
/* 842 */       if (propName != null) {
/* 843 */         if (propName.length() == 0) {
/* 844 */           propName = af.getName();
/*     */           
/* 846 */           if (naming != null) {
/* 847 */             propName = naming.nameForField(this._config, af, propName);
/*     */           }
/*     */         }
/*     */       } else {
/* 851 */         if (!vchecker.isFieldVisible(af)) {
/*     */           continue;
/*     */         }
/* 854 */         propName = af.getName();
/*     */         
/* 856 */         if (naming != null) {
/* 857 */           propName = naming.nameForField(this._config, af, propName);
/*     */         }
/*     */       }
/*     */       
/* 861 */       if ((ignoredProperties == null) || 
/* 862 */         (!ignoredProperties.contains(propName)))
/*     */       {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 871 */         AnnotatedField old = (AnnotatedField)results.put(propName, af);
/* 872 */         if (old != null)
/*     */         {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 882 */           if (old.getDeclaringClass() == af.getDeclaringClass()) {
/* 883 */             String oldDesc = old.getFullName();
/* 884 */             String newDesc = af.getFullName();
/* 885 */             throw new IllegalArgumentException("Multiple fields representing property \"" + propName + "\": " + oldDesc + " vs " + newDesc);
/*     */           } }
/*     */       }
/*     */     }
/* 889 */     return results;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String manglePropertyName(String basename)
/*     */   {
/* 907 */     int len = basename.length();
/*     */     
/*     */ 
/* 910 */     if (len == 0) {
/* 911 */       return null;
/*     */     }
/*     */     
/* 914 */     StringBuilder sb = null;
/* 915 */     for (int i = 0; i < len; i++) {
/* 916 */       char upper = basename.charAt(i);
/* 917 */       char lower = Character.toLowerCase(upper);
/* 918 */       if (upper == lower) {
/*     */         break;
/*     */       }
/* 921 */       if (sb == null) {
/* 922 */         sb = new StringBuilder(basename);
/*     */       }
/* 924 */       sb.setCharAt(i, lower);
/*     */     }
/* 926 */     return sb == null ? basename : sb.toString();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String descFor(AnnotatedElement elem)
/*     */   {
/* 935 */     if ((elem instanceof Class)) {
/* 936 */       return "class " + ((Class)elem).getName();
/*     */     }
/* 938 */     if ((elem instanceof Method)) {
/* 939 */       Method m = (Method)elem;
/* 940 */       return "method " + m.getName() + " (from class " + m.getDeclaringClass().getName() + ")";
/*     */     }
/* 942 */     if ((elem instanceof Constructor)) {
/* 943 */       Constructor<?> ctor = (Constructor)elem;
/*     */       
/* 945 */       return "constructor() (from class " + ctor.getDeclaringClass().getName() + ")";
/*     */     }
/*     */     
/* 948 */     return "unknown type [" + elem.getClass() + "]";
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\introspect\BasicBeanDescription.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */