/*     */ package org.codehaus.commons.compiler;
/*     */ 
/*     */ import java.io.File;
/*     */ import java.io.PrintStream;
/*     */ import java.lang.reflect.Method;
/*     */ import java.security.ProtectionDomain;
/*     */ import java.util.ArrayList;
/*     */ import java.util.List;
/*     */ import java.util.StringTokenizer;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class AbstractJavaSourceClassLoader
/*     */   extends ClassLoader
/*     */ {
/*  43 */   protected ProtectionDomainFactory optionalProtectionDomainFactory = null;
/*     */   
/*     */   public AbstractJavaSourceClassLoader() {}
/*     */   
/*     */   public AbstractJavaSourceClassLoader(ClassLoader parentClassLoader)
/*     */   {
/*  49 */     super(parentClassLoader);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract void setSourcePath(File[] paramArrayOfFile);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract void setSourceFileCharacterEncoding(String paramString);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract void setDebuggingInfo(boolean paramBoolean1, boolean paramBoolean2, boolean paramBoolean3);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void setProtectionDomainFactory(ProtectionDomainFactory optionalProtectionDomainFactory)
/*     */   {
/*  73 */     this.optionalProtectionDomainFactory = optionalProtectionDomainFactory;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static void main(String[] args)
/*     */     throws Exception
/*     */   {
/* 101 */     File[] optionalSourcePath = null;
/* 102 */     String optionalCharacterEncoding = null;
/*     */     
/* 104 */     boolean debuggingInfoLines = false;
/* 105 */     boolean debuggingInfoVars = false;
/* 106 */     boolean debuggingInfoSource = false;
/* 107 */     boolean haveDebuggingInfo = false;
/*     */     
/*     */ 
/*     */ 
/* 111 */     for (int i = 0; i < args.length; i++) {
/* 112 */       String arg = args[i];
/* 113 */       if (!arg.startsWith("-"))
/*     */         break;
/* 115 */       if ("-sourcepath".equals(arg)) {
/* 116 */         optionalSourcePath = splitPath(args[(++i)]);
/*     */       }
/* 118 */       else if ("-encoding".equals(arg)) {
/* 119 */         optionalCharacterEncoding = args[(++i)];
/*     */       }
/* 121 */       else if (arg.equals("-g")) {
/* 122 */         debuggingInfoLines = true;
/* 123 */         debuggingInfoVars = true;
/* 124 */         debuggingInfoSource = true;
/* 125 */         haveDebuggingInfo = true;
/*     */       }
/* 127 */       else if (arg.equals("-g:none")) {
/* 128 */         debuggingInfoLines = false;
/* 129 */         debuggingInfoVars = false;
/* 130 */         debuggingInfoSource = false;
/* 131 */         haveDebuggingInfo = true;
/*     */       }
/* 133 */       else if (arg.startsWith("-g:")) {
/* 134 */         debuggingInfoLines = arg.contains("lines");
/* 135 */         debuggingInfoVars = arg.contains("vars");
/* 136 */         debuggingInfoSource = arg.contains("source");
/* 137 */         haveDebuggingInfo = true;
/*     */       }
/* 139 */       else if ("-help".equals(arg)) {
/* 140 */         System.out.println("Usage:");
/* 141 */         System.out.println("  java " + AbstractJavaSourceClassLoader.class.getName() + " { <option> } <class-name> { <argument> }");
/*     */         
/*     */ 
/*     */ 
/*     */ 
/* 146 */         System.out.println("Loads the named class by name and invoke its \"main(String[])\" method, passing");
/* 147 */         System.out.println("the given <argument>s.");
/* 148 */         System.out.println("  <option>:");
/* 149 */         System.out.println("    -sourcepath <" + File.pathSeparator + "-separated-list-of-source-directories>");
/* 150 */         System.out.println("    -encoding <character-encoding>");
/* 151 */         System.out.println("    -g                     Generate all debugging info");
/* 152 */         System.out.println("    -g:none                Generate no debugging info");
/* 153 */         System.out.println("    -g:{source,lines,vars} Generate only some debugging info");
/* 154 */         System.exit(0);
/*     */       }
/*     */       else {
/* 157 */         System.err.println("Invalid command line option \"" + arg + "\"; try \"-help\"");
/* 158 */         System.exit(1);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/* 163 */     if (i == args.length) {
/* 164 */       System.err.println("No class name given, try \"-help\"");
/* 165 */       System.exit(1);
/*     */     }
/* 167 */     String className = args[(i++)];
/*     */     
/*     */ 
/* 170 */     String[] mainArgs = new String[args.length - i];
/* 171 */     System.arraycopy(args, i, mainArgs, 0, args.length - i);
/*     */     
/*     */ 
/* 174 */     AbstractJavaSourceClassLoader ajscl = CompilerFactoryFactory.getDefaultCompilerFactory().newJavaSourceClassLoader();
/*     */     
/*     */ 
/* 177 */     if (haveDebuggingInfo) ajscl.setDebuggingInfo(debuggingInfoLines, debuggingInfoVars, debuggingInfoSource);
/* 178 */     if (optionalCharacterEncoding != null) ajscl.setSourceFileCharacterEncoding(optionalCharacterEncoding);
/* 179 */     if (optionalSourcePath != null) { ajscl.setSourcePath(optionalSourcePath);
/*     */     }
/*     */     
/* 182 */     Class clazz = ajscl.loadClass(className);
/*     */     
/*     */     Method mainMethod;
/*     */     try
/*     */     {
/* 187 */       mainMethod = clazz.getMethod("main", new Class[] { new String[0].getClass() });
/*     */     } catch (NoSuchMethodException ex) {
/* 189 */       System.err.println("Class \"" + className + "\" has not public method \"main(String[])\".");
/* 190 */       System.exit(1);
/* 191 */       return;
/*     */     }
/*     */     
/*     */ 
/* 195 */     mainMethod.invoke(null, new Object[] { mainArgs });
/*     */   }
/*     */   
/*     */   private static File[] splitPath(String string) {
/* 199 */     List l = new ArrayList();
/* 200 */     for (StringTokenizer st = new StringTokenizer(string, File.pathSeparator); st.hasMoreTokens();) {
/* 201 */       l.add(new File(st.nextToken()));
/*     */     }
/* 203 */     return (File[])l.toArray(new File[l.size()]);
/*     */   }
/*     */   
/*     */   public static abstract interface ProtectionDomainFactory
/*     */   {
/*     */     public abstract ProtectionDomain getProtectionDomain(String paramString);
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\commons\compiler\AbstractJavaSourceClassLoader.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */