/*     */ package org.codehaus.jackson.map.deser;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.reflect.Constructor;
/*     */ import java.lang.reflect.Method;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.io.NumberInput;
/*     */ import org.codehaus.jackson.map.DeserializationContext;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.KeyDeserializer;
/*     */ 
/*     */ public abstract class StdKeyDeserializer
/*     */   extends KeyDeserializer
/*     */ {
/*     */   protected final Class<?> _keyClass;
/*     */   
/*     */   protected StdKeyDeserializer(Class<?> cls)
/*     */   {
/*  19 */     this._keyClass = cls;
/*     */   }
/*     */   
/*     */   public final Object deserializeKey(String key, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/*  25 */     if (key == null) {
/*  26 */       return null;
/*     */     }
/*     */     try {
/*  29 */       Object result = _parse(key, ctxt);
/*  30 */       if (result != null) {
/*  31 */         return result;
/*     */       }
/*     */     } catch (Exception re) {
/*  34 */       throw ctxt.weirdKeyException(this._keyClass, key, "not a valid representation: " + re.getMessage());
/*     */     }
/*  36 */     throw ctxt.weirdKeyException(this._keyClass, key, "not a valid representation");
/*     */   }
/*     */   
/*  39 */   public Class<?> getKeyClass() { return this._keyClass; }
/*     */   
/*     */ 
/*     */ 
/*     */   protected abstract Object _parse(String paramString, DeserializationContext paramDeserializationContext)
/*     */     throws Exception;
/*     */   
/*     */ 
/*     */ 
/*     */   protected int _parseInt(String key)
/*     */     throws IllegalArgumentException
/*     */   {
/*  51 */     return Integer.parseInt(key);
/*     */   }
/*     */   
/*     */   protected long _parseLong(String key) throws IllegalArgumentException
/*     */   {
/*  56 */     return Long.parseLong(key);
/*     */   }
/*     */   
/*     */   protected double _parseDouble(String key) throws IllegalArgumentException
/*     */   {
/*  61 */     return NumberInput.parseDouble(key);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   static final class BoolKD
/*     */     extends StdKeyDeserializer
/*     */   {
/*     */     BoolKD()
/*     */     {
/*  72 */       super();
/*     */     }
/*     */     
/*     */     public Boolean _parse(String key, DeserializationContext ctxt) throws JsonMappingException
/*     */     {
/*  77 */       if ("true".equals(key)) {
/*  78 */         return Boolean.TRUE;
/*     */       }
/*  80 */       if ("false".equals(key)) {
/*  81 */         return Boolean.FALSE;
/*     */       }
/*  83 */       throw ctxt.weirdKeyException(this._keyClass, key, "value not 'true' or 'false'");
/*     */     }
/*     */   }
/*     */   
/*     */   static final class ByteKD extends StdKeyDeserializer {
/*     */     ByteKD() {
/*  89 */       super();
/*     */     }
/*     */     
/*     */     public Byte _parse(String key, DeserializationContext ctxt) throws JsonMappingException
/*     */     {
/*  94 */       int value = _parseInt(key);
/*  95 */       if ((value < -128) || (value > 127)) {
/*  96 */         throw ctxt.weirdKeyException(this._keyClass, key, "overflow, value can not be represented as 8-bit value");
/*     */       }
/*  98 */       return Byte.valueOf((byte)value);
/*     */     }
/*     */   }
/*     */   
/*     */   static final class ShortKD extends StdKeyDeserializer {
/*     */     ShortKD() {
/* 104 */       super();
/*     */     }
/*     */     
/*     */     public Short _parse(String key, DeserializationContext ctxt) throws JsonMappingException
/*     */     {
/* 109 */       int value = _parseInt(key);
/* 110 */       if ((value < 32768) || (value > 32767)) {
/* 111 */         throw ctxt.weirdKeyException(this._keyClass, key, "overflow, value can not be represented as 16-bit value");
/*     */       }
/* 113 */       return Short.valueOf((short)value);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   static final class CharKD
/*     */     extends StdKeyDeserializer
/*     */   {
/*     */     CharKD()
/*     */     {
/* 123 */       super();
/*     */     }
/*     */     
/*     */     public Character _parse(String key, DeserializationContext ctxt) throws JsonMappingException
/*     */     {
/* 128 */       if (key.length() == 1) {
/* 129 */         return Character.valueOf(key.charAt(0));
/*     */       }
/* 131 */       throw ctxt.weirdKeyException(this._keyClass, key, "can only convert 1-character Strings");
/*     */     }
/*     */   }
/*     */   
/*     */   static final class IntKD extends StdKeyDeserializer {
/*     */     IntKD() {
/* 137 */       super();
/*     */     }
/*     */     
/*     */     public Integer _parse(String key, DeserializationContext ctxt) throws JsonMappingException
/*     */     {
/* 142 */       return Integer.valueOf(_parseInt(key));
/*     */     }
/*     */   }
/*     */   
/*     */   static final class LongKD extends StdKeyDeserializer {
/*     */     LongKD() {
/* 148 */       super();
/*     */     }
/*     */     
/*     */     public Long _parse(String key, DeserializationContext ctxt) throws JsonMappingException
/*     */     {
/* 153 */       return Long.valueOf(_parseLong(key));
/*     */     }
/*     */   }
/*     */   
/*     */   static final class DoubleKD extends StdKeyDeserializer {
/*     */     DoubleKD() {
/* 159 */       super();
/*     */     }
/*     */     
/*     */     public Double _parse(String key, DeserializationContext ctxt) throws JsonMappingException
/*     */     {
/* 164 */       return Double.valueOf(_parseDouble(key));
/*     */     }
/*     */   }
/*     */   
/*     */   static final class FloatKD extends StdKeyDeserializer {
/*     */     FloatKD() {
/* 170 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     public Float _parse(String key, DeserializationContext ctxt)
/*     */       throws JsonMappingException
/*     */     {
/* 178 */       return Float.valueOf((float)_parseDouble(key));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   static final class EnumKD
/*     */     extends StdKeyDeserializer
/*     */   {
/*     */     final EnumResolver<?> _resolver;
/*     */     
/*     */ 
/*     */ 
/*     */     EnumKD(EnumResolver<?> er)
/*     */     {
/* 194 */       super();
/* 195 */       this._resolver = er;
/*     */     }
/*     */     
/*     */     public Enum<?> _parse(String key, DeserializationContext ctxt)
/*     */       throws JsonMappingException
/*     */     {
/* 201 */       Enum<?> e = this._resolver.findEnum(key);
/* 202 */       if (e == null) {
/* 203 */         throw ctxt.weirdKeyException(this._keyClass, key, "not one of values for Enum class");
/*     */       }
/* 205 */       return e;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   static final class StringCtorKeyDeserializer
/*     */     extends StdKeyDeserializer
/*     */   {
/*     */     final Constructor<?> _ctor;
/*     */     
/*     */ 
/*     */     public StringCtorKeyDeserializer(Constructor<?> ctor)
/*     */     {
/* 218 */       super();
/* 219 */       this._ctor = ctor;
/*     */     }
/*     */     
/*     */     public Object _parse(String key, DeserializationContext ctxt)
/*     */       throws Exception
/*     */     {
/* 225 */       return this._ctor.newInstance(new Object[] { key });
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   static final class StringFactoryKeyDeserializer
/*     */     extends StdKeyDeserializer
/*     */   {
/*     */     final Method _factoryMethod;
/*     */     
/*     */ 
/*     */     public StringFactoryKeyDeserializer(Method fm)
/*     */     {
/* 238 */       super();
/* 239 */       this._factoryMethod = fm;
/*     */     }
/*     */     
/*     */     public Object _parse(String key, DeserializationContext ctxt)
/*     */       throws Exception
/*     */     {
/* 245 */       return this._factoryMethod.invoke(null, new Object[] { key });
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\StdKeyDeserializer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */