/*      */ package org.codehaus.janino;
/*      */ 
/*      */ import java.io.DataOutputStream;
/*      */ import java.io.File;
/*      */ import java.io.IOException;
/*      */ import java.util.AbstractMap;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Arrays;
/*      */ import java.util.Collection;
/*      */ import java.util.Collections;
/*      */ import java.util.HashMap;
/*      */ import java.util.HashSet;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.Map.Entry;
/*      */ import java.util.Set;
/*      */ import java.util.SortedMap;
/*      */ import java.util.StringTokenizer;
/*      */ import java.util.TreeMap;
/*      */ import org.codehaus.commons.compiler.CompileException;
/*      */ import org.codehaus.commons.compiler.Location;
/*      */ import org.codehaus.janino.util.ClassFile;
/*      */ import org.codehaus.janino.util.ClassFile.AttributeInfo;
/*      */ import org.codehaus.janino.util.ClassFile.DeprecatedAttribute;
/*      */ import org.codehaus.janino.util.ClassFile.ExceptionsAttribute;
/*      */ import org.codehaus.janino.util.ClassFile.FieldInfo;
/*      */ import org.codehaus.janino.util.ClassFile.InnerClassesAttribute.Entry;
/*      */ import org.codehaus.janino.util.ClassFile.MethodInfo;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class UnitCompiler
/*      */ {
/*      */   private static final boolean DEBUG = false;
/*      */   private static final int STRING_CONCAT_LIMIT = 3;
/*      */   
/*      */   public UnitCompiler(Java.CompilationUnit compilationUnit, IClassLoader iClassLoader)
/*      */     throws CompileException
/*      */   {
/*  130 */     this.compilationUnit = compilationUnit;
/*  131 */     this.iClassLoader = iClassLoader;
/*      */     try
/*      */     {
/*  134 */       boolean allowStringBuilder = true;
/*  135 */       String targetVersion = System.getProperty("Janino.TargetVersion");
/*  136 */       if ((targetVersion != null) && (Double.parseDouble(targetVersion) <= 1.4D)) {
/*  137 */         allowStringBuilder = false;
/*      */       }
/*      */       
/*  140 */       if ((iClassLoader.loadIClass("Ljava/lang/StringBuilder;") != null) && (allowStringBuilder)) {
/*  141 */         this.isStringBuilderAvailable = true;
/*      */       }
/*  143 */       else if (iClassLoader.loadIClass("Ljava/lang/StringBuffer;") != null) {
/*  144 */         this.isStringBuilderAvailable = false;
/*      */       }
/*      */       else {
/*  147 */         throw new JaninoRuntimeException("SNO: Could neither load \"StringBuffer\" nor \"StringBuilder\"");
/*      */       }
/*      */     } catch (ClassNotFoundException ex) {
/*  150 */       throw new JaninoRuntimeException("SNO: Error loading \"StringBuffer\" or \"StringBuilder\": " + ex.getMessage());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  158 */     this.typeImportsOnDemand = new ArrayList();
/*  159 */     this.typeImportsOnDemand.add(new String[] { "java", "lang" });
/*  160 */     for (Iterator it = this.compilationUnit.importDeclarations.iterator(); it.hasNext();) {
/*  161 */       Java.CompilationUnit.ImportDeclaration id = (Java.CompilationUnit.ImportDeclaration)it.next();
/*      */       try
/*      */       {
/*  164 */         id.accept(new Visitor.ImportVisitor() {
/*      */           public void visitSingleTypeImportDeclaration(Java.CompilationUnit.SingleTypeImportDeclaration stid) {
/*  166 */             try { UnitCompiler.this.import2(stid); } catch (CompileException e) { throw new UnitCompiler.1UCE(UnitCompiler.this, e); } }
/*  167 */           public void visitTypeImportOnDemandDeclaration(Java.CompilationUnit.TypeImportOnDemandDeclaration tiodd) { UnitCompiler.this.import2(tiodd); }
/*      */           
/*      */           public void visitSingleStaticImportDeclaration(Java.CompilationUnit.SingleStaticImportDeclaration ssid) {}
/*      */           
/*      */           public void visitStaticImportOnDemandDeclaration(Java.CompilationUnit.StaticImportOnDemandDeclaration siodd) {}
/*      */         }); } catch (1UCE uce) { throw uce.ce;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*  177 */   private void import2(Java.CompilationUnit.SingleTypeImportDeclaration stid) throws CompileException { String[] ids = stid.identifiers;
/*  178 */     String name = last(ids);
/*      */     
/*  180 */     String[] prev = (String[])this.singleTypeImports.put(name, ids);
/*      */     
/*      */ 
/*  183 */     if ((prev != null) && (!Arrays.equals(prev, ids))) {
/*  184 */       compileError("Class \"" + name + "\" was previously imported as " + "\"" + Java.join(prev, ".") + "\", now as \"" + Java.join(ids, ".") + "\"", stid.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  190 */     IClass iClass = loadFullyQualifiedClass(ids);
/*  191 */     if (iClass == null) {
/*  192 */       compileError("Imported class \"" + Java.join(ids, ".") + "\" could not be loaded", stid.getLocation());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*  198 */   private void import2(Java.CompilationUnit.TypeImportOnDemandDeclaration tiodd) { this.typeImportsOnDemand.add(tiodd.identifiers); }
/*      */   
/*      */   private void import2(Java.CompilationUnit.SingleStaticImportDeclaration ssid) throws CompileException {
/*  201 */     String name = last(ssid.identifiers);
/*      */     
/*  203 */     List importedObjects = (List)this.singleStaticImports.get(name);
/*  204 */     if (importedObjects == null) {
/*  205 */       importedObjects = new ArrayList();
/*  206 */       this.singleStaticImports.put(name, importedObjects);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  211 */     IClass iClass = loadFullyQualifiedClass(ssid.identifiers);
/*  212 */     if (iClass != null) {
/*  213 */       importedObjects.add(iClass);
/*  214 */       return;
/*      */     }
/*      */     
/*      */ 
/*  218 */     String[] typeName = allButLast(ssid.identifiers);
/*  219 */     IClass iClass = loadFullyQualifiedClass(typeName);
/*  220 */     if (iClass == null) {
/*  221 */       compileError("Could not load \"" + Java.join(typeName, ".") + "\"", ssid.getLocation());
/*  222 */       return;
/*      */     }
/*      */     
/*      */ 
/*  226 */     IClass.IField iField = iClass.getDeclaredIField(name);
/*  227 */     if (iField != null) {
/*  228 */       if (!iField.isStatic()) {
/*  229 */         compileError("Field \"" + name + "\" of \"" + Java.join(typeName, ".") + "\" must be static", ssid.getLocation());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*  234 */       importedObjects.add(iField);
/*  235 */       return;
/*      */     }
/*      */     
/*      */ 
/*  239 */     IClass.IMethod[] ms = iClass.getDeclaredIMethods(name);
/*  240 */     if (ms.length > 0) {
/*  241 */       importedObjects.addAll(Arrays.asList(ms));
/*  242 */       return;
/*      */     }
/*      */     
/*      */ 
/*  246 */     compileError("\"" + Java.join(typeName, ".") + "\" has no static member \"" + name + "\"", ssid.getLocation());
/*      */   }
/*      */   
/*      */   private void import2(Java.CompilationUnit.StaticImportOnDemandDeclaration siodd)
/*      */     throws CompileException
/*      */   {
/*  252 */     IClass iClass = loadFullyQualifiedClass(siodd.identifiers);
/*  253 */     if (iClass == null) {
/*  254 */       compileError("Could not load \"" + Java.join(siodd.identifiers, ".") + "\"", siodd.getLocation());
/*  255 */       return;
/*      */     }
/*  257 */     this.staticImportsOnDemand.add(iClass);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public ClassFile[] compileUnit(boolean debugSource, boolean debugLines, boolean debugVars)
/*      */     throws CompileException
/*      */   {
/*  265 */     this.debugSource = debugSource;
/*  266 */     this.debugLines = debugLines;
/*  267 */     this.debugVars = debugVars;
/*      */     
/*      */ 
/*  270 */     for (Iterator it = this.compilationUnit.importDeclarations.iterator(); it.hasNext();) {
/*  271 */       Java.CompilationUnit.ImportDeclaration id = (Java.CompilationUnit.ImportDeclaration)it.next();
/*      */       try
/*      */       {
/*  274 */         id.accept(new Visitor.ImportVisitor() { public void visitSingleTypeImportDeclaration(Java.CompilationUnit.SingleTypeImportDeclaration stid) {}
/*      */           
/*      */           public void visitTypeImportOnDemandDeclaration(Java.CompilationUnit.TypeImportOnDemandDeclaration tiodd) {}
/*      */           
/*  278 */           public void visitSingleStaticImportDeclaration(Java.CompilationUnit.SingleStaticImportDeclaration ssid) { try { UnitCompiler.this.import2(ssid); } catch (CompileException e) { throw new UnitCompiler.2UCE(UnitCompiler.this, e); } }
/*  279 */           public void visitStaticImportOnDemandDeclaration(Java.CompilationUnit.StaticImportOnDemandDeclaration siodd) { try { UnitCompiler.this.import2(siodd); } catch (CompileException e) { throw new UnitCompiler.2UCE(UnitCompiler.this, e);
/*      */             }
/*      */           }
/*      */         }); } catch (2UCE uce) { throw uce.ce;
/*      */       }
/*      */     }
/*  285 */     this.generatedClassFiles = new ArrayList();
/*      */     
/*  287 */     for (Iterator it = this.compilationUnit.packageMemberTypeDeclarations.iterator(); it.hasNext();) {
/*  288 */       compile((Java.PackageMemberTypeDeclaration)it.next());
/*      */     }
/*      */     
/*  291 */     if (this.compileErrorCount > 0) {
/*  292 */       throw new CompileException(this.compileErrorCount + " error(s) while compiling unit \"" + this.compilationUnit.optionalFileName + "\"", null);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  300 */     List l = this.generatedClassFiles;
/*  301 */     return (ClassFile[])l.toArray(new ClassFile[l.size()]);
/*      */   }
/*      */   
/*      */ 
/*      */   private void compile(Java.TypeDeclaration td)
/*      */     throws CompileException
/*      */   {
/*  308 */     Visitor.TypeDeclarationVisitor tdv = new Visitor.TypeDeclarationVisitor() {
/*      */       public void visitAnonymousClassDeclaration(Java.AnonymousClassDeclaration acd) {
/*  310 */         try { UnitCompiler.this.compile2(acd); } catch (CompileException e) { throw new UnitCompiler.3UCE(UnitCompiler.this, e); } }
/*  311 */       public void visitLocalClassDeclaration(Java.LocalClassDeclaration lcd) { try { UnitCompiler.this.compile2(lcd); } catch (CompileException e) { throw new UnitCompiler.3UCE(UnitCompiler.this, e); } }
/*  312 */       public void visitPackageMemberClassDeclaration(Java.PackageMemberClassDeclaration pmcd) { try { UnitCompiler.this.compile2(pmcd); } catch (CompileException e) { throw new UnitCompiler.3UCE(UnitCompiler.this, e); } }
/*  313 */       public void visitMemberInterfaceDeclaration(Java.MemberInterfaceDeclaration mid) { try { UnitCompiler.this.compile2(mid); } catch (CompileException e) { throw new UnitCompiler.3UCE(UnitCompiler.this, e); } }
/*  314 */       public void visitPackageMemberInterfaceDeclaration(Java.PackageMemberInterfaceDeclaration pmid) { try { UnitCompiler.this.compile2(pmid); } catch (CompileException e) { throw new UnitCompiler.3UCE(UnitCompiler.this, e); } }
/*  315 */       public void visitMemberClassDeclaration(Java.MemberClassDeclaration mcd) { try { UnitCompiler.this.compile2(mcd); } catch (CompileException e) { throw new UnitCompiler.3UCE(UnitCompiler.this, e);
/*      */         }
/*      */       }
/*      */     };
/*  319 */     try { td.accept(tdv);
/*      */     } catch (3UCE uce) {
/*  321 */       throw uce.ce;
/*      */     }
/*      */   }
/*      */   
/*  325 */   public void compile2(Java.PackageMemberTypeDeclaration pmtd) throws CompileException { Java.CompilationUnit declaringCompilationUnit = pmtd.getDeclaringCompilationUnit();
/*      */     
/*      */ 
/*      */ 
/*  329 */     String[] ss = getSingleTypeImport(pmtd.getName());
/*  330 */     if (ss != null) {
/*  331 */       compileError("Package member type declaration \"" + pmtd.getName() + "\" conflicts with single-type-import \"" + Java.join(ss, ".") + "\"", pmtd.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  343 */     Java.PackageMemberTypeDeclaration otherPMTD = declaringCompilationUnit.getPackageMemberTypeDeclaration(pmtd.getName());
/*      */     
/*      */ 
/*  346 */     if (otherPMTD != pmtd) {
/*  347 */       compileError("Redeclaration of type \"" + pmtd.getName() + "\", previously declared in " + otherPMTD.getLocation(), pmtd.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  356 */     if ((pmtd instanceof Java.NamedClassDeclaration)) {
/*  357 */       compile2((Java.NamedClassDeclaration)pmtd);
/*      */     }
/*  359 */     else if ((pmtd instanceof Java.InterfaceDeclaration)) {
/*  360 */       compile2((Java.InterfaceDeclaration)pmtd);
/*      */     }
/*      */     else {
/*  363 */       throw new JaninoRuntimeException("PMTD of unexpected type " + pmtd.getClass().getName());
/*      */     }
/*      */   }
/*      */   
/*      */   public void compile2(Java.ClassDeclaration cd) throws CompileException {
/*  368 */     IClass iClass = resolve(cd);
/*      */     
/*      */ 
/*  371 */     if ((cd.getModifiers() & 0x400) == 0) {
/*  372 */       IClass.IMethod[] ms = iClass.getIMethods();
/*  373 */       for (int i = 0; i < ms.length; i++) {
/*  374 */         IClass.IMethod base = ms[i];
/*  375 */         if (base.isAbstract()) {
/*  376 */           IClass.IMethod override = iClass.findIMethod(base.getName(), base.getParameterTypes());
/*  377 */           if ((override == null) || (override.isAbstract()) || (!base.getReturnType().isAssignableFrom(override.getReturnType())))
/*      */           {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  383 */             compileError("Non-abstract class \"" + iClass + "\" must implement method \"" + base + "\"", cd.getLocation());
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  393 */     ClassFile cf = new ClassFile((short)(cd.getModifiers() | 0x20), iClass.getDescriptor(), iClass.getSuperclass().getDescriptor(), IClass.getDescriptors(iClass.getInterfaces()));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  401 */     if (!(cd.getEnclosingScope() instanceof Java.CompilationUnit))
/*      */     {
/*      */ 
/*  404 */       if ((cd.getEnclosingScope() instanceof Java.Block)) {
/*  405 */         short innerClassInfoIndex = cf.addConstantClassInfo(iClass.getDescriptor());
/*  406 */         short innerNameIndex = (this instanceof Java.NamedTypeDeclaration) ? cf.addConstantUtf8Info(((Java.NamedTypeDeclaration)this).getName()) : 0;
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*  411 */         cf.addInnerClassesAttributeEntry(new ClassFile.InnerClassesAttribute.Entry(innerClassInfoIndex, (short)0, innerNameIndex, cd.getModifiers()));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       }
/*  418 */       else if ((cd.getEnclosingScope() instanceof Java.TypeDeclaration)) {
/*  419 */         short innerClassInfoIndex = cf.addConstantClassInfo(iClass.getDescriptor());
/*  420 */         short outerClassInfoIndex = cf.addConstantClassInfo(resolve((Java.TypeDeclaration)cd.getEnclosingScope()).getDescriptor());
/*      */         
/*      */ 
/*  423 */         short innerNameIndex = cf.addConstantUtf8Info(((Java.MemberTypeDeclaration)cd).getName());
/*  424 */         cf.addInnerClassesAttributeEntry(new ClassFile.InnerClassesAttribute.Entry(innerClassInfoIndex, outerClassInfoIndex, innerNameIndex, cd.getModifiers()));
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  433 */     if (this.debugSource)
/*      */     {
/*      */ 
/*  436 */       String s = cd.getLocation().getFileName();
/*  437 */       String sourceFileName; String sourceFileName; if (s != null) {
/*  438 */         sourceFileName = new File(s).getName(); } else { String sourceFileName;
/*  439 */         if ((cd instanceof Java.NamedTypeDeclaration)) {
/*  440 */           sourceFileName = ((Java.NamedTypeDeclaration)cd).getName() + ".java";
/*      */         } else {
/*  442 */           sourceFileName = "ANONYMOUS.java";
/*      */         }
/*      */       }
/*  445 */       cf.addSourceFileAttribute(sourceFileName);
/*      */     }
/*      */     
/*      */ 
/*  449 */     if (((cd instanceof Java.DocCommentable)) && 
/*  450 */       (((Java.DocCommentable)cd).hasDeprecatedDocTag())) { cf.addDeprecatedAttribute();
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  455 */     List statements = new ArrayList();
/*  456 */     for (Iterator it = cd.variableDeclaratorsAndInitializers.iterator(); it.hasNext();) {
/*  457 */       Java.TypeBodyDeclaration tbd = (Java.TypeBodyDeclaration)it.next();
/*  458 */       if (tbd.isStatic()) { statements.add((Java.BlockStatement)tbd);
/*      */       }
/*      */     }
/*  461 */     maybeCreateInitMethod(cd, cf, statements);
/*      */     
/*      */ 
/*  464 */     compileDeclaredMethods(cd, cf);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  470 */     int declaredMethodCount = cd.getMethodDeclarations().size();
/*      */     
/*  472 */     int syntheticFieldCount = cd.syntheticFields.size();
/*  473 */     Java.ConstructorDeclarator[] cds = cd.getConstructors();
/*  474 */     for (int i = 0; i < cds.length; i++) {
/*  475 */       compile(cds[i], cf);
/*  476 */       if (syntheticFieldCount != cd.syntheticFields.size()) {
/*  477 */         throw new JaninoRuntimeException("SNO: Compilation of constructor \"" + cds[i] + "\" (" + cds[i].getLocation() + ") added synthetic fields!?");
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  490 */     compileDeclaredMemberTypes(cd, cf);
/*      */     
/*      */ 
/*  493 */     compileDeclaredMethods(cd, cf, declaredMethodCount);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  499 */     IClass.IMethod[] ms = iClass.getIMethods();
/*  500 */     for (int i = 0; i < ms.length; i++) {
/*  501 */       IClass.IMethod base = ms[i];
/*  502 */       if (!base.isStatic()) {
/*  503 */         IClass.IMethod override = iClass.findIMethod(base.getName(), base.getParameterTypes());
/*      */         
/*  505 */         if ((override != null) && (!base.getReturnType().equals(override.getReturnType())))
/*      */         {
/*  507 */           compileBridgeMethod(cf, base, override);
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  516 */     for (Iterator it = cd.variableDeclaratorsAndInitializers.iterator(); it.hasNext();) {
/*  517 */       Java.TypeBodyDeclaration tbd = (Java.TypeBodyDeclaration)it.next();
/*  518 */       if ((tbd instanceof Java.FieldDeclaration)) {
/*  519 */         addFields((Java.FieldDeclaration)tbd, cf);
/*      */       }
/*      */     }
/*      */     
/*  523 */     for (Iterator it = cd.syntheticFields.values().iterator(); it.hasNext();) {
/*  524 */       IClass.IField f = (IClass.IField)it.next();
/*  525 */       cf.addFieldInfo((short)0, f.getName(), f.getType().getDescriptor(), null);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  535 */     this.generatedClassFiles.add(cf);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void addFields(Java.FieldDeclaration fd, ClassFile cf)
/*      */     throws CompileException
/*      */   {
/*  544 */     for (int j = 0; j < fd.variableDeclarators.length; j++) {
/*  545 */       Java.VariableDeclarator vd = fd.variableDeclarators[j];
/*  546 */       Java.Type type = fd.type;
/*  547 */       for (int k = 0; k < vd.brackets; k++) { type = new Java.ArrayType(type);
/*      */       }
/*  549 */       Object ocv = null;
/*  550 */       if (((fd.modifiers & 0x10) != 0) && (vd.optionalInitializer != null))
/*      */       {
/*      */ 
/*      */ 
/*  554 */         if ((vd.optionalInitializer instanceof Java.Rvalue)) {
/*  555 */           ocv = getConstantValue((Java.Rvalue)vd.optionalInitializer);
/*      */         }
/*  557 */         if (ocv == Java.Rvalue.CONSTANT_VALUE_NULL) ocv = null;
/*      */       }
/*      */       ClassFile.FieldInfo fi;
/*      */       ClassFile.FieldInfo fi;
/*  561 */       if (Mod.isPrivateAccess(fd.modifiers))
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*  566 */         fi = cf.addFieldInfo(Mod.changeAccess(fd.modifiers, (short)0), vd.name, getType(type).getDescriptor(), ocv);
/*      */ 
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/*      */ 
/*      */ 
/*  574 */         fi = cf.addFieldInfo(fd.modifiers, vd.name, getType(type).getDescriptor(), ocv);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  583 */       if (fd.hasDeprecatedDocTag()) {
/*  584 */         fi.addAttribute(new ClassFile.DeprecatedAttribute(cf.addConstantUtf8Info("Deprecated")));
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public void compile2(Java.AnonymousClassDeclaration acd) throws CompileException {
/*  590 */     compile2(acd);
/*      */   }
/*      */   
/*      */   public void compile2(Java.LocalClassDeclaration lcd) throws CompileException {
/*  594 */     compile2(lcd);
/*      */   }
/*      */   
/*      */ 
/*      */   public void compile2(Java.InnerClassDeclaration icd)
/*      */     throws CompileException
/*      */   {
/*  601 */     List ocs = getOuterClasses(icd);
/*  602 */     int nesting = ocs.size();
/*  603 */     if (nesting >= 2) {
/*  604 */       icd.defineSyntheticField(new SimpleIField(resolve(icd), "this$" + (nesting - 2), resolve((Java.TypeDeclaration)ocs.get(1))));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  613 */     if (((icd instanceof Java.AnonymousClassDeclaration)) || ((icd instanceof Java.LocalClassDeclaration))) {
/*  614 */       Java.ClassDeclaration cd = (Java.ClassDeclaration)icd;
/*  615 */       for (int i = 0; i < cd.variableDeclaratorsAndInitializers.size(); i++) {
/*  616 */         Java.TypeBodyDeclaration tbd = (Java.TypeBodyDeclaration)cd.variableDeclaratorsAndInitializers.get(i);
/*  617 */         if ((tbd instanceof Java.FieldDeclaration)) {
/*  618 */           Java.FieldDeclaration fd = (Java.FieldDeclaration)tbd;
/*  619 */           for (int j = 0; j < fd.variableDeclarators.length; j++) {
/*  620 */             Java.VariableDeclarator vd = fd.variableDeclarators[j];
/*  621 */             if (vd.optionalInitializer != null) {
/*  622 */               fakeCompile(vd.optionalInitializer);
/*      */             }
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*  629 */     compile2((Java.ClassDeclaration)icd);
/*      */   }
/*      */   
/*      */   public void compile2(Java.MemberClassDeclaration mcd) throws CompileException {
/*  633 */     compile2(mcd);
/*      */   }
/*      */   
/*      */   public void compile2(Java.InterfaceDeclaration id) throws CompileException {
/*  637 */     IClass iClass = resolve(id);
/*      */     
/*      */ 
/*  640 */     id.interfaces = new IClass[id.extendedTypes.length];
/*  641 */     String[] interfaceDescriptors = new String[id.interfaces.length];
/*  642 */     for (int i = 0; i < id.extendedTypes.length; i++) {
/*  643 */       id.interfaces[i] = getType(id.extendedTypes[i]);
/*  644 */       interfaceDescriptors[i] = id.interfaces[i].getDescriptor();
/*      */     }
/*      */     
/*      */ 
/*  648 */     ClassFile cf = new ClassFile((short)(id.getModifiers() | 0x20 | 0x200 | 0x400), iClass.getDescriptor(), "Ljava/lang/Object;", interfaceDescriptors);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  661 */     if (this.debugSource)
/*      */     {
/*      */ 
/*  664 */       String s = id.getLocation().getFileName();
/*  665 */       String sourceFileName; String sourceFileName; if (s != null) {
/*  666 */         sourceFileName = new File(s).getName();
/*      */       } else {
/*  668 */         sourceFileName = id.getName() + ".java";
/*      */       }
/*      */       
/*  671 */       cf.addSourceFileAttribute(sourceFileName);
/*      */     }
/*      */     
/*      */ 
/*  675 */     if (id.hasDeprecatedDocTag()) { cf.addDeprecatedAttribute();
/*      */     }
/*      */     
/*  678 */     if (!id.constantDeclarations.isEmpty()) {
/*  679 */       List statements = new ArrayList();
/*  680 */       statements.addAll(id.constantDeclarations);
/*      */       
/*  682 */       maybeCreateInitMethod(id, cf, statements);
/*      */     }
/*      */     
/*  685 */     compileDeclaredMethods(id, cf);
/*      */     
/*      */ 
/*  688 */     for (int i = 0; i < id.constantDeclarations.size(); i++) {
/*  689 */       Java.BlockStatement bs = (Java.BlockStatement)id.constantDeclarations.get(i);
/*  690 */       if ((bs instanceof Java.FieldDeclaration)) {
/*  691 */         addFields((Java.FieldDeclaration)bs, cf);
/*      */       }
/*      */     }
/*  694 */     compileDeclaredMemberTypes(id, cf);
/*      */     
/*      */ 
/*  697 */     this.generatedClassFiles.add(cf);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void maybeCreateInitMethod(Java.AbstractTypeDeclaration decl, ClassFile cf, List statements)
/*      */     throws CompileException
/*      */   {
/*  713 */     if (generatesCode2ListStatements(statements)) {
/*  714 */       Java.MethodDeclarator md = new Java.MethodDeclarator(decl.getLocation(), null, (short)9, new Java.BasicType(decl.getLocation(), 0), "<clinit>", new Java.FunctionDeclarator.FormalParameter[0], new Java.ReferenceType[0], statements);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  727 */       md.setDeclaringType(decl);
/*  728 */       compile(md, cf);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void compileDeclaredMemberTypes(Java.TypeDeclaration decl, ClassFile cf)
/*      */     throws CompileException
/*      */   {
/*  743 */     for (Iterator it = decl.getMemberTypeDeclarations().iterator(); it.hasNext();) {
/*  744 */       Java.TypeDeclaration td = (Java.MemberTypeDeclaration)it.next();
/*  745 */       compile(td);
/*      */       
/*      */ 
/*  748 */       short innerClassInfoIndex = cf.addConstantClassInfo(resolve(td).getDescriptor());
/*  749 */       short outerClassInfoIndex = cf.addConstantClassInfo(resolve(decl).getDescriptor());
/*  750 */       short innerNameIndex = cf.addConstantUtf8Info(((Java.MemberTypeDeclaration)td).getName());
/*  751 */       cf.addInnerClassesAttributeEntry(new ClassFile.InnerClassesAttribute.Entry(innerClassInfoIndex, outerClassInfoIndex, innerNameIndex, td.getModifiers()));
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void compileDeclaredMethods(Java.AbstractTypeDeclaration typeDeclaration, ClassFile cf)
/*      */     throws CompileException
/*      */   {
/*  770 */     compileDeclaredMethods(typeDeclaration, cf, 0);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void compileDeclaredMethods(Java.TypeDeclaration typeDeclaration, ClassFile cf, int startPos)
/*      */     throws CompileException
/*      */   {
/*  788 */     for (int i = startPos; i < typeDeclaration.getMethodDeclarations().size(); i++) {
/*  789 */       compile((Java.MethodDeclarator)typeDeclaration.getMethodDeclarations().get(i), cf);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void compileBridgeMethod(ClassFile cf, IClass.IMethod base, IClass.IMethod override)
/*      */     throws CompileException
/*      */   {
/*  800 */     ClassFile.MethodInfo mi = cf.addMethodInfo((short)4097, base.getName(), base.getDescriptor());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  807 */     IClass[] thrownExceptions = base.getThrownExceptions();
/*  808 */     if (thrownExceptions.length > 0) {
/*  809 */       short eani = cf.addConstantUtf8Info("Exceptions");
/*  810 */       short[] tecciis = new short[thrownExceptions.length];
/*  811 */       for (int i = 0; i < thrownExceptions.length; i++) {
/*  812 */         tecciis[i] = cf.addConstantClassInfo(thrownExceptions[i].getDescriptor());
/*      */       }
/*  814 */       mi.addAttribute(new ClassFile.ExceptionsAttribute(eani, tecciis));
/*      */     }
/*      */     
/*  817 */     CodeContext codeContext = new CodeContext(mi.getClassFile());
/*  818 */     CodeContext savedCodeContext = replaceCodeContext(codeContext);
/*      */     
/*      */ 
/*  821 */     codeContext.saveLocalVariables();
/*  822 */     codeContext.allocateLocalVariable((short)1, "this", override.getDeclaringIClass());
/*  823 */     IClass[] paramTypes = override.getParameterTypes();
/*  824 */     Java.LocalVariableSlot[] locals = new Java.LocalVariableSlot[paramTypes.length];
/*  825 */     for (int i = 0; i < paramTypes.length; i++) {
/*  826 */       locals[i] = codeContext.allocateLocalVariable(Descriptor.size(paramTypes[i].getDescriptor()), "param" + i, paramTypes[i]);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  832 */     writeOpcode(Java.Located.NOWHERE, 42);
/*  833 */     for (int i = 0; i < locals.length; i++) {
/*  834 */       load(Java.Located.NOWHERE, locals[i].getType(), locals[i].getSlotIndex());
/*      */     }
/*  836 */     writeOpcode(Java.Located.NOWHERE, -74);
/*  837 */     writeConstantMethodrefInfo(override.getDeclaringIClass().getDescriptor(), override.getName(), override.getDescriptor());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  842 */     writeOpcode(Java.Located.NOWHERE, -80);
/*  843 */     replaceCodeContext(savedCodeContext);
/*  844 */     codeContext.flowAnalysis(override.getName());
/*      */     
/*      */ 
/*  847 */     mi.addAttribute(new ClassFile.AttributeInfo(cf.addConstantUtf8Info("Code")) { private final CodeContext val$codeContext;
/*      */       
/*  849 */       protected void storeBody(DataOutputStream dos) throws IOException { this.val$codeContext.storeCodeAttributeBody(dos, (short)0, (short)0); }
/*      */     });
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean compile(Java.BlockStatement bs)
/*      */     throws CompileException
/*      */   {
/*  860 */     boolean[] res = new boolean[1];
/*      */     
/*  862 */     Visitor.BlockStatementVisitor bsv = new Visitor.BlockStatementVisitor() { private final boolean[] val$res;
/*      */       
/*  864 */       public void visitInitializer(Java.Initializer i) { try { this.val$res[0] = UnitCompiler.this.compile2(i); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  865 */       public void visitFieldDeclaration(Java.FieldDeclaration fd) { try { this.val$res[0] = UnitCompiler.this.compile2(fd); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  866 */       public void visitLabeledStatement(Java.LabeledStatement ls) { try { this.val$res[0] = UnitCompiler.this.compile2(ls); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  867 */       public void visitBlock(Java.Block b) { try { this.val$res[0] = UnitCompiler.this.compile2(b); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  868 */       public void visitExpressionStatement(Java.ExpressionStatement es) { try { this.val$res[0] = UnitCompiler.this.compile2(es); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  869 */       public void visitIfStatement(Java.IfStatement is) { try { this.val$res[0] = UnitCompiler.this.compile2(is); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  870 */       public void visitForStatement(Java.ForStatement fs) { try { this.val$res[0] = UnitCompiler.this.compile2(fs); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  871 */       public void visitWhileStatement(Java.WhileStatement ws) { try { this.val$res[0] = UnitCompiler.this.compile2(ws); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  872 */       public void visitTryStatement(Java.TryStatement ts) { try { this.val$res[0] = UnitCompiler.this.compile2(ts); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  873 */       public void visitSwitchStatement(Java.SwitchStatement ss) { try { this.val$res[0] = UnitCompiler.this.compile2(ss); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  874 */       public void visitSynchronizedStatement(Java.SynchronizedStatement ss) { try { this.val$res[0] = UnitCompiler.this.compile2(ss); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  875 */       public void visitDoStatement(Java.DoStatement ds) { try { this.val$res[0] = UnitCompiler.this.compile2(ds); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  876 */       public void visitLocalVariableDeclarationStatement(Java.LocalVariableDeclarationStatement lvds) { try { this.val$res[0] = UnitCompiler.this.compile2(lvds); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  877 */       public void visitReturnStatement(Java.ReturnStatement rs) { try { this.val$res[0] = UnitCompiler.this.compile2(rs); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  878 */       public void visitThrowStatement(Java.ThrowStatement ts) { try { this.val$res[0] = UnitCompiler.this.compile2(ts); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  879 */       public void visitBreakStatement(Java.BreakStatement bs) { try { this.val$res[0] = UnitCompiler.this.compile2(bs); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  880 */       public void visitContinueStatement(Java.ContinueStatement cs) { try { this.val$res[0] = UnitCompiler.this.compile2(cs); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  881 */       public void visitEmptyStatement(Java.EmptyStatement es) { this.val$res[0] = UnitCompiler.this.compile2(es); }
/*  882 */       public void visitLocalClassDeclarationStatement(Java.LocalClassDeclarationStatement lcds) { try { this.val$res[0] = UnitCompiler.this.compile2(lcds); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  883 */       public void visitAlternateConstructorInvocation(Java.AlternateConstructorInvocation aci) { try { this.val$res[0] = UnitCompiler.this.compile2(aci); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e); } }
/*  884 */       public void visitSuperConstructorInvocation(Java.SuperConstructorInvocation sci) { try { this.val$res[0] = UnitCompiler.this.compile2(sci); } catch (CompileException e) { throw new UnitCompiler.4UCE(UnitCompiler.this, e);
/*      */         }
/*      */       }
/*      */     };
/*  888 */     try { bs.accept(bsv);
/*  889 */       return res[0];
/*      */     } catch (4UCE uce) {
/*  891 */       throw uce.ce;
/*      */     }
/*      */   }
/*      */   
/*  895 */   private boolean compile2(Java.Initializer i) throws CompileException { return compile(i.block); }
/*      */   
/*      */   private boolean compile2(Java.Block b) throws CompileException {
/*  898 */     this.codeContext.saveLocalVariables();
/*      */     try {
/*  900 */       return compileStatements(b.statements);
/*      */     } finally {
/*  902 */       this.codeContext.restoreLocalVariables();
/*      */     }
/*      */   }
/*      */   
/*      */   private boolean compileStatements(List statements) throws CompileException {
/*  907 */     boolean previousStatementCanCompleteNormally = true;
/*  908 */     for (int i = 0; i < statements.size(); i++) {
/*  909 */       Java.BlockStatement bs = (Java.BlockStatement)statements.get(i);
/*  910 */       if ((!previousStatementCanCompleteNormally) && (generatesCode(bs))) {
/*  911 */         compileError("Statement is unreachable", bs.getLocation());
/*  912 */         break;
/*      */       }
/*  914 */       previousStatementCanCompleteNormally = compile(bs);
/*      */     }
/*  916 */     return previousStatementCanCompleteNormally;
/*      */   }
/*      */   
/*  919 */   private boolean compile2(Java.DoStatement ds) throws CompileException { Object cvc = getConstantValue(ds.condition);
/*  920 */     if (cvc != null) {
/*  921 */       if (Boolean.TRUE.equals(cvc)) {
/*  922 */         warning("DSTC", "Condition of DO statement is always TRUE; the proper way of declaring an unconditional loop is \"for (;;)\"", ds.getLocation());
/*      */         
/*      */ 
/*      */ 
/*  926 */         return compileUnconditionalLoop(ds, ds.body, null);
/*      */       }
/*      */       
/*  929 */       warning("DSNR", "DO statement never repeats", ds.getLocation());
/*      */     }
/*      */     
/*      */ 
/*  933 */     CodeContext tmp71_68 = this.codeContext;tmp71_68.getClass();ds.whereToContinue = new CodeContext.Offset(tmp71_68);
/*  934 */     ds.bodyHasContinue = false;
/*      */     
/*  936 */     CodeContext.Offset bodyOffset = this.codeContext.newOffset();
/*      */     
/*      */ 
/*  939 */     if ((!compile(ds.body)) && (!ds.bodyHasContinue)) {
/*  940 */       warning("DSNTC", "\"do\" statement never tests its condition", ds.getLocation());
/*  941 */       if (ds.whereToBreak == null) return false;
/*  942 */       ds.whereToBreak.set();
/*  943 */       return true;
/*      */     }
/*      */     
/*      */ 
/*  947 */     ds.whereToContinue.set();
/*  948 */     compileBoolean(ds.condition, bodyOffset, true);
/*      */     
/*  950 */     if (ds.whereToBreak != null) { ds.whereToBreak.set();
/*      */     }
/*  952 */     return true;
/*      */   }
/*      */   
/*  955 */   private boolean compile2(Java.ForStatement fs) throws CompileException { this.codeContext.saveLocalVariables();
/*      */     
/*      */     try
/*      */     {
/*  959 */       if (fs.optionalInit != null) { compile(fs.optionalInit);
/*      */       }
/*  961 */       if (fs.optionalCondition == null) {
/*  962 */         return compileUnconditionalLoop(fs, fs.body, fs.optionalUpdate);
/*      */       }
/*      */       
/*  965 */       Object cvc = getConstantValue(fs.optionalCondition);
/*  966 */       if (cvc != null) {
/*  967 */         if (Boolean.TRUE.equals(cvc)) {
/*  968 */           warning("FSTC", "Condition of FOR statement is always TRUE; the proper way of declaring an unconditional loop is \"for (;;)\"", fs.getLocation());
/*      */           
/*      */ 
/*      */ 
/*  972 */           return compileUnconditionalLoop(fs, fs.body, fs.optionalUpdate);
/*      */         }
/*      */         
/*  975 */         warning("FSNR", "FOR statement never repeats", fs.getLocation());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*  980 */       CodeContext tmp136_133 = this.codeContext;tmp136_133.getClass();CodeContext.Offset toCondition = new CodeContext.Offset(tmp136_133);
/*  981 */       writeBranch(fs, -89, toCondition); CodeContext 
/*      */       
/*      */ 
/*  984 */         tmp162_159 = this.codeContext;tmp162_159.getClass();fs.whereToContinue = new CodeContext.Offset(tmp162_159);
/*  985 */       fs.bodyHasContinue = false;
/*  986 */       CodeContext.Offset bodyOffset = this.codeContext.newOffset();
/*  987 */       boolean bodyCCN = compile(fs.body);
/*      */       
/*      */ 
/*  990 */       fs.whereToContinue.set();
/*  991 */       if (fs.optionalUpdate != null) {
/*  992 */         if ((!bodyCCN) && (!fs.bodyHasContinue)) {
/*  993 */           warning("FUUR", "For update is unreachable", fs.getLocation());
/*      */         }
/*      */         else {
/*  996 */           for (int i = 0; i < fs.optionalUpdate.length; i++) {
/*  997 */             compile(fs.optionalUpdate[i]);
/*      */           }
/*      */         }
/*      */       }
/*      */       
/*      */ 
/* 1003 */       toCondition.set();
/* 1004 */       compileBoolean(fs.optionalCondition, bodyOffset, true);
/*      */     } finally {
/* 1006 */       this.codeContext.restoreLocalVariables();
/*      */     }
/*      */     
/* 1009 */     if (fs.whereToBreak != null) { fs.whereToBreak.set();
/*      */     }
/* 1011 */     return true;
/*      */   }
/*      */   
/* 1014 */   private boolean compile2(Java.WhileStatement ws) throws CompileException { Object cvc = getConstantValue(ws.condition);
/* 1015 */     if (cvc != null) {
/* 1016 */       if (Boolean.TRUE.equals(cvc)) {
/* 1017 */         warning("WSTC", "Condition of WHILE statement is always TRUE; the proper way of declaring an unconditional loop is \"for (;;)\"", ws.getLocation());
/*      */         
/*      */ 
/*      */ 
/* 1021 */         return compileUnconditionalLoop(ws, ws.body, null);
/*      */       }
/*      */       
/* 1024 */       warning("WSNR", "WHILE statement never repeats", ws.getLocation());
/*      */     }
/*      */     
/*      */ 
/* 1028 */     CodeContext tmp71_68 = this.codeContext;tmp71_68.getClass();ws.whereToContinue = new CodeContext.Offset(tmp71_68);
/* 1029 */     ws.bodyHasContinue = false;
/* 1030 */     writeBranch(ws, -89, ws.whereToContinue);
/*      */     
/*      */ 
/* 1033 */     CodeContext.Offset bodyOffset = this.codeContext.newOffset();
/* 1034 */     compile(ws.body);
/*      */     
/*      */ 
/* 1037 */     ws.whereToContinue.set();
/* 1038 */     compileBoolean(ws.condition, bodyOffset, true);
/*      */     
/* 1040 */     if (ws.whereToBreak != null) ws.whereToBreak.set();
/* 1041 */     return true;
/*      */   }
/*      */   
/*      */ 
/*      */   private boolean compileUnconditionalLoop(Java.ContinuableStatement cs, Java.BlockStatement body, Java.Rvalue[] optionalUpdate)
/*      */     throws CompileException
/*      */   {
/* 1048 */     if (optionalUpdate != null) { return compileUnconditionalLoopWithUpdate(cs, body, optionalUpdate);
/*      */     }
/* 1050 */     cs.whereToContinue = this.codeContext.newOffset();
/* 1051 */     cs.bodyHasContinue = false;
/*      */     
/*      */ 
/* 1054 */     if (compile(body)) { writeBranch(cs, -89, cs.whereToContinue);
/*      */     }
/* 1056 */     if (cs.whereToBreak == null) return false;
/* 1057 */     cs.whereToBreak.set();
/* 1058 */     return true;
/*      */   }
/*      */   
/*      */ 
/*      */   private boolean compileUnconditionalLoopWithUpdate(Java.ContinuableStatement cs, Java.BlockStatement body, Java.Rvalue[] update)
/*      */     throws CompileException
/*      */   {
/* 1065 */     CodeContext tmp9_6 = this.codeContext;tmp9_6.getClass();cs.whereToContinue = new CodeContext.Offset(tmp9_6);
/* 1066 */     cs.bodyHasContinue = false;
/*      */     
/*      */ 
/* 1069 */     CodeContext.Offset bodyOffset = this.codeContext.newOffset();
/* 1070 */     boolean bodyCCN = compile(body);
/*      */     
/*      */ 
/* 1073 */     cs.whereToContinue.set();
/* 1074 */     if ((!bodyCCN) && (!cs.bodyHasContinue)) {
/* 1075 */       warning("LUUR", "Loop update is unreachable", update[0].getLocation());
/*      */     }
/*      */     else {
/* 1078 */       for (int i = 0; i < update.length; i++) compile(update[i]);
/* 1079 */       writeBranch(cs, -89, bodyOffset);
/*      */     }
/*      */     
/* 1082 */     if (cs.whereToBreak == null) return false;
/* 1083 */     cs.whereToBreak.set();
/* 1084 */     return true;
/*      */   }
/*      */   
/*      */   private boolean compile2(Java.LabeledStatement ls) throws CompileException {
/* 1088 */     boolean canCompleteNormally = compile(ls.body);
/* 1089 */     if (ls.whereToBreak != null) {
/* 1090 */       ls.whereToBreak.set();
/* 1091 */       canCompleteNormally = true;
/*      */     }
/* 1093 */     return canCompleteNormally;
/*      */   }
/*      */   
/*      */   private boolean compile2(Java.SwitchStatement ss) throws CompileException
/*      */   {
/* 1098 */     IClass switchExpressionType = compileGetValue(ss.condition);
/* 1099 */     assignmentConversion(ss, switchExpressionType, IClass.INT, null);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1107 */     TreeMap caseLabelMap = new TreeMap();
/* 1108 */     CodeContext.Offset defaultLabelOffset = null;
/* 1109 */     CodeContext.Offset[] sbsgOffsets = new CodeContext.Offset[ss.sbsgs.size()];
/* 1110 */     for (int i = 0; i < ss.sbsgs.size(); i++) {
/* 1111 */       Java.SwitchStatement.SwitchBlockStatementGroup sbsg = (Java.SwitchStatement.SwitchBlockStatementGroup)ss.sbsgs.get(i); CodeContext 
/*      */       
/*      */ 
/* 1114 */         tmp89_86 = this.codeContext;tmp89_86.getClass();sbsgOffsets[i] = new CodeContext.Offset(tmp89_86);
/* 1115 */       for (int j = 0; j < sbsg.caseLabels.size(); j++)
/*      */       {
/*      */ 
/* 1118 */         Java.Rvalue rv = (Java.Rvalue)sbsg.caseLabels.get(j);
/* 1119 */         Object cv = getConstantValue(rv);
/* 1120 */         if (cv == null) {
/* 1121 */           compileError("Value of \"case\" label does not pose a constant value", rv.getLocation());
/* 1122 */           cv = new Integer(99);
/*      */         }
/*      */         
/*      */ 
/* 1126 */         IClass rvType = getType(rv);
/* 1127 */         assignmentConversion(ss, rvType, switchExpressionType, cv);
/*      */         
/*      */ 
/*      */         Integer civ;
/*      */         
/*      */ 
/*      */         Integer civ;
/*      */         
/*      */ 
/* 1136 */         if ((cv instanceof Integer)) {
/* 1137 */           civ = (Integer)cv;
/*      */         } else { Integer civ;
/* 1139 */           if ((cv instanceof Number)) {
/* 1140 */             civ = new Integer(((Number)cv).intValue());
/*      */           } else { Integer civ;
/* 1142 */             if ((cv instanceof Character)) {
/* 1143 */               civ = new Integer(((Character)cv).charValue());
/*      */             } else {
/* 1145 */               compileError("Value of case label must be a char, byte, short or int constant", rv.getLocation());
/*      */               
/*      */ 
/*      */ 
/* 1149 */               civ = new Integer(99);
/*      */             }
/*      */           }
/*      */         }
/* 1153 */         if (caseLabelMap.containsKey(civ)) {
/* 1154 */           compileError("Duplicate \"case\" switch label value", rv.getLocation());
/*      */         }
/* 1156 */         caseLabelMap.put(civ, sbsgOffsets[i]);
/*      */       }
/* 1158 */       if (sbsg.hasDefaultLabel) {
/* 1159 */         if (defaultLabelOffset != null) {
/* 1160 */           compileError("Duplicate \"default\" switch label", sbsg.getLocation());
/*      */         }
/* 1162 */         defaultLabelOffset = sbsgOffsets[i];
/*      */       }
/*      */     }
/* 1165 */     if (defaultLabelOffset == null) { defaultLabelOffset = getWhereToBreak(ss);
/*      */     }
/*      */     
/* 1168 */     CodeContext.Offset switchOffset = this.codeContext.newOffset();
/* 1169 */     if (!caseLabelMap.isEmpty())
/*      */     {
/*      */ 
/*      */ 
/* 1173 */       if (((Integer)caseLabelMap.firstKey()).intValue() + caseLabelMap.size() >= ((Integer)caseLabelMap.lastKey()).intValue() - caseLabelMap.size())
/*      */       {
/*      */ 
/*      */ 
/* 1177 */         int low = ((Integer)caseLabelMap.firstKey()).intValue();
/* 1178 */         int high = ((Integer)caseLabelMap.lastKey()).intValue();
/*      */         
/* 1180 */         writeOpcode(ss, -86);
/* 1181 */         new Java.Padder(this.codeContext).set();
/* 1182 */         writeOffset(switchOffset, defaultLabelOffset);
/* 1183 */         writeInt(low);
/* 1184 */         writeInt(high);
/* 1185 */         Iterator si = caseLabelMap.entrySet().iterator();
/* 1186 */         int cur = low;
/* 1187 */         while (si.hasNext()) {
/* 1188 */           Map.Entry me = (Map.Entry)si.next();
/* 1189 */           int val = ((Integer)me.getKey()).intValue();
/* 1190 */           while (cur < val) {
/* 1191 */             writeOffset(switchOffset, defaultLabelOffset);
/* 1192 */             cur++;
/*      */           }
/* 1194 */           writeOffset(switchOffset, (CodeContext.Offset)me.getValue());
/* 1195 */           cur++;
/*      */         }
/*      */       } else {
/* 1198 */         writeOpcode(ss, -85);
/* 1199 */         new Java.Padder(this.codeContext).set();
/* 1200 */         writeOffset(switchOffset, defaultLabelOffset);
/* 1201 */         writeInt(caseLabelMap.size());
/* 1202 */         Iterator si = caseLabelMap.entrySet().iterator();
/* 1203 */         while (si.hasNext()) {
/* 1204 */           Map.Entry me = (Map.Entry)si.next();
/* 1205 */           writeInt(((Integer)me.getKey()).intValue());
/* 1206 */           writeOffset(switchOffset, (CodeContext.Offset)me.getValue());
/*      */         }
/*      */       }
/*      */     }
/*      */     
/* 1211 */     boolean canCompleteNormally = true;
/* 1212 */     for (int i = 0; i < ss.sbsgs.size(); i++) {
/* 1213 */       Java.SwitchStatement.SwitchBlockStatementGroup sbsg = (Java.SwitchStatement.SwitchBlockStatementGroup)ss.sbsgs.get(i);
/*      */       
/*      */ 
/* 1216 */       sbsgOffsets[i].set();
/* 1217 */       canCompleteNormally = true;
/* 1218 */       for (int j = 0; j < sbsg.blockStatements.size(); j++) {
/* 1219 */         Java.BlockStatement bs = (Java.BlockStatement)sbsg.blockStatements.get(j);
/* 1220 */         if (!canCompleteNormally) {
/* 1221 */           compileError("Statement is unreachable", bs.getLocation());
/* 1222 */           break;
/*      */         }
/* 1224 */         canCompleteNormally = compile(bs);
/*      */       }
/*      */     }
/* 1227 */     if (ss.whereToBreak != null) {
/* 1228 */       ss.whereToBreak.set();
/* 1229 */       canCompleteNormally = true;
/*      */     }
/* 1231 */     return canCompleteNormally;
/*      */   }
/*      */   
/*      */   private boolean compile2(Java.BreakStatement bs) throws CompileException
/*      */   {
/* 1236 */     Java.BreakableStatement brokenStatement = null;
/* 1237 */     if (bs.optionalLabel == null)
/*      */     {
/* 1239 */       for (Java.Scope s = bs.getEnclosingScope(); 
/* 1240 */           ((s instanceof Java.Statement)) || ((s instanceof Java.CatchClause)); 
/* 1241 */           s = s.getEnclosingScope())
/*      */       {
/* 1243 */         if ((s instanceof Java.BreakableStatement)) {
/* 1244 */           brokenStatement = (Java.BreakableStatement)s;
/* 1245 */           break;
/*      */         }
/*      */       }
/* 1248 */       if (brokenStatement == null) {
/* 1249 */         compileError("\"break\" statement is not enclosed by a breakable statement", bs.getLocation());
/* 1250 */         return false;
/*      */       }
/*      */     }
/*      */     else {
/* 1254 */       for (Java.Scope s = bs.getEnclosingScope(); 
/* 1255 */           ((s instanceof Java.Statement)) || ((s instanceof Java.CatchClause)); 
/* 1256 */           s = s.getEnclosingScope())
/*      */       {
/* 1258 */         if ((s instanceof Java.LabeledStatement)) {
/* 1259 */           Java.LabeledStatement ls = (Java.LabeledStatement)s;
/* 1260 */           if (ls.label.equals(bs.optionalLabel)) {
/* 1261 */             brokenStatement = ls;
/* 1262 */             break;
/*      */           }
/*      */         }
/*      */       }
/* 1266 */       if (brokenStatement == null) {
/* 1267 */         compileError("Statement \"break " + bs.optionalLabel + "\" is not enclosed by a breakable statement with label \"" + bs.optionalLabel + "\"", bs.getLocation());
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1274 */         return false;
/*      */       }
/*      */     }
/*      */     
/* 1278 */     leaveStatements(bs.getEnclosingScope(), brokenStatement.getEnclosingScope(), null);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1283 */     writeBranch(bs, -89, getWhereToBreak(brokenStatement));
/* 1284 */     return false;
/*      */   }
/*      */   
/*      */   private boolean compile2(Java.ContinueStatement cs) throws CompileException
/*      */   {
/* 1289 */     Java.ContinuableStatement continuedStatement = null;
/* 1290 */     if (cs.optionalLabel == null)
/*      */     {
/* 1292 */       for (Java.Scope s = cs.getEnclosingScope(); 
/* 1293 */           ((s instanceof Java.Statement)) || ((s instanceof Java.CatchClause)); 
/* 1294 */           s = s.getEnclosingScope())
/*      */       {
/* 1296 */         if ((s instanceof Java.ContinuableStatement)) {
/* 1297 */           continuedStatement = (Java.ContinuableStatement)s;
/* 1298 */           break;
/*      */         }
/*      */       }
/* 1301 */       if (continuedStatement == null) {
/* 1302 */         compileError("\"continue\" statement is not enclosed by a continuable statement", cs.getLocation());
/*      */         
/*      */ 
/*      */ 
/* 1306 */         return false;
/*      */       }
/*      */     }
/*      */     else {
/* 1310 */       for (Java.Scope s = cs.getEnclosingScope(); 
/* 1311 */           ((s instanceof Java.Statement)) || ((s instanceof Java.CatchClause)); 
/* 1312 */           s = s.getEnclosingScope())
/*      */       {
/* 1314 */         if ((s instanceof Java.LabeledStatement)) {
/* 1315 */           Java.LabeledStatement ls = (Java.LabeledStatement)s;
/* 1316 */           if (ls.label.equals(cs.optionalLabel)) {
/* 1317 */             Java.Statement st = ls.body;
/* 1318 */             while ((st instanceof Java.LabeledStatement)) st = ((Java.LabeledStatement)st).body;
/* 1319 */             if (!(st instanceof Java.ContinuableStatement)) {
/* 1320 */               compileError("Labeled statement is not continuable", st.getLocation());
/* 1321 */               return false;
/*      */             }
/* 1323 */             continuedStatement = (Java.ContinuableStatement)st;
/* 1324 */             break;
/*      */           }
/*      */         }
/*      */       }
/* 1328 */       if (continuedStatement == null) {
/* 1329 */         compileError("Statement \"continue " + cs.optionalLabel + "\" is not enclosed by a continuable statement with label \"" + cs.optionalLabel + "\"", cs.getLocation());
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1336 */         return false;
/*      */       }
/*      */     }
/*      */     
/* 1340 */     continuedStatement.bodyHasContinue = true;
/* 1341 */     leaveStatements(cs.getEnclosingScope(), continuedStatement.getEnclosingScope(), null);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1346 */     writeBranch(cs, -89, continuedStatement.whereToContinue);
/* 1347 */     return false;
/*      */   }
/*      */   
/* 1350 */   private boolean compile2(Java.EmptyStatement es) { return true; }
/*      */   
/*      */   private boolean compile2(Java.ExpressionStatement ee) throws CompileException {
/* 1353 */     compile(ee.rvalue);
/* 1354 */     return true;
/*      */   }
/*      */   
/* 1357 */   private boolean compile2(Java.FieldDeclaration fd) throws CompileException { for (int i = 0; i < fd.variableDeclarators.length; i++) {
/* 1358 */       Java.VariableDeclarator vd = fd.variableDeclarators[i];
/*      */       
/* 1360 */       Java.ArrayInitializerOrRvalue initializer = getNonConstantFinalInitializer(fd, vd);
/* 1361 */       if (initializer != null)
/*      */       {
/* 1363 */         if ((fd.modifiers & 0x8) == 0) {
/* 1364 */           writeOpcode(fd, 42);
/*      */         }
/* 1366 */         IClass fieldType = getType(fd.type);
/* 1367 */         if ((initializer instanceof Java.Rvalue)) {
/* 1368 */           Java.Rvalue rvalue = (Java.Rvalue)initializer;
/* 1369 */           IClass initializerType = compileGetValue(rvalue);
/* 1370 */           fieldType = fieldType.getArrayIClass(vd.brackets, this.iClassLoader.OBJECT);
/* 1371 */           assignmentConversion(fd, initializerType, fieldType, getConstantValue(rvalue));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         }
/* 1378 */         else if ((initializer instanceof Java.ArrayInitializer)) {
/* 1379 */           compileGetValue((Java.ArrayInitializer)initializer, fieldType);
/*      */         }
/*      */         else {
/* 1382 */           throw new JaninoRuntimeException("Unexpected array initializer or rvalue class " + initializer.getClass().getName());
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1391 */         if ((fd.modifiers & 0x8) != 0) {
/* 1392 */           writeOpcode(fd, -77);
/*      */         } else {
/* 1394 */           writeOpcode(fd, -75);
/*      */         }
/* 1396 */         writeConstantFieldrefInfo(resolve(fd.getDeclaringType()).getDescriptor(), vd.name, fieldType.getDescriptor());
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1402 */     return true;
/*      */   }
/*      */   
/* 1405 */   private boolean compile2(Java.IfStatement is) throws CompileException { Object cv = getConstantValue(is.condition);
/* 1406 */     Java.BlockStatement es = is.optionalElseStatement != null ? is.optionalElseStatement : new Java.EmptyStatement(is.thenStatement.getLocation());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1411 */     if ((cv instanceof Boolean))
/*      */     {
/*      */ 
/* 1414 */       fakeCompile(is.condition);
/*      */       Java.BlockStatement blindStatement;
/* 1416 */       Java.BlockStatement seeingStatement; Java.BlockStatement blindStatement; if (((Boolean)cv).booleanValue()) {
/* 1417 */         Java.BlockStatement seeingStatement = is.thenStatement;
/* 1418 */         blindStatement = es;
/*      */       } else {
/* 1420 */         seeingStatement = es;
/* 1421 */         blindStatement = is.thenStatement;
/*      */       }
/*      */       
/*      */ 
/* 1425 */       CodeContext.Inserter ins = this.codeContext.newInserter();
/* 1426 */       boolean ssccn = compile(seeingStatement);
/* 1427 */       if (ssccn) { return true;
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1436 */       Java.Scope s = is.getEnclosingScope();
/* 1437 */       while ((s instanceof Java.Block)) s = s.getEnclosingScope();
/* 1438 */       if ((s instanceof Java.FunctionDeclarator))
/*      */       {
/*      */ 
/* 1441 */         throw STOP_COMPILING_CODE;
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 1446 */       CodeContext.Offset off = this.codeContext.newOffset();
/* 1447 */       this.codeContext.pushInserter(ins);
/*      */       try {
/* 1449 */         pushConstant(is, new Integer(0));
/* 1450 */         writeBranch(is, -102, off);
/*      */       } finally {
/* 1452 */         this.codeContext.popInserter();
/*      */       }
/*      */       
/* 1455 */       return compile(blindStatement);
/*      */     }
/*      */     
/*      */ 
/* 1459 */     if (generatesCode(is.thenStatement)) {
/* 1460 */       if (generatesCode(es))
/*      */       {
/*      */ 
/* 1463 */         CodeContext tmp245_242 = this.codeContext;tmp245_242.getClass();CodeContext.Offset eso = new CodeContext.Offset(tmp245_242); CodeContext 
/* 1464 */           tmp263_260 = this.codeContext;tmp263_260.getClass();CodeContext.Offset end = new CodeContext.Offset(tmp263_260);
/* 1465 */         compileBoolean(is.condition, eso, false);
/* 1466 */         boolean tsccn = compile(is.thenStatement);
/* 1467 */         if (tsccn) writeBranch(is, -89, end);
/* 1468 */         eso.set();
/* 1469 */         boolean esccn = compile(es);
/* 1470 */         end.set();
/* 1471 */         return (tsccn) || (esccn);
/*      */       }
/*      */       
/*      */ 
/* 1475 */       CodeContext tmp349_346 = this.codeContext;tmp349_346.getClass();CodeContext.Offset end = new CodeContext.Offset(tmp349_346);
/* 1476 */       compileBoolean(is.condition, end, false);
/* 1477 */       compile(is.thenStatement);
/* 1478 */       end.set();
/* 1479 */       return true;
/*      */     }
/*      */     
/* 1482 */     if (generatesCode(es))
/*      */     {
/*      */ 
/* 1485 */       CodeContext tmp402_399 = this.codeContext;tmp402_399.getClass();CodeContext.Offset end = new CodeContext.Offset(tmp402_399);
/* 1486 */       compileBoolean(is.condition, end, true);
/* 1487 */       compile(es);
/* 1488 */       end.set();
/* 1489 */       return true;
/*      */     }
/*      */     
/*      */ 
/* 1493 */     IClass conditionType = compileGetValue(is.condition);
/* 1494 */     if (conditionType != IClass.BOOLEAN) compileError("Not a boolean expression", is.getLocation());
/* 1495 */     pop(is, conditionType);
/* 1496 */     return true;
/*      */   }
/*      */   
/*      */ 
/* 1500 */   private static final RuntimeException STOP_COMPILING_CODE = new RuntimeException("SNO: This exception should have been caught and processed");
/*      */   
/*      */ 
/*      */ 
/*      */   private boolean compile2(Java.LocalClassDeclarationStatement lcds)
/*      */     throws CompileException
/*      */   {
/* 1507 */     Java.LocalClassDeclaration otherLCD = findLocalClassDeclaration(lcds, lcds.lcd.name);
/* 1508 */     if (otherLCD != lcds.lcd) {
/* 1509 */       compileError("Redeclaration of local class \"" + lcds.lcd.name + "\"; previously declared in " + otherLCD.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1517 */     compile(lcds.lcd);
/* 1518 */     return true;
/*      */   }
/*      */   
/*      */ 
/*      */   private Java.LocalClassDeclaration findLocalClassDeclaration(Java.Scope s, String name)
/*      */   {
/*      */     for (;;)
/*      */     {
/* 1526 */       Java.Scope es = s.getEnclosingScope();
/* 1527 */       if ((es instanceof Java.CompilationUnit)) break;
/* 1528 */       Java.BlockStatement bs; Iterator it; if (((s instanceof Java.BlockStatement)) && (((es instanceof Java.Block)) || ((es instanceof Java.FunctionDeclarator))))
/*      */       {
/*      */ 
/*      */ 
/* 1532 */         bs = (Java.BlockStatement)s;
/* 1533 */         List statements = (es instanceof Java.BlockStatement) ? ((Java.Block)es).statements : ((Java.FunctionDeclarator)es).optionalStatements;
/*      */         
/*      */ 
/*      */ 
/*      */ 
/* 1538 */         for (it = statements.iterator(); it.hasNext();) {
/* 1539 */           Java.BlockStatement bs2 = (Java.BlockStatement)it.next();
/* 1540 */           if ((bs2 instanceof Java.LocalClassDeclarationStatement)) {
/* 1541 */             Java.LocalClassDeclarationStatement lcds = (Java.LocalClassDeclarationStatement)bs2;
/* 1542 */             if (lcds.lcd.name.equals(name)) return lcds.lcd;
/*      */           }
/* 1544 */           if (bs2 == bs) break;
/*      */         }
/*      */       }
/* 1547 */       s = es;
/*      */     }
/* 1549 */     return null;
/*      */   }
/*      */   
/*      */   private boolean compile2(Java.LocalVariableDeclarationStatement lvds) throws CompileException {
/* 1553 */     if ((lvds.modifiers & 0xFFFFFFEF) != 0) {
/* 1554 */       compileError("The only allowed modifier in local variable declarations is \"final\"", lvds.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1560 */     for (int j = 0; j < lvds.variableDeclarators.length; j++) {
/* 1561 */       Java.VariableDeclarator vd = lvds.variableDeclarators[j];
/*      */       
/* 1563 */       Java.LocalVariable lv = getLocalVariable(lvds, vd);
/* 1564 */       lv.setSlot(this.codeContext.allocateLocalVariable(Descriptor.size(lv.type.getDescriptor()), vd.name, lv.type));
/*      */       
/*      */ 
/*      */ 
/* 1568 */       if (vd.optionalInitializer != null) {
/* 1569 */         if ((vd.optionalInitializer instanceof Java.Rvalue)) {
/* 1570 */           Java.Rvalue rhs = (Java.Rvalue)vd.optionalInitializer;
/* 1571 */           assignmentConversion(lvds, compileGetValue(rhs), lv.type, getConstantValue(rhs));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         }
/* 1578 */         else if ((vd.optionalInitializer instanceof Java.ArrayInitializer)) {
/* 1579 */           compileGetValue((Java.ArrayInitializer)vd.optionalInitializer, lv.type);
/*      */         }
/*      */         else {
/* 1582 */           throw new JaninoRuntimeException("Unexpected rvalue or array initialized class " + vd.optionalInitializer.getClass().getName());
/*      */         }
/*      */         
/*      */ 
/*      */ 
/* 1587 */         store(lvds, lv.type, lv);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1594 */     return true;
/*      */   }
/*      */   
/*      */ 
/*      */   public Java.LocalVariable getLocalVariable(Java.LocalVariableDeclarationStatement lvds, Java.VariableDeclarator vd)
/*      */     throws CompileException
/*      */   {
/* 1601 */     if (vd.localVariable == null)
/*      */     {
/*      */ 
/* 1604 */       Java.Type variableType = lvds.type;
/* 1605 */       for (int k = 0; k < vd.brackets; k++) { variableType = new Java.ArrayType(variableType);
/*      */       }
/* 1607 */       vd.localVariable = new Java.LocalVariable((lvds.modifiers & 0x10) != 0, getType(variableType));
/*      */     }
/* 1609 */     return vd.localVariable;
/*      */   }
/*      */   
/*      */   private boolean compile2(Java.ReturnStatement rs)
/*      */     throws CompileException
/*      */   {
/* 1615 */     Java.FunctionDeclarator enclosingFunction = null;
/*      */     
/* 1617 */     Java.Scope s = rs.getEnclosingScope();
/* 1618 */     while (((s instanceof Java.Statement)) || ((s instanceof Java.CatchClause))) s = s.getEnclosingScope();
/* 1619 */     enclosingFunction = (Java.FunctionDeclarator)s;
/*      */     
/*      */ 
/* 1622 */     IClass returnType = getReturnType(enclosingFunction);
/* 1623 */     if (returnType == IClass.VOID) {
/* 1624 */       if (rs.optionalReturnValue != null) compileError("Method must not return a value", rs.getLocation());
/* 1625 */       leaveStatements(rs.getEnclosingScope(), enclosingFunction, null);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 1630 */       writeOpcode(rs, -79);
/* 1631 */       return false;
/*      */     }
/* 1633 */     if (rs.optionalReturnValue == null) {
/* 1634 */       compileError("Method must return a value", rs.getLocation());
/* 1635 */       return false;
/*      */     }
/* 1637 */     IClass type = compileGetValue(rs.optionalReturnValue);
/* 1638 */     assignmentConversion(rs, type, returnType, getConstantValue(rs.optionalReturnValue));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1645 */     leaveStatements(rs.getEnclosingScope(), enclosingFunction, returnType);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1650 */     writeOpcode(rs, -84 + ilfda(returnType));
/* 1651 */     return false;
/*      */   }
/*      */   
/*      */   private boolean compile2(Java.SynchronizedStatement ss) throws CompileException
/*      */   {
/* 1656 */     if (!this.iClassLoader.OBJECT.isAssignableFrom(compileGetValue(ss.expression))) {
/* 1657 */       compileError("Monitor object of \"synchronized\" statement is not a subclass of \"Object\"", ss.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1663 */     this.codeContext.saveLocalVariables();
/* 1664 */     boolean canCompleteNormally = false;
/*      */     
/*      */     try
/*      */     {
/* 1668 */       ss.monitorLvIndex = this.codeContext.allocateLocalVariable((short)1);
/*      */       
/*      */ 
/* 1671 */       writeOpcode(ss, 89);
/* 1672 */       store(ss, this.iClassLoader.OBJECT, ss.monitorLvIndex);
/*      */       
/*      */ 
/* 1675 */       writeOpcode(ss, -62); CodeContext 
/*      */       
/*      */ 
/* 1678 */         tmp92_89 = this.codeContext;tmp92_89.getClass();CodeContext.Offset monitorExitOffset = new CodeContext.Offset(tmp92_89);
/* 1679 */       CodeContext.Offset beginningOfBody = this.codeContext.newOffset();
/* 1680 */       canCompleteNormally = compile(ss.body);
/* 1681 */       if (canCompleteNormally) {
/* 1682 */         writeBranch(ss, -89, monitorExitOffset);
/*      */       }
/*      */       
/*      */ 
/* 1686 */       CodeContext.Offset here = this.codeContext.newOffset();
/* 1687 */       this.codeContext.addExceptionTableEntry(beginningOfBody, here, here, null);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1693 */       leave(ss, this.iClassLoader.THROWABLE);
/* 1694 */       writeOpcode(ss, -65);
/*      */       
/*      */ 
/* 1697 */       if (canCompleteNormally) {
/* 1698 */         monitorExitOffset.set();
/* 1699 */         leave(ss, null);
/*      */       }
/*      */     } finally {
/* 1702 */       this.codeContext.restoreLocalVariables();
/*      */     }
/*      */     
/* 1705 */     return canCompleteNormally;
/*      */   }
/*      */   
/* 1708 */   private boolean compile2(Java.ThrowStatement ts) throws CompileException { IClass expressionType = compileGetValue(ts.expression);
/* 1709 */     checkThrownException(ts, expressionType, ts.getEnclosingScope());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1714 */     writeOpcode(ts, -65);
/* 1715 */     return false;
/*      */   }
/*      */   
/* 1718 */   private boolean compile2(Java.TryStatement ts) throws CompileException { if (ts.optionalFinally != null) { CodeContext tmp16_13 = this.codeContext;tmp16_13.getClass();ts.finallyOffset = new CodeContext.Offset(tmp16_13);
/*      */     }
/* 1720 */     CodeContext.Offset beginningOfBody = this.codeContext.newOffset(); CodeContext 
/* 1721 */       tmp43_40 = this.codeContext;tmp43_40.getClass();CodeContext.Offset afterStatement = new CodeContext.Offset(tmp43_40);
/*      */     
/* 1723 */     this.codeContext.saveLocalVariables();
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     try
/*      */     {
/* 1734 */       short pcLVIndex = ts.optionalFinally != null ? this.codeContext.allocateLocalVariable((short)1) : 0;
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1740 */       boolean canCompleteNormally = compile(ts.body);
/* 1741 */       CodeContext.Offset afterBody = this.codeContext.newOffset();
/* 1742 */       if (canCompleteNormally) {
/* 1743 */         writeBranch(ts, -89, afterStatement);
/*      */       }
/*      */       
/* 1746 */       if (beginningOfBody.offset != afterBody.offset) {
/* 1747 */         this.codeContext.saveLocalVariables();
/*      */         try {
/* 1749 */           for (int i = 0; i < ts.catchClauses.size(); i++) {
/*      */             try {
/* 1751 */               this.codeContext.saveLocalVariables();
/*      */               
/* 1753 */               Java.CatchClause cc = (Java.CatchClause)ts.catchClauses.get(i);
/* 1754 */               IClass caughtExceptionType = getType(cc.caughtException.type);
/*      */               
/* 1756 */               Java.LocalVariableSlot exceptionVarSlot = this.codeContext.allocateLocalVariable((short)1, cc.caughtException.name, caughtExceptionType);
/*      */               
/*      */ 
/*      */ 
/*      */ 
/* 1761 */               short evi = exceptionVarSlot.getSlotIndex();
/*      */               
/*      */ 
/* 1764 */               getLocalVariable(cc.caughtException).setSlot(exceptionVarSlot);
/*      */               
/* 1766 */               this.codeContext.addExceptionTableEntry(beginningOfBody, afterBody, this.codeContext.newOffset(), caughtExceptionType.getDescriptor());
/*      */               
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1772 */               store(cc, caughtExceptionType, evi);
/*      */               
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1779 */               if (compile(cc.body)) {
/* 1780 */                 canCompleteNormally = true;
/* 1781 */                 if ((i < ts.catchClauses.size() - 1) || (ts.optionalFinally != null))
/*      */                 {
/*      */ 
/* 1784 */                   writeBranch(cc, -89, afterStatement);
/*      */                 }
/*      */               }
/*      */             }
/*      */             finally {}
/*      */           }
/*      */         }
/*      */         finally {}
/*      */       }
/*      */       
/*      */       CodeContext.Offset here;
/* 1795 */       if (ts.optionalFinally != null) {
/* 1796 */         here = this.codeContext.newOffset();
/* 1797 */         this.codeContext.addExceptionTableEntry(beginningOfBody, here, here, null);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1804 */         this.codeContext.saveLocalVariables();
/*      */         
/*      */         try
/*      */         {
/* 1808 */           short evi = this.codeContext.allocateLocalVariable((short)1);
/* 1809 */           store(ts.optionalFinally, this.iClassLoader.OBJECT, evi);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 1814 */           writeBranch(ts.optionalFinally, -88, ts.finallyOffset);
/* 1815 */           load(ts.optionalFinally, this.iClassLoader.OBJECT, evi);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 1820 */           writeOpcode(ts.optionalFinally, -65);
/*      */           
/*      */ 
/* 1823 */           ts.finallyOffset.set();
/* 1824 */           store(ts.optionalFinally, this.iClassLoader.OBJECT, pcLVIndex);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 1829 */           if (compile(ts.optionalFinally)) {
/* 1830 */             if (pcLVIndex > 255) {
/* 1831 */               writeOpcode(ts.optionalFinally, -60);
/* 1832 */               writeOpcode(ts.optionalFinally, -87);
/* 1833 */               writeShort(pcLVIndex);
/*      */             } else {
/* 1835 */               writeOpcode(ts.optionalFinally, -87);
/* 1836 */               writeByte(pcLVIndex);
/*      */             }
/*      */           }
/*      */         }
/*      */         finally {}
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1848 */       afterStatement.set();
/* 1849 */       if (canCompleteNormally) leave(ts, null);
/* 1850 */       return canCompleteNormally;
/*      */     } finally {
/* 1852 */       this.codeContext.restoreLocalVariables();
/*      */     }
/*      */   }
/*      */   
/*      */   private void compile(Java.FunctionDeclarator fd, ClassFile classFile)
/*      */     throws CompileException
/*      */   {
/*      */     ClassFile.MethodInfo mi;
/*      */     ClassFile.MethodInfo mi;
/* 1861 */     if (Mod.isPrivateAccess(fd.modifiers)) { ClassFile.MethodInfo mi;
/* 1862 */       if (((fd instanceof Java.MethodDeclarator)) && (!fd.isStatic()))
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1870 */         mi = classFile.addMethodInfo((short)(Mod.changeAccess(fd.modifiers, (short)0) | 0x8), fd.name + '$', MethodDescriptor.prependParameter(toIMethod((Java.MethodDeclarator)fd).getDescriptor(), resolve(fd.getDeclaringType()).getDescriptor()));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1884 */         mi = classFile.addMethodInfo(Mod.changeAccess(fd.modifiers, (short)0), fd.name, toIInvocable(fd).getDescriptor());
/*      */       }
/*      */       
/*      */ 
/*      */     }
/*      */     else
/*      */     {
/*      */ 
/* 1892 */       mi = classFile.addMethodInfo(fd.modifiers, fd.name, toIInvocable(fd).getDescriptor());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1901 */     if (fd.thrownExceptions.length > 0) {
/* 1902 */       short eani = classFile.addConstantUtf8Info("Exceptions");
/* 1903 */       short[] tecciis = new short[fd.thrownExceptions.length];
/* 1904 */       for (int i = 0; i < fd.thrownExceptions.length; i++) {
/* 1905 */         tecciis[i] = classFile.addConstantClassInfo(getType(fd.thrownExceptions[i]).getDescriptor());
/*      */       }
/* 1907 */       mi.addAttribute(new ClassFile.ExceptionsAttribute(eani, tecciis));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1912 */     if (fd.hasDeprecatedDocTag()) {
/* 1913 */       mi.addAttribute(new ClassFile.DeprecatedAttribute(classFile.addConstantUtf8Info("Deprecated")));
/*      */     }
/*      */     
/* 1916 */     if ((fd.modifiers & 0x500) != 0) { return;
/*      */     }
/*      */     
/* 1919 */     CodeContext codeContext = new CodeContext(mi.getClassFile());
/*      */     
/* 1921 */     CodeContext savedCodeContext = replaceCodeContext(codeContext);
/*      */     try {
/* 1923 */       this.codeContext.saveLocalVariables();
/*      */       
/*      */ 
/* 1926 */       if ((fd.modifiers & 0x8) == 0) {
/* 1927 */         this.codeContext.allocateLocalVariable((short)1, "this", resolve(fd.getDeclaringType()));
/*      */       }
/*      */       
/* 1930 */       if ((fd instanceof Java.ConstructorDeclarator)) {
/* 1931 */         Java.ConstructorDeclarator constructorDeclarator = (Java.ConstructorDeclarator)fd;
/*      */         
/*      */ 
/*      */ 
/* 1935 */         Iterator it = constructorDeclarator.getDeclaringClass().syntheticFields.values().iterator();
/* 1936 */         while (it.hasNext())
/*      */         {
/* 1938 */           IClass.IField sf = (IClass.IField)it.next();
/* 1939 */           Java.LocalVariable lv = new Java.LocalVariable(true, sf.getType());
/*      */           
/* 1941 */           lv.setSlot(this.codeContext.allocateLocalVariable(Descriptor.size(sf.getDescriptor()), null, null));
/* 1942 */           constructorDeclarator.syntheticParameters.put(sf.getName(), lv);
/*      */         }
/*      */       }
/*      */       
/* 1946 */       buildLocalVariableMap(fd);
/*      */       
/*      */ 
/* 1949 */       if ((fd instanceof Java.ConstructorDeclarator)) {
/* 1950 */         Java.ConstructorDeclarator cd = (Java.ConstructorDeclarator)fd;
/* 1951 */         if (cd.optionalConstructorInvocation != null) {
/* 1952 */           compile(cd.optionalConstructorInvocation);
/* 1953 */           if ((cd.optionalConstructorInvocation instanceof Java.SuperConstructorInvocation)) {
/* 1954 */             assignSyntheticParametersToSyntheticFields(cd);
/* 1955 */             initializeInstanceVariablesAndInvokeInstanceInitializers(cd);
/*      */           }
/*      */         }
/*      */         else
/*      */         {
/* 1960 */           Java.QualifiedThisReference qualification = null;
/* 1961 */           IClass outerClassOfSuperclass = resolve(cd.getDeclaringClass()).getSuperclass().getOuterIClass();
/*      */           
/*      */ 
/* 1964 */           if (outerClassOfSuperclass != null)
/*      */           {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1971 */             qualification = new Java.QualifiedThisReference(cd.getLocation(), new Java.SimpleType(cd.getLocation(), outerClassOfSuperclass));
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1981 */           Java.SuperConstructorInvocation sci = new Java.SuperConstructorInvocation(cd.getLocation(), qualification, new Java.Rvalue[0]);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 1986 */           sci.setEnclosingScope(fd);
/* 1987 */           compile(sci);
/* 1988 */           assignSyntheticParametersToSyntheticFields(cd);
/* 1989 */           initializeInstanceVariablesAndInvokeInstanceInitializers(cd);
/*      */         }
/*      */       }
/*      */       
/*      */       try
/*      */       {
/* 1995 */         if (fd.optionalStatements == null) {
/* 1996 */           compileError("Method must have a body", fd.getLocation()); return;
/*      */         }
/*      */         
/* 1999 */         if (compileStatements(fd.optionalStatements)) {
/* 2000 */           if (getReturnType(fd) != IClass.VOID) {
/* 2001 */             compileError("Method must return a value", fd.getLocation());
/*      */           }
/* 2003 */           writeOpcode(fd, -79);
/*      */         }
/*      */       } catch (RuntimeException ex) {
/* 2006 */         if (ex != STOP_COMPILING_CODE) { throw ex;
/*      */         }
/*      */         
/*      */       }
/*      */     }
/*      */     finally
/*      */     {
/* 2013 */       this.codeContext.restoreLocalVariables();
/* 2014 */       replaceCodeContext(savedCodeContext);
/*      */     }
/*      */     
/*      */ 
/* 2018 */     if (this.compileErrorCount > 0) { return;
/*      */     }
/*      */     
/* 2021 */     codeContext.fixUpAndRelocate();
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2032 */     codeContext.flowAnalysis(fd.toString());
/*      */     
/*      */     short lntani;
/*      */     short lntani;
/* 2036 */     if (this.debugLines) {
/* 2037 */       lntani = classFile.addConstantUtf8Info("LineNumberTable");
/*      */     } else {
/* 2039 */       lntani = 0;
/*      */     }
/*      */     short lvtani;
/*      */     short lvtani;
/* 2043 */     if (this.debugVars) {
/* 2044 */       makeLocalVariableNames(codeContext, mi);
/* 2045 */       lvtani = classFile.addConstantUtf8Info("LocalVariableTable");
/*      */     } else {
/* 2047 */       lvtani = 0;
/*      */     }
/*      */     
/*      */ 
/* 2051 */     mi.addAttribute(new ClassFile.AttributeInfo(classFile.addConstantUtf8Info("Code")) { private final CodeContext val$codeContext;
/*      */       
/* 2053 */       protected void storeBody(DataOutputStream dos) throws IOException { this.val$codeContext.storeCodeAttributeBody(dos, this.val$lntani, this.val$lvtani); }
/*      */     });
/*      */   }
/*      */   
/*      */ 
/*      */   private void makeLocalVariableNames(CodeContext cc, ClassFile.MethodInfo mi)
/*      */   {
/* 2060 */     ClassFile cf = mi.getClassFile();
/* 2061 */     Iterator iter = cc.getAllLocalVars().iterator();
/*      */     
/* 2063 */     cf.addConstantUtf8Info("LocalVariableTable");
/*      */     
/* 2065 */     while (iter.hasNext()) {
/* 2066 */       Java.LocalVariableSlot slot = (Java.LocalVariableSlot)iter.next();
/*      */       
/* 2068 */       if (slot.getName() != null) {
/* 2069 */         String typeName = slot.getType().getDescriptor();
/*      */         
/* 2071 */         cf.addConstantUtf8Info(typeName);
/* 2072 */         cf.addConstantUtf8Info(slot.getName());
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private void buildLocalVariableMap(Java.FunctionDeclarator fd) throws CompileException {
/* 2078 */     Map localVars = new HashMap();
/*      */     
/*      */ 
/* 2081 */     for (int i = 0; i < fd.formalParameters.length; i++) {
/* 2082 */       Java.FunctionDeclarator.FormalParameter fp = fd.formalParameters[i];
/* 2083 */       Java.LocalVariable lv = getLocalVariable(fp);
/* 2084 */       lv.setSlot(this.codeContext.allocateLocalVariable(Descriptor.size(lv.type.getDescriptor()), fp.name, getType(fp.type)));
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2090 */       if (localVars.put(fp.name, lv) != null) {
/* 2091 */         compileError("Redefinition of parameter \"" + fp.name + "\"", fd.getLocation());
/*      */       }
/*      */     }
/*      */     
/* 2095 */     fd.localVariables = localVars;
/* 2096 */     if ((fd instanceof Java.ConstructorDeclarator)) {
/* 2097 */       Java.ConstructorDeclarator cd = (Java.ConstructorDeclarator)fd;
/* 2098 */       if (cd.optionalConstructorInvocation != null)
/* 2099 */         buildLocalVariableMap(cd.optionalConstructorInvocation, localVars);
/*      */     }
/*      */     Iterator it;
/* 2102 */     if (fd.optionalStatements != null) {
/* 2103 */       for (it = fd.optionalStatements.iterator(); it.hasNext();) {
/* 2104 */         Java.BlockStatement bs = (Java.BlockStatement)it.next();
/* 2105 */         localVars = buildLocalVariableMap(bs, localVars);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private Map buildLocalVariableMap(Java.BlockStatement bs, Map localVars) throws CompileException {
/* 2111 */     Map[] resVars = { localVars };
/*      */     
/* 2113 */     Visitor.BlockStatementVisitor bsv = new Visitor.BlockStatementVisitor() {
/*      */       private final Map val$localVars;
/*      */       private final Map[] val$resVars;
/*      */       
/* 2117 */       public void visitAlternateConstructorInvocation(Java.AlternateConstructorInvocation aci) { UnitCompiler.this.buildLocalVariableMap(aci, this.val$localVars); }
/* 2118 */       public void visitBreakStatement(Java.BreakStatement bs) { UnitCompiler.this.buildLocalVariableMap(bs, this.val$localVars); }
/* 2119 */       public void visitContinueStatement(Java.ContinueStatement cs) { UnitCompiler.this.buildLocalVariableMap(cs, this.val$localVars); }
/* 2120 */       public void visitEmptyStatement(Java.EmptyStatement es) { UnitCompiler.this.buildLocalVariableMap(es, this.val$localVars); }
/* 2121 */       public void visitExpressionStatement(Java.ExpressionStatement es) { UnitCompiler.this.buildLocalVariableMap(es, this.val$localVars); }
/* 2122 */       public void visitFieldDeclaration(Java.FieldDeclaration fd) { UnitCompiler.this.buildLocalVariableMap(fd, this.val$localVars); }
/* 2123 */       public void visitReturnStatement(Java.ReturnStatement rs) { UnitCompiler.this.buildLocalVariableMap(rs, this.val$localVars); }
/* 2124 */       public void visitSuperConstructorInvocation(Java.SuperConstructorInvocation sci) { UnitCompiler.this.buildLocalVariableMap(sci, this.val$localVars); }
/* 2125 */       public void visitThrowStatement(Java.ThrowStatement ts) { UnitCompiler.this.buildLocalVariableMap(ts, this.val$localVars); }
/* 2126 */       public void visitLocalClassDeclarationStatement(Java.LocalClassDeclarationStatement lcds) { UnitCompiler.this.buildLocalVariableMap(lcds, this.val$localVars); }
/*      */       
/*      */       public void visitBlock(Java.Block b) {
/* 2129 */         try { UnitCompiler.this.buildLocalVariableMap(b, this.val$localVars); } catch (CompileException e) { throw new UnitCompiler.5UCE(UnitCompiler.this, e); } }
/* 2130 */       public void visitDoStatement(Java.DoStatement ds) { try { UnitCompiler.this.buildLocalVariableMap(ds, this.val$localVars); } catch (CompileException e) { throw new UnitCompiler.5UCE(UnitCompiler.this, e); } }
/* 2131 */       public void visitForStatement(Java.ForStatement fs) { try { UnitCompiler.this.buildLocalVariableMap(fs, this.val$localVars); } catch (CompileException e) { throw new UnitCompiler.5UCE(UnitCompiler.this, e); } }
/* 2132 */       public void visitIfStatement(Java.IfStatement is) { try { UnitCompiler.this.buildLocalVariableMap(is, this.val$localVars); } catch (CompileException e) { throw new UnitCompiler.5UCE(UnitCompiler.this, e); } }
/* 2133 */       public void visitInitializer(Java.Initializer i) { try { UnitCompiler.this.buildLocalVariableMap(i, this.val$localVars); } catch (CompileException e) { throw new UnitCompiler.5UCE(UnitCompiler.this, e); } }
/* 2134 */       public void visitSwitchStatement(Java.SwitchStatement ss) { try { UnitCompiler.this.buildLocalVariableMap(ss, this.val$localVars); } catch (CompileException e) { throw new UnitCompiler.5UCE(UnitCompiler.this, e); } }
/* 2135 */       public void visitSynchronizedStatement(Java.SynchronizedStatement ss) { try { UnitCompiler.this.buildLocalVariableMap(ss, this.val$localVars); } catch (CompileException e) { throw new UnitCompiler.5UCE(UnitCompiler.this, e); } }
/* 2136 */       public void visitTryStatement(Java.TryStatement ts) { try { UnitCompiler.this.buildLocalVariableMap(ts, this.val$localVars); } catch (CompileException e) { throw new UnitCompiler.5UCE(UnitCompiler.this, e); } }
/* 2137 */       public void visitWhileStatement(Java.WhileStatement ws) { try { UnitCompiler.this.buildLocalVariableMap(ws, this.val$localVars); } catch (CompileException e) { throw new UnitCompiler.5UCE(UnitCompiler.this, e);
/*      */         } }
/*      */       
/* 2140 */       public void visitLabeledStatement(Java.LabeledStatement ls) { try { this.val$resVars[0] = UnitCompiler.this.buildLocalVariableMap(ls, this.val$localVars); } catch (CompileException e) { throw new UnitCompiler.5UCE(UnitCompiler.this, e); } }
/* 2141 */       public void visitLocalVariableDeclarationStatement(Java.LocalVariableDeclarationStatement lvds) { try { this.val$resVars[0] = UnitCompiler.this.buildLocalVariableMap(lvds, this.val$localVars); } catch (CompileException e) { throw new UnitCompiler.5UCE(UnitCompiler.this, e);
/*      */         }
/*      */       } };
/* 2144 */     try { bs.accept(bsv); } catch (5UCE uce) { throw uce.ce; }
/* 2145 */     return resVars[0];
/*      */   }
/*      */   
/* 2148 */   private Map buildLocalVariableMap(Java.Statement s, Map localVars) { return s.localVariables = localVars; }
/*      */   
/* 2150 */   private Map buildLocalVariableMap(Java.ConstructorInvocation ci, Map localVars) { return ci.localVariables = localVars; }
/*      */   
/*      */   private void buildLocalVariableMap(Java.Block block, Map localVars)
/*      */     throws CompileException
/*      */   {
/* 2155 */     block.localVariables = localVars;
/* 2156 */     for (Iterator it = block.statements.iterator(); it.hasNext();) {
/* 2157 */       Java.BlockStatement bs = (Java.BlockStatement)it.next();
/* 2158 */       localVars = buildLocalVariableMap(bs, localVars);
/*      */     }
/*      */   }
/*      */   
/* 2162 */   private void buildLocalVariableMap(Java.DoStatement ds, Map localVars) throws CompileException { ds.localVariables = localVars;
/* 2163 */     buildLocalVariableMap(ds.body, localVars);
/*      */   }
/*      */   
/* 2166 */   private void buildLocalVariableMap(Java.ForStatement fs, Map localVars) throws CompileException { Map inner = localVars;
/* 2167 */     if (fs.optionalInit != null) {
/* 2168 */       inner = buildLocalVariableMap(fs.optionalInit, localVars);
/*      */     }
/* 2170 */     fs.localVariables = inner;
/* 2171 */     buildLocalVariableMap(fs.body, inner);
/*      */   }
/*      */   
/* 2174 */   private void buildLocalVariableMap(Java.IfStatement is, Map localVars) throws CompileException { is.localVariables = localVars;
/* 2175 */     buildLocalVariableMap(is.thenStatement, localVars);
/* 2176 */     if (is.optionalElseStatement != null) {
/* 2177 */       buildLocalVariableMap(is.optionalElseStatement, localVars);
/*      */     }
/*      */   }
/*      */   
/* 2181 */   private void buildLocalVariableMap(Java.Initializer i, Map localVars) throws CompileException { buildLocalVariableMap(i.block, localVars); }
/*      */   
/*      */   private void buildLocalVariableMap(Java.SwitchStatement ss, Map localVars) throws CompileException {
/* 2184 */     ss.localVariables = localVars;
/* 2185 */     Map vars = localVars;
/* 2186 */     for (Iterator cases = ss.sbsgs.iterator(); cases.hasNext();) {
/* 2187 */       Java.SwitchStatement.SwitchBlockStatementGroup sbsg = (Java.SwitchStatement.SwitchBlockStatementGroup)cases.next();
/* 2188 */       for (stmts = sbsg.blockStatements.iterator(); stmts.hasNext();) {
/* 2189 */         Java.BlockStatement bs = (Java.BlockStatement)stmts.next();
/* 2190 */         vars = buildLocalVariableMap(bs, vars);
/*      */       }
/*      */     }
/*      */     Iterator stmts; }
/*      */   
/* 2195 */   private void buildLocalVariableMap(Java.SynchronizedStatement ss, Map localVars) throws CompileException { ss.localVariables = localVars;
/* 2196 */     buildLocalVariableMap(ss.body, localVars);
/*      */   }
/*      */   
/* 2199 */   private void buildLocalVariableMap(Java.TryStatement ts, Map localVars) throws CompileException { ts.localVariables = localVars;
/* 2200 */     buildLocalVariableMap(ts.body, localVars);
/* 2201 */     for (Iterator it = ts.catchClauses.iterator(); it.hasNext();) {
/* 2202 */       Java.CatchClause cc = (Java.CatchClause)it.next();
/* 2203 */       buildLocalVariableMap(cc, localVars);
/*      */     }
/* 2205 */     if (ts.optionalFinally != null)
/* 2206 */       buildLocalVariableMap(ts.optionalFinally, localVars);
/*      */   }
/*      */   
/*      */   private void buildLocalVariableMap(Java.WhileStatement ws, Map localVars) throws CompileException {
/* 2210 */     ws.localVariables = localVars;
/* 2211 */     buildLocalVariableMap(ws.body, localVars);
/*      */   }
/*      */   
/*      */   private Map buildLocalVariableMap(Java.LabeledStatement ls, Map localVars) throws CompileException {
/* 2215 */     ls.localVariables = localVars;
/* 2216 */     return buildLocalVariableMap(ls.body, localVars);
/*      */   }
/*      */   
/*      */   private Map buildLocalVariableMap(Java.LocalVariableDeclarationStatement lvds, Map localVars)
/*      */     throws CompileException
/*      */   {
/* 2222 */     Map newVars = new HashMap();
/* 2223 */     newVars.putAll(localVars);
/* 2224 */     for (int i = 0; i < lvds.variableDeclarators.length; i++) {
/* 2225 */       Java.VariableDeclarator vd = lvds.variableDeclarators[i];
/* 2226 */       Java.LocalVariable lv = getLocalVariable(lvds, vd);
/* 2227 */       if (newVars.put(vd.name, lv) != null) {
/* 2228 */         compileError("Redefinition of local variable \"" + vd.name + "\" ", vd.getLocation());
/*      */       }
/*      */     }
/* 2231 */     lvds.localVariables = newVars;
/* 2232 */     return newVars;
/*      */   }
/*      */   
/* 2235 */   protected void buildLocalVariableMap(Java.CatchClause cc, Map localVars) throws CompileException { Map vars = new HashMap();
/* 2236 */     vars.putAll(localVars);
/* 2237 */     Java.LocalVariable lv = getLocalVariable(cc.caughtException);
/* 2238 */     vars.put(cc.caughtException.name, lv);
/* 2239 */     buildLocalVariableMap(cc.body, vars);
/*      */   }
/*      */   
/*      */   public Java.LocalVariable getLocalVariable(Java.FunctionDeclarator.FormalParameter fp)
/*      */     throws CompileException
/*      */   {
/* 2245 */     if (fp.localVariable == null) {
/* 2246 */       fp.localVariable = new Java.LocalVariable(fp.finaL, getType(fp.type));
/*      */     }
/* 2248 */     return fp.localVariable;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void fakeCompile(Java.ArrayInitializerOrRvalue aior)
/*      */     throws CompileException
/*      */   {
/* 2257 */     if ((aior instanceof Java.Rvalue)) {
/* 2258 */       Java.Rvalue rv = (Java.Rvalue)aior;
/* 2259 */       fakeCompile(rv);
/*      */     }
/* 2261 */     if ((aior instanceof Java.ArrayInitializer)) {
/* 2262 */       Java.ArrayInitializer ai = (Java.ArrayInitializer)aior;
/* 2263 */       for (int i = 0; i < ai.values.length; i++) {
/* 2264 */         fakeCompile(ai.values[i]);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private void fakeCompile(Java.Rvalue rv)
/*      */     throws CompileException
/*      */   {
/* 2273 */     CodeContext savedCodeContext = replaceCodeContext(createDummyCodeContext());
/*      */     try {
/* 2275 */       compileContext(rv);
/* 2276 */       compileGet(rv);
/*      */     } finally {
/* 2278 */       replaceCodeContext(savedCodeContext);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void compile(Java.Rvalue rv)
/*      */     throws CompileException
/*      */   {
/* 2288 */     Visitor.RvalueVisitor rvv = new Visitor.RvalueVisitor() {
/*      */       public void visitArrayLength(Java.ArrayLength al) {
/* 2290 */         try { UnitCompiler.this.compile2(al); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2291 */       public void visitAssignment(Java.Assignment a) { try { UnitCompiler.this.compile2(a); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2292 */       public void visitUnaryOperation(Java.UnaryOperation uo) { try { UnitCompiler.this.compile2(uo); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2293 */       public void visitBinaryOperation(Java.BinaryOperation bo) { try { UnitCompiler.this.compile2(bo); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2294 */       public void visitCast(Java.Cast c) { try { UnitCompiler.this.compile2(c); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2295 */       public void visitClassLiteral(Java.ClassLiteral cl) { try { UnitCompiler.this.compile2(cl); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2296 */       public void visitConditionalExpression(Java.ConditionalExpression ce) { try { UnitCompiler.this.compile2(ce); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2297 */       public void visitCrement(Java.Crement c) { try { UnitCompiler.this.compile2(c); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2298 */       public void visitInstanceof(Java.Instanceof io) { try { UnitCompiler.this.compile2(io); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2299 */       public void visitMethodInvocation(Java.MethodInvocation mi) { try { UnitCompiler.this.compile2(mi); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2300 */       public void visitSuperclassMethodInvocation(Java.SuperclassMethodInvocation smi) { try { UnitCompiler.this.compile2(smi); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2301 */       public void visitLiteral(Java.Literal l) { try { UnitCompiler.this.compile2(l); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2302 */       public void visitNewAnonymousClassInstance(Java.NewAnonymousClassInstance naci) { try { UnitCompiler.this.compile2(naci); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2303 */       public void visitNewArray(Java.NewArray na) { try { UnitCompiler.this.compile2(na); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2304 */       public void visitNewInitializedArray(Java.NewInitializedArray nia) { try { UnitCompiler.this.compile2(nia); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2305 */       public void visitNewClassInstance(Java.NewClassInstance nci) { try { UnitCompiler.this.compile2(nci); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2306 */       public void visitParameterAccess(Java.ParameterAccess pa) { try { UnitCompiler.this.compile2(pa); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2307 */       public void visitQualifiedThisReference(Java.QualifiedThisReference qtr) { try { UnitCompiler.this.compile2(qtr); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2308 */       public void visitThisReference(Java.ThisReference tr) { try { UnitCompiler.this.compile2(tr); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/*      */       
/* 2310 */       public void visitAmbiguousName(Java.AmbiguousName an) { try { UnitCompiler.this.compile2(an); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2311 */       public void visitArrayAccessExpression(Java.ArrayAccessExpression aae) { try { UnitCompiler.this.compile2(aae); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2312 */       public void visitFieldAccess(Java.FieldAccess fa) { try { UnitCompiler.this.compile2(fa); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2313 */       public void visitFieldAccessExpression(Java.FieldAccessExpression fae) { try { UnitCompiler.this.compile2(fae); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2314 */       public void visitSuperclassFieldAccessExpression(Java.SuperclassFieldAccessExpression scfae) { try { UnitCompiler.this.compile2(scfae); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2315 */       public void visitLocalVariableAccess(Java.LocalVariableAccess lva) { try { UnitCompiler.this.compile2(lva); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e); } }
/* 2316 */       public void visitParenthesizedExpression(Java.ParenthesizedExpression pe) { try { UnitCompiler.this.compile2(pe); } catch (CompileException e) { throw new UnitCompiler.6UCE(UnitCompiler.this, e);
/*      */         }
/*      */       }
/*      */     };
/* 2320 */     try { rv.accept(rvv);
/*      */     } catch (6UCE uce) {
/* 2322 */       throw uce.ce;
/*      */     }
/*      */   }
/*      */   
/* 2326 */   private void compile2(Java.Rvalue rv) throws CompileException { pop(rv, compileGetValue(rv)); }
/*      */   
/*      */   private void compile2(Java.Assignment a) throws CompileException {
/* 2329 */     if (a.operator == "=") {
/* 2330 */       compileContext(a.lhs);
/* 2331 */       assignmentConversion(a, compileGetValue(a.rhs), getType(a.lhs), getConstantValue(a.rhs));
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2337 */       compileSet(a.lhs);
/* 2338 */       return;
/*      */     }
/*      */     
/*      */ 
/* 2342 */     int lhsCS = compileContext(a.lhs);
/* 2343 */     dup(a, lhsCS);
/* 2344 */     IClass lhsType = compileGet(a.lhs);
/* 2345 */     IClass resultType = compileArithmeticBinaryOperation(a, lhsType, a.operator.substring(0, a.operator.length() - 1).intern(), a.rhs);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2355 */     if ((!tryIdentityConversion(resultType, lhsType)) && (!tryNarrowingPrimitiveConversion(a, resultType, lhsType)))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2362 */       throw new JaninoRuntimeException("SNO: \"" + a.operator + "\" reconversion failed"); }
/* 2363 */     compileSet(a.lhs);
/*      */   }
/*      */   
/*      */   private void compile2(Java.Crement c) throws CompileException
/*      */   {
/* 2368 */     Java.LocalVariable lv = isIntLV(c);
/* 2369 */     if (lv != null) {
/* 2370 */       compileLocalVariableCrement(c, lv);
/* 2371 */       return;
/*      */     }
/*      */     
/* 2374 */     int cs = compileContext(c.operand);
/* 2375 */     dup(c, cs);
/* 2376 */     IClass type = compileGet(c.operand);
/* 2377 */     IClass promotedType = unaryNumericPromotion(c, type);
/* 2378 */     writeOpcode(c, ilfd(promotedType, 4, 10, 12, 15));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2385 */     if (c.operator == "++") {
/* 2386 */       writeOpcode(c, 96 + ilfd(promotedType));
/*      */     }
/* 2388 */     else if (c.operator == "--") {
/* 2389 */       writeOpcode(c, 100 + ilfd(promotedType));
/*      */     } else {
/* 2391 */       compileError("Unexpected operator \"" + c.operator + "\"", c.getLocation());
/*      */     }
/*      */     
/* 2394 */     reverseUnaryNumericPromotion(c, promotedType, type);
/* 2395 */     compileSet(c.operand);
/*      */   }
/*      */   
/* 2398 */   private void compile2(Java.ParenthesizedExpression pe) throws CompileException { compile(pe.value); }
/*      */   
/*      */   private boolean compile2(Java.AlternateConstructorInvocation aci) throws CompileException
/*      */   {
/* 2402 */     Java.ConstructorDeclarator declaringConstructor = (Java.ConstructorDeclarator)aci.getEnclosingScope();
/* 2403 */     IClass declaringIClass = resolve(declaringConstructor.getDeclaringClass());
/*      */     
/* 2405 */     writeOpcode(aci, 42);
/* 2406 */     if (declaringIClass.getOuterIClass() != null) writeOpcode(aci, 43);
/* 2407 */     invokeConstructor(aci, declaringConstructor, (Java.Rvalue)null, declaringIClass, aci.arguments);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2414 */     return true;
/*      */   }
/*      */   
/* 2417 */   private boolean compile2(Java.SuperConstructorInvocation sci) throws CompileException { Java.ConstructorDeclarator declaringConstructor = (Java.ConstructorDeclarator)sci.getEnclosingScope();
/* 2418 */     writeOpcode(sci, 42);
/* 2419 */     Java.ClassDeclaration declaringClass = declaringConstructor.getDeclaringClass();
/* 2420 */     IClass superclass = resolve(declaringClass).getSuperclass();
/*      */     Java.Rvalue optionalEnclosingInstance;
/*      */     Java.Rvalue optionalEnclosingInstance;
/* 2423 */     if (sci.optionalQualification != null) {
/* 2424 */       optionalEnclosingInstance = sci.optionalQualification;
/*      */     } else {
/* 2426 */       IClass outerIClassOfSuperclass = superclass.getOuterIClass();
/* 2427 */       Java.Rvalue optionalEnclosingInstance; if (outerIClassOfSuperclass == null) {
/* 2428 */         optionalEnclosingInstance = null;
/*      */       } else {
/* 2430 */         optionalEnclosingInstance = new Java.QualifiedThisReference(sci.getLocation(), new Java.SimpleType(sci.getLocation(), outerIClassOfSuperclass));
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2437 */         optionalEnclosingInstance.setEnclosingBlockStatement(sci);
/*      */       }
/*      */     }
/* 2440 */     invokeConstructor(sci, declaringConstructor, optionalEnclosingInstance, superclass, sci.arguments);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2447 */     return true;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void compileBoolean(Java.Rvalue rv, CodeContext.Offset dst, boolean orientation)
/*      */     throws CompileException
/*      */   {
/* 2462 */     Visitor.RvalueVisitor rvv = new Visitor.RvalueVisitor() { private final CodeContext.Offset val$dst;
/*      */       
/* 2464 */       public void visitArrayLength(Java.ArrayLength al) { try { UnitCompiler.this.compileBoolean2(al, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2465 */       public void visitAssignment(Java.Assignment a) { try { UnitCompiler.this.compileBoolean2(a, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2466 */       public void visitUnaryOperation(Java.UnaryOperation uo) { try { UnitCompiler.this.compileBoolean2(uo, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2467 */       public void visitBinaryOperation(Java.BinaryOperation bo) { try { UnitCompiler.this.compileBoolean2(bo, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2468 */       public void visitCast(Java.Cast c) { try { UnitCompiler.this.compileBoolean2(c, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2469 */       public void visitClassLiteral(Java.ClassLiteral cl) { try { UnitCompiler.this.compileBoolean2(cl, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2470 */       public void visitConditionalExpression(Java.ConditionalExpression ce) { try { UnitCompiler.this.compileBoolean2(ce, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2471 */       public void visitCrement(Java.Crement c) { try { UnitCompiler.this.compileBoolean2(c, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2472 */       public void visitInstanceof(Java.Instanceof io) { try { UnitCompiler.this.compileBoolean2(io, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2473 */       public void visitMethodInvocation(Java.MethodInvocation mi) { try { UnitCompiler.this.compileBoolean2(mi, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2474 */       public void visitSuperclassMethodInvocation(Java.SuperclassMethodInvocation smi) { try { UnitCompiler.this.compileBoolean2(smi, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2475 */       public void visitLiteral(Java.Literal l) { try { UnitCompiler.this.compileBoolean2(l, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2476 */       public void visitNewAnonymousClassInstance(Java.NewAnonymousClassInstance naci) { try { UnitCompiler.this.compileBoolean2(naci, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2477 */       public void visitNewArray(Java.NewArray na) { try { UnitCompiler.this.compileBoolean2(na, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2478 */       public void visitNewInitializedArray(Java.NewInitializedArray nia) { try { UnitCompiler.this.compileBoolean2(nia, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2479 */       public void visitNewClassInstance(Java.NewClassInstance nci) { try { UnitCompiler.this.compileBoolean2(nci, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2480 */       public void visitParameterAccess(Java.ParameterAccess pa) { try { UnitCompiler.this.compileBoolean2(pa, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2481 */       public void visitQualifiedThisReference(Java.QualifiedThisReference qtr) { try { UnitCompiler.this.compileBoolean2(qtr, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2482 */       public void visitThisReference(Java.ThisReference tr) { try { UnitCompiler.this.compileBoolean2(tr, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/*      */       
/* 2484 */       public void visitAmbiguousName(Java.AmbiguousName an) { try { UnitCompiler.this.compileBoolean2(an, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2485 */       public void visitArrayAccessExpression(Java.ArrayAccessExpression aae) { try { UnitCompiler.this.compileBoolean2(aae, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2486 */       public void visitFieldAccess(Java.FieldAccess fa) { try { UnitCompiler.this.compileBoolean2(fa, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2487 */       public void visitFieldAccessExpression(Java.FieldAccessExpression fae) { try { UnitCompiler.this.compileBoolean2(fae, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2488 */       public void visitSuperclassFieldAccessExpression(Java.SuperclassFieldAccessExpression scfae) { try { UnitCompiler.this.compileBoolean2(scfae, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2489 */       public void visitLocalVariableAccess(Java.LocalVariableAccess lva) { try { UnitCompiler.this.compileBoolean2(lva, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e); } }
/* 2490 */       public void visitParenthesizedExpression(Java.ParenthesizedExpression pe) { try { UnitCompiler.this.compileBoolean2(pe, this.val$dst, this.val$orientation); } catch (CompileException e) { throw new UnitCompiler.7UCE(UnitCompiler.this, e);
/*      */         }
/*      */       }
/*      */     };
/* 2494 */     try { rv.accept(rvv);
/*      */     } catch (7UCE uce) {
/* 2496 */       throw uce.ce;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private void compileBoolean2(Java.Rvalue rv, CodeContext.Offset dst, boolean orientation)
/*      */     throws CompileException
/*      */   {
/* 2504 */     IClass type = compileGetValue(rv);
/* 2505 */     IClassLoader icl = this.iClassLoader;
/* 2506 */     if (type == icl.BOOLEAN) {
/* 2507 */       unboxingConversion(rv, icl.BOOLEAN, IClass.BOOLEAN);
/*      */     }
/* 2509 */     else if (type != IClass.BOOLEAN) {
/* 2510 */       compileError("Not a boolean expression", rv.getLocation());
/*      */     }
/* 2512 */     writeBranch(rv, orientation == true ? -102 : -103, dst);
/*      */   }
/*      */   
/*      */ 
/*      */   private void compileBoolean2(Java.UnaryOperation ue, CodeContext.Offset dst, boolean orientation)
/*      */     throws CompileException
/*      */   {
/* 2519 */     if (ue.operator == "!") {
/* 2520 */       compileBoolean(ue.operand, dst, !orientation);
/* 2521 */       return;
/*      */     }
/*      */     
/* 2524 */     compileError("Boolean expression expected", ue.getLocation());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void compileBoolean2(Java.BinaryOperation bo, CodeContext.Offset dst, boolean orientation)
/*      */     throws CompileException
/*      */   {
/* 2532 */     if ((bo.op == "|") || (bo.op == "^") || (bo.op == "&")) {
/* 2533 */       compileBoolean2(bo, dst, orientation);
/* 2534 */       return;
/*      */     }
/*      */     
/* 2537 */     if ((bo.op == "||") || (bo.op == "&&")) {
/* 2538 */       Object lhsCV = getConstantValue(bo.lhs);
/* 2539 */       if ((lhsCV instanceof Boolean)) {
/* 2540 */         if ((((Boolean)lhsCV).booleanValue() ^ bo.op == "||"))
/*      */         {
/* 2542 */           compileBoolean(bo.rhs, dst, 0x1 ^ (!orientation ? 1 : 0));
/*      */ 
/*      */ 
/*      */         }
/*      */         else
/*      */         {
/*      */ 
/* 2549 */           compileBoolean(bo.lhs, dst, 0x1 ^ (!orientation ? 1 : 0));
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 2554 */           fakeCompile(bo.rhs);
/*      */         }
/* 2556 */         return;
/*      */       }
/* 2558 */       Object rhsCV = getConstantValue(bo.rhs);
/* 2559 */       if ((rhsCV instanceof Boolean)) {
/* 2560 */         if ((((Boolean)rhsCV).booleanValue() ^ bo.op == "||"))
/*      */         {
/* 2562 */           compileBoolean(bo.lhs, dst, 0x1 ^ (!orientation ? 1 : 0));
/*      */ 
/*      */ 
/*      */         }
/*      */         else
/*      */         {
/*      */ 
/* 2569 */           pop(bo.lhs, compileGetValue(bo.lhs));
/* 2570 */           compileBoolean(bo.rhs, dst, 0x1 ^ (!orientation ? 1 : 0));
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/* 2576 */         return;
/*      */       }
/* 2578 */       if (((bo.op == "||" ? 1 : 0) ^ (!orientation ? 1 : 0)) != 0) {
/* 2579 */         compileBoolean(bo.lhs, dst, 0x1 ^ (!orientation ? 1 : 0));
/* 2580 */         compileBoolean(bo.rhs, dst, 0x1 ^ (!orientation ? 1 : 0));
/*      */       } else {
/* 2582 */         CodeContext tmp339_336 = this.codeContext;tmp339_336.getClass();CodeContext.Offset end = new CodeContext.Offset(tmp339_336);
/* 2583 */         compileBoolean(bo.lhs, end, 0x0 ^ (!orientation ? 1 : 0));
/* 2584 */         compileBoolean(bo.rhs, dst, 0x1 ^ (!orientation ? 1 : 0));
/* 2585 */         end.set();
/*      */       }
/* 2587 */       return;
/*      */     }
/*      */     
/* 2590 */     if ((bo.op == "==") || (bo.op == "!=") || (bo.op == "<=") || (bo.op == ">=") || (bo.op == "<") || (bo.op == ">"))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2598 */       int opIdx = bo.op == "<=" ? 5 : bo.op == ">" ? 4 : bo.op == ">=" ? 3 : bo.op == "<" ? 2 : bo.op == "!=" ? 1 : bo.op == "==" ? 0 : Integer.MIN_VALUE;
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2606 */       if (!orientation) { opIdx ^= 0x1;
/*      */       }
/*      */       
/*      */ 
/* 2610 */       boolean lhsIsNull = getConstantValue(bo.lhs) == Java.Rvalue.CONSTANT_VALUE_NULL;
/* 2611 */       boolean rhsIsNull = getConstantValue(bo.rhs) == Java.Rvalue.CONSTANT_VALUE_NULL;
/*      */       
/* 2613 */       if ((lhsIsNull) || (rhsIsNull)) {
/* 2614 */         if ((bo.op != "==") && (bo.op != "!=")) {
/* 2615 */           compileError("Operator \"" + bo.op + "\" not allowed on operand \"null\"", bo.getLocation());
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2622 */         if ((!lhsIsNull) || (!rhsIsNull)) {
/* 2623 */           IClass ohsType = compileGetValue(lhsIsNull ? bo.rhs : bo.lhs);
/* 2624 */           if (ohsType.isPrimitive()) {
/* 2625 */             compileError("Cannot compare \"null\" with primitive type \"" + ohsType.toString() + "\"", bo.getLocation());
/*      */           }
/*      */         }
/*      */         
/*      */ 
/*      */ 
/* 2631 */         writeBranch(bo, -58 + opIdx, dst);
/* 2632 */         return;
/*      */       }
/*      */       
/*      */ 
/* 2636 */       IClass lhsType = compileGetValue(bo.lhs);
/* 2637 */       CodeContext.Inserter convertLhsInserter = this.codeContext.newInserter();
/* 2638 */       IClass rhsType = compileGetValue(bo.rhs);
/*      */       
/*      */ 
/* 2641 */       if ((getUnboxedType(lhsType).isPrimitiveNumeric()) && (getUnboxedType(rhsType).isPrimitiveNumeric()) && (((bo.op != "==") && (bo.op != "!=")) || (lhsType.isPrimitive()) || (rhsType.isPrimitive())))
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/* 2646 */         IClass promotedType = binaryNumericPromotion(bo, lhsType, convertLhsInserter, rhsType);
/* 2647 */         if (promotedType == IClass.INT) {
/* 2648 */           writeBranch(bo, -97 + opIdx, dst);
/*      */         }
/* 2650 */         else if (promotedType == IClass.LONG) {
/* 2651 */           writeOpcode(bo, -108);
/* 2652 */           writeBranch(bo, -103 + opIdx, dst);
/*      */         }
/* 2654 */         else if (promotedType == IClass.FLOAT) {
/* 2655 */           if ((bo.op == ">") || (bo.op == ">=")) {
/* 2656 */             writeOpcode(bo, -107);
/*      */           } else {
/* 2658 */             writeOpcode(bo, -106);
/*      */           }
/* 2660 */           writeBranch(bo, -103 + opIdx, dst);
/*      */         }
/* 2662 */         else if (promotedType == IClass.DOUBLE) {
/* 2663 */           if ((bo.op == ">") || (bo.op == ">=")) {
/* 2664 */             writeOpcode(bo, -105);
/*      */           } else {
/* 2666 */             writeOpcode(bo, -104);
/*      */           }
/* 2668 */           writeBranch(bo, -103 + opIdx, dst);
/*      */         }
/*      */         else {
/* 2671 */           throw new JaninoRuntimeException("Unexpected promoted type \"" + promotedType + "\"");
/*      */         }
/* 2673 */         return;
/*      */       }
/*      */       
/*      */ 
/* 2677 */       if (((lhsType == IClass.BOOLEAN) && (getUnboxedType(rhsType) == IClass.BOOLEAN)) || ((rhsType == IClass.BOOLEAN) && (getUnboxedType(lhsType) == IClass.BOOLEAN)))
/*      */       {
/*      */ 
/*      */ 
/* 2681 */         if ((bo.op != "==") && (bo.op != "!=")) {
/* 2682 */           compileError("Operator \"" + bo.op + "\" not allowed on boolean operands", bo.getLocation());
/*      */         }
/* 2684 */         IClassLoader icl = this.iClassLoader;
/*      */         
/*      */ 
/* 2687 */         if (lhsType == icl.BOOLEAN) {
/* 2688 */           this.codeContext.pushInserter(convertLhsInserter);
/*      */           try {
/* 2690 */             unboxingConversion(bo, icl.BOOLEAN, IClass.BOOLEAN);
/*      */           } finally {
/* 2692 */             this.codeContext.popInserter();
/*      */           }
/*      */         }
/*      */         
/*      */ 
/* 2697 */         if (rhsType == icl.BOOLEAN) {
/* 2698 */           unboxingConversion(bo, icl.BOOLEAN, IClass.BOOLEAN);
/*      */         }
/*      */         
/* 2701 */         writeBranch(bo, -97 + opIdx, dst);
/* 2702 */         return;
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 2707 */       if ((!lhsType.isPrimitive()) && (!rhsType.isPrimitive()))
/*      */       {
/*      */ 
/*      */ 
/* 2711 */         if ((bo.op != "==") && (bo.op != "!=")) {
/* 2712 */           compileError("Operator \"" + bo.op + "\" not allowed on reference operands", bo.getLocation());
/*      */         }
/* 2714 */         writeBranch(bo, -91 + opIdx, dst);
/* 2715 */         return;
/*      */       }
/*      */       
/* 2718 */       compileError("Cannot compare types \"" + lhsType + "\" and \"" + rhsType + "\"", bo.getLocation());
/*      */     }
/*      */     
/* 2721 */     compileError("Boolean expression expected", bo.getLocation());
/*      */   }
/*      */   
/*      */ 
/*      */   private void compileBoolean2(Java.ParenthesizedExpression pe, CodeContext.Offset dst, boolean orientation)
/*      */     throws CompileException
/*      */   {
/* 2728 */     compileBoolean(pe.value, dst, orientation);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private int compileContext(Java.Rvalue rv)
/*      */     throws CompileException
/*      */   {
/* 2741 */     int[] res = new int[1];
/*      */     
/* 2743 */     Visitor.RvalueVisitor rvv = new Visitor.RvalueVisitor() { private final int[] val$res;
/*      */       
/* 2745 */       public void visitArrayLength(Java.ArrayLength al) { try { this.val$res[0] = UnitCompiler.this.compileContext2(al); } catch (CompileException e) { throw new UnitCompiler.8UCE(UnitCompiler.this, e); } }
/* 2746 */       public void visitAssignment(Java.Assignment a) { this.val$res[0] = UnitCompiler.this.compileContext2(a); }
/* 2747 */       public void visitUnaryOperation(Java.UnaryOperation uo) { this.val$res[0] = UnitCompiler.this.compileContext2(uo); }
/* 2748 */       public void visitBinaryOperation(Java.BinaryOperation bo) { this.val$res[0] = UnitCompiler.this.compileContext2(bo); }
/* 2749 */       public void visitCast(Java.Cast c) { this.val$res[0] = UnitCompiler.this.compileContext2(c); }
/* 2750 */       public void visitClassLiteral(Java.ClassLiteral cl) { this.val$res[0] = UnitCompiler.this.compileContext2(cl); }
/* 2751 */       public void visitConditionalExpression(Java.ConditionalExpression ce) { this.val$res[0] = UnitCompiler.this.compileContext2(ce); }
/* 2752 */       public void visitCrement(Java.Crement c) { this.val$res[0] = UnitCompiler.this.compileContext2(c); }
/* 2753 */       public void visitInstanceof(Java.Instanceof io) { this.val$res[0] = UnitCompiler.this.compileContext2(io); }
/* 2754 */       public void visitMethodInvocation(Java.MethodInvocation mi) { this.val$res[0] = UnitCompiler.this.compileContext2(mi); }
/* 2755 */       public void visitSuperclassMethodInvocation(Java.SuperclassMethodInvocation smi) { this.val$res[0] = UnitCompiler.this.compileContext2(smi); }
/* 2756 */       public void visitLiteral(Java.Literal l) { this.val$res[0] = UnitCompiler.this.compileContext2(l); }
/* 2757 */       public void visitNewAnonymousClassInstance(Java.NewAnonymousClassInstance naci) { this.val$res[0] = UnitCompiler.this.compileContext2(naci); }
/* 2758 */       public void visitNewArray(Java.NewArray na) { this.val$res[0] = UnitCompiler.this.compileContext2(na); }
/* 2759 */       public void visitNewInitializedArray(Java.NewInitializedArray nia) { this.val$res[0] = UnitCompiler.this.compileContext2(nia); }
/* 2760 */       public void visitNewClassInstance(Java.NewClassInstance nci) { this.val$res[0] = UnitCompiler.this.compileContext2(nci); }
/* 2761 */       public void visitParameterAccess(Java.ParameterAccess pa) { this.val$res[0] = UnitCompiler.this.compileContext2(pa); }
/* 2762 */       public void visitQualifiedThisReference(Java.QualifiedThisReference qtr) { this.val$res[0] = UnitCompiler.this.compileContext2(qtr); }
/* 2763 */       public void visitThisReference(Java.ThisReference tr) { this.val$res[0] = UnitCompiler.this.compileContext2(tr); }
/*      */       
/* 2765 */       public void visitAmbiguousName(Java.AmbiguousName an) { try { this.val$res[0] = UnitCompiler.this.compileContext2(an); } catch (CompileException e) { throw new UnitCompiler.8UCE(UnitCompiler.this, e); } }
/* 2766 */       public void visitArrayAccessExpression(Java.ArrayAccessExpression aae) { try { this.val$res[0] = UnitCompiler.this.compileContext2(aae); } catch (CompileException e) { throw new UnitCompiler.8UCE(UnitCompiler.this, e); } }
/* 2767 */       public void visitFieldAccess(Java.FieldAccess fa) { try { this.val$res[0] = UnitCompiler.this.compileContext2(fa); } catch (CompileException e) { throw new UnitCompiler.8UCE(UnitCompiler.this, e); } }
/* 2768 */       public void visitFieldAccessExpression(Java.FieldAccessExpression fae) { try { this.val$res[0] = UnitCompiler.this.compileContext2(fae); } catch (CompileException e) { throw new UnitCompiler.8UCE(UnitCompiler.this, e); } }
/* 2769 */       public void visitSuperclassFieldAccessExpression(Java.SuperclassFieldAccessExpression scfae) { try { this.val$res[0] = UnitCompiler.this.compileContext2(scfae); } catch (CompileException e) { throw new UnitCompiler.8UCE(UnitCompiler.this, e); } }
/* 2770 */       public void visitLocalVariableAccess(Java.LocalVariableAccess lva) { this.val$res[0] = UnitCompiler.this.compileContext2(lva); }
/* 2771 */       public void visitParenthesizedExpression(Java.ParenthesizedExpression pe) { try { this.val$res[0] = UnitCompiler.this.compileContext2(pe); } catch (CompileException e) { throw new UnitCompiler.8UCE(UnitCompiler.this, e);
/*      */         }
/*      */       }
/*      */     };
/* 2775 */     try { rv.accept(rvv);
/* 2776 */       return res[0];
/*      */     } catch (8UCE uce) {
/* 2778 */       throw uce.ce;
/*      */     }
/*      */   }
/*      */   
/* 2782 */   private int compileContext2(Java.Rvalue rv) { return 0; }
/*      */   
/*      */ 
/* 2785 */   private int compileContext2(Java.AmbiguousName an) throws CompileException { return compileContext(toRvalueOrCE(reclassify(an))); }
/*      */   
/*      */   private int compileContext2(Java.FieldAccess fa) throws CompileException {
/* 2788 */     if (fa.field.isStatic()) {
/* 2789 */       Java.Rvalue rv = fa.lhs.toRvalue();
/* 2790 */       if (rv != null) {
/* 2791 */         warning("CNSFA", "Left-hand side of static field access should be a type, not an rvalue", fa.lhs.getLocation());
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2797 */         pop(fa.lhs, compileGetValue(rv));
/*      */       }
/* 2799 */       return 0;
/*      */     }
/* 2801 */     compileGetValue(toRvalueOrCE(fa.lhs));
/* 2802 */     return 1;
/*      */   }
/*      */   
/*      */   private int compileContext2(Java.ArrayLength al) throws CompileException {
/* 2806 */     if (!compileGetValue(al.lhs).isArray()) {
/* 2807 */       compileError("Cannot determine length of non-array type", al.getLocation());
/*      */     }
/* 2809 */     return 1;
/*      */   }
/*      */   
/* 2812 */   private int compileContext2(Java.ArrayAccessExpression aae) throws CompileException { IClass lhsType = compileGetValue(aae.lhs);
/* 2813 */     if (!lhsType.isArray()) {
/* 2814 */       compileError("Subscript not allowed on non-array type \"" + lhsType.toString() + "\"", aae.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 2820 */     IClass indexType = compileGetValue(aae.index);
/* 2821 */     if ((!tryIdentityConversion(indexType, IClass.INT)) && (!tryWideningPrimitiveConversion(aae, indexType, IClass.INT)))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2828 */       compileError("Index expression of type \"" + indexType + "\" cannot be widened to \"int\"", aae.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 2833 */     return 2;
/*      */   }
/*      */   
/* 2836 */   private int compileContext2(Java.FieldAccessExpression fae) throws CompileException { determineValue(fae);
/* 2837 */     return compileContext(fae.value);
/*      */   }
/*      */   
/* 2840 */   private int compileContext2(Java.SuperclassFieldAccessExpression scfae) throws CompileException { determineValue(scfae);
/* 2841 */     return compileContext(scfae.value);
/*      */   }
/*      */   
/* 2844 */   private int compileContext2(Java.ParenthesizedExpression pe) throws CompileException { return compileContext(pe.value); }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass compileGet(Java.Rvalue rv)
/*      */     throws CompileException
/*      */   {
/* 2856 */     IClass[] res = new IClass[1];
/*      */     
/* 2858 */     Visitor.RvalueVisitor rvv = new Visitor.RvalueVisitor() { private final IClass[] val$res;
/*      */       
/* 2860 */       public void visitArrayLength(Java.ArrayLength al) { this.val$res[0] = UnitCompiler.this.compileGet2(al); }
/* 2861 */       public void visitAssignment(Java.Assignment a) { try { this.val$res[0] = UnitCompiler.this.compileGet2(a); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2862 */       public void visitUnaryOperation(Java.UnaryOperation uo) { try { this.val$res[0] = UnitCompiler.this.compileGet2(uo); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2863 */       public void visitBinaryOperation(Java.BinaryOperation bo) { try { this.val$res[0] = UnitCompiler.this.compileGet2(bo); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2864 */       public void visitCast(Java.Cast c) { try { this.val$res[0] = UnitCompiler.this.compileGet2(c); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2865 */       public void visitClassLiteral(Java.ClassLiteral cl) { try { this.val$res[0] = UnitCompiler.this.compileGet2(cl); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2866 */       public void visitConditionalExpression(Java.ConditionalExpression ce) { try { this.val$res[0] = UnitCompiler.this.compileGet2(ce); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2867 */       public void visitCrement(Java.Crement c) { try { this.val$res[0] = UnitCompiler.this.compileGet2(c); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2868 */       public void visitInstanceof(Java.Instanceof io) { try { this.val$res[0] = UnitCompiler.this.compileGet2(io); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2869 */       public void visitMethodInvocation(Java.MethodInvocation mi) { try { this.val$res[0] = UnitCompiler.this.compileGet2(mi); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2870 */       public void visitSuperclassMethodInvocation(Java.SuperclassMethodInvocation smi) { try { this.val$res[0] = UnitCompiler.this.compileGet2(smi); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2871 */       public void visitLiteral(Java.Literal l) { try { this.val$res[0] = UnitCompiler.this.compileGet2(l); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2872 */       public void visitNewAnonymousClassInstance(Java.NewAnonymousClassInstance naci) { try { this.val$res[0] = UnitCompiler.this.compileGet2(naci); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2873 */       public void visitNewArray(Java.NewArray na) { try { this.val$res[0] = UnitCompiler.this.compileGet2(na); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2874 */       public void visitNewInitializedArray(Java.NewInitializedArray nia) { try { this.val$res[0] = UnitCompiler.this.compileGet2(nia); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2875 */       public void visitNewClassInstance(Java.NewClassInstance nci) { try { this.val$res[0] = UnitCompiler.this.compileGet2(nci); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2876 */       public void visitParameterAccess(Java.ParameterAccess pa) { try { this.val$res[0] = UnitCompiler.this.compileGet2(pa); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2877 */       public void visitQualifiedThisReference(Java.QualifiedThisReference qtr) { try { this.val$res[0] = UnitCompiler.this.compileGet2(qtr); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2878 */       public void visitThisReference(Java.ThisReference tr) { try { this.val$res[0] = UnitCompiler.this.compileGet2(tr); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/*      */       
/* 2880 */       public void visitAmbiguousName(Java.AmbiguousName an) { try { this.val$res[0] = UnitCompiler.this.compileGet2(an); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2881 */       public void visitArrayAccessExpression(Java.ArrayAccessExpression aae) { try { this.val$res[0] = UnitCompiler.this.compileGet2(aae); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2882 */       public void visitFieldAccess(Java.FieldAccess fa) { try { this.val$res[0] = UnitCompiler.this.compileGet2(fa); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2883 */       public void visitFieldAccessExpression(Java.FieldAccessExpression fae) { try { this.val$res[0] = UnitCompiler.this.compileGet2(fae); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2884 */       public void visitSuperclassFieldAccessExpression(Java.SuperclassFieldAccessExpression scfae) { try { this.val$res[0] = UnitCompiler.this.compileGet2(scfae); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e); } }
/* 2885 */       public void visitLocalVariableAccess(Java.LocalVariableAccess lva) { this.val$res[0] = UnitCompiler.this.compileGet2(lva); }
/* 2886 */       public void visitParenthesizedExpression(Java.ParenthesizedExpression pe) { try { this.val$res[0] = UnitCompiler.this.compileGet2(pe); } catch (CompileException e) { throw new UnitCompiler.9UCE(UnitCompiler.this, e);
/*      */         }
/*      */       }
/*      */     };
/* 2890 */     try { rv.accept(rvv);
/* 2891 */       return res[0];
/*      */     } catch (9UCE uce) {
/* 2893 */       throw uce.ce;
/*      */     }
/*      */   }
/*      */   
/* 2897 */   private IClass compileGet2(Java.BooleanRvalue brv) throws CompileException { CodeContext tmp8_5 = this.codeContext;tmp8_5.getClass();CodeContext.Offset isTrue = new CodeContext.Offset(tmp8_5);
/* 2898 */     compileBoolean(brv, isTrue, true);
/* 2899 */     writeOpcode(brv, 3); CodeContext 
/* 2900 */       tmp38_35 = this.codeContext;tmp38_35.getClass();CodeContext.Offset end = new CodeContext.Offset(tmp38_35);
/* 2901 */     writeBranch(brv, -89, end);
/* 2902 */     isTrue.set();
/* 2903 */     writeOpcode(brv, 4);
/* 2904 */     end.set();
/*      */     
/* 2906 */     return IClass.BOOLEAN;
/*      */   }
/*      */   
/* 2909 */   private IClass compileGet2(Java.AmbiguousName an) throws CompileException { return compileGet(toRvalueOrCE(reclassify(an))); }
/*      */   
/*      */ 
/* 2912 */   private IClass compileGet2(Java.LocalVariableAccess lva) { return load(lva, lva.localVariable); }
/*      */   
/*      */   private IClass compileGet2(Java.FieldAccess fa) throws CompileException {
/* 2915 */     checkAccessible(fa.field, fa.getEnclosingBlockStatement());
/* 2916 */     if (fa.field.isStatic()) {
/* 2917 */       writeOpcode(fa, -78);
/*      */     } else {
/* 2919 */       writeOpcode(fa, -76);
/*      */     }
/* 2921 */     writeConstantFieldrefInfo(fa.field.getDeclaringIClass().getDescriptor(), fa.field.getName(), fa.field.getType().getDescriptor());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 2926 */     return fa.field.getType();
/*      */   }
/*      */   
/* 2929 */   private IClass compileGet2(Java.ArrayLength al) { writeOpcode(al, -66);
/* 2930 */     return IClass.INT;
/*      */   }
/*      */   
/* 2933 */   private IClass compileGet2(Java.ThisReference tr) throws CompileException { referenceThis(tr);
/* 2934 */     return getIClass(tr);
/*      */   }
/*      */   
/* 2937 */   private IClass compileGet2(Java.QualifiedThisReference qtr) throws CompileException { referenceThis(qtr, getDeclaringClass(qtr), getDeclaringTypeBodyDeclaration(qtr), getTargetIClass(qtr));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2943 */     return getTargetIClass(qtr);
/*      */   }
/*      */   
/* 2946 */   private IClass compileGet2(Java.ClassLiteral cl) throws CompileException { Location loc = cl.getLocation();
/* 2947 */     IClassLoader icl = this.iClassLoader;
/* 2948 */     IClass iClass = getType(cl.type);
/*      */     
/* 2950 */     if (iClass.isPrimitive())
/*      */     {
/*      */ 
/* 2953 */       writeOpcode(cl, -78);
/* 2954 */       String wrapperClassDescriptor = iClass == IClass.BOOLEAN ? "Ljava/lang/Boolean;" : iClass == IClass.SHORT ? "Ljava/lang/Short;" : iClass == IClass.LONG ? "Ljava/lang/Long;" : iClass == IClass.INT ? "Ljava/lang/Integer;" : iClass == IClass.FLOAT ? "Ljava/lang/Float;" : iClass == IClass.DOUBLE ? "Ljava/lang/Double;" : iClass == IClass.CHAR ? "Ljava/lang/Character;" : iClass == IClass.BYTE ? "Ljava/lang/Byte;" : iClass == IClass.VOID ? "Ljava/lang/Void;" : null;
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2966 */       if (wrapperClassDescriptor == null) {
/* 2967 */         throw new JaninoRuntimeException("SNO: Unidentifiable primitive type \"" + iClass + "\"");
/*      */       }
/*      */       
/* 2970 */       writeConstantFieldrefInfo(wrapperClassDescriptor, "TYPE", "Ljava/lang/Class;");
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 2975 */       return icl.CLASS;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 2981 */     for (Java.Scope s = cl.getEnclosingBlockStatement();; s = s.getEnclosingScope()) {
/* 2982 */       if ((s instanceof Java.TypeDeclaration)) {
/* 2983 */         Java.TypeDeclaration declaringType = (Java.AbstractTypeDeclaration)s;
/* 2984 */         break;
/*      */       }
/*      */     }
/*      */     
/*      */     Java.TypeDeclaration declaringType;
/*      */     
/* 2990 */     if (declaringType.getMethodDeclaration("class$") == null) { declareClassDollarMethod(cl);
/*      */     }
/*      */     
/*      */     List statics;
/*      */     
/* 2995 */     if ((declaringType instanceof Java.ClassDeclaration)) {
/* 2996 */       statics = ((Java.ClassDeclaration)declaringType).variableDeclaratorsAndInitializers;
/*      */     } else { List statics;
/* 2998 */       if ((declaringType instanceof Java.InterfaceDeclaration)) {
/* 2999 */         statics = ((Java.InterfaceDeclaration)declaringType).constantDeclarations;
/*      */       } else {
/* 3001 */         throw new JaninoRuntimeException("SNO: AbstractTypeDeclaration is neither ClassDeclaration nor InterfaceDeclaration");
/*      */       }
/*      */     }
/*      */     
/*      */     List statics;
/* 3006 */     String className = Descriptor.toClassName(iClass.getDescriptor());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     String classDollarFieldName;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3018 */     if (className.startsWith("[")) {
/* 3019 */       String classDollarFieldName = "array" + className.replace('.', '$').replace('[', '$');
/* 3020 */       if (classDollarFieldName.endsWith(";")) {
/* 3021 */         classDollarFieldName = classDollarFieldName.substring(0, classDollarFieldName.length() - 1);
/*      */       }
/*      */     }
/*      */     else {
/* 3025 */       classDollarFieldName = "class$" + className.replace('.', '$');
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 3031 */     boolean hasClassDollarField = false;
/* 3032 */     for (Iterator it = statics.iterator(); it.hasNext();) {
/* 3033 */       Java.TypeBodyDeclaration tbd = (Java.TypeBodyDeclaration)it.next();
/* 3034 */       if (tbd.isStatic())
/* 3035 */         if ((tbd instanceof Java.FieldDeclaration)) {
/* 3036 */           Java.FieldDeclaration fd = (Java.FieldDeclaration)tbd;
/* 3037 */           IClass.IField[] fds = getIFields(fd);
/* 3038 */           for (int j = 0; j < fds.length; j++)
/* 3039 */             if (fds[j].getName().equals(classDollarFieldName)) {
/* 3040 */               hasClassDollarField = true;
/*      */               break label556;
/*      */             }
/*      */         }
/*      */     }
/*      */     label556:
/* 3046 */     if (!hasClassDollarField) {
/* 3047 */       Java.Type classType = new Java.SimpleType(loc, icl.CLASS);
/* 3048 */       Java.FieldDeclaration fd = new Java.FieldDeclaration(loc, null, (short)8, classType, new Java.VariableDeclarator[] { new Java.VariableDeclarator(loc, classDollarFieldName, 0, (Java.Rvalue)null) });
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3062 */       if ((declaringType instanceof Java.ClassDeclaration)) {
/* 3063 */         ((Java.ClassDeclaration)declaringType).addVariableDeclaratorOrInitializer(fd);
/*      */       }
/* 3065 */       else if ((declaringType instanceof Java.InterfaceDeclaration)) {
/* 3066 */         ((Java.InterfaceDeclaration)declaringType).addConstantDeclaration(fd);
/*      */       } else {
/* 3068 */         throw new JaninoRuntimeException("SNO: AbstractTypeDeclaration is neither ClassDeclaration nor InterfaceDeclaration");
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3076 */     Java.Type declaringClassOrInterfaceType = new Java.SimpleType(loc, resolve(declaringType));
/* 3077 */     Java.Lvalue classDollarFieldAccess = new Java.FieldAccessExpression(loc, declaringClassOrInterfaceType, classDollarFieldName);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 3082 */     Java.ConditionalExpression ce = new Java.ConditionalExpression(loc, new Java.BinaryOperation(loc, classDollarFieldAccess, "!=", new Java.Literal(loc, null)), classDollarFieldAccess, new Java.Assignment(loc, classDollarFieldAccess, "=", new Java.MethodInvocation(loc, declaringClassOrInterfaceType, "class$", new Java.Rvalue[] { new Java.Literal(loc, className) })));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3108 */     ce.setEnclosingBlockStatement(cl.getEnclosingBlockStatement());
/* 3109 */     return compileGet(ce);
/*      */   }
/*      */   
/* 3112 */   private IClass compileGet2(Java.Assignment a) throws CompileException { if (a.operator == "=") {
/* 3113 */       int lhsCS = compileContext(a.lhs);
/* 3114 */       IClass rhsType = compileGetValue(a.rhs);
/* 3115 */       IClass lhsType = getType(a.lhs);
/* 3116 */       Object rhsCV = getConstantValue(a.rhs);
/* 3117 */       assignmentConversion(a, rhsType, lhsType, rhsCV);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3123 */       dupx(a, lhsType, lhsCS);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 3128 */       compileSet(a.lhs);
/* 3129 */       return lhsType;
/*      */     }
/*      */     
/*      */ 
/* 3133 */     int lhsCS = compileContext(a.lhs);
/* 3134 */     dup(a, lhsCS);
/* 3135 */     IClass lhsType = compileGet(a.lhs);
/* 3136 */     IClass resultType = compileArithmeticBinaryOperation(a, lhsType, a.operator.substring(0, a.operator.length() - 1).intern(), a.rhs);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3146 */     if ((!tryIdentityConversion(resultType, lhsType)) && (!tryNarrowingPrimitiveConversion(a, resultType, lhsType)))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3153 */       throw new JaninoRuntimeException("SNO: \"" + a.operator + "\" reconversion failed"); }
/* 3154 */     dupx(a, lhsType, lhsCS);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 3159 */     compileSet(a.lhs);
/* 3160 */     return lhsType;
/*      */   }
/*      */   
/*      */   private IClass compileGet2(Java.ConditionalExpression ce) throws CompileException
/*      */   {
/* 3165 */     CodeContext tmp8_5 = this.codeContext;tmp8_5.getClass();CodeContext.Offset toEnd = new CodeContext.Offset(tmp8_5);
/* 3166 */     Object cv = getConstantValue(ce.lhs);
/* 3167 */     CodeContext.Inserter rhsConvertInserter; IClass mhsType; CodeContext.Inserter mhsConvertInserter; IClass rhsType; CodeContext.Inserter rhsConvertInserter; if ((cv instanceof Boolean)) { CodeContext.Inserter rhsConvertInserter;
/* 3168 */       if (((Boolean)cv).booleanValue()) {
/* 3169 */         IClass mhsType = compileGetValue(ce.mhs);
/* 3170 */         CodeContext.Inserter mhsConvertInserter = this.codeContext.newInserter();
/* 3171 */         IClass rhsType = getType(ce.rhs);
/* 3172 */         rhsConvertInserter = null;
/*      */       } else {
/* 3174 */         IClass mhsType = getType(ce.mhs);
/* 3175 */         CodeContext.Inserter mhsConvertInserter = null;
/* 3176 */         IClass rhsType = compileGetValue(ce.rhs);
/* 3177 */         rhsConvertInserter = this.codeContext.currentInserter();
/*      */       }
/*      */     } else {
/* 3180 */       CodeContext tmp121_118 = this.codeContext;tmp121_118.getClass();CodeContext.Offset toRhs = new CodeContext.Offset(tmp121_118);
/*      */       
/* 3182 */       compileBoolean(ce.lhs, toRhs, false);
/* 3183 */       mhsType = compileGetValue(ce.mhs);
/* 3184 */       mhsConvertInserter = this.codeContext.newInserter();
/* 3185 */       writeBranch(ce, -89, toEnd);
/* 3186 */       toRhs.set();
/* 3187 */       rhsType = compileGetValue(ce.rhs);
/* 3188 */       rhsConvertInserter = this.codeContext.currentInserter();
/*      */     }
/*      */     
/*      */     IClass expressionType;
/* 3192 */     if (mhsType == rhsType)
/*      */     {
/*      */ 
/* 3195 */       expressionType = mhsType;
/*      */     } else { IClass expressionType;
/* 3197 */       if ((mhsType.isPrimitiveNumeric()) && (rhsType.isPrimitiveNumeric()))
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3206 */         expressionType = binaryNumericPromotion(ce, mhsType, mhsConvertInserter, rhsType, rhsConvertInserter);
/*      */       }
/*      */       else
/*      */       {
/*      */         IClass expressionType;
/*      */         
/*      */ 
/*      */ 
/* 3214 */         if ((getConstantValue(ce.mhs) == Java.Rvalue.CONSTANT_VALUE_NULL) && (!rhsType.isPrimitive()))
/*      */         {
/*      */ 
/* 3217 */           expressionType = rhsType;
/*      */         } else { IClass expressionType;
/* 3219 */           if ((!mhsType.isPrimitive()) && (getConstantValue(ce.rhs) == Java.Rvalue.CONSTANT_VALUE_NULL))
/*      */           {
/*      */ 
/* 3222 */             expressionType = mhsType;
/*      */           }
/* 3224 */           else if ((!mhsType.isPrimitive()) && (!rhsType.isPrimitive())) { IClass expressionType;
/* 3225 */             if (mhsType.isAssignableFrom(rhsType)) {
/* 3226 */               expressionType = mhsType;
/*      */             } else { IClass expressionType;
/* 3228 */               if (rhsType.isAssignableFrom(mhsType)) {
/* 3229 */                 expressionType = rhsType;
/*      */               } else {
/* 3231 */                 compileError("Reference types \"" + mhsType + "\" and \"" + rhsType + "\" don't match", ce.getLocation());
/*      */                 
/*      */ 
/*      */ 
/* 3235 */                 return this.iClassLoader.OBJECT;
/*      */               }
/*      */             }
/*      */           } else {
/* 3239 */             compileError("Incompatible expression types \"" + mhsType + "\" and \"" + rhsType + "\"", ce.getLocation());
/*      */             
/*      */ 
/*      */ 
/* 3243 */             return this.iClassLoader.OBJECT; } } } }
/*      */     IClass expressionType;
/* 3245 */     toEnd.set();
/*      */     
/* 3247 */     return expressionType;
/*      */   }
/*      */   
/*      */   private IClass compileGet2(Java.Crement c) throws CompileException
/*      */   {
/* 3252 */     Java.LocalVariable lv = isIntLV(c);
/* 3253 */     if (lv != null) {
/* 3254 */       if (!c.pre) load(c, lv);
/* 3255 */       compileLocalVariableCrement(c, lv);
/* 3256 */       if (c.pre) load(c, lv);
/* 3257 */       return lv.type;
/*      */     }
/*      */     
/*      */ 
/* 3261 */     int cs = compileContext(c.operand);
/*      */     
/* 3263 */     dup(c, cs);
/*      */     
/* 3265 */     IClass type = compileGet(c.operand);
/*      */     
/* 3267 */     if (!c.pre) { dupx(c, type, cs);
/*      */     }
/* 3269 */     IClass promotedType = unaryNumericPromotion(c, type);
/*      */     
/* 3271 */     writeOpcode(c, ilfd(promotedType, 4, 10, 12, 15));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3278 */     if (c.operator == "++") {
/* 3279 */       writeOpcode(c, 96 + ilfd(promotedType));
/*      */     }
/* 3281 */     else if (c.operator == "--") {
/* 3282 */       writeOpcode(c, 100 + ilfd(promotedType));
/*      */     } else {
/* 3284 */       compileError("Unexpected operator \"" + c.operator + "\"", c.getLocation());
/*      */     }
/* 3286 */     reverseUnaryNumericPromotion(c, promotedType, type);
/*      */     
/* 3288 */     if (c.pre) { dupx(c, type, cs);
/*      */     }
/* 3290 */     compileSet(c.operand);
/*      */     
/* 3292 */     return type;
/*      */   }
/*      */   
/* 3295 */   private void compileLocalVariableCrement(Java.Crement c, Java.LocalVariable lv) { if (lv.getSlotIndex() > 255) {
/* 3296 */       writeOpcode(c, -60);
/* 3297 */       writeOpcode(c, -124);
/* 3298 */       writeShort(lv.getSlotIndex());
/* 3299 */       writeShort(c.operator == "++" ? 1 : -1);
/*      */     } else {
/* 3301 */       writeOpcode(c, -124);
/* 3302 */       writeByte(lv.getSlotIndex());
/* 3303 */       writeByte(c.operator == "++" ? 1 : -1);
/*      */     }
/*      */   }
/*      */   
/* 3307 */   private IClass compileGet2(Java.ArrayAccessExpression aae) throws CompileException { IClass lhsComponentType = getType(aae);
/* 3308 */     writeOpcode(aae, 46 + ilfdabcs(lhsComponentType));
/* 3309 */     return lhsComponentType;
/*      */   }
/*      */   
/* 3312 */   private IClass compileGet2(Java.FieldAccessExpression fae) throws CompileException { determineValue(fae);
/* 3313 */     return compileGet(fae.value);
/*      */   }
/*      */   
/* 3316 */   private IClass compileGet2(Java.SuperclassFieldAccessExpression scfae) throws CompileException { determineValue(scfae);
/* 3317 */     return compileGet(scfae.value);
/*      */   }
/*      */   
/* 3320 */   private IClass compileGet2(Java.UnaryOperation uo) throws CompileException { if (uo.operator == "!") {
/* 3321 */       return compileGet2(uo);
/*      */     }
/*      */     
/* 3324 */     if (uo.operator == "+") {
/* 3325 */       return unaryNumericPromotion(uo, convertToPrimitiveNumericType(uo, compileGetValue(uo.operand)));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 3331 */     if (uo.operator == "-")
/*      */     {
/*      */ 
/* 3334 */       if ((uo.operand instanceof Java.Literal)) {
/* 3335 */         Java.Literal l = (Java.Literal)uo.operand;
/* 3336 */         pushConstant(uo, getNegatedConstantValue2(l));
/* 3337 */         return unaryNumericPromotion(uo, getType2(l));
/*      */       }
/*      */       
/* 3340 */       IClass promotedType = unaryNumericPromotion(uo, convertToPrimitiveNumericType(uo, compileGetValue(uo.operand)));
/*      */       
/*      */ 
/*      */ 
/* 3344 */       writeOpcode(uo, 116 + ilfd(promotedType));
/* 3345 */       return promotedType;
/*      */     }
/*      */     
/* 3348 */     if (uo.operator == "~") {
/* 3349 */       IClass operandType = compileGetValue(uo.operand);
/*      */       
/* 3351 */       IClass promotedType = unaryNumericPromotion(uo, operandType);
/* 3352 */       if (promotedType == IClass.INT) {
/* 3353 */         writeOpcode(uo, 2);
/* 3354 */         writeOpcode(uo, -126);
/* 3355 */         return IClass.INT;
/*      */       }
/* 3357 */       if (promotedType == IClass.LONG) {
/* 3358 */         writeOpcode(uo, 20);
/* 3359 */         writeConstantLongInfo(-1L);
/* 3360 */         writeOpcode(uo, -125);
/* 3361 */         return IClass.LONG;
/*      */       }
/* 3363 */       compileError("Operator \"~\" not applicable to type \"" + promotedType + "\"", uo.getLocation());
/*      */     }
/*      */     
/* 3366 */     compileError("Unexpected operator \"" + uo.operator + "\"", uo.getLocation());
/* 3367 */     return this.iClassLoader.OBJECT;
/*      */   }
/*      */   
/* 3370 */   private IClass compileGet2(Java.Instanceof io) throws CompileException { IClass lhsType = compileGetValue(io.lhs);
/* 3371 */     IClass rhsType = getType(io.rhs);
/* 3372 */     if ((lhsType.isInterface()) || (rhsType.isInterface()) || (lhsType.isAssignableFrom(rhsType)) || (rhsType.isAssignableFrom(lhsType)))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3379 */       writeOpcode(io, -63);
/* 3380 */       writeConstantClassInfo(rhsType.getDescriptor());
/*      */     } else {
/* 3382 */       compileError("\"" + lhsType + "\" can never be an instance of \"" + rhsType + "\"", io.getLocation());
/*      */     }
/* 3384 */     return IClass.BOOLEAN;
/*      */   }
/*      */   
/* 3387 */   private IClass compileGet2(Java.BinaryOperation bo) throws CompileException { if ((bo.op == "||") || (bo.op == "&&") || (bo.op == "==") || (bo.op == "!=") || (bo.op == "<") || (bo.op == ">") || (bo.op == "<=") || (bo.op == ">="))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3398 */       return compileGet2(bo);
/*      */     }
/*      */     
/*      */ 
/* 3402 */     return compileArithmeticOperation(bo, null, bo.unrollLeftAssociation(), bo.op);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass compileGet2(Java.Cast c)
/*      */     throws CompileException
/*      */   {
/* 3412 */     IClass tt = getType(c.targetType);
/* 3413 */     IClass vt = compileGetValue(c.value);
/* 3414 */     if ((!tryIdentityConversion(vt, tt)) && (!tryWideningPrimitiveConversion(c, vt, tt)) && (!tryNarrowingPrimitiveConversion(c, vt, tt)) && (!tryWideningReferenceConversion(vt, tt)) && (!tryNarrowingReferenceConversion(c, vt, tt)) && (!tryBoxingConversion(c, vt, tt)) && (!tryUnboxingConversion(c, vt, tt)))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3422 */       compileError("Cannot cast \"" + vt + "\" to \"" + tt + "\"", c.getLocation()); }
/* 3423 */     return tt;
/*      */   }
/*      */   
/* 3426 */   private IClass compileGet2(Java.ParenthesizedExpression pe) throws CompileException { return compileGet(pe.value); }
/*      */   
/*      */   private IClass compileGet2(Java.MethodInvocation mi) throws CompileException {
/* 3429 */     IClass.IMethod iMethod = findIMethod(mi);
/*      */     
/* 3431 */     if (mi.optionalTarget == null)
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3439 */       Java.Scope s = mi.getEnclosingBlockStatement();
/* 3440 */       while (!(s instanceof Java.TypeBodyDeclaration)) {
/* 3441 */         s = s.getEnclosingScope();
/*      */       }
/* 3443 */       Java.TypeBodyDeclaration scopeTBD = (Java.TypeBodyDeclaration)s;
/* 3444 */       if (!(s instanceof Java.ClassDeclaration)) s = s.getEnclosingScope();
/* 3445 */       Java.ClassDeclaration scopeClassDeclaration = (Java.ClassDeclaration)s;
/*      */       
/* 3447 */       if (iMethod.isStatic()) {
/* 3448 */         warning("IASM", "Implicit access to static method \"" + iMethod.toString() + "\"", mi.getLocation());
/*      */ 
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/*      */ 
/*      */ 
/* 3456 */         warning("IANSM", "Implicit access to non-static method \"" + iMethod.toString() + "\"", mi.getLocation());
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3462 */         if (scopeTBD.isStatic()) {
/* 3463 */           compileError("Instance method \"" + iMethod.toString() + "\" cannot be invoked in static context", mi.getLocation());
/*      */         }
/*      */         
/*      */ 
/*      */ 
/* 3468 */         referenceThis(mi, scopeClassDeclaration, scopeTBD, iMethod.getDeclaringIClass());
/*      */ 
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */     }
/*      */     else
/*      */     {
/*      */ 
/* 3478 */       boolean staticContext = isType(mi.optionalTarget);
/* 3479 */       if (staticContext) {
/* 3480 */         getType(toTypeOrCE(mi.optionalTarget));
/*      */       }
/*      */       else {
/* 3483 */         compileGetValue(toRvalueOrCE(mi.optionalTarget));
/*      */       }
/* 3485 */       if (iMethod.isStatic()) {
/* 3486 */         if (!staticContext)
/*      */         {
/* 3488 */           pop(mi.optionalTarget, getType(mi.optionalTarget));
/*      */         }
/*      */       }
/* 3491 */       else if (staticContext) {
/* 3492 */         compileError("Instance method \"" + mi.methodName + "\" cannot be invoked in static context", mi.getLocation());
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3501 */     IClass[] parameterTypes = iMethod.getParameterTypes();
/* 3502 */     for (int i = 0; i < mi.arguments.length; i++) {
/* 3503 */       assignmentConversion(mi, compileGetValue(mi.arguments[i]), parameterTypes[i], getConstantValue(mi.arguments[i]));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3512 */     checkAccessible(iMethod, mi.getEnclosingBlockStatement());
/* 3513 */     if (iMethod.getDeclaringIClass().isInterface()) {
/* 3514 */       writeOpcode(mi, -71);
/* 3515 */       writeConstantInterfaceMethodrefInfo(iMethod.getDeclaringIClass().getDescriptor(), iMethod.getName(), iMethod.getDescriptor());
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 3520 */       IClass[] pts = iMethod.getParameterTypes();
/* 3521 */       int count = 1;
/* 3522 */       for (int i = 0; i < pts.length; i++) count += Descriptor.size(pts[i].getDescriptor());
/* 3523 */       writeByte(count);
/* 3524 */       writeByte(0);
/*      */     }
/* 3526 */     else if ((!iMethod.isStatic()) && (iMethod.getAccess() == Access.PRIVATE))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3536 */       writeOpcode(mi, -72);
/* 3537 */       writeConstantMethodrefInfo(iMethod.getDeclaringIClass().getDescriptor(), iMethod.getName() + '$', MethodDescriptor.prependParameter(iMethod.getDescriptor(), iMethod.getDeclaringIClass().getDescriptor()));
/*      */ 
/*      */ 
/*      */ 
/*      */     }
/*      */     else
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/* 3547 */       byte opcode = iMethod.isStatic() ? -72 : -74;
/* 3548 */       writeOpcode(mi, opcode);
/* 3549 */       writeConstantMethodrefInfo(iMethod.getDeclaringIClass().getDescriptor(), iMethod.getName(), iMethod.getDescriptor());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3556 */     return iMethod.getReturnType();
/*      */   }
/*      */   
/* 3559 */   private IClass compileGet2(Java.SuperclassMethodInvocation scmi) throws CompileException { IClass.IMethod iMethod = findIMethod(scmi);
/*      */     
/*      */ 
/*      */ 
/* 3563 */     Java.Scope s = scmi.getEnclosingBlockStatement();
/* 3564 */     while (((s instanceof Java.Statement)) || ((s instanceof Java.CatchClause))) {
/* 3565 */       s = s.getEnclosingScope();
/*      */     }
/* 3567 */     Java.FunctionDeclarator fd = (s instanceof Java.FunctionDeclarator) ? (Java.FunctionDeclarator)s : null;
/* 3568 */     if (fd == null) {
/* 3569 */       compileError("Cannot invoke superclass method in non-method scope", scmi.getLocation());
/* 3570 */       return IClass.INT;
/*      */     }
/* 3572 */     if ((fd.modifiers & 0x8) != 0) {
/* 3573 */       compileError("Cannot invoke superclass method in static context", scmi.getLocation());
/*      */     }
/* 3575 */     load(scmi, resolve(fd.getDeclaringType()), 0);
/*      */     
/*      */ 
/* 3578 */     IClass[] parameterTypes = iMethod.getParameterTypes();
/* 3579 */     for (int i = 0; i < scmi.arguments.length; i++) {
/* 3580 */       assignmentConversion(scmi, compileGetValue(scmi.arguments[i]), parameterTypes[i], getConstantValue(scmi.arguments[i]));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3589 */     writeOpcode(scmi, -73);
/* 3590 */     writeConstantMethodrefInfo(iMethod.getDeclaringIClass().getDescriptor(), scmi.methodName, iMethod.getDescriptor());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 3595 */     return iMethod.getReturnType();
/*      */   }
/*      */   
/* 3598 */   private IClass compileGet2(Java.NewClassInstance nci) throws CompileException { if (nci.iClass == null) { nci.iClass = getType(nci.type);
/*      */     }
/* 3600 */     writeOpcode(nci, -69);
/* 3601 */     writeConstantClassInfo(nci.iClass.getDescriptor());
/* 3602 */     writeOpcode(nci, 89);
/*      */     
/* 3604 */     if (nci.iClass.isInterface()) compileError("Cannot instantiate \"" + nci.iClass + "\"", nci.getLocation());
/* 3605 */     checkAccessible(nci.iClass, nci.getEnclosingBlockStatement());
/* 3606 */     if (nci.iClass.isAbstract()) {
/* 3607 */       compileError("Cannot instantiate abstract \"" + nci.iClass + "\"", nci.getLocation());
/*      */     }
/*      */     
/*      */     Java.Rvalue optionalEnclosingInstance;
/*      */     Java.Rvalue optionalEnclosingInstance;
/* 3612 */     if (nci.optionalQualification != null) {
/* 3613 */       if (nci.iClass.getOuterIClass() == null) {
/* 3614 */         compileError("Static member class cannot be instantiated with qualified NEW");
/*      */       }
/*      */       
/*      */ 
/* 3618 */       optionalEnclosingInstance = nci.optionalQualification;
/*      */     } else {
/* 3620 */       Java.Scope s = nci.getEnclosingBlockStatement();
/* 3621 */       while (!(s instanceof Java.TypeBodyDeclaration)) s = s.getEnclosingScope();
/* 3622 */       Java.TypeBodyDeclaration enclosingTypeBodyDeclaration = (Java.TypeBodyDeclaration)s;
/* 3623 */       Java.TypeDeclaration enclosingTypeDeclaration = (Java.TypeDeclaration)s.getEnclosingScope();
/*      */       Java.Rvalue optionalEnclosingInstance;
/* 3625 */       if ((!(enclosingTypeDeclaration instanceof Java.ClassDeclaration)) || (enclosingTypeBodyDeclaration.isStatic()))
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3634 */         if (nci.iClass.getOuterIClass() != null) {
/* 3635 */           compileError("Instantiation of \"" + nci.type + "\" requires an enclosing instance", nci.getLocation());
/*      */         }
/*      */         
/*      */ 
/*      */ 
/* 3640 */         optionalEnclosingInstance = null;
/*      */       }
/*      */       else
/*      */       {
/* 3644 */         IClass optionalOuterIClass = nci.iClass.getDeclaringIClass();
/* 3645 */         Java.Rvalue optionalEnclosingInstance; if (optionalOuterIClass == null)
/*      */         {
/*      */ 
/* 3648 */           optionalEnclosingInstance = null;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         }
/*      */         else
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3661 */           optionalEnclosingInstance = new Java.QualifiedThisReference(nci.getLocation(), new Java.SimpleType(nci.getLocation(), optionalOuterIClass));
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3668 */           optionalEnclosingInstance.setEnclosingBlockStatement(nci.getEnclosingBlockStatement());
/*      */         }
/*      */       }
/*      */     }
/*      */     
/* 3673 */     invokeConstructor(nci, nci.getEnclosingBlockStatement(), optionalEnclosingInstance, nci.iClass, nci.arguments);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3680 */     return nci.iClass;
/*      */   }
/*      */   
/*      */   private IClass compileGet2(Java.NewAnonymousClassInstance naci) throws CompileException
/*      */   {
/* 3685 */     Java.AnonymousClassDeclaration acd = naci.anonymousClassDeclaration;
/* 3686 */     IClass sc = resolve(acd).getSuperclass();
/* 3687 */     IClass.IConstructor[] iConstructors = sc.getDeclaredIConstructors();
/* 3688 */     if (iConstructors.length == 0) { throw new JaninoRuntimeException("SNO: Base class has no constructors");
/*      */     }
/*      */     
/* 3691 */     IClass.IConstructor iConstructor = (IClass.IConstructor)findMostSpecificIInvocable(naci, iConstructors, naci.arguments, acd);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3698 */     IClass[] pts = iConstructor.getParameterTypes();
/*      */     
/*      */ 
/*      */ 
/* 3702 */     Location loc = naci.getLocation();
/*      */     
/* 3704 */     List l = new ArrayList();
/*      */     
/*      */ 
/* 3707 */     if (naci.optionalQualification != null) { l.add(new Java.FunctionDeclarator.FormalParameter(loc, true, new Java.SimpleType(loc, getType(naci.optionalQualification)), "this$base"));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 3713 */     for (int i = 0; i < pts.length; i++) { l.add(new Java.FunctionDeclarator.FormalParameter(loc, true, new Java.SimpleType(loc, pts[i]), "p" + i));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 3719 */     Java.FunctionDeclarator.FormalParameter[] fps = (Java.FunctionDeclarator.FormalParameter[])l.toArray(new Java.FunctionDeclarator.FormalParameter[l.size()]);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3725 */     IClass[] tes = iConstructor.getThrownExceptions();
/* 3726 */     Java.Type[] tets = new Java.Type[tes.length];
/* 3727 */     for (int i = 0; i < tes.length; i++) { tets[i] = new Java.SimpleType(loc, tes[i]);
/*      */     }
/*      */     
/* 3730 */     int j = 0;
/*      */     Java.Rvalue optionalQualificationAccess;
/* 3732 */     Java.Rvalue optionalQualificationAccess; if (naci.optionalQualification == null) {
/* 3733 */       optionalQualificationAccess = null;
/*      */     }
/*      */     else {
/* 3736 */       optionalQualificationAccess = new Java.ParameterAccess(loc, fps[(j++)]);
/*      */     }
/* 3738 */     Java.Rvalue[] parameterAccesses = new Java.Rvalue[pts.length];
/* 3739 */     for (int i = 0; i < pts.length; i++) {
/* 3740 */       parameterAccesses[i] = new Java.ParameterAccess(loc, fps[(j++)]);
/*      */     }
/*      */     
/*      */ 
/* 3744 */     Java.ConstructorDeclarator anonymousConstructor = new Java.ConstructorDeclarator(loc, null, (short)0, fps, tets, new Java.SuperConstructorInvocation(loc, optionalQualificationAccess, parameterAccesses), Collections.EMPTY_LIST);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3759 */     acd.addConstructor(anonymousConstructor);
/*      */     try {
/* 3761 */       compile(acd);
/*      */       
/*      */ 
/* 3764 */       writeOpcode(naci, -69);
/* 3765 */       writeConstantClassInfo(resolve(naci.anonymousClassDeclaration).getDescriptor());
/*      */       
/*      */ 
/* 3768 */       writeOpcode(naci, 89);
/*      */       Java.Rvalue[] arguments2;
/* 3770 */       Java.Rvalue[] arguments2; if (naci.optionalQualification == null) {
/* 3771 */         arguments2 = naci.arguments;
/*      */       } else {
/* 3773 */         arguments2 = new Java.Rvalue[naci.arguments.length + 1];
/* 3774 */         arguments2[0] = naci.optionalQualification;
/* 3775 */         System.arraycopy(naci.arguments, 0, arguments2, 1, naci.arguments.length);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3782 */       Java.Scope s = naci.getEnclosingBlockStatement();
/* 3783 */       while (!(s instanceof Java.TypeBodyDeclaration))
/* 3784 */         s = s.getEnclosingScope();
/*      */       Java.ThisReference oei;
/*      */       Java.ThisReference oei;
/* 3787 */       if (((Java.TypeBodyDeclaration)s).isStatic()) {
/* 3788 */         oei = null;
/*      */       }
/*      */       else {
/* 3791 */         oei = new Java.ThisReference(loc);
/* 3792 */         oei.setEnclosingBlockStatement(naci.getEnclosingBlockStatement());
/*      */       }
/* 3794 */       invokeConstructor(naci, naci.getEnclosingBlockStatement(), oei, resolve(naci.anonymousClassDeclaration), arguments2);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     }
/*      */     finally
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3806 */       acd.constructors.remove(acd.constructors.size() - 1);
/*      */     }
/* 3808 */     return resolve(naci.anonymousClassDeclaration);
/*      */   }
/*      */   
/* 3811 */   private IClass compileGet2(Java.ParameterAccess pa) throws CompileException { Java.LocalVariable lv = getLocalVariable(pa.formalParameter);
/* 3812 */     load(pa, lv);
/* 3813 */     return lv.type;
/*      */   }
/*      */   
/* 3816 */   private IClass compileGet2(Java.NewArray na) throws CompileException { for (int i = 0; i < na.dimExprs.length; i++) {
/* 3817 */       IClass dimType = compileGetValue(na.dimExprs[i]);
/* 3818 */       if ((dimType != IClass.INT) && (unaryNumericPromotion(na, dimType) != IClass.INT))
/*      */       {
/*      */ 
/* 3821 */         compileError("Invalid array size expression type", na.getLocation());
/*      */       }
/*      */     }
/* 3824 */     return newArray(na, na.dimExprs.length, na.dims, getType(na.type));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private IClass compileGet2(Java.NewInitializedArray nia)
/*      */     throws CompileException
/*      */   {
/* 3832 */     IClass at = getType(nia.arrayType);
/* 3833 */     compileGetValue(nia.arrayInitializer, at);
/* 3834 */     return at;
/*      */   }
/*      */   
/* 3837 */   private void compileGetValue(Java.ArrayInitializer ai, IClass arrayType) throws CompileException { if (!arrayType.isArray()) {
/* 3838 */       compileError("Array initializer not allowed for non-array type \"" + arrayType.toString() + "\"");
/*      */     }
/* 3840 */     IClass ct = arrayType.getComponentType();
/*      */     
/* 3842 */     pushConstant(ai, new Integer(ai.values.length));
/* 3843 */     newArray(ai, 1, 0, ct);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 3850 */     for (int i = 0; i < ai.values.length; i++) {
/* 3851 */       writeOpcode(ai, 89);
/* 3852 */       pushConstant(ai, new Integer(i));
/* 3853 */       Java.ArrayInitializerOrRvalue aiorv = ai.values[i];
/* 3854 */       if ((aiorv instanceof Java.Rvalue)) {
/* 3855 */         Java.Rvalue rv = (Java.Rvalue)aiorv;
/* 3856 */         assignmentConversion(ai, compileGetValue(rv), ct, getConstantValue(rv));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       }
/* 3863 */       else if ((aiorv instanceof Java.ArrayInitializer)) {
/* 3864 */         compileGetValue((Java.ArrayInitializer)aiorv, ct);
/*      */       }
/*      */       else {
/* 3867 */         throw new JaninoRuntimeException("Unexpected array initializer or rvalue class " + aiorv.getClass().getName());
/*      */       }
/*      */       
/*      */ 
/* 3871 */       writeOpcode(ai, 79 + ilfdabcs(ct));
/*      */     }
/*      */   }
/*      */   
/* 3875 */   private IClass compileGet2(Java.Literal l) throws CompileException { if ((l.value == Scanner.MAGIC_INTEGER) || (l.value == Scanner.MAGIC_LONG))
/*      */     {
/*      */ 
/* 3878 */       compileError("This literal value may only appear in a negated context", l.getLocation()); }
/* 3879 */     return pushConstant(l, l.value == null ? Java.Rvalue.CONSTANT_VALUE_NULL : l.value);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass compileGetValue(Java.Rvalue rv)
/*      */     throws CompileException
/*      */   {
/* 3889 */     Object cv = getConstantValue(rv);
/* 3890 */     if (cv != null) {
/* 3891 */       fakeCompile(rv);
/* 3892 */       pushConstant(rv, cv);
/* 3893 */       return getType(rv);
/*      */     }
/*      */     
/* 3896 */     compileContext(rv);
/* 3897 */     return compileGet(rv);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final boolean val$orientation;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final Object getConstantValue(Java.Rvalue rv)
/*      */     throws CompileException
/*      */   {
/* 3922 */     if (rv.constantValue != Java.Rvalue.CONSTANT_VALUE_UNKNOWN) { return rv.constantValue;
/*      */     }
/* 3924 */     Object[] res = new Object[1];
/*      */     
/* 3926 */     Visitor.RvalueVisitor rvv = new Visitor.RvalueVisitor() { private final Object[] val$res;
/*      */       
/* 3928 */       public void visitArrayLength(Java.ArrayLength al) { this.val$res[0] = UnitCompiler.this.getConstantValue2(al); }
/* 3929 */       public void visitAssignment(Java.Assignment a) { this.val$res[0] = UnitCompiler.this.getConstantValue2(a); }
/* 3930 */       public void visitUnaryOperation(Java.UnaryOperation uo) { try { this.val$res[0] = UnitCompiler.this.getConstantValue2(uo); } catch (CompileException e) { throw new UnitCompiler.10UCE(UnitCompiler.this, e); } }
/* 3931 */       public void visitBinaryOperation(Java.BinaryOperation bo) { try { this.val$res[0] = UnitCompiler.this.getConstantValue2(bo); } catch (CompileException e) { throw new UnitCompiler.10UCE(UnitCompiler.this, e); } }
/* 3932 */       public void visitCast(Java.Cast c) { try { this.val$res[0] = UnitCompiler.this.getConstantValue2(c); } catch (CompileException e) { throw new UnitCompiler.10UCE(UnitCompiler.this, e); } }
/* 3933 */       public void visitClassLiteral(Java.ClassLiteral cl) { this.val$res[0] = UnitCompiler.this.getConstantValue2(cl); }
/* 3934 */       public void visitConditionalExpression(Java.ConditionalExpression ce) { try { this.val$res[0] = UnitCompiler.this.getConstantValue2(ce); } catch (CompileException e) { throw new UnitCompiler.10UCE(UnitCompiler.this, e); } }
/* 3935 */       public void visitCrement(Java.Crement c) { this.val$res[0] = UnitCompiler.this.getConstantValue2(c); }
/* 3936 */       public void visitInstanceof(Java.Instanceof io) { this.val$res[0] = UnitCompiler.this.getConstantValue2(io); }
/* 3937 */       public void visitMethodInvocation(Java.MethodInvocation mi) { this.val$res[0] = UnitCompiler.this.getConstantValue2(mi); }
/* 3938 */       public void visitSuperclassMethodInvocation(Java.SuperclassMethodInvocation smi) { this.val$res[0] = UnitCompiler.this.getConstantValue2(smi); }
/* 3939 */       public void visitLiteral(Java.Literal l) { try { this.val$res[0] = UnitCompiler.this.getConstantValue2(l); } catch (CompileException e) { throw new UnitCompiler.10UCE(UnitCompiler.this, e); } }
/* 3940 */       public void visitNewAnonymousClassInstance(Java.NewAnonymousClassInstance naci) { this.val$res[0] = UnitCompiler.this.getConstantValue2(naci); }
/* 3941 */       public void visitNewArray(Java.NewArray na) { this.val$res[0] = UnitCompiler.this.getConstantValue2(na); }
/* 3942 */       public void visitNewInitializedArray(Java.NewInitializedArray nia) { this.val$res[0] = UnitCompiler.this.getConstantValue2(nia); }
/* 3943 */       public void visitNewClassInstance(Java.NewClassInstance nci) { this.val$res[0] = UnitCompiler.this.getConstantValue2(nci); }
/* 3944 */       public void visitParameterAccess(Java.ParameterAccess pa) { this.val$res[0] = UnitCompiler.this.getConstantValue2(pa); }
/* 3945 */       public void visitQualifiedThisReference(Java.QualifiedThisReference qtr) { this.val$res[0] = UnitCompiler.this.getConstantValue2(qtr); }
/* 3946 */       public void visitThisReference(Java.ThisReference tr) { this.val$res[0] = UnitCompiler.this.getConstantValue2(tr); }
/*      */       
/* 3948 */       public void visitAmbiguousName(Java.AmbiguousName an) { try { this.val$res[0] = UnitCompiler.this.getConstantValue2(an); } catch (CompileException e) { throw new UnitCompiler.10UCE(UnitCompiler.this, e); } }
/* 3949 */       public void visitArrayAccessExpression(Java.ArrayAccessExpression aae) { this.val$res[0] = UnitCompiler.this.getConstantValue2(aae); }
/* 3950 */       public void visitFieldAccess(Java.FieldAccess fa) { try { this.val$res[0] = UnitCompiler.this.getConstantValue2(fa); } catch (CompileException e) { throw new UnitCompiler.10UCE(UnitCompiler.this, e); } }
/* 3951 */       public void visitFieldAccessExpression(Java.FieldAccessExpression fae) { this.val$res[0] = UnitCompiler.this.getConstantValue2(fae); }
/* 3952 */       public void visitSuperclassFieldAccessExpression(Java.SuperclassFieldAccessExpression scfae) { this.val$res[0] = UnitCompiler.this.getConstantValue2(scfae); }
/* 3953 */       public void visitLocalVariableAccess(Java.LocalVariableAccess lva) { this.val$res[0] = UnitCompiler.this.getConstantValue2(lva); }
/* 3954 */       public void visitParenthesizedExpression(Java.ParenthesizedExpression pe) { try { this.val$res[0] = UnitCompiler.this.getConstantValue2(pe); } catch (CompileException e) { throw new UnitCompiler.10UCE(UnitCompiler.this, e);
/*      */         }
/*      */       }
/*      */     };
/* 3958 */     try { rv.accept(rvv);
/* 3959 */       rv.constantValue = res[0];
/* 3960 */       return rv.constantValue;
/*      */     } catch (10UCE uce) {
/* 3962 */       throw uce.ce;
/*      */     }
/*      */   }
/*      */   
/* 3966 */   private Object getConstantValue2(Java.Rvalue rv) { return null; }
/*      */   
/*      */   private Object getConstantValue2(Java.AmbiguousName an) throws CompileException {
/* 3969 */     return getConstantValue(toRvalueOrCE(reclassify(an)));
/*      */   }
/*      */   
/* 3972 */   private Object getConstantValue2(Java.FieldAccess fa) throws CompileException { return fa.field.getConstantValue(); }
/*      */   
/*      */   private Object getConstantValue2(Java.UnaryOperation uo) throws CompileException {
/* 3975 */     if (uo.operator.equals("+")) return getConstantValue(uo.operand);
/* 3976 */     if (uo.operator.equals("-")) return getNegatedConstantValue(uo.operand);
/* 3977 */     if (uo.operator.equals("!")) {
/* 3978 */       Object cv = getConstantValue(uo.operand);
/* 3979 */       return (cv instanceof Boolean) ? Boolean.TRUE : ((Boolean)cv).booleanValue() ? Boolean.FALSE : null;
/*      */     }
/*      */     
/*      */ 
/* 3983 */     return null;
/*      */   }
/*      */   
/*      */   private Object getConstantValue2(Java.ConditionalExpression ce) throws CompileException {
/* 3987 */     Object lhsValue = getConstantValue(ce.lhs);
/* 3988 */     if ((lhsValue instanceof Boolean)) {
/* 3989 */       return ((Boolean)lhsValue).booleanValue() ? getConstantValue(ce.mhs) : getConstantValue(ce.rhs);
/*      */     }
/*      */     
/*      */ 
/* 3993 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */   private Object getConstantValue2(Java.BinaryOperation bo)
/*      */     throws CompileException
/*      */   {
/* 4000 */     if (((bo.op == "==") || (bo.op == "!=")) && (getConstantValue(bo.lhs) == Java.Rvalue.CONSTANT_VALUE_NULL) && (getConstantValue(bo.rhs) == Java.Rvalue.CONSTANT_VALUE_NULL))
/*      */     {
/*      */ 
/*      */ 
/* 4004 */       return bo.op == "==" ? Boolean.TRUE : Boolean.FALSE;
/*      */     }
/*      */     
/* 4007 */     if ((bo.op == "|") || (bo.op == "^") || (bo.op == "&") || (bo.op == "*") || (bo.op == "/") || (bo.op == "%") || (bo.op == "+") || (bo.op == "-"))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4019 */       List cvs = new ArrayList();
/* 4020 */       for (Iterator it = bo.unrollLeftAssociation(); it.hasNext();) {
/* 4021 */         Object cv = getConstantValue((Java.Rvalue)it.next());
/* 4022 */         if (cv == null) return null;
/* 4023 */         cvs.add(cv);
/*      */       }
/*      */       
/*      */ 
/* 4027 */       Iterator it = cvs.iterator();
/* 4028 */       Object lhs = it.next();
/* 4029 */       while (it.hasNext()) {
/* 4030 */         Object rhs = it.next();
/*      */         
/*      */ 
/* 4033 */         if ((bo.op == "+") && (((lhs instanceof String)) || ((rhs instanceof String)))) {
/* 4034 */           StringBuffer sb = new StringBuffer();
/* 4035 */           sb.append(lhs.toString()).append(rhs.toString());
/* 4036 */           while (it.hasNext()) sb.append(it.next().toString());
/* 4037 */           return sb.toString();
/*      */         }
/*      */         
/* 4040 */         if ((!(lhs instanceof Number)) || (!(rhs instanceof Number))) { return null;
/*      */         }
/*      */         try
/*      */         {
/* 4044 */           if (((lhs instanceof Double)) || ((rhs instanceof Double))) {
/* 4045 */             double lhsD = ((Number)lhs).doubleValue();
/* 4046 */             double rhsD = ((Number)rhs).doubleValue();
/* 4047 */             lhs = bo.op == "-" ? new Double(lhsD - rhsD) : bo.op == "+" ? new Double(lhsD + rhsD) : bo.op == "%" ? new Double(lhsD % rhsD) : bo.op == "/" ? new Double(lhsD / rhsD) : bo.op == "*" ? new Double(lhsD * rhsD) : null;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           }
/* 4056 */           else if (((lhs instanceof Float)) || ((rhs instanceof Float))) {
/* 4057 */             float lhsF = ((Number)lhs).floatValue();
/* 4058 */             float rhsF = ((Number)rhs).floatValue();
/* 4059 */             lhs = bo.op == "-" ? new Float(lhsF - rhsF) : bo.op == "+" ? new Float(lhsF + rhsF) : bo.op == "%" ? new Float(lhsF % rhsF) : bo.op == "/" ? new Float(lhsF / rhsF) : bo.op == "*" ? new Float(lhsF * rhsF) : null;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           }
/* 4068 */           else if (((lhs instanceof Long)) || ((rhs instanceof Long))) {
/* 4069 */             long lhsL = ((Number)lhs).longValue();
/* 4070 */             long rhsL = ((Number)rhs).longValue();
/* 4071 */             lhs = bo.op == "-" ? new Long(lhsL - rhsL) : bo.op == "+" ? new Long(lhsL + rhsL) : bo.op == "%" ? new Long(lhsL % rhsL) : bo.op == "/" ? new Long(lhsL / rhsL) : bo.op == "*" ? new Long(lhsL * rhsL) : bo.op == "&" ? new Long(lhsL & rhsL) : bo.op == "^" ? new Long(lhsL ^ rhsL) : bo.op == "|" ? new Long(lhsL | rhsL) : null;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           }
/*      */           else
/*      */           {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4084 */             int lhsI = ((Number)lhs).intValue();
/* 4085 */             int rhsI = ((Number)rhs).intValue();
/* 4086 */             lhs = bo.op == "-" ? new Integer(lhsI - rhsI) : bo.op == "+" ? new Integer(lhsI + rhsI) : bo.op == "%" ? new Integer(lhsI % rhsI) : bo.op == "/" ? new Integer(lhsI / rhsI) : bo.op == "*" ? new Integer(lhsI * rhsI) : bo.op == "&" ? new Integer(lhsI & rhsI) : bo.op == "^" ? new Integer(lhsI ^ rhsI) : bo.op == "|" ? new Integer(lhsI | rhsI) : null;
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4098 */           if (lhs == null) return null;
/*      */         }
/*      */         catch (ArithmeticException ae)
/*      */         {
/* 4102 */           return null;
/*      */         }
/*      */       }
/* 4105 */       return lhs;
/*      */     }
/*      */     
/*      */ 
/* 4109 */     if ((bo.op == "&&") || (bo.op == "||"))
/*      */     {
/*      */ 
/*      */ 
/* 4113 */       Object lhsValue = getConstantValue(bo.lhs);
/* 4114 */       if ((lhsValue instanceof Boolean)) {
/* 4115 */         boolean lhsBV = ((Boolean)lhsValue).booleanValue();
/* 4116 */         return lhsBV ? Boolean.TRUE : bo.op == "&&" ? Boolean.FALSE : lhsBV ? getConstantValue(bo.rhs) : getConstantValue(bo.rhs);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4124 */     return null;
/*      */   }
/*      */   
/* 4127 */   private Object getConstantValue2(Java.Cast c) throws CompileException { Object cv = getConstantValue(c.value);
/* 4128 */     if (cv == null) { return null;
/*      */     }
/* 4130 */     if ((cv instanceof Number)) {
/* 4131 */       IClass tt = getType(c.targetType);
/* 4132 */       if (tt == IClass.BYTE) return new Byte(((Number)cv).byteValue());
/* 4133 */       if (tt == IClass.SHORT) return new Short(((Number)cv).shortValue());
/* 4134 */       if (tt == IClass.INT) return new Integer(((Number)cv).intValue());
/* 4135 */       if (tt == IClass.LONG) return new Long(((Number)cv).longValue());
/* 4136 */       if (tt == IClass.FLOAT) return new Float(((Number)cv).floatValue());
/* 4137 */       if (tt == IClass.DOUBLE) { return new Double(((Number)cv).doubleValue());
/*      */       }
/*      */     }
/* 4140 */     return null;
/*      */   }
/*      */   
/* 4143 */   private Object getConstantValue2(Java.ParenthesizedExpression pe) throws CompileException { return getConstantValue(pe.value); }
/*      */   
/*      */   private Object getConstantValue2(Java.Literal l) throws CompileException {
/* 4146 */     if ((l.value == Scanner.MAGIC_INTEGER) || (l.value == Scanner.MAGIC_LONG))
/*      */     {
/*      */ 
/* 4149 */       compileError("This literal value may only appear in a negated context", l.getLocation()); }
/* 4150 */     return l.value == null ? Java.Rvalue.CONSTANT_VALUE_NULL : l.value;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private Object getNegatedConstantValue(Java.Rvalue rv)
/*      */     throws CompileException
/*      */   {
/* 4162 */     Object[] res = new Object[1];
/*      */     
/* 4164 */     Visitor.RvalueVisitor rvv = new Visitor.RvalueVisitor() { private final Object[] val$res;
/*      */       
/* 4166 */       public void visitArrayLength(Java.ArrayLength al) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(al); }
/* 4167 */       public void visitAssignment(Java.Assignment a) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(a); }
/* 4168 */       public void visitUnaryOperation(Java.UnaryOperation uo) { try { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(uo); } catch (CompileException e) { throw new UnitCompiler.11UCE(UnitCompiler.this, e); } }
/* 4169 */       public void visitBinaryOperation(Java.BinaryOperation bo) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(bo); }
/* 4170 */       public void visitCast(Java.Cast c) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(c); }
/* 4171 */       public void visitClassLiteral(Java.ClassLiteral cl) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(cl); }
/* 4172 */       public void visitConditionalExpression(Java.ConditionalExpression ce) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(ce); }
/* 4173 */       public void visitCrement(Java.Crement c) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(c); }
/* 4174 */       public void visitInstanceof(Java.Instanceof io) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(io); }
/* 4175 */       public void visitMethodInvocation(Java.MethodInvocation mi) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(mi); }
/* 4176 */       public void visitSuperclassMethodInvocation(Java.SuperclassMethodInvocation smi) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(smi); }
/* 4177 */       public void visitLiteral(Java.Literal l) { try { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(l); } catch (CompileException e) { throw new UnitCompiler.11UCE(UnitCompiler.this, e); } }
/* 4178 */       public void visitNewAnonymousClassInstance(Java.NewAnonymousClassInstance naci) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(naci); }
/* 4179 */       public void visitNewArray(Java.NewArray na) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(na); }
/* 4180 */       public void visitNewInitializedArray(Java.NewInitializedArray nia) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(nia); }
/* 4181 */       public void visitNewClassInstance(Java.NewClassInstance nci) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(nci); }
/* 4182 */       public void visitParameterAccess(Java.ParameterAccess pa) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(pa); }
/* 4183 */       public void visitQualifiedThisReference(Java.QualifiedThisReference qtr) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(qtr); }
/* 4184 */       public void visitThisReference(Java.ThisReference tr) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(tr); }
/*      */       
/* 4186 */       public void visitAmbiguousName(Java.AmbiguousName an) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(an); }
/* 4187 */       public void visitArrayAccessExpression(Java.ArrayAccessExpression aae) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(aae); }
/* 4188 */       public void visitFieldAccess(Java.FieldAccess fa) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(fa); }
/* 4189 */       public void visitFieldAccessExpression(Java.FieldAccessExpression fae) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(fae); }
/* 4190 */       public void visitSuperclassFieldAccessExpression(Java.SuperclassFieldAccessExpression scfae) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(scfae); }
/* 4191 */       public void visitLocalVariableAccess(Java.LocalVariableAccess lva) { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(lva); }
/* 4192 */       public void visitParenthesizedExpression(Java.ParenthesizedExpression pe) { try { this.val$res[0] = UnitCompiler.this.getNegatedConstantValue2(pe); } catch (CompileException e) { throw new UnitCompiler.11UCE(UnitCompiler.this, e);
/*      */         }
/*      */       }
/*      */     };
/* 4196 */     try { rv.accept(rvv);
/* 4197 */       return res[0];
/*      */     } catch (11UCE uce) {
/* 4199 */       throw uce.ce;
/*      */     }
/*      */   }
/*      */   
/* 4203 */   private Object getNegatedConstantValue2(Java.Rvalue rv) { return null; }
/*      */   
/*      */   private Object getNegatedConstantValue2(Java.UnaryOperation uo) throws CompileException {
/* 4206 */     return uo.operator.equals("-") ? getConstantValue(uo.operand) : uo.operator.equals("+") ? getNegatedConstantValue(uo.operand) : null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/* 4213 */   private Object getNegatedConstantValue2(Java.ParenthesizedExpression pe)
/* 4213 */     throws CompileException { return getNegatedConstantValue(pe.value); }
/*      */   
/*      */   private Object getNegatedConstantValue2(Java.Literal l) throws CompileException {
/* 4216 */     if ((l.value instanceof Byte)) return new Byte((byte)-((Byte)l.value).byteValue());
/* 4217 */     if ((l.value instanceof Short)) return new Short((short)-((Short)l.value).shortValue());
/* 4218 */     if ((l.value instanceof Integer)) return new Integer(-((Integer)l.value).intValue());
/* 4219 */     if ((l.value instanceof Long)) return new Long(-((Long)l.value).longValue());
/* 4220 */     if ((l.value instanceof Float)) return new Float(-((Float)l.value).floatValue());
/* 4221 */     if ((l.value instanceof Double)) { return new Double(-((Double)l.value).doubleValue());
/*      */     }
/* 4223 */     compileError("Cannot negate this literal", l.getLocation());
/* 4224 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean generatesCode(Java.BlockStatement bs)
/*      */     throws CompileException
/*      */   {
/* 4234 */     boolean[] res = new boolean[1];
/*      */     
/* 4236 */     Visitor.BlockStatementVisitor bsv = new Visitor.BlockStatementVisitor() { private final boolean[] val$res;
/*      */       
/* 4238 */       public void visitInitializer(Java.Initializer i) { try { this.val$res[0] = UnitCompiler.this.generatesCode2(i); } catch (CompileException e) { throw new UnitCompiler.12UCE(UnitCompiler.this, e); } }
/* 4239 */       public void visitFieldDeclaration(Java.FieldDeclaration fd) { try { this.val$res[0] = UnitCompiler.this.generatesCode2(fd); } catch (CompileException e) { throw new UnitCompiler.12UCE(UnitCompiler.this, e); } }
/* 4240 */       public void visitLabeledStatement(Java.LabeledStatement ls) { this.val$res[0] = UnitCompiler.this.generatesCode2(ls); }
/* 4241 */       public void visitBlock(Java.Block b) { try { this.val$res[0] = UnitCompiler.this.generatesCode2(b); } catch (CompileException e) { throw new UnitCompiler.12UCE(UnitCompiler.this, e); } }
/* 4242 */       public void visitExpressionStatement(Java.ExpressionStatement es) { this.val$res[0] = UnitCompiler.this.generatesCode2(es); }
/* 4243 */       public void visitIfStatement(Java.IfStatement is) { this.val$res[0] = UnitCompiler.this.generatesCode2(is); }
/* 4244 */       public void visitForStatement(Java.ForStatement fs) { this.val$res[0] = UnitCompiler.this.generatesCode2(fs); }
/* 4245 */       public void visitWhileStatement(Java.WhileStatement ws) { this.val$res[0] = UnitCompiler.this.generatesCode2(ws); }
/* 4246 */       public void visitTryStatement(Java.TryStatement ts) { this.val$res[0] = UnitCompiler.this.generatesCode2(ts); }
/* 4247 */       public void visitSwitchStatement(Java.SwitchStatement ss) { this.val$res[0] = UnitCompiler.this.generatesCode2(ss); }
/* 4248 */       public void visitSynchronizedStatement(Java.SynchronizedStatement ss) { this.val$res[0] = UnitCompiler.this.generatesCode2(ss); }
/* 4249 */       public void visitDoStatement(Java.DoStatement ds) { this.val$res[0] = UnitCompiler.this.generatesCode2(ds); }
/* 4250 */       public void visitLocalVariableDeclarationStatement(Java.LocalVariableDeclarationStatement lvds) { this.val$res[0] = UnitCompiler.this.generatesCode2(lvds); }
/* 4251 */       public void visitReturnStatement(Java.ReturnStatement rs) { this.val$res[0] = UnitCompiler.this.generatesCode2(rs); }
/* 4252 */       public void visitThrowStatement(Java.ThrowStatement ts) { this.val$res[0] = UnitCompiler.this.generatesCode2(ts); }
/* 4253 */       public void visitBreakStatement(Java.BreakStatement bs) { this.val$res[0] = UnitCompiler.this.generatesCode2(bs); }
/* 4254 */       public void visitContinueStatement(Java.ContinueStatement cs) { this.val$res[0] = UnitCompiler.this.generatesCode2(cs); }
/* 4255 */       public void visitEmptyStatement(Java.EmptyStatement es) { this.val$res[0] = UnitCompiler.this.generatesCode2(es); }
/* 4256 */       public void visitLocalClassDeclarationStatement(Java.LocalClassDeclarationStatement lcds) { this.val$res[0] = UnitCompiler.this.generatesCode2(lcds); }
/* 4257 */       public void visitAlternateConstructorInvocation(Java.AlternateConstructorInvocation aci) { this.val$res[0] = UnitCompiler.this.generatesCode2(aci); }
/* 4258 */       public void visitSuperConstructorInvocation(Java.SuperConstructorInvocation sci) { this.val$res[0] = UnitCompiler.this.generatesCode2(sci); }
/*      */     };
/*      */     try
/*      */     {
/* 4262 */       bs.accept(bsv);
/* 4263 */       return res[0];
/*      */     } catch (12UCE uce) {
/* 4265 */       throw uce.ce;
/*      */     } }
/*      */   
/* 4268 */   public boolean generatesCode2(Java.BlockStatement bs) { return true; }
/* 4269 */   public boolean generatesCode2(Java.EmptyStatement es) { return false; }
/* 4270 */   public boolean generatesCode2(Java.LocalClassDeclarationStatement lcds) { return false; }
/* 4271 */   public boolean generatesCode2(Java.Initializer i) throws CompileException { return generatesCode(i.block); }
/*      */   
/* 4273 */   public boolean generatesCode2ListStatements(List l) throws CompileException { for (int i = 0; i < l.size(); i++) {
/* 4274 */       if (generatesCode((Java.BlockStatement)l.get(i))) return true;
/*      */     }
/* 4276 */     return false;
/*      */   }
/*      */   
/* 4279 */   public boolean generatesCode2(Java.Block b) throws CompileException { return generatesCode2ListStatements(b.statements); }
/*      */   
/*      */   public boolean generatesCode2(Java.FieldDeclaration fd)
/*      */     throws CompileException
/*      */   {
/* 4284 */     for (int i = 0; i < fd.variableDeclarators.length; i++) {
/* 4285 */       Java.VariableDeclarator vd = fd.variableDeclarators[i];
/* 4286 */       if (getNonConstantFinalInitializer(fd, vd) != null) return true;
/*      */     }
/* 4288 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void leave(Java.BlockStatement bs, IClass optionalStackValueType)
/*      */   {
/* 4305 */     Visitor.BlockStatementVisitor bsv = new Visitor.BlockStatementVisitor() { private final IClass val$optionalStackValueType;
/*      */       
/* 4307 */       public void visitInitializer(Java.Initializer i) { UnitCompiler.this.leave2(i, this.val$optionalStackValueType); }
/* 4308 */       public void visitFieldDeclaration(Java.FieldDeclaration fd) { UnitCompiler.this.leave2(fd, this.val$optionalStackValueType); }
/* 4309 */       public void visitLabeledStatement(Java.LabeledStatement ls) { UnitCompiler.this.leave2(ls, this.val$optionalStackValueType); }
/* 4310 */       public void visitBlock(Java.Block b) { UnitCompiler.this.leave2(b, this.val$optionalStackValueType); }
/* 4311 */       public void visitExpressionStatement(Java.ExpressionStatement es) { UnitCompiler.this.leave2(es, this.val$optionalStackValueType); }
/* 4312 */       public void visitIfStatement(Java.IfStatement is) { UnitCompiler.this.leave2(is, this.val$optionalStackValueType); }
/* 4313 */       public void visitForStatement(Java.ForStatement fs) { UnitCompiler.this.leave2(fs, this.val$optionalStackValueType); }
/* 4314 */       public void visitWhileStatement(Java.WhileStatement ws) { UnitCompiler.this.leave2(ws, this.val$optionalStackValueType); }
/* 4315 */       public void visitTryStatement(Java.TryStatement ts) { UnitCompiler.this.leave2(ts, this.val$optionalStackValueType); }
/* 4316 */       public void visitSwitchStatement(Java.SwitchStatement ss) { UnitCompiler.this.leave2(ss, this.val$optionalStackValueType); }
/* 4317 */       public void visitSynchronizedStatement(Java.SynchronizedStatement ss) { UnitCompiler.this.leave2(ss, this.val$optionalStackValueType); }
/* 4318 */       public void visitDoStatement(Java.DoStatement ds) { UnitCompiler.this.leave2(ds, this.val$optionalStackValueType); }
/* 4319 */       public void visitLocalVariableDeclarationStatement(Java.LocalVariableDeclarationStatement lvds) { UnitCompiler.this.leave2(lvds, this.val$optionalStackValueType); }
/* 4320 */       public void visitReturnStatement(Java.ReturnStatement rs) { UnitCompiler.this.leave2(rs, this.val$optionalStackValueType); }
/* 4321 */       public void visitThrowStatement(Java.ThrowStatement ts) { UnitCompiler.this.leave2(ts, this.val$optionalStackValueType); }
/* 4322 */       public void visitBreakStatement(Java.BreakStatement bs) { UnitCompiler.this.leave2(bs, this.val$optionalStackValueType); }
/* 4323 */       public void visitContinueStatement(Java.ContinueStatement cs) { UnitCompiler.this.leave2(cs, this.val$optionalStackValueType); }
/* 4324 */       public void visitEmptyStatement(Java.EmptyStatement es) { UnitCompiler.this.leave2(es, this.val$optionalStackValueType); }
/* 4325 */       public void visitLocalClassDeclarationStatement(Java.LocalClassDeclarationStatement lcds) { UnitCompiler.this.leave2(lcds, this.val$optionalStackValueType); }
/* 4326 */       public void visitAlternateConstructorInvocation(Java.AlternateConstructorInvocation aci) { UnitCompiler.this.leave2(aci, this.val$optionalStackValueType); }
/* 4327 */       public void visitSuperConstructorInvocation(Java.SuperConstructorInvocation sci) { UnitCompiler.this.leave2(sci, this.val$optionalStackValueType);
/*      */       }
/* 4329 */     };
/* 4330 */     bs.accept(bsv);
/*      */   }
/*      */   
/*      */   public void leave2(Java.SynchronizedStatement ss, IClass optionalStackValueType) {
/* 4334 */     load(ss, this.iClassLoader.OBJECT, ss.monitorLvIndex);
/* 4335 */     writeOpcode(ss, -61);
/*      */   }
/*      */   
/* 4338 */   public void leave2(Java.TryStatement ts, IClass optionalStackValueType) { if (ts.finallyOffset != null)
/*      */     {
/* 4340 */       this.codeContext.saveLocalVariables();
/*      */       try {
/* 4342 */         short sv = 0;
/*      */         
/*      */ 
/*      */ 
/*      */ 
/* 4347 */         if (optionalStackValueType != null) {
/* 4348 */           sv = this.codeContext.allocateLocalVariable(Descriptor.size(optionalStackValueType.getDescriptor()));
/*      */           
/*      */ 
/* 4351 */           store(ts, optionalStackValueType, sv);
/*      */         }
/*      */         
/* 4354 */         writeBranch(ts, -88, ts.finallyOffset);
/*      */         
/* 4356 */         if (optionalStackValueType != null) {
/* 4357 */           load(ts, optionalStackValueType, sv);
/*      */         }
/*      */       } finally {
/* 4360 */         this.codeContext.restoreLocalVariables();
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void compileSet(Java.Lvalue lv)
/*      */     throws CompileException
/*      */   {
/* 4375 */     Visitor.LvalueVisitor lvv = new Visitor.LvalueVisitor() {
/*      */       public void visitAmbiguousName(Java.AmbiguousName an) {
/* 4377 */         try { UnitCompiler.this.compileSet2(an); } catch (CompileException e) { throw new UnitCompiler.13UCE(UnitCompiler.this, e); } }
/* 4378 */       public void visitArrayAccessExpression(Java.ArrayAccessExpression aae) { try { UnitCompiler.this.compileSet2(aae); } catch (CompileException e) { throw new UnitCompiler.13UCE(UnitCompiler.this, e); } }
/* 4379 */       public void visitFieldAccess(Java.FieldAccess fa) { try { UnitCompiler.this.compileSet2(fa); } catch (CompileException e) { throw new UnitCompiler.13UCE(UnitCompiler.this, e); } }
/* 4380 */       public void visitFieldAccessExpression(Java.FieldAccessExpression fae) { try { UnitCompiler.this.compileSet2(fae); } catch (CompileException e) { throw new UnitCompiler.13UCE(UnitCompiler.this, e); } }
/* 4381 */       public void visitSuperclassFieldAccessExpression(Java.SuperclassFieldAccessExpression scfae) { try { UnitCompiler.this.compileSet2(scfae); } catch (CompileException e) { throw new UnitCompiler.13UCE(UnitCompiler.this, e); } }
/* 4382 */       public void visitLocalVariableAccess(Java.LocalVariableAccess lva) { UnitCompiler.this.compileSet2(lva); }
/* 4383 */       public void visitParenthesizedExpression(Java.ParenthesizedExpression pe) { try { UnitCompiler.this.compileSet2(pe); } catch (CompileException e) { throw new UnitCompiler.13UCE(UnitCompiler.this, e);
/*      */         }
/*      */       }
/*      */     };
/* 4387 */     try { lv.accept(lvv);
/*      */     } catch (13UCE uce) {
/* 4389 */       throw uce.ce;
/*      */     }
/*      */   }
/*      */   
/* 4393 */   private void compileSet2(Java.AmbiguousName an) throws CompileException { compileSet(toLvalueOrCE(reclassify(an))); }
/*      */   
/*      */   private void compileSet2(Java.LocalVariableAccess lva) {
/* 4396 */     store(lva, lva.localVariable.type, lva.localVariable);
/*      */   }
/*      */   
/*      */ 
/*      */   private void compileSet2(Java.FieldAccess fa)
/*      */     throws CompileException
/*      */   {
/* 4403 */     checkAccessible(fa.field, fa.getEnclosingBlockStatement());
/* 4404 */     writeOpcode(fa, fa.field.isStatic() ? -77 : -75);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 4409 */     writeConstantFieldrefInfo(fa.field.getDeclaringIClass().getDescriptor(), fa.field.getName(), fa.field.getDescriptor());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/* 4416 */   private void compileSet2(Java.ArrayAccessExpression aae)
/* 4416 */     throws CompileException { writeOpcode(aae, 79 + ilfdabcs(getType(aae))); }
/*      */   
/*      */   private void compileSet2(Java.FieldAccessExpression fae) throws CompileException {
/* 4419 */     determineValue(fae);
/* 4420 */     compileSet(toLvalueOrCE(fae.value));
/*      */   }
/*      */   
/* 4423 */   private void compileSet2(Java.SuperclassFieldAccessExpression scfae) throws CompileException { determineValue(scfae);
/* 4424 */     compileSet(toLvalueOrCE(scfae.value));
/*      */   }
/*      */   
/* 4427 */   private void compileSet2(Java.ParenthesizedExpression pe) throws CompileException { compileSet(toLvalueOrCE(pe.value)); }
/*      */   
/*      */ 
/*      */   private IClass getType(Java.Atom a)
/*      */     throws CompileException
/*      */   {
/* 4433 */     IClass[] res = new IClass[1];
/*      */     
/* 4435 */     Visitor.AtomVisitor av = new Visitor.AtomVisitor() {
/*      */       private final IClass[] val$res;
/*      */       
/* 4438 */       public void visitPackage(Java.Package p) { try { this.val$res[0] = UnitCompiler.this.getType2(p); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/*      */       
/* 4440 */       public void visitArrayType(Java.ArrayType at) { try { this.val$res[0] = UnitCompiler.this.getType2(at); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4441 */       public void visitBasicType(Java.BasicType bt) { this.val$res[0] = UnitCompiler.this.getType2(bt); }
/* 4442 */       public void visitReferenceType(Java.ReferenceType rt) { try { this.val$res[0] = UnitCompiler.this.getType2(rt); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4443 */       public void visitRvalueMemberType(Java.RvalueMemberType rmt) { try { this.val$res[0] = UnitCompiler.this.getType2(rmt); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4444 */       public void visitSimpleType(Java.SimpleType st) { this.val$res[0] = UnitCompiler.this.getType2(st); }
/*      */       
/* 4446 */       public void visitArrayLength(Java.ArrayLength al) { this.val$res[0] = UnitCompiler.this.getType2(al); }
/* 4447 */       public void visitAssignment(Java.Assignment a) { try { this.val$res[0] = UnitCompiler.this.getType2(a); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4448 */       public void visitUnaryOperation(Java.UnaryOperation uo) { try { this.val$res[0] = UnitCompiler.this.getType2(uo); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4449 */       public void visitBinaryOperation(Java.BinaryOperation bo) { try { this.val$res[0] = UnitCompiler.this.getType2(bo); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4450 */       public void visitCast(Java.Cast c) { try { this.val$res[0] = UnitCompiler.this.getType2(c); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4451 */       public void visitClassLiteral(Java.ClassLiteral cl) { this.val$res[0] = UnitCompiler.this.getType2(cl); }
/* 4452 */       public void visitConditionalExpression(Java.ConditionalExpression ce) { try { this.val$res[0] = UnitCompiler.this.getType2(ce); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4453 */       public void visitCrement(Java.Crement c) { try { this.val$res[0] = UnitCompiler.this.getType2(c); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4454 */       public void visitInstanceof(Java.Instanceof io) { this.val$res[0] = UnitCompiler.this.getType2(io); }
/* 4455 */       public void visitMethodInvocation(Java.MethodInvocation mi) { try { this.val$res[0] = UnitCompiler.this.getType2(mi); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4456 */       public void visitSuperclassMethodInvocation(Java.SuperclassMethodInvocation smi) { try { this.val$res[0] = UnitCompiler.this.getType2(smi); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4457 */       public void visitLiteral(Java.Literal l) { this.val$res[0] = UnitCompiler.this.getType2(l); }
/* 4458 */       public void visitNewAnonymousClassInstance(Java.NewAnonymousClassInstance naci) { this.val$res[0] = UnitCompiler.this.getType2(naci); }
/* 4459 */       public void visitNewArray(Java.NewArray na) { try { this.val$res[0] = UnitCompiler.this.getType2(na); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4460 */       public void visitNewInitializedArray(Java.NewInitializedArray nia) { try { this.val$res[0] = UnitCompiler.this.getType2(nia); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4461 */       public void visitNewClassInstance(Java.NewClassInstance nci) { try { this.val$res[0] = UnitCompiler.this.getType2(nci); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4462 */       public void visitParameterAccess(Java.ParameterAccess pa) { try { this.val$res[0] = UnitCompiler.this.getType2(pa); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4463 */       public void visitQualifiedThisReference(Java.QualifiedThisReference qtr) { try { this.val$res[0] = UnitCompiler.this.getType2(qtr); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4464 */       public void visitThisReference(Java.ThisReference tr) { try { this.val$res[0] = UnitCompiler.this.getType2(tr); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/*      */       
/* 4466 */       public void visitAmbiguousName(Java.AmbiguousName an) { try { this.val$res[0] = UnitCompiler.this.getType2(an); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4467 */       public void visitArrayAccessExpression(Java.ArrayAccessExpression aae) { try { this.val$res[0] = UnitCompiler.this.getType2(aae); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4468 */       public void visitFieldAccess(Java.FieldAccess fa) { try { this.val$res[0] = UnitCompiler.this.getType2(fa); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4469 */       public void visitFieldAccessExpression(Java.FieldAccessExpression fae) { try { this.val$res[0] = UnitCompiler.this.getType2(fae); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4470 */       public void visitSuperclassFieldAccessExpression(Java.SuperclassFieldAccessExpression scfae) { try { this.val$res[0] = UnitCompiler.this.getType2(scfae); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e); } }
/* 4471 */       public void visitLocalVariableAccess(Java.LocalVariableAccess lva) { this.val$res[0] = UnitCompiler.this.getType2(lva); }
/* 4472 */       public void visitParenthesizedExpression(Java.ParenthesizedExpression pe) { try { this.val$res[0] = UnitCompiler.this.getType2(pe); } catch (CompileException e) { throw new UnitCompiler.14UCE(UnitCompiler.this, e);
/*      */         }
/*      */       }
/*      */     };
/* 4476 */     try { a.accept(av);
/* 4477 */       return res[0] != null ? res[0] : this.iClassLoader.OBJECT;
/*      */     } catch (14UCE uce) {
/* 4479 */       throw uce.ce;
/*      */     } }
/*      */   
/* 4482 */   private IClass getType2(Java.SimpleType st) { return st.iClass; }
/*      */   
/* 4484 */   private IClass getType2(Java.BasicType bt) { switch (bt.index) {
/* 4485 */     case 0:  return IClass.VOID;
/* 4486 */     case 1:  return IClass.BYTE;
/* 4487 */     case 2:  return IClass.SHORT;
/* 4488 */     case 3:  return IClass.CHAR;
/* 4489 */     case 4:  return IClass.INT;
/* 4490 */     case 5:  return IClass.LONG;
/* 4491 */     case 6:  return IClass.FLOAT;
/* 4492 */     case 7:  return IClass.DOUBLE;
/* 4493 */     case 8:  return IClass.BOOLEAN; }
/* 4494 */     throw new JaninoRuntimeException("Invalid index " + bt.index);
/*      */   }
/*      */   
/*      */   private IClass getType2(Java.ReferenceType rt) throws CompileException {
/* 4498 */     Java.BlockStatement scopeBlockStatement = null;
/* 4499 */     Java.TypeDeclaration scopeTypeDeclaration = null;
/*      */     
/* 4501 */     for (Java.Scope s = rt.getEnclosingScope();; s = s.getEnclosingScope()) {
/* 4502 */       if (((s instanceof Java.BlockStatement)) && (scopeBlockStatement == null)) {
/* 4503 */         scopeBlockStatement = (Java.BlockStatement)s;
/*      */       }
/* 4505 */       if (((s instanceof Java.TypeDeclaration)) && (scopeTypeDeclaration == null)) {
/* 4506 */         scopeTypeDeclaration = (Java.TypeDeclaration)s;
/*      */       }
/* 4508 */       if ((s instanceof Java.CompilationUnit)) {
/* 4509 */         Java.CompilationUnit scopeCompilationUnit = (Java.CompilationUnit)s;
/* 4510 */         break;
/*      */       }
/*      */     }
/*      */     Java.CompilationUnit scopeCompilationUnit;
/* 4514 */     if (rt.identifiers.length == 1)
/*      */     {
/*      */ 
/* 4517 */       String simpleTypeName = rt.identifiers[0];
/*      */       
/*      */ 
/*      */ 
/* 4521 */       Java.LocalClassDeclaration lcd = findLocalClassDeclaration(rt.getEnclosingScope(), simpleTypeName);
/* 4522 */       if (lcd != null) { return resolve(lcd);
/*      */       }
/*      */       
/*      */ 
/* 4526 */       if (scopeTypeDeclaration != null)
/*      */       {
/* 4528 */         for (Java.Scope s = scopeTypeDeclaration; 
/* 4529 */             !(s instanceof Java.CompilationUnit); 
/* 4530 */             s = s.getEnclosingScope())
/*      */         {
/* 4532 */           if ((s instanceof Java.TypeDeclaration)) {
/* 4533 */             IClass mt = findMemberType(resolve((Java.AbstractTypeDeclaration)s), simpleTypeName, rt.getLocation());
/*      */             
/*      */ 
/*      */ 
/*      */ 
/* 4538 */             if (mt != null) { return mt;
/*      */             }
/*      */           }
/*      */         }
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 4547 */       IClass importedClass = importSingleType(simpleTypeName, rt.getLocation());
/* 4548 */       if (importedClass != null) { return importedClass;
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 4553 */       Java.PackageMemberTypeDeclaration pmtd = scopeCompilationUnit.getPackageMemberTypeDeclaration(simpleTypeName);
/*      */       
/*      */ 
/* 4556 */       if (pmtd != null) { return resolve((Java.AbstractTypeDeclaration)pmtd);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4563 */       String pkg = scopeCompilationUnit.optionalPackageDeclaration == null ? null : scopeCompilationUnit.optionalPackageDeclaration.packageName;
/*      */       
/*      */ 
/*      */ 
/* 4567 */       String className = pkg + "." + simpleTypeName;
/* 4568 */       IClass result = findClassByName(rt.getLocation(), className);
/* 4569 */       if (result != null) { return result;
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 4574 */       IClass importedClass = importTypeOnDemand(simpleTypeName, rt.getLocation());
/* 4575 */       if (importedClass != null) { return importedClass;
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 4580 */       List l = (List)this.singleStaticImports.get(simpleTypeName);
/* 4581 */       if (l != null) {
/* 4582 */         IClass importedMemberType = null;
/* 4583 */         for (Iterator it = l.iterator(); it.hasNext();) {
/* 4584 */           Object o = it.next();
/* 4585 */           if ((o instanceof IClass)) {
/* 4586 */             IClass mt = (IClass)o;
/* 4587 */             if (isAccessible(mt, scopeBlockStatement)) {
/* 4588 */               if ((importedMemberType != null) && (importedMemberType != mt)) {
/* 4589 */                 compileError("Ambiguous static imports: \"" + importedMemberType.toString() + "\" vs. \"" + mt.toString() + "\"");
/*      */               }
/*      */               
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4597 */               importedMemberType = mt;
/*      */             }
/*      */           } }
/* 4600 */         if (importedMemberType != null) { return importedMemberType;
/*      */         }
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 4606 */       IClass importedMemberType = null;
/* 4607 */       for (Iterator it = this.staticImportsOnDemand.iterator(); it.hasNext();) {
/* 4608 */         IClass ic = (IClass)it.next();
/* 4609 */         IClass[] memberTypes = ic.getDeclaredIClasses();
/* 4610 */         for (int i = 0; i < memberTypes.length; i++) {
/* 4611 */           IClass mt = memberTypes[i];
/* 4612 */           if ((isAccessible(mt, scopeBlockStatement)) && 
/* 4613 */             (mt.getDescriptor().endsWith('$' + simpleTypeName + ';'))) {
/* 4614 */             if (importedMemberType != null) {
/* 4615 */               compileError("Ambiguous static imports: \"" + importedMemberType.toString() + "\" vs. \"" + mt.toString() + "\"");
/*      */             }
/*      */             
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4623 */             importedMemberType = mt;
/*      */           }
/*      */         }
/*      */       }
/* 4627 */       if (importedMemberType != null) { return importedMemberType;
/*      */       }
/*      */       
/*      */ 
/* 4631 */       compileError("Cannot determine simple type name \"" + simpleTypeName + "\"", rt.getLocation());
/* 4632 */       return this.iClassLoader.OBJECT;
/*      */     }
/*      */     
/*      */ 
/* 4636 */     Java.Atom q = reclassifyName(rt.getLocation(), rt.getEnclosingScope(), rt.identifiers, rt.identifiers.length - 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4644 */     if ((q instanceof Java.Package)) {
/* 4645 */       String className = Java.join(rt.identifiers, ".");
/* 4646 */       IClass result = findClassByName(rt.getLocation(), className);
/* 4647 */       if (result != null) { return result;
/*      */       }
/* 4649 */       compileError("Class \"" + className + "\" not found", rt.getLocation());
/* 4650 */       return this.iClassLoader.OBJECT;
/*      */     }
/*      */     
/*      */ 
/* 4654 */     String memberTypeName = rt.identifiers[(rt.identifiers.length - 1)];
/* 4655 */     IClass[] types = getType(toTypeOrCE(q)).findMemberType(memberTypeName);
/* 4656 */     if (types.length == 1) return types[0];
/* 4657 */     if (types.length == 0) {
/* 4658 */       compileError("\"" + q + "\" declares no member type \"" + memberTypeName + "\"", rt.getLocation());
/*      */     }
/*      */     else {
/* 4661 */       compileError("\"" + q + "\" and its supertypes declare more than one member type \"" + memberTypeName + "\"", rt.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 4666 */     return this.iClassLoader.OBJECT;
/*      */   }
/*      */   
/*      */   private IClass getType2(Java.RvalueMemberType rvmt) throws CompileException
/*      */   {
/* 4671 */     IClass rvt = getType(rvmt.rvalue);
/* 4672 */     IClass memberType = findMemberType(rvt, rvmt.identifier, rvmt.getLocation());
/* 4673 */     if (memberType == null) {
/* 4674 */       compileError("\"" + rvt + "\" has no member type \"" + rvmt.identifier + "\"", rvmt.getLocation());
/*      */     }
/* 4676 */     return memberType;
/*      */   }
/*      */   
/* 4679 */   private IClass getType2(Java.ArrayType at) throws CompileException { return getType(at.componentType).getArrayIClass(this.iClassLoader.OBJECT); }
/*      */   
/*      */ 
/* 4682 */   private IClass getType2(Java.AmbiguousName an) throws CompileException { return getType(reclassify(an)); }
/*      */   
/*      */   private IClass getType2(Java.Package p) throws CompileException {
/* 4685 */     compileError("Unknown variable or type \"" + p.name + "\"", p.getLocation());
/* 4686 */     return this.iClassLoader.OBJECT;
/*      */   }
/*      */   
/* 4689 */   private IClass getType2(Java.LocalVariableAccess lva) { return lva.localVariable.type; }
/*      */   
/*      */   private IClass getType2(Java.FieldAccess fa) throws CompileException {
/* 4692 */     return fa.field.getType();
/*      */   }
/*      */   
/* 4695 */   private IClass getType2(Java.ArrayLength al) { return IClass.INT; }
/*      */   
/*      */   private IClass getType2(Java.ThisReference tr) throws CompileException {
/* 4698 */     return getIClass(tr);
/*      */   }
/*      */   
/* 4701 */   private IClass getType2(Java.QualifiedThisReference qtr) throws CompileException { return getTargetIClass(qtr); }
/*      */   
/*      */   private IClass getType2(Java.ClassLiteral cl) {
/* 4704 */     return this.iClassLoader.CLASS;
/*      */   }
/*      */   
/* 4707 */   private IClass getType2(Java.Assignment a) throws CompileException { return getType(a.lhs); }
/*      */   
/*      */   private IClass getType2(Java.ConditionalExpression ce) throws CompileException {
/* 4710 */     IClass mhsType = getType(ce.mhs);
/* 4711 */     IClass rhsType = getType(ce.rhs);
/*      */     
/* 4713 */     if (mhsType == rhsType)
/*      */     {
/*      */ 
/* 4716 */       return mhsType;
/*      */     }
/* 4718 */     if ((mhsType.isPrimitiveNumeric()) && (rhsType.isPrimitiveNumeric()))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4727 */       return binaryNumericPromotionType(ce, mhsType, rhsType);
/*      */     }
/* 4729 */     if ((getConstantValue(ce.mhs) == Java.Rvalue.CONSTANT_VALUE_NULL) && (!rhsType.isPrimitive()))
/*      */     {
/*      */ 
/* 4732 */       return rhsType;
/*      */     }
/* 4734 */     if ((!mhsType.isPrimitive()) && (getConstantValue(ce.rhs) == Java.Rvalue.CONSTANT_VALUE_NULL))
/*      */     {
/*      */ 
/* 4737 */       return mhsType;
/*      */     }
/* 4739 */     if ((!mhsType.isPrimitive()) && (!rhsType.isPrimitive())) {
/* 4740 */       if (mhsType.isAssignableFrom(rhsType)) {
/* 4741 */         return mhsType;
/*      */       }
/* 4743 */       if (rhsType.isAssignableFrom(mhsType)) {
/* 4744 */         return rhsType;
/*      */       }
/* 4746 */       compileError("Reference types \"" + mhsType + "\" and \"" + rhsType + "\" don't match", ce.getLocation());
/*      */       
/*      */ 
/*      */ 
/* 4750 */       return this.iClassLoader.OBJECT;
/*      */     }
/*      */     
/*      */ 
/* 4754 */     compileError("Incompatible expression types \"" + mhsType + "\" and \"" + rhsType + "\"", ce.getLocation());
/*      */     
/*      */ 
/*      */ 
/* 4758 */     return this.iClassLoader.OBJECT;
/*      */   }
/*      */   
/*      */   private IClass getType2(Java.Crement c) throws CompileException {
/* 4762 */     return getType(c.operand);
/*      */   }
/*      */   
/* 4765 */   private IClass getType2(Java.ArrayAccessExpression aae) throws CompileException { return getType(aae.lhs).getComponentType(); }
/*      */   
/*      */   private IClass getType2(Java.FieldAccessExpression fae) throws CompileException {
/* 4768 */     determineValue(fae);
/* 4769 */     return getType(fae.value);
/*      */   }
/*      */   
/* 4772 */   private IClass getType2(Java.SuperclassFieldAccessExpression scfae) throws CompileException { determineValue(scfae);
/* 4773 */     return getType(scfae.value);
/*      */   }
/*      */   
/* 4776 */   private IClass getType2(Java.UnaryOperation uo) throws CompileException { if (uo.operator == "!") { return IClass.BOOLEAN;
/*      */     }
/* 4778 */     if ((uo.operator == "+") || (uo.operator == "-") || (uo.operator == "~"))
/*      */     {
/*      */ 
/*      */ 
/* 4782 */       return unaryNumericPromotionType(uo, getUnboxedType(getType(uo.operand)));
/*      */     }
/* 4784 */     compileError("Unexpected operator \"" + uo.operator + "\"", uo.getLocation());
/* 4785 */     return IClass.BOOLEAN;
/*      */   }
/*      */   
/* 4788 */   private IClass getType2(Java.Instanceof io) { return IClass.BOOLEAN; }
/*      */   
/*      */   private IClass getType2(Java.BinaryOperation bo) throws CompileException {
/* 4791 */     if ((bo.op == "||") || (bo.op == "&&") || (bo.op == "==") || (bo.op == "!=") || (bo.op == "<") || (bo.op == ">") || (bo.op == "<=") || (bo.op == ">="))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4800 */       return IClass.BOOLEAN;
/*      */     }
/* 4802 */     if ((bo.op == "|") || (bo.op == "^") || (bo.op == "&"))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/* 4807 */       IClass lhsType = getType(bo.lhs);
/* 4808 */       return (lhsType == IClass.BOOLEAN) || (lhsType == this.iClassLoader.BOOLEAN) ? IClass.BOOLEAN : binaryNumericPromotionType(bo, lhsType, getType(bo.rhs));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4819 */     if ((bo.op == "*") || (bo.op == "/") || (bo.op == "%") || (bo.op == "+") || (bo.op == "-"))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 4826 */       IClassLoader icl = this.iClassLoader;
/*      */       
/*      */ 
/* 4829 */       Iterator ops = bo.unrollLeftAssociation();
/*      */       
/*      */ 
/* 4832 */       IClass lhsType = getUnboxedType(getType((Java.Rvalue)ops.next()));
/* 4833 */       if ((bo.op == "+") && (lhsType == icl.STRING)) { return icl.STRING;
/*      */       }
/*      */       do
/*      */       {
/* 4837 */         IClass rhsType = getUnboxedType(getType((Java.Rvalue)ops.next()));
/* 4838 */         if ((bo.op == "+") && (rhsType == icl.STRING)) return icl.STRING;
/* 4839 */         lhsType = binaryNumericPromotionType(bo, lhsType, rhsType);
/* 4840 */       } while (ops.hasNext());
/* 4841 */       return lhsType;
/*      */     }
/*      */     
/* 4844 */     if ((bo.op == "<<") || (bo.op == ">>") || (bo.op == ">>>"))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/* 4849 */       IClass lhsType = getType(bo.lhs);
/* 4850 */       return unaryNumericPromotionType(bo, lhsType);
/*      */     }
/*      */     
/* 4853 */     compileError("Unexpected operator \"" + bo.op + "\"", bo.getLocation());
/* 4854 */     return this.iClassLoader.OBJECT;
/*      */   }
/*      */   
/* 4857 */   private IClass getUnboxedType(IClass type) { IClass c = isUnboxingConvertible(type);
/* 4858 */     return c != null ? c : type;
/*      */   }
/*      */   
/* 4861 */   private IClass getType2(Java.Cast c) throws CompileException { return getType(c.targetType); }
/*      */   
/*      */ 
/* 4864 */   private IClass getType2(Java.ParenthesizedExpression pe) throws CompileException { return getType(pe.value); }
/*      */   
/*      */   private IClass getType2(Java.MethodInvocation mi) throws CompileException {
/* 4867 */     if (mi.iMethod == null) {
/* 4868 */       mi.iMethod = findIMethod(mi);
/*      */     }
/* 4870 */     return mi.iMethod.getReturnType();
/*      */   }
/*      */   
/* 4873 */   private IClass getType2(Java.SuperclassMethodInvocation scmi) throws CompileException { return findIMethod(scmi).getReturnType(); }
/*      */   
/*      */   private IClass getType2(Java.NewClassInstance nci) throws CompileException {
/* 4876 */     if (nci.iClass == null) nci.iClass = getType(nci.type);
/* 4877 */     return nci.iClass;
/*      */   }
/*      */   
/* 4880 */   private IClass getType2(Java.NewAnonymousClassInstance naci) { return resolve(naci.anonymousClassDeclaration); }
/*      */   
/*      */ 
/* 4883 */   private IClass getType2(Java.ParameterAccess pa) throws CompileException { return getLocalVariable(pa.formalParameter).type; }
/*      */   
/*      */   private IClass getType2(Java.NewArray na) throws CompileException {
/* 4886 */     IClass res = getType(na.type);
/* 4887 */     return res.getArrayIClass(na.dimExprs.length + na.dims, this.iClassLoader.OBJECT);
/*      */   }
/*      */   
/* 4890 */   private IClass getType2(Java.NewInitializedArray nia) throws CompileException { return getType(nia.arrayType); }
/*      */   
/*      */   private IClass getType2(Java.Literal l) {
/* 4893 */     if ((l.value instanceof Short)) return IClass.SHORT;
/* 4894 */     if ((l.value instanceof Byte)) return IClass.BYTE;
/* 4895 */     if ((l.value instanceof Integer)) return IClass.INT;
/* 4896 */     if ((l.value instanceof Long)) return IClass.LONG;
/* 4897 */     if ((l.value instanceof Float)) return IClass.FLOAT;
/* 4898 */     if ((l.value instanceof Double)) return IClass.DOUBLE;
/* 4899 */     if ((l.value instanceof String)) return this.iClassLoader.STRING;
/* 4900 */     if ((l.value instanceof Character)) return IClass.CHAR;
/* 4901 */     if ((l.value instanceof Boolean)) return IClass.BOOLEAN;
/* 4902 */     if (l.value == null) return IClass.VOID;
/* 4903 */     throw new JaninoRuntimeException("SNO: Unidentifiable literal type \"" + l.value.getClass().getName() + "\"");
/*      */   }
/*      */   
/*      */   private boolean isType(Java.Atom a)
/*      */     throws CompileException
/*      */   {
/* 4909 */     boolean[] res = new boolean[1];
/*      */     
/* 4911 */     Visitor.AtomVisitor av = new Visitor.AtomVisitor() {
/*      */       private final boolean[] val$res;
/*      */       
/* 4914 */       public void visitPackage(Java.Package p) { this.val$res[0] = UnitCompiler.this.isType2(p); }
/*      */       
/* 4916 */       public void visitArrayType(Java.ArrayType at) { this.val$res[0] = UnitCompiler.this.isType2(at); }
/* 4917 */       public void visitBasicType(Java.BasicType bt) { this.val$res[0] = UnitCompiler.this.isType2(bt); }
/* 4918 */       public void visitReferenceType(Java.ReferenceType rt) { this.val$res[0] = UnitCompiler.this.isType2(rt); }
/* 4919 */       public void visitRvalueMemberType(Java.RvalueMemberType rmt) { this.val$res[0] = UnitCompiler.this.isType2(rmt); }
/* 4920 */       public void visitSimpleType(Java.SimpleType st) { this.val$res[0] = UnitCompiler.this.isType2(st); }
/*      */       
/* 4922 */       public void visitArrayLength(Java.ArrayLength al) { this.val$res[0] = UnitCompiler.this.isType2(al); }
/* 4923 */       public void visitAssignment(Java.Assignment a) { this.val$res[0] = UnitCompiler.this.isType2(a); }
/* 4924 */       public void visitUnaryOperation(Java.UnaryOperation uo) { this.val$res[0] = UnitCompiler.this.isType2(uo); }
/* 4925 */       public void visitBinaryOperation(Java.BinaryOperation bo) { this.val$res[0] = UnitCompiler.this.isType2(bo); }
/* 4926 */       public void visitCast(Java.Cast c) { this.val$res[0] = UnitCompiler.this.isType2(c); }
/* 4927 */       public void visitClassLiteral(Java.ClassLiteral cl) { this.val$res[0] = UnitCompiler.this.isType2(cl); }
/* 4928 */       public void visitConditionalExpression(Java.ConditionalExpression ce) { this.val$res[0] = UnitCompiler.this.isType2(ce); }
/* 4929 */       public void visitCrement(Java.Crement c) { this.val$res[0] = UnitCompiler.this.isType2(c); }
/* 4930 */       public void visitInstanceof(Java.Instanceof io) { this.val$res[0] = UnitCompiler.this.isType2(io); }
/* 4931 */       public void visitMethodInvocation(Java.MethodInvocation mi) { this.val$res[0] = UnitCompiler.this.isType2(mi); }
/* 4932 */       public void visitSuperclassMethodInvocation(Java.SuperclassMethodInvocation smi) { this.val$res[0] = UnitCompiler.this.isType2(smi); }
/* 4933 */       public void visitLiteral(Java.Literal l) { this.val$res[0] = UnitCompiler.this.isType2(l); }
/* 4934 */       public void visitNewAnonymousClassInstance(Java.NewAnonymousClassInstance naci) { this.val$res[0] = UnitCompiler.this.isType2(naci); }
/* 4935 */       public void visitNewArray(Java.NewArray na) { this.val$res[0] = UnitCompiler.this.isType2(na); }
/* 4936 */       public void visitNewInitializedArray(Java.NewInitializedArray nia) { this.val$res[0] = UnitCompiler.this.isType2(nia); }
/* 4937 */       public void visitNewClassInstance(Java.NewClassInstance nci) { this.val$res[0] = UnitCompiler.this.isType2(nci); }
/* 4938 */       public void visitParameterAccess(Java.ParameterAccess pa) { this.val$res[0] = UnitCompiler.this.isType2(pa); }
/* 4939 */       public void visitQualifiedThisReference(Java.QualifiedThisReference qtr) { this.val$res[0] = UnitCompiler.this.isType2(qtr); }
/* 4940 */       public void visitThisReference(Java.ThisReference tr) { this.val$res[0] = UnitCompiler.this.isType2(tr); }
/*      */       
/* 4942 */       public void visitAmbiguousName(Java.AmbiguousName an) { try { this.val$res[0] = UnitCompiler.this.isType2(an); } catch (CompileException e) { throw new UnitCompiler.15UCE(UnitCompiler.this, e); } }
/* 4943 */       public void visitArrayAccessExpression(Java.ArrayAccessExpression aae) { this.val$res[0] = UnitCompiler.this.isType2(aae); }
/* 4944 */       public void visitFieldAccess(Java.FieldAccess fa) { this.val$res[0] = UnitCompiler.this.isType2(fa); }
/* 4945 */       public void visitFieldAccessExpression(Java.FieldAccessExpression fae) { this.val$res[0] = UnitCompiler.this.isType2(fae); }
/* 4946 */       public void visitSuperclassFieldAccessExpression(Java.SuperclassFieldAccessExpression scfae) { this.val$res[0] = UnitCompiler.this.isType2(scfae); }
/* 4947 */       public void visitLocalVariableAccess(Java.LocalVariableAccess lva) { this.val$res[0] = UnitCompiler.this.isType2(lva); }
/* 4948 */       public void visitParenthesizedExpression(Java.ParenthesizedExpression pe) { this.val$res[0] = UnitCompiler.this.isType2(pe); }
/*      */     };
/*      */     try
/*      */     {
/* 4952 */       a.accept(av);
/* 4953 */       return res[0];
/*      */     } catch (15UCE uce) {
/* 4955 */       throw uce.ce;
/*      */     }
/*      */   }
/*      */   
/* 4959 */   private boolean isType2(Java.Atom a) { return a instanceof Java.Type; }
/*      */   
/*      */   private boolean isType2(Java.AmbiguousName an) throws CompileException {
/* 4962 */     return isType(reclassify(an));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean isAccessible(IClass.IMember member, Java.Scope contextScope)
/*      */     throws CompileException
/*      */   {
/* 4974 */     IClass declaringIClass = member.getDeclaringIClass();
/* 4975 */     boolean acc = isAccessible(declaringIClass, contextScope);
/* 4976 */     acc = (acc) && (isAccessible(declaringIClass, member.getAccess(), contextScope));
/* 4977 */     return acc;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void checkAccessible(IClass.IMember member, Java.BlockStatement contextBlockStatement)
/*      */     throws CompileException
/*      */   {
/* 4989 */     IClass declaringIClass = member.getDeclaringIClass();
/* 4990 */     checkAccessible(declaringIClass, contextBlockStatement);
/* 4991 */     checkAccessible(declaringIClass, member.getAccess(), contextBlockStatement);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean isAccessible(IClass iClassDeclaringMember, Access memberAccess, Java.Scope contextScope)
/*      */     throws CompileException
/*      */   {
/* 5004 */     return null == internalCheckAccessible(iClassDeclaringMember, memberAccess, contextScope);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void checkAccessible(IClass iClassDeclaringMember, Access memberAccess, Java.BlockStatement contextBlockStatement)
/*      */     throws CompileException
/*      */   {
/* 5017 */     String message = internalCheckAccessible(iClassDeclaringMember, memberAccess, contextBlockStatement);
/* 5018 */     if (message != null) { compileError(message, contextBlockStatement.getLocation());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private String internalCheckAccessible(IClass iClassDeclaringMember, Access memberAccess, Java.Scope contextScope)
/*      */     throws CompileException
/*      */   {
/* 5033 */     if (memberAccess == Access.PUBLIC) { return null;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 5039 */     for (Java.Scope s = contextScope;; s = s.getEnclosingScope()) {
/* 5040 */       if ((s instanceof Java.TypeDeclaration)) {
/* 5041 */         IClass iClassDeclaringContextBlockStatement = resolve((Java.TypeDeclaration)s);
/* 5042 */         break;
/*      */       }
/*      */     }
/*      */     
/*      */     IClass iClassDeclaringContextBlockStatement;
/* 5047 */     if (iClassDeclaringContextBlockStatement == iClassDeclaringMember) { return null;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 5052 */     IClass topLevelIClassEnclosingMember = iClassDeclaringMember;
/* 5053 */     for (IClass c = iClassDeclaringMember.getDeclaringIClass(); c != null; c = c.getDeclaringIClass()) {
/* 5054 */       topLevelIClassEnclosingMember = c;
/*      */     }
/* 5056 */     IClass topLevelIClassEnclosingContextBlockStatement = iClassDeclaringContextBlockStatement;
/*      */     
/* 5058 */     for (IClass c = iClassDeclaringContextBlockStatement.getDeclaringIClass(); 
/* 5059 */         c != null; 
/* 5060 */         c = c.getDeclaringIClass()) {
/* 5061 */       topLevelIClassEnclosingContextBlockStatement = c;
/*      */     }
/* 5063 */     if (topLevelIClassEnclosingMember == topLevelIClassEnclosingContextBlockStatement) { return null;
/*      */     }
/*      */     
/* 5066 */     if (memberAccess == Access.PRIVATE) {
/* 5067 */       return "Private member cannot be accessed from type \"" + iClassDeclaringContextBlockStatement + "\".";
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5074 */     if (Descriptor.areInSamePackage(iClassDeclaringMember.getDescriptor(), iClassDeclaringContextBlockStatement.getDescriptor()))
/*      */     {
/*      */ 
/* 5077 */       return null;
/*      */     }
/* 5079 */     if (memberAccess == Access.DEFAULT) {
/* 5080 */       return "Member with \"" + memberAccess + "\" access cannot be accessed from type \"" + iClassDeclaringContextBlockStatement + "\".";
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5093 */     IClass parentClass = iClassDeclaringContextBlockStatement;
/*      */     do {
/* 5095 */       if (iClassDeclaringMember.isAssignableFrom(parentClass)) {
/* 5096 */         return null;
/*      */       }
/* 5098 */       parentClass = parentClass.getOuterIClass();
/* 5099 */     } while (parentClass != null);
/*      */     
/* 5101 */     return "Protected member cannot be accessed from type \"" + iClassDeclaringContextBlockStatement + "\", which is neither declared in the same package as nor is a subclass of \"" + iClassDeclaringMember + "\".";
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean isAccessible(IClass type, Java.Scope contextScope)
/*      */     throws CompileException
/*      */   {
/* 5118 */     return null == internalCheckAccessible(type, contextScope);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void checkAccessible(IClass type, Java.BlockStatement contextBlockStatement)
/*      */     throws CompileException
/*      */   {
/* 5129 */     String message = internalCheckAccessible(type, contextBlockStatement);
/* 5130 */     if (message != null) { compileError(message, contextBlockStatement.getLocation());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private String internalCheckAccessible(IClass type, Java.Scope contextScope)
/*      */     throws CompileException
/*      */   {
/* 5139 */     IClass iClassDeclaringType = type.getDeclaringIClass();
/*      */     
/*      */ 
/* 5142 */     if (iClassDeclaringType == null) {
/* 5143 */       if (type.getAccess() == Access.PUBLIC) {
/* 5144 */         return null;
/*      */       }
/* 5146 */       if (type.getAccess() == Access.DEFAULT)
/*      */       {
/*      */ 
/*      */ 
/* 5150 */         for (Java.Scope s = contextScope;; s = s.getEnclosingScope()) {
/* 5151 */           if ((s instanceof Java.TypeDeclaration)) {
/* 5152 */             IClass iClassDeclaringContextBlockStatement = resolve((Java.TypeDeclaration)s);
/* 5153 */             break;
/*      */           }
/*      */         }
/*      */         
/*      */         IClass iClassDeclaringContextBlockStatement;
/* 5158 */         String packageDeclaringType = Descriptor.getPackageName(type.getDescriptor());
/* 5159 */         String contextPackage = Descriptor.getPackageName(iClassDeclaringContextBlockStatement.getDescriptor());
/* 5160 */         if (packageDeclaringType == null ? contextPackage != null : !packageDeclaringType.equals(contextPackage))
/*      */         {
/*      */ 
/*      */ 
/* 5164 */           return "\"" + type + "\" is inaccessible from this package"; }
/* 5165 */         return null;
/*      */       }
/*      */       
/* 5168 */       throw new JaninoRuntimeException("\"" + type + "\" has unexpected access \"" + type.getAccess() + "\"");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 5173 */     return internalCheckAccessible(iClassDeclaringType, type.getAccess(), contextScope);
/*      */   }
/*      */   
/*      */   private Java.Type toTypeOrCE(Java.Atom a) throws CompileException {
/* 5177 */     Java.Type result = a.toType();
/* 5178 */     if (result == null) {
/* 5179 */       compileError("Expression \"" + a.toString() + "\" is not a type", a.getLocation());
/* 5180 */       return new Java.SimpleType(a.getLocation(), this.iClassLoader.OBJECT);
/*      */     }
/* 5182 */     return result;
/*      */   }
/*      */   
/* 5185 */   private Java.Rvalue toRvalueOrCE(Java.Atom a) throws CompileException { Java.Rvalue result = a.toRvalue();
/* 5186 */     if (result == null) {
/* 5187 */       compileError("Expression \"" + a.toString() + "\" is not an rvalue", a.getLocation());
/* 5188 */       return new Java.Literal(a.getLocation(), "X");
/*      */     }
/* 5190 */     return result;
/*      */   }
/*      */   
/* 5193 */   public final Java.Lvalue toLvalueOrCE(Java.Atom a) throws CompileException { Java.Lvalue result = a.toLvalue();
/* 5194 */     if (result == null) {
/* 5195 */       compileError("Expression \"" + a.toString() + "\" is not an lvalue", a.getLocation());
/* 5196 */       new Java.Lvalue(a.getLocation()) {
/* 5197 */         public String toString() { return this.val$a.toString(); }
/*      */         
/*      */         public void accept(Visitor.AtomVisitor visitor) {}
/*      */         
/*      */         public void accept(Visitor.RvalueVisitor visitor) {}
/*      */         
/*      */         public void accept(Visitor.LvalueVisitor visitor) {} }; } return result;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   void assignSyntheticParametersToSyntheticFields(Java.ConstructorDeclarator cd)
/*      */     throws CompileException
/*      */   {
/* 5211 */     for (Iterator it = cd.getDeclaringClass().syntheticFields.values().iterator(); it.hasNext();) {
/* 5212 */       IClass.IField sf = (IClass.IField)it.next();
/* 5213 */       Java.LocalVariable syntheticParameter = (Java.LocalVariable)cd.syntheticParameters.get(sf.getName());
/* 5214 */       if (syntheticParameter == null) {
/* 5215 */         throw new JaninoRuntimeException("SNO: Synthetic parameter for synthetic field \"" + sf.getName() + "\" not found");
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 5221 */       Java.ExpressionStatement es = new Java.ExpressionStatement(new Java.Assignment(cd.getLocation(), new Java.FieldAccess(cd.getLocation(), new Java.ThisReference(cd.getLocation()), sf), "=", new Java.LocalVariableAccess(cd.getLocation(), syntheticParameter)));
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5234 */       es.setEnclosingScope(cd);
/* 5235 */       compile(es);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   void initializeInstanceVariablesAndInvokeInstanceInitializers(Java.ConstructorDeclarator cd)
/*      */     throws CompileException
/*      */   {
/* 5246 */     for (int i = 0; i < cd.getDeclaringClass().variableDeclaratorsAndInitializers.size(); i++) {
/* 5247 */       Java.TypeBodyDeclaration tbd = (Java.TypeBodyDeclaration)cd.getDeclaringClass().variableDeclaratorsAndInitializers.get(i);
/*      */       
/*      */ 
/* 5250 */       if (!tbd.isStatic()) {
/* 5251 */         Java.BlockStatement bs = (Java.BlockStatement)tbd;
/* 5252 */         if (!compile(bs)) {
/* 5253 */           compileError("Instance variable declarator or instance initializer does not complete normally", bs.getLocation());
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void leaveStatements(Java.Scope from, Java.Scope to, IClass optionalStackValueType)
/*      */   {
/* 5272 */     for (Java.Scope s = from; s != to; s = s.getEnclosingScope()) {
/* 5273 */       if ((s instanceof Java.BlockStatement)) {
/* 5274 */         leave((Java.BlockStatement)s, optionalStackValueType);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass compileArithmeticBinaryOperation(Java.Locatable l, IClass lhsType, String operator, Java.Rvalue rhs)
/*      */     throws CompileException
/*      */   {
/* 5291 */     return compileArithmeticOperation(l, lhsType, Arrays.asList(new Java.Rvalue[] { rhs }).iterator(), operator);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass compileArithmeticOperation(Java.Locatable l, IClass type, Iterator operands, String operator)
/*      */     throws CompileException
/*      */   {
/* 5312 */     if ((operator == "|") || (operator == "^") || (operator == "&"))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/* 5317 */       int iopcode = operator == "^" ? -126 : operator == "|" ? -128 : operator == "&" ? 126 : Integer.MAX_VALUE;
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */       do
/*      */       {
/* 5324 */         Java.Rvalue operand = (Java.Rvalue)operands.next();
/*      */         
/* 5326 */         if (type == null) {
/* 5327 */           type = compileGetValue(operand);
/*      */         } else {
/* 5329 */           CodeContext.Inserter convertLhsInserter = this.codeContext.newInserter();
/* 5330 */           IClass rhsType = compileGetValue(operand);
/*      */           
/* 5332 */           if ((type.isPrimitiveNumeric()) && (rhsType.isPrimitiveNumeric()))
/*      */           {
/*      */ 
/*      */ 
/* 5336 */             IClass promotedType = binaryNumericPromotion(l, type, convertLhsInserter, rhsType);
/* 5337 */             if (promotedType == IClass.INT) {
/* 5338 */               writeOpcode(l, iopcode);
/*      */             }
/* 5340 */             else if (promotedType == IClass.LONG) {
/* 5341 */               writeOpcode(l, iopcode + 1);
/*      */             }
/*      */             else {
/* 5344 */               compileError("Operator \"" + operator + "\" not defined on types \"" + type + "\" and \"" + rhsType + "\"", l.getLocation());
/*      */             }
/*      */             
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5354 */             type = promotedType;
/*      */           }
/* 5356 */           else if (((type == IClass.BOOLEAN) && (getUnboxedType(rhsType) == IClass.BOOLEAN)) || ((getUnboxedType(type) == IClass.BOOLEAN) && (rhsType == IClass.BOOLEAN)))
/*      */           {
/*      */ 
/*      */ 
/* 5360 */             IClassLoader icl = this.iClassLoader;
/* 5361 */             if (type == icl.BOOLEAN) {
/* 5362 */               this.codeContext.pushInserter(convertLhsInserter);
/*      */               try {
/* 5364 */                 unboxingConversion(l, icl.BOOLEAN, IClass.BOOLEAN);
/*      */               } finally {
/* 5366 */                 this.codeContext.popInserter();
/*      */               }
/*      */             }
/* 5369 */             if (rhsType == icl.BOOLEAN) {
/* 5370 */               unboxingConversion(l, icl.BOOLEAN, IClass.BOOLEAN);
/*      */             }
/* 5372 */             writeOpcode(l, iopcode);
/* 5373 */             type = IClass.BOOLEAN;
/*      */           }
/*      */           else {
/* 5376 */             compileError("Operator \"" + operator + "\" not defined on types \"" + type + "\" and \"" + rhsType + "\"", l.getLocation());
/*      */             
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5385 */             type = IClass.INT;
/*      */           }
/*      */         }
/* 5388 */       } while (operands.hasNext());
/* 5389 */       return type;
/*      */     }
/*      */     
/* 5392 */     if ((operator == "*") || (operator == "/") || (operator == "%") || (operator == "+") || (operator == "-"))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5399 */       int iopcode = operator == "-" ? 100 : operator == "+" ? 96 : operator == "%" ? 112 : operator == "/" ? 108 : operator == "*" ? 104 : Integer.MAX_VALUE;
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       do
/*      */       {
/* 5408 */         Java.Rvalue operand = (Java.Rvalue)operands.next();
/*      */         
/* 5410 */         IClass operandType = getType(operand);
/* 5411 */         IClassLoader icl = this.iClassLoader;
/*      */         
/*      */ 
/* 5414 */         if ((operator == "+") && ((type == icl.STRING) || (operandType == icl.STRING))) {
/* 5415 */           return compileStringConcatenation(l, type, operand, operands);
/*      */         }
/*      */         
/* 5418 */         if (type == null) {
/* 5419 */           type = compileGetValue(operand);
/*      */         } else {
/* 5421 */           CodeContext.Inserter convertLhsInserter = this.codeContext.newInserter();
/* 5422 */           IClass rhsType = compileGetValue(operand);
/*      */           
/* 5424 */           type = binaryNumericPromotion(l, type, convertLhsInserter, rhsType);
/*      */           int opcode;
/*      */           int opcode;
/* 5427 */           if (type == IClass.INT) {
/* 5428 */             opcode = iopcode;
/*      */           } else { int opcode;
/* 5430 */             if (type == IClass.LONG) {
/* 5431 */               opcode = iopcode + 1;
/*      */             } else { int opcode;
/* 5433 */               if (type == IClass.FLOAT) {
/* 5434 */                 opcode = iopcode + 2;
/*      */               } else { int opcode;
/* 5436 */                 if (type == IClass.DOUBLE) {
/* 5437 */                   opcode = iopcode + 3;
/*      */                 }
/*      */                 else {
/* 5440 */                   compileError("Unexpected promoted type \"" + type + "\"", l.getLocation());
/* 5441 */                   opcode = iopcode;
/*      */                 } } } }
/* 5443 */           writeOpcode(l, opcode);
/*      */         }
/* 5445 */       } while (operands.hasNext());
/* 5446 */       return type;
/*      */     }
/*      */     
/* 5449 */     if ((operator == "<<") || (operator == ">>") || (operator == ">>>"))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/* 5454 */       int iopcode = operator == ">>>" ? 124 : operator == ">>" ? 122 : operator == "<<" ? 120 : Integer.MAX_VALUE;
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */       do
/*      */       {
/* 5461 */         Java.Rvalue operand = (Java.Rvalue)operands.next();
/*      */         
/* 5463 */         if (type == null) {
/* 5464 */           type = compileGetValue(operand);
/*      */         } else {
/* 5466 */           CodeContext.Inserter convertLhsInserter = this.codeContext.newInserter();
/* 5467 */           IClass rhsType = compileGetValue(operand);
/*      */           
/*      */ 
/* 5470 */           this.codeContext.pushInserter(convertLhsInserter);
/*      */           IClass promotedLhsType;
/* 5472 */           try { promotedLhsType = unaryNumericPromotion(l, type);
/*      */           } finally {
/* 5474 */             this.codeContext.popInserter();
/*      */           }
/* 5476 */           if ((promotedLhsType != IClass.INT) && (promotedLhsType != IClass.LONG)) {
/* 5477 */             compileError("Shift operation not allowed on operand type \"" + type + "\"", l.getLocation());
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 5483 */           IClass promotedRhsType = unaryNumericPromotion(l, rhsType);
/* 5484 */           if ((promotedRhsType != IClass.INT) && (promotedRhsType != IClass.LONG)) {
/* 5485 */             compileError("Shift distance of type \"" + rhsType + "\" is not allowed", l.getLocation());
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 5491 */           if (promotedRhsType == IClass.LONG) { writeOpcode(l, -120);
/*      */           }
/* 5493 */           writeOpcode(l, promotedLhsType == IClass.LONG ? iopcode + 1 : iopcode);
/* 5494 */           type = promotedLhsType;
/*      */         }
/* 5496 */       } while (operands.hasNext());
/* 5497 */       return type;
/*      */     }
/*      */     
/* 5500 */     throw new JaninoRuntimeException("Unexpected operator \"" + operator + "\"");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass compileStringConcatenation(Java.Locatable l, IClass type, Java.Rvalue operand, Iterator operands)
/*      */     throws CompileException
/*      */   {
/*      */     boolean operandOnStack;
/*      */     
/*      */ 
/*      */     boolean operandOnStack;
/*      */     
/*      */ 
/* 5515 */     if (type != null) {
/* 5516 */       stringConversion(l, type);
/* 5517 */       operandOnStack = true;
/*      */     }
/*      */     else {
/* 5520 */       operandOnStack = false;
/*      */     }
/*      */     
/*      */ 
/* 5524 */     List tmp = new ArrayList();
/*      */     do {
/* 5526 */       Object cv = getConstantValue(operand);
/* 5527 */       if (cv == null)
/*      */       {
/* 5529 */         Java.Rvalue finalOperand = operand;
/* 5530 */         tmp.add(new Compilable() { private final Java.Locatable val$l;
/*      */           
/* 5532 */           public void compile() throws CompileException { UnitCompiler.this.stringConversion(this.val$l, UnitCompiler.access$13400(UnitCompiler.this, this.val$finalOperand));
/*      */           }
/*      */ 
/* 5535 */         });
/* 5536 */         operand = operands.hasNext() ? (Java.Rvalue)operands.next() : null;
/*      */       }
/*      */       else
/*      */       {
/* 5540 */         if (operands.hasNext()) {
/* 5541 */           operand = (Java.Rvalue)operands.next();
/* 5542 */           Object cv2 = getConstantValue(operand);
/* 5543 */           if (cv2 != null) {
/* 5544 */             StringBuffer sb = new StringBuffer(cv.toString()).append(cv2);
/*      */             for (;;) {
/* 5546 */               if (!operands.hasNext()) {
/* 5547 */                 operand = null;
/* 5548 */                 break;
/*      */               }
/* 5550 */               operand = (Java.Rvalue)operands.next();
/* 5551 */               Object cv3 = getConstantValue(operand);
/* 5552 */               if (cv3 == null) break;
/* 5553 */               sb.append(cv3);
/*      */             }
/* 5555 */             cv = sb.toString();
/*      */           }
/*      */         }
/*      */         else {
/* 5559 */           operand = null;
/*      */         }
/*      */         
/* 5562 */         String[] ss = makeUTF8Able(cv.toString());
/* 5563 */         for (int i = 0; i < ss.length; i++) {
/* 5564 */           String s = ss[i];
/* 5565 */           tmp.add(new Compilable() { private final Java.Rvalue val$finalOperand;
/*      */             
/* 5567 */             public void compile() throws CompileException { UnitCompiler.this.pushConstant(this.val$l, this.val$s);
/*      */             }
/*      */           });
/*      */         }
/*      */       }
/* 5572 */     } while (operand != null);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 5577 */     if (tmp.size() <= (operandOnStack ? 2 : 3))
/*      */     {
/*      */ 
/* 5580 */       for (Iterator it = tmp.iterator(); it.hasNext();) {
/* 5581 */         Compilable c = (Compilable)it.next();
/* 5582 */         c.compile();
/*      */         
/*      */ 
/* 5585 */         if (operandOnStack) {
/* 5586 */           writeOpcode(l, -74);
/* 5587 */           writeConstantMethodrefInfo("Ljava/lang/String;", "concat", "(Ljava/lang/String;)Ljava/lang/String;");
/*      */ 
/*      */ 
/*      */         }
/*      */         else
/*      */         {
/*      */ 
/* 5594 */           operandOnStack = true;
/*      */         }
/*      */       }
/* 5597 */       return this.iClassLoader.STRING;
/*      */     }
/*      */     
/*      */ 
/* 5601 */     Iterator it = tmp.iterator();
/*      */     
/* 5603 */     String stringBuilferFD = this.isStringBuilderAvailable ? "Ljava/lang/StringBuilder;" : "Ljava/lang/StringBuffer;";
/*      */     
/* 5605 */     if (operandOnStack) {
/* 5606 */       writeOpcode(l, -69);
/* 5607 */       writeConstantClassInfo(stringBuilferFD);
/* 5608 */       writeOpcode(l, 90);
/* 5609 */       writeOpcode(l, 95);
/*      */     }
/*      */     else {
/* 5612 */       writeOpcode(l, -69);
/* 5613 */       writeConstantClassInfo(stringBuilferFD);
/* 5614 */       writeOpcode(l, 89);
/* 5615 */       ((Compilable)it.next()).compile();
/*      */     }
/* 5617 */     writeOpcode(l, -73);
/* 5618 */     writeConstantMethodrefInfo(stringBuilferFD, "<init>", "(Ljava/lang/String;)V");
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 5623 */     while (it.hasNext()) {
/* 5624 */       ((Compilable)it.next()).compile();
/*      */       
/*      */ 
/* 5627 */       writeOpcode(l, -74);
/* 5628 */       writeConstantMethodrefInfo(stringBuilferFD, "append", "(Ljava/lang/String;)" + stringBuilferFD);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5636 */     writeOpcode(l, -74);
/* 5637 */     writeConstantMethodrefInfo(stringBuilferFD, "toString", "()Ljava/lang/String;");
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 5642 */     return this.iClassLoader.STRING;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void stringConversion(Java.Locatable l, IClass sourceType)
/*      */   {
/* 5653 */     writeOpcode(l, -72);
/* 5654 */     writeConstantMethodrefInfo("Ljava/lang/String;", "valueOf", "(" + ((sourceType == IClass.BYTE) || (sourceType == IClass.SHORT) || (sourceType == IClass.INT) ? "I" : (sourceType == IClass.BOOLEAN) || (sourceType == IClass.CHAR) || (sourceType == IClass.LONG) || (sourceType == IClass.FLOAT) || (sourceType == IClass.DOUBLE) ? sourceType.getDescriptor() : "Ljava/lang/Object;") + ")" + "Ljava/lang/String;");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final Java.Locatable val$l;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private final String val$s;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private final Java.Atom val$a;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private final short val$lntani;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private final short val$lvtani;
/*      */   
/*      */ 
/*      */ 
/*      */   private void invokeConstructor(Java.Locatable l, Java.Scope scope, Java.Rvalue optionalEnclosingInstance, IClass targetClass, Java.Rvalue[] arguments)
/*      */     throws CompileException
/*      */   {
/* 5688 */     IClass.IConstructor[] iConstructors = targetClass.getDeclaredIConstructors();
/* 5689 */     if (iConstructors.length == 0) {
/* 5690 */       throw new JaninoRuntimeException("SNO: Target class \"" + targetClass.getDescriptor() + "\" has no constructors");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 5695 */     IClass.IConstructor iConstructor = (IClass.IConstructor)findMostSpecificIInvocable(l, iConstructors, arguments, scope);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5703 */     IClass[] thrownExceptions = iConstructor.getThrownExceptions();
/* 5704 */     for (int i = 0; i < thrownExceptions.length; i++) {
/* 5705 */       checkThrownException(l, thrownExceptions[i], scope);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5713 */     if (optionalEnclosingInstance != null) {
/* 5714 */       IClass outerIClass = targetClass.getOuterIClass();
/* 5715 */       if (outerIClass != null) {
/* 5716 */         IClass eiic = compileGetValue(optionalEnclosingInstance);
/* 5717 */         if (!outerIClass.isAssignableFrom(eiic)) {
/* 5718 */           compileError("Type of enclosing instance (\"" + eiic + "\") is not assignable to \"" + outerIClass + "\"", l.getLocation());
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5728 */     IClass.IField[] syntheticFields = targetClass.getSyntheticIFields();
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5734 */     Java.Scope s = scope;
/* 5735 */     while (!(s instanceof Java.TypeBodyDeclaration)) s = s.getEnclosingScope();
/* 5736 */     Java.TypeBodyDeclaration scopeTBD = (Java.TypeBodyDeclaration)s;
/* 5737 */     Java.TypeDeclaration scopeTypeDeclaration = scopeTBD.getDeclaringType();
/*      */     
/*      */ 
/* 5740 */     if (!(scopeTypeDeclaration instanceof Java.ClassDeclaration)) {
/* 5741 */       if (syntheticFields.length > 0) {
/* 5742 */         throw new JaninoRuntimeException("SNO: Target class has synthetic fields");
/*      */       }
/*      */     } else {
/* 5745 */       Java.ClassDeclaration scopeClassDeclaration = (Java.ClassDeclaration)scopeTypeDeclaration;
/* 5746 */       for (int i = 0; i < syntheticFields.length; i++) {
/* 5747 */         IClass.IField sf = syntheticFields[i];
/* 5748 */         if (sf.getName().startsWith("val$")) {
/* 5749 */           IClass.IField eisf = (IClass.IField)scopeClassDeclaration.syntheticFields.get(sf.getName());
/* 5750 */           if (eisf != null) {
/* 5751 */             if ((scopeTBD instanceof Java.MethodDeclarator)) {
/* 5752 */               load(l, resolve(scopeClassDeclaration), 0);
/* 5753 */               writeOpcode(l, -76);
/* 5754 */               writeConstantFieldrefInfo(resolve(scopeClassDeclaration).getDescriptor(), sf.getName(), sf.getDescriptor());
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */             }
/* 5760 */             else if ((scopeTBD instanceof Java.ConstructorDeclarator)) {
/* 5761 */               Java.ConstructorDeclarator constructorDeclarator = (Java.ConstructorDeclarator)scopeTBD;
/* 5762 */               Java.LocalVariable syntheticParameter = (Java.LocalVariable)constructorDeclarator.syntheticParameters.get(sf.getName());
/*      */               
/*      */ 
/* 5765 */               if (syntheticParameter == null) {
/* 5766 */                 compileError("Compiler limitation: Constructor cannot access local variable \"" + sf.getName().substring(4) + "\" declared in an enclosing block because none of the methods accesses it. " + "As a workaround, declare a dummy method that accesses the local variable.", l.getLocation());
/*      */                 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5772 */                 writeOpcode(l, 1);
/*      */               } else {
/* 5774 */                 load(l, syntheticParameter);
/*      */               }
/*      */             } else {
/* 5777 */               compileError("Compiler limitation: Initializers cannot access local variables declared in an enclosing block.", l.getLocation());
/*      */               
/*      */ 
/*      */ 
/* 5781 */               writeOpcode(l, 1);
/*      */             }
/*      */           } else {
/* 5784 */             String localVariableName = sf.getName().substring(4);
/*      */             
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5790 */             for (Java.Scope s = scope; (s instanceof Java.BlockStatement); s = s.getEnclosingScope()) {
/* 5791 */               Java.BlockStatement bs = (Java.BlockStatement)s;
/* 5792 */               Java.Scope es = bs.getEnclosingScope();
/*      */               List statements;
/* 5794 */               List statements; if ((es instanceof Java.Block)) {
/* 5795 */                 statements = ((Java.Block)es).statements;
/*      */               } else {
/* 5797 */                 if (!(es instanceof Java.FunctionDeclarator)) continue;
/* 5798 */                 statements = ((Java.FunctionDeclarator)es).optionalStatements;
/*      */               }
/*      */               
/*      */ 
/*      */ 
/*      */ 
/* 5804 */               Iterator it = statements.iterator();
/* 5805 */               for (;;) { Java.BlockStatement bs2 = (Java.BlockStatement)it.next();
/* 5806 */                 if (bs2 == bs) break;
/* 5807 */                 if ((bs2 instanceof Java.LocalVariableDeclarationStatement)) {
/* 5808 */                   Java.LocalVariableDeclarationStatement lvds = (Java.LocalVariableDeclarationStatement)bs2;
/*      */                   
/*      */ 
/* 5811 */                   Java.VariableDeclarator[] vds = lvds.variableDeclarators;
/* 5812 */                   for (int j = 0; j < vds.length; j++) {
/* 5813 */                     if (vds[j].name.equals(localVariableName)) {
/* 5814 */                       Java.LocalVariable lv = getLocalVariable(lvds, vds[j]);
/*      */                       
/*      */                       break label817;
/*      */                     }
/*      */                   }
/*      */                 }
/*      */               }
/*      */             }
/*      */             
/* 5823 */             while (!(s instanceof Java.FunctionDeclarator)) s = s.getEnclosingScope();
/* 5824 */             Java.FunctionDeclarator fd = (Java.FunctionDeclarator)s;
/* 5825 */             for (int j = 0; j < fd.formalParameters.length; j++) {
/* 5826 */               Java.FunctionDeclarator.FormalParameter fp = fd.formalParameters[j];
/* 5827 */               if (fp.name.equals(localVariableName)) {
/* 5828 */                 Java.LocalVariable lv = getLocalVariable(fp);
/*      */                 break label817;
/*      */               }
/*      */             }
/* 5832 */             throw new JaninoRuntimeException("SNO: Synthetic field \"" + sf.getName() + "\" neither maps a synthetic field of an enclosing instance nor a local variable");
/*      */             
/*      */             label817:
/*      */             
/*      */             Java.LocalVariable lv;
/*      */             
/* 5838 */             load(l, lv);
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/* 5845 */     IClass[] parameterTypes = iConstructor.getParameterTypes();
/* 5846 */     for (int i = 0; i < arguments.length; i++) {
/* 5847 */       assignmentConversion(l, compileGetValue(arguments[i]), parameterTypes[i], getConstantValue(arguments[i]));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5858 */     writeOpcode(l, -73);
/* 5859 */     writeConstantMethodrefInfo(targetClass.getDescriptor(), "<init>", iConstructor.getDescriptor());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   IClass.IField[] getIFields(Java.FieldDeclaration fd)
/*      */   {
/* 5867 */     IClass.IField[] res = new IClass.IField[fd.variableDeclarators.length];
/* 5868 */     for (int i = 0; i < res.length; i++) {
/* 5869 */       Java.VariableDeclarator vd = fd.variableDeclarators[i]; IClass 
/* 5870 */         tmp40_37 = resolve(fd.getDeclaringType());tmp40_37.getClass();res[i = new IClass.IField(tmp40_37) {
/*      */         private final Java.FieldDeclaration val$fd;
/*      */         private final Java.VariableDeclarator val$vd;
/*      */         
/* 5874 */         public Access getAccess() { switch (this.val$fd.modifiers & 0x7) {
/*      */           case 2: 
/* 5876 */             return Access.PRIVATE;
/*      */           case 4: 
/* 5878 */             return Access.PROTECTED;
/*      */           case 0: 
/* 5880 */             return Access.DEFAULT;
/*      */           case 1: 
/* 5882 */             return Access.PUBLIC;
/*      */           }
/* 5884 */           throw new JaninoRuntimeException("Invalid access");
/*      */         }
/*      */         
/*      */ 
/*      */ 
/* 5889 */         public boolean isStatic() { return (this.val$fd.modifiers & 0x8) != 0; }
/*      */         
/* 5891 */         public IClass getType() throws CompileException { return UnitCompiler.this.getType(this.val$fd.type).getArrayIClass(this.val$vd.brackets, UnitCompiler.this.iClassLoader.OBJECT); }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/* 5896 */         public String getName() { return this.val$vd.name; }
/*      */         
/* 5898 */         public Object getConstantValue() throws CompileException { if (((this.val$fd.modifiers & 0x10) != 0) && ((this.val$vd.optionalInitializer instanceof Java.Rvalue)))
/*      */           {
/*      */ 
/*      */ 
/* 5902 */             Object constantInitializerValue = UnitCompiler.this.getConstantValue((Java.Rvalue)this.val$vd.optionalInitializer);
/*      */             
/*      */ 
/* 5905 */             if (constantInitializerValue != null) { return UnitCompiler.this.assignmentConversion((Java.Locatable)this.val$vd.optionalInitializer, constantInitializerValue, getType());
/*      */             }
/*      */           }
/*      */           
/*      */ 
/*      */ 
/* 5911 */           return null;
/*      */         }
/*      */       };
/*      */     }
/* 5915 */     return res;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   Java.ArrayInitializerOrRvalue getNonConstantFinalInitializer(Java.FieldDeclaration fd, Java.VariableDeclarator vd)
/*      */     throws CompileException
/*      */   {
/* 5930 */     if (vd.optionalInitializer == null) { return null;
/*      */     }
/*      */     
/* 5933 */     if (((fd.modifiers & 0x8) != 0) && ((fd.modifiers & 0x10) != 0) && ((vd.optionalInitializer instanceof Java.Rvalue)) && (getConstantValue((Java.Rvalue)vd.optionalInitializer) != null))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/* 5938 */       return null;
/*      */     }
/* 5940 */     return vd.optionalInitializer;
/*      */   }
/*      */   
/*      */   private Java.Atom reclassify(Java.AmbiguousName an) throws CompileException {
/* 5944 */     if (an.reclassified == null) {
/* 5945 */       an.reclassified = reclassifyName(an.getLocation(), an.getEnclosingBlockStatement(), an.identifiers, an.n);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 5951 */     return an.reclassified;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private Java.Atom reclassifyName(Location location, Java.Scope scope, String[] identifiers, int n)
/*      */     throws CompileException
/*      */   {
/* 5972 */     if (n == 1) { return reclassifyName(location, scope, identifiers[0]);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 5979 */     Java.Atom lhs = reclassifyName(location, scope, identifiers, n - 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 5984 */     String rhs = identifiers[(n - 1)];
/*      */     
/*      */ 
/*      */ 
/* 5988 */     if ((lhs instanceof Java.Package)) {
/* 5989 */       String className = ((Java.Package)lhs).name + '.' + rhs;
/* 5990 */       IClass result = findClassByName(location, className);
/* 5991 */       if (result != null) { return new Java.SimpleType(location, result);
/*      */       }
/* 5993 */       return new Java.Package(location, className);
/*      */     }
/*      */     
/*      */ 
/* 5997 */     if ((rhs.equals("length")) && (getType(lhs).isArray())) {
/* 5998 */       Java.ArrayLength al = new Java.ArrayLength(location, toRvalueOrCE(lhs));
/* 5999 */       if (!(scope instanceof Java.BlockStatement)) {
/* 6000 */         compileError("\".length\" only allowed in expression context");
/* 6001 */         return al;
/*      */       }
/* 6003 */       al.setEnclosingBlockStatement((Java.BlockStatement)scope);
/* 6004 */       return al;
/*      */     }
/*      */     
/* 6007 */     IClass lhsType = getType(lhs);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6013 */     IClass.IField field = findIField(lhsType, rhs, location);
/* 6014 */     if (field != null)
/*      */     {
/*      */ 
/* 6017 */       Java.FieldAccess fa = new Java.FieldAccess(location, lhs, field);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 6022 */       fa.setEnclosingBlockStatement((Java.BlockStatement)scope);
/* 6023 */       return fa;
/*      */     }
/*      */     
/*      */ 
/* 6027 */     IClass[] classes = lhsType.getDeclaredIClasses();
/* 6028 */     for (int i = 0; i < classes.length; i++) {
/* 6029 */       IClass memberType = classes[i];
/* 6030 */       String name = Descriptor.toClassName(memberType.getDescriptor());
/* 6031 */       name = name.substring(name.lastIndexOf('$') + 1);
/* 6032 */       if (name.equals(rhs))
/*      */       {
/*      */ 
/*      */ 
/* 6036 */         return new Java.SimpleType(location, memberType);
/*      */       }
/*      */     }
/*      */     
/* 6040 */     compileError("\"" + rhs + "\" is neither a method, a field, nor a member class of \"" + lhsType + "\"", location);
/*      */     
/*      */ 
/*      */ 
/* 6044 */     new Java.Atom(location) {
/* 6045 */       public String toString() { return Java.join(this.val$identifiers, "."); }
/*      */       
/*      */       public final void accept(Visitor.AtomVisitor visitor) {}
/*      */     };
/*      */   }
/*      */   
/* 6051 */   private IClass findClassByName(Location location, String className) throws CompileException { IClass res = findClass(className);
/* 6052 */     if (res != null) return res;
/*      */     try {
/* 6054 */       return this.iClassLoader.loadIClass(Descriptor.fromClassName(className));
/*      */     } catch (ClassNotFoundException ex) {
/* 6056 */       if ((ex.getException() instanceof CompileException)) throw ((CompileException)ex.getException());
/* 6057 */       throw new CompileException(className, location, ex);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private Java.Atom reclassifyName(Location location, Java.Scope scope, String identifier)
/*      */     throws CompileException
/*      */   {
/* 6075 */     Java.BlockStatement scopeBlockStatement = null;
/* 6076 */     Java.TypeBodyDeclaration scopeTBD = null;
/* 6077 */     Java.AbstractTypeDeclaration scopeTypeDeclaration = null;
/*      */     
/*      */ 
/* 6080 */     Java.Scope s = scope;
/* 6081 */     if ((s instanceof Java.BlockStatement)) { scopeBlockStatement = (Java.BlockStatement)s;
/*      */     }
/*      */     
/* 6084 */     while ((((s instanceof Java.BlockStatement)) || ((s instanceof Java.CatchClause))) && (!(s instanceof Java.TypeBodyDeclaration)))
/* 6085 */       s = s.getEnclosingScope();
/* 6086 */     if ((s instanceof Java.TypeBodyDeclaration)) {
/* 6087 */       scopeTBD = (Java.TypeBodyDeclaration)s;
/* 6088 */       s = s.getEnclosingScope();
/*      */     }
/* 6090 */     if ((s instanceof Java.TypeDeclaration)) {
/* 6091 */       scopeTypeDeclaration = (Java.AbstractTypeDeclaration)s;
/* 6092 */       s = s.getEnclosingScope();
/*      */     }
/* 6094 */     while (!(s instanceof Java.CompilationUnit)) s = s.getEnclosingScope();
/* 6095 */     Java.CompilationUnit scopeCompilationUnit = (Java.CompilationUnit)s;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6103 */     Java.Scope s = scope;
/* 6104 */     if ((s instanceof Java.BlockStatement)) {
/* 6105 */       Java.BlockStatement bs = (Java.BlockStatement)s;
/* 6106 */       Java.LocalVariable lv = bs.findLocalVariable(identifier);
/* 6107 */       if (lv != null) {
/* 6108 */         Java.LocalVariableAccess lva = new Java.LocalVariableAccess(location, lv);
/* 6109 */         lva.setEnclosingBlockStatement(bs);
/* 6110 */         return lva;
/*      */       }
/* 6112 */       s = s.getEnclosingScope();
/*      */     }
/* 6114 */     while (((s instanceof Java.BlockStatement)) || ((s instanceof Java.CatchClause))) s = s.getEnclosingScope();
/* 6115 */     if ((s instanceof Java.FunctionDeclarator)) {
/* 6116 */       s = s.getEnclosingScope();
/*      */     }
/* 6118 */     if ((s instanceof Java.InnerClassDeclaration)) {
/* 6119 */       Java.InnerClassDeclaration icd = (Java.InnerClassDeclaration)s;
/* 6120 */       s = s.getEnclosingScope();
/* 6121 */       if ((s instanceof Java.AnonymousClassDeclaration)) s = s.getEnclosingScope();
/* 6122 */       while ((s instanceof Java.BlockStatement)) {
/* 6123 */         Java.LocalVariable lv = ((Java.BlockStatement)s).findLocalVariable(identifier);
/* 6124 */         if (lv != null) {
/* 6125 */           if (!lv.finaL) {
/* 6126 */             compileError("Cannot access non-final local variable \"" + identifier + "\" from inner class");
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 6132 */           IClass lvType = lv.type;
/* 6133 */           IClass.IField iField = new SimpleIField(resolve(icd), "val$" + identifier, lvType);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 6138 */           icd.defineSyntheticField(iField);
/* 6139 */           Java.FieldAccess fa = new Java.FieldAccess(location, new Java.QualifiedThisReference(location, new Java.SimpleType(location, resolve(icd))), iField);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6147 */           fa.setEnclosingBlockStatement((Java.BlockStatement)scope);
/* 6148 */           return fa;
/*      */         }
/* 6150 */         s = s.getEnclosingScope();
/* 6151 */         while ((s instanceof Java.BlockStatement)) s = s.getEnclosingScope();
/* 6152 */         if (!(s instanceof Java.FunctionDeclarator)) break;
/* 6153 */         s = s.getEnclosingScope();
/* 6154 */         if (!(s instanceof Java.InnerClassDeclaration)) break;
/* 6155 */         icd = (Java.InnerClassDeclaration)s;
/* 6156 */         s = s.getEnclosingScope();
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 6162 */     Java.BlockStatement enclosingBlockStatement = null;
/* 6163 */     for (Java.Scope s = scope; !(s instanceof Java.CompilationUnit); s = s.getEnclosingScope()) {
/* 6164 */       if (((s instanceof Java.BlockStatement)) && (enclosingBlockStatement == null)) {
/* 6165 */         enclosingBlockStatement = (Java.BlockStatement)s;
/*      */       }
/* 6167 */       if ((s instanceof Java.TypeDeclaration)) {
/* 6168 */         Java.AbstractTypeDeclaration enclosingTypeDecl = (Java.AbstractTypeDeclaration)s;
/* 6169 */         IClass etd = resolve(enclosingTypeDecl);
/* 6170 */         IClass.IField f = findIField(etd, identifier, location);
/* 6171 */         if (f != null) {
/* 6172 */           if (f.isStatic()) {
/* 6173 */             warning("IASF", "Implicit access to static field \"" + identifier + "\" of declaring class (better write \"" + f.getDeclaringIClass() + '.' + f.getName() + "\")", location);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           }
/* 6183 */           else if (f.getDeclaringIClass() == etd) {
/* 6184 */             warning("IANSF", "Implicit access to non-static field \"" + identifier + "\" of declaring class (better write \"this." + f.getName() + "\")", location);
/*      */ 
/*      */ 
/*      */           }
/*      */           else
/*      */           {
/*      */ 
/*      */ 
/* 6192 */             warning("IANSFEI", "Implicit access to non-static field \"" + identifier + "\" of enclosing instance (better write \"" + f.getDeclaringIClass() + ".this." + f.getName() + "\")", location);
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6203 */           Java.SimpleType ct = new Java.SimpleType(scopeTypeDeclaration.getLocation(), etd);
/*      */           Java.Atom lhs;
/* 6205 */           Java.Atom lhs; if (scopeTBD.isStatic())
/*      */           {
/*      */ 
/* 6208 */             lhs = ct;
/*      */           }
/*      */           else
/*      */           {
/*      */             Java.Atom lhs;
/* 6213 */             if (f.isStatic())
/*      */             {
/*      */ 
/* 6216 */               lhs = ct;
/*      */             }
/*      */             else
/*      */             {
/* 6220 */               lhs = new Java.QualifiedThisReference(location, ct);
/*      */             }
/*      */           }
/* 6223 */           Java.Rvalue res = new Java.FieldAccess(location, lhs, f);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 6228 */           res.setEnclosingBlockStatement(enclosingBlockStatement);
/* 6229 */           return res;
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 6236 */     List l = (List)this.singleStaticImports.get(identifier);
/* 6237 */     Iterator it; if (l != null) {
/* 6238 */       for (it = l.iterator(); it.hasNext();) {
/* 6239 */         Object o = it.next();
/* 6240 */         if ((o instanceof IClass.IField)) {
/* 6241 */           Java.FieldAccess fieldAccess = new Java.FieldAccess(location, new Java.SimpleType(location, ((IClass.IField)o).getDeclaringIClass()), (IClass.IField)o);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 6246 */           fieldAccess.setEnclosingBlockStatement(enclosingBlockStatement);
/* 6247 */           return fieldAccess;
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 6255 */     IClass.IField importedField = null;
/* 6256 */     for (Iterator it = this.staticImportsOnDemand.iterator(); it.hasNext();) {
/* 6257 */       IClass iClass = (IClass)it.next();
/* 6258 */       IClass.IField f = iClass.getDeclaredIField(identifier);
/* 6259 */       if (f != null)
/*      */       {
/* 6261 */         if (isAccessible(f, enclosingBlockStatement))
/*      */         {
/* 6263 */           if (importedField != null) {
/* 6264 */             compileError("Ambiguous static field import: \"" + importedField.toString() + "\" vs. \"" + f.toString() + "\"");
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6272 */           importedField = f;
/*      */         } }
/*      */     }
/* 6275 */     if (importedField != null) {
/* 6276 */       if (!importedField.isStatic()) compileError("Cannot static-import non-static field");
/* 6277 */       Java.FieldAccess fieldAccess = new Java.FieldAccess(location, new Java.SimpleType(location, importedField.getDeclaringIClass()), importedField);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 6282 */       fieldAccess.setEnclosingBlockStatement(enclosingBlockStatement);
/* 6283 */       return fieldAccess;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 6288 */     if (identifier.equals("java")) { return new Java.Package(location, identifier);
/*      */     }
/*      */     
/*      */ 
/* 6292 */     Java.LocalClassDeclaration lcd = findLocalClassDeclaration(scope, identifier);
/* 6293 */     if (lcd != null) { return new Java.SimpleType(location, resolve(lcd));
/*      */     }
/*      */     
/*      */ 
/* 6297 */     if (scopeTypeDeclaration != null) {
/* 6298 */       IClass memberType = findMemberType(resolve(scopeTypeDeclaration), identifier, location);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 6303 */       if (memberType != null) { return new Java.SimpleType(location, memberType);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/* 6308 */     IClass iClass = importSingleType(identifier, location);
/* 6309 */     if (iClass != null) { return new Java.SimpleType(location, iClass);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6316 */     Java.PackageMemberTypeDeclaration pmtd = scopeCompilationUnit.getPackageMemberTypeDeclaration(identifier);
/* 6317 */     if (pmtd != null) { return new Java.SimpleType(location, resolve((Java.AbstractTypeDeclaration)pmtd));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 6323 */     String className = scopeCompilationUnit.optionalPackageDeclaration.packageName + '.' + identifier;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 6328 */     IClass result = findClassByName(location, className);
/* 6329 */     if (result != null) { return new Java.SimpleType(location, result);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 6334 */     IClass importedClass = importTypeOnDemand(identifier, location);
/* 6335 */     if (importedClass != null) {
/* 6336 */       return new Java.SimpleType(location, importedClass);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 6342 */     List l = (List)this.singleStaticImports.get(identifier);
/* 6343 */     Iterator it; if (l != null) {
/* 6344 */       for (it = l.iterator(); it.hasNext();) {
/* 6345 */         Object o = it.next();
/* 6346 */         if ((o instanceof IClass)) { return new Java.SimpleType(null, (IClass)o);
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 6353 */     IClass importedType = null;
/* 6354 */     for (Iterator it = this.staticImportsOnDemand.iterator(); it.hasNext();) {
/* 6355 */       IClass ic = (IClass)it.next();
/* 6356 */       IClass[] memberTypes = ic.getDeclaredIClasses();
/* 6357 */       for (int i = 0; i < memberTypes.length; i++) {
/* 6358 */         IClass mt = memberTypes[i];
/* 6359 */         if ((isAccessible(mt, scopeBlockStatement)) && 
/* 6360 */           (mt.getDescriptor().endsWith('$' + identifier + ';'))) {
/* 6361 */           if (importedType != null) {
/* 6362 */             compileError("Ambiguous static type import: \"" + importedType.toString() + "\" vs. \"" + mt.toString() + "\"");
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6370 */           importedType = mt;
/*      */         }
/*      */       }
/*      */     }
/* 6374 */     if (importedType != null) { return new Java.SimpleType(null, importedType);
/*      */     }
/*      */     
/*      */ 
/* 6378 */     return new Java.Package(location, identifier);
/*      */   }
/*      */   
/*      */   private void determineValue(Java.FieldAccessExpression fae) throws CompileException {
/* 6382 */     if (fae.value != null) { return;
/*      */     }
/* 6384 */     IClass lhsType = getType(fae.lhs);
/*      */     
/* 6386 */     if ((fae.fieldName.equals("length")) && (lhsType.isArray())) {
/* 6387 */       fae.value = new Java.ArrayLength(fae.getLocation(), toRvalueOrCE(fae.lhs));
/*      */ 
/*      */     }
/*      */     else
/*      */     {
/* 6392 */       IClass.IField iField = findIField(lhsType, fae.fieldName, fae.getLocation());
/* 6393 */       if (iField == null) {
/* 6394 */         compileError("\"" + getType(fae.lhs).toString() + "\" has no field \"" + fae.fieldName + "\"", fae.getLocation());
/*      */         
/*      */ 
/*      */ 
/* 6398 */         fae.value = new Java.Rvalue(fae.getLocation())
/*      */         {
/* 6400 */           public String toString() { return "???"; }
/*      */           
/*      */           public final void accept(Visitor.AtomVisitor visitor) {}
/* 6403 */           public final void accept(Visitor.RvalueVisitor visitor) {} };
/* 6404 */         return;
/*      */       }
/*      */       
/* 6407 */       fae.value = new Java.FieldAccess(fae.getLocation(), fae.lhs, iField);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 6413 */     fae.value.setEnclosingBlockStatement(fae.getEnclosingBlockStatement());
/*      */   }
/*      */   
/*      */   private void determineValue(Java.SuperclassFieldAccessExpression scfae) throws CompileException
/*      */   {
/* 6418 */     if (scfae.value != null) { return;
/*      */     }
/*      */     
/*      */ 
/* 6422 */     Java.ThisReference tr = new Java.ThisReference(scfae.getLocation());
/* 6423 */     tr.setEnclosingBlockStatement(scfae.getEnclosingBlockStatement());
/*      */     IClass type;
/* 6425 */     IClass type; if (scfae.optionalQualification != null) {
/* 6426 */       type = getType(scfae.optionalQualification);
/*      */     }
/*      */     else {
/* 6429 */       type = getType(tr);
/*      */     }
/* 6431 */     Java.Rvalue lhs = new Java.Cast(scfae.getLocation(), new Java.SimpleType(scfae.getLocation(), type.getSuperclass()), tr);
/*      */     
/*      */ 
/* 6434 */     IClass.IField iField = findIField(getType(lhs), scfae.fieldName, scfae.getLocation());
/* 6435 */     if (iField == null) {
/* 6436 */       compileError("Class has no field \"" + scfae.fieldName + "\"", scfae.getLocation());
/* 6437 */       scfae.value = new Java.Rvalue(scfae.getLocation()) {
/* 6438 */         public String toString() { return "???"; }
/*      */         
/*      */         public final void accept(Visitor.AtomVisitor visitor) {}
/* 6441 */         public final void accept(Visitor.RvalueVisitor visitor) {} };
/* 6442 */       return;
/*      */     }
/* 6444 */     scfae.value = new Java.FieldAccess(scfae.getLocation(), lhs, iField);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 6449 */     scfae.value.setEnclosingBlockStatement(scfae.getEnclosingBlockStatement());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IClass.IMethod findIMethod(Java.MethodInvocation mi)
/*      */     throws CompileException
/*      */   {
/* 6464 */     if (mi.optionalTarget == null)
/*      */     {
/*      */ 
/*      */ 
/* 6468 */       for (Java.Scope s = mi.getEnclosingBlockStatement(); 
/* 6469 */           !(s instanceof Java.CompilationUnit); 
/* 6470 */           s = s.getEnclosingScope())
/*      */       {
/* 6472 */         if ((s instanceof Java.TypeDeclaration)) {
/* 6473 */           Java.TypeDeclaration td = (Java.TypeDeclaration)s;
/*      */           
/*      */ 
/* 6476 */           IClass.IMethod iMethod = findIMethod(resolve(td), mi);
/*      */           
/*      */ 
/*      */ 
/* 6480 */           if (iMethod != null) {
/*      */             break label402;
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */     else
/*      */     {
/* 6488 */       IClass.IMethod iMethod = findIMethod(getType(mi.optionalTarget), mi);
/*      */       
/*      */ 
/*      */ 
/* 6492 */       if (iMethod != null) {
/*      */         break label402;
/*      */       }
/*      */     }
/*      */     
/* 6497 */     List l = (List)this.singleStaticImports.get(mi.methodName);
/* 6498 */     IClass.IMethod iMethod; if (l != null) {
/* 6499 */       IClass.IMethod iMethod = null;
/* 6500 */       for (Iterator it = l.iterator(); it.hasNext();) {
/* 6501 */         Object o = it.next();
/* 6502 */         if ((o instanceof IClass.IMethod)) {
/* 6503 */           IClass declaringIClass = ((IClass.IMethod)o).getDeclaringIClass();
/* 6504 */           IClass.IMethod im = findIMethod(declaringIClass, mi);
/*      */           
/*      */ 
/*      */ 
/* 6508 */           if (im != null) {
/* 6509 */             if ((iMethod != null) && (iMethod != im)) {
/* 6510 */               compileError("Ambiguous static method import: \"" + iMethod.toString() + "\" vs. \"" + im.toString() + "\"");
/*      */             }
/*      */             
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6518 */             iMethod = im;
/*      */           }
/*      */         }
/*      */       }
/* 6522 */       if (iMethod != null) {}
/*      */ 
/*      */     }
/*      */     else
/*      */     {
/* 6527 */       iMethod = null;
/* 6528 */       for (Iterator it = this.staticImportsOnDemand.iterator(); it.hasNext();) {
/* 6529 */         IClass iClass = (IClass)it.next();
/* 6530 */         IClass.IMethod im = findIMethod(iClass, mi);
/*      */         
/*      */ 
/*      */ 
/* 6534 */         if (im != null) {
/* 6535 */           if (iMethod != null) {
/* 6536 */             compileError("Ambiguous static method import: \"" + iMethod.toString() + "\" vs. \"" + im.toString() + "\"");
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6544 */           iMethod = im;
/*      */         }
/*      */       }
/* 6547 */       if (iMethod == null)
/*      */       {
/* 6549 */         compileError("A method named \"" + mi.methodName + "\" is not declared in any enclosing class nor any supertype, nor through a static import", mi.getLocation());
/*      */         
/*      */ 
/*      */ 
/*      */ 
/* 6554 */         return fakeIMethod(this.iClassLoader.OBJECT, mi.methodName, mi.arguments);
/*      */       } }
/*      */     label402:
/* 6557 */     checkThrownExceptions(mi, iMethod);
/* 6558 */     return iMethod;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass.IMethod findIMethod(IClass targetType, Java.Invocation invocation)
/*      */     throws CompileException
/*      */   {
/* 6574 */     List ms = new ArrayList();
/* 6575 */     getIMethods(targetType, invocation.methodName, ms);
/*      */     
/*      */ 
/* 6578 */     if (targetType.isInterface()) {
/* 6579 */       IClass.IMethod[] oms = this.iClassLoader.OBJECT.getDeclaredIMethods(invocation.methodName);
/* 6580 */       for (int i = 0; i < oms.length; i++) {
/* 6581 */         IClass.IMethod om = oms[i];
/* 6582 */         if ((!om.isStatic()) && (om.getAccess() == Access.PUBLIC)) { ms.add(om);
/*      */         }
/*      */       }
/*      */     }
/* 6586 */     if (ms.size() == 0) { return null;
/*      */     }
/*      */     
/* 6589 */     return (IClass.IMethod)findMostSpecificIInvocable(invocation, (IClass.IMethod[])ms.toArray(new IClass.IMethod[ms.size()]), invocation.arguments, invocation.getEnclosingBlockStatement());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass.IMethod fakeIMethod(IClass targetType, String name, Java.Rvalue[] arguments)
/*      */     throws CompileException
/*      */   {
/* 6598 */     IClass[] pts = new IClass[arguments.length];
/* 6599 */     for (int i = 0; i < arguments.length; i++) pts[i] = getType(arguments[i]);
/* 6600 */     IClass tmp42_41 = targetType;tmp42_41.getClass();new IClass.IMethod(tmp42_41) {
/* 6601 */       public String getName() { return this.val$name; }
/* 6602 */       public IClass getReturnType() throws CompileException { return IClass.INT; }
/* 6603 */       public boolean isStatic() { return false; }
/* 6604 */       public boolean isAbstract() { return false; }
/* 6605 */       public IClass[] getParameterTypes() throws CompileException { return this.val$pts; }
/* 6606 */       public IClass[] getThrownExceptions() throws CompileException { return new IClass[0]; }
/* 6607 */       public Access getAccess() { return Access.PUBLIC; }
/*      */     };
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final String val$name;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private final String[] val$identifiers;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public void getIMethods(IClass type, String methodName, List v)
/*      */     throws CompileException
/*      */   {
/* 6628 */     IClass.IMethod[] ims = type.getDeclaredIMethods(methodName);
/* 6629 */     for (int i = 0; i < ims.length; i++) { v.add(ims[i]);
/*      */     }
/*      */     
/*      */ 
/* 6633 */     IClass superclass = type.getSuperclass();
/* 6634 */     if (superclass != null) { getIMethods(superclass, methodName, v);
/*      */     }
/*      */     
/* 6637 */     IClass[] interfaces = type.getInterfaces();
/* 6638 */     for (int i = 0; i < interfaces.length; i++) getIMethods(interfaces[i], methodName, v);
/*      */   }
/*      */   
/*      */   public IClass.IMethod findIMethod(Java.SuperclassMethodInvocation scmi) throws CompileException
/*      */   {
/* 6643 */     for (Java.Scope s = scmi.getEnclosingBlockStatement();; s = s.getEnclosingScope()) {
/* 6644 */       if ((s instanceof Java.FunctionDeclarator)) {
/* 6645 */         Java.FunctionDeclarator fd = (Java.FunctionDeclarator)s;
/* 6646 */         if ((fd.modifiers & 0x8) != 0) {
/* 6647 */           compileError("Superclass method cannot be invoked in static context", scmi.getLocation());
/*      */         }
/*      */       }
/* 6650 */       if ((s instanceof Java.ClassDeclaration)) {
/* 6651 */         Java.ClassDeclaration declaringClass = (Java.ClassDeclaration)s;
/* 6652 */         break;
/*      */       } }
/*      */     Java.ClassDeclaration declaringClass;
/* 6655 */     IClass superclass = resolve(declaringClass).getSuperclass();
/* 6656 */     IClass.IMethod iMethod = findIMethod(superclass, scmi);
/*      */     
/*      */ 
/*      */ 
/* 6660 */     if (iMethod == null) {
/* 6661 */       compileError("Class \"" + superclass + "\" has no method named \"" + scmi.methodName + "\"", scmi.getLocation());
/*      */       
/*      */ 
/*      */ 
/* 6665 */       return fakeIMethod(superclass, scmi.methodName, scmi.arguments);
/*      */     }
/* 6667 */     checkThrownExceptions(scmi, iMethod);
/* 6668 */     return iMethod;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass.IInvocable findMostSpecificIInvocable(Java.Locatable l, IClass.IInvocable[] iInvocables, Java.Rvalue[] arguments, Java.Scope contextScope)
/*      */     throws CompileException
/*      */   {
/* 6686 */     IClass[] argumentTypes = new IClass[arguments.length];
/* 6687 */     for (int i = 0; i < arguments.length; i++) {
/* 6688 */       argumentTypes[i] = getType(arguments[i]);
/*      */     }
/*      */     
/*      */ 
/* 6692 */     IClass.IInvocable ii = findMostSpecificIInvocable(l, iInvocables, argumentTypes, false, contextScope);
/* 6693 */     if (ii != null) { return ii;
/*      */     }
/*      */     
/* 6696 */     ii = findMostSpecificIInvocable(l, iInvocables, argumentTypes, true, contextScope);
/* 6697 */     if (ii != null) { return ii;
/*      */     }
/*      */     
/* 6700 */     StringBuffer sb = new StringBuffer("No applicable constructor/method found for ");
/* 6701 */     if (argumentTypes.length == 0) {
/* 6702 */       sb.append("zero actual parameters");
/*      */     } else {
/* 6704 */       sb.append("actual parameters \"").append(argumentTypes[0]);
/* 6705 */       for (int i = 1; i < argumentTypes.length; i++) {
/* 6706 */         sb.append(", ").append(argumentTypes[i]);
/*      */       }
/* 6708 */       sb.append("\"");
/*      */     }
/* 6710 */     sb.append("; candidates are: ").append('"' + iInvocables[0].toString() + '"');
/* 6711 */     for (int i = 1; i < iInvocables.length; i++) {
/* 6712 */       sb.append(", ").append('"' + iInvocables[i].toString() + '"');
/*      */     }
/* 6714 */     compileError(sb.toString(), l.getLocation());
/*      */     
/*      */ 
/*      */ 
/* 6718 */     if ((iInvocables[0] instanceof IClass.IConstructor)) {
/* 6719 */       IClass tmp300_297 = iInvocables[0].getDeclaringIClass();tmp300_297.getClass();new IClass.IConstructor(tmp300_297) {
/* 6720 */         public IClass[] getParameterTypes() { return this.val$argumentTypes; }
/* 6721 */         public Access getAccess() { return Access.PUBLIC; }
/* 6722 */         public IClass[] getThrownExceptions() { return new IClass[0]; }
/*      */       };
/*      */     }
/* 6725 */     if ((iInvocables[0] instanceof IClass.IMethod)) {
/* 6726 */       IClass tmp331_328 = iInvocables[0].getDeclaringIClass();tmp331_328.getClass();new IClass.IMethod(tmp331_328) {
/* 6727 */         public boolean isStatic() { return true; }
/* 6728 */         public boolean isAbstract() { return false; }
/* 6729 */         public IClass getReturnType() { return IClass.INT; }
/* 6730 */         public String getName() { return ((IClass.IMethod)this.val$iInvocables[0]).getName(); }
/* 6731 */         public Access getAccess() { return Access.PUBLIC; }
/* 6732 */         public IClass[] getParameterTypes() { return this.val$argumentTypes; }
/* 6733 */         public IClass[] getThrownExceptions() { return new IClass[0]; }
/*      */       };
/*      */     }
/*      */     
/* 6737 */     return iInvocables[0];
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private final IClass[] val$argumentTypes;
/*      */   
/*      */ 
/*      */ 
/*      */   private final IClass.IInvocable[] val$iInvocables;
/*      */   
/*      */ 
/*      */ 
/*      */   private final IClass[] val$argumentTypes;
/*      */   
/*      */ 
/*      */ 
/*      */   private final IClass[] val$pts;
/*      */   
/*      */ 
/*      */ 
/*      */   public IClass.IInvocable findMostSpecificIInvocable(Java.Locatable l, IClass.IInvocable[] iInvocables, IClass[] argumentTypes, boolean boxingPermitted, Java.Scope contextScope)
/*      */     throws CompileException
/*      */   {
/* 6763 */     List applicableIInvocables = new ArrayList();
/*      */     label104:
/* 6765 */     for (int i = 0; i < iInvocables.length; i++) {
/* 6766 */       IClass.IInvocable ii = iInvocables[i];
/*      */       
/*      */ 
/* 6769 */       IClass[] parameterTypes = ii.getParameterTypes();
/* 6770 */       if ((parameterTypes.length == argumentTypes.length) && 
/* 6771 */         (isAccessible(ii, contextScope)))
/*      */       {
/*      */ 
/*      */ 
/* 6775 */         for (int j = 0; j < argumentTypes.length; j++)
/*      */         {
/*      */ 
/* 6778 */           if (!isMethodInvocationConvertible(argumentTypes[j], parameterTypes[j], boxingPermitted)) {
/*      */             break label104;
/*      */           }
/*      */         }
/*      */         
/*      */ 
/*      */ 
/* 6785 */         applicableIInvocables.add(ii);
/*      */       } }
/* 6787 */     if (applicableIInvocables.size() == 0) { return null;
/*      */     }
/*      */     
/* 6790 */     if (applicableIInvocables.size() == 1) {
/* 6791 */       return (IClass.IInvocable)applicableIInvocables.get(0);
/*      */     }
/*      */     
/*      */ 
/* 6795 */     List maximallySpecificIInvocables = new ArrayList();
/* 6796 */     for (int i = 0; i < applicableIInvocables.size(); i++) {
/* 6797 */       IClass.IInvocable applicableIInvocable = (IClass.IInvocable)applicableIInvocables.get(i);
/* 6798 */       int moreSpecific = 0;int lessSpecific = 0;
/* 6799 */       for (int j = 0; j < maximallySpecificIInvocables.size(); j++) {
/* 6800 */         IClass.IInvocable mostSpecificIInvocable = (IClass.IInvocable)maximallySpecificIInvocables.get(j);
/* 6801 */         if (applicableIInvocable.isMoreSpecificThan(mostSpecificIInvocable)) {
/* 6802 */           moreSpecific++;
/*      */         }
/* 6804 */         else if (applicableIInvocable.isLessSpecificThan(mostSpecificIInvocable)) {
/* 6805 */           lessSpecific++;
/*      */         }
/*      */       }
/* 6808 */       if (moreSpecific == maximallySpecificIInvocables.size()) {
/* 6809 */         maximallySpecificIInvocables.clear();
/* 6810 */         maximallySpecificIInvocables.add(applicableIInvocable);
/*      */       }
/* 6812 */       else if (lessSpecific < maximallySpecificIInvocables.size()) {
/* 6813 */         maximallySpecificIInvocables.add(applicableIInvocable);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6821 */     if (maximallySpecificIInvocables.size() == 1) { return (IClass.IInvocable)maximallySpecificIInvocables.get(0);
/*      */     }
/*      */     
/* 6824 */     if ((maximallySpecificIInvocables.size() > 1) && ((iInvocables[0] instanceof IClass.IMethod)))
/*      */     {
/*      */ 
/*      */ 
/* 6828 */       IClass.IMethod theNonAbstractMethod = null;
/*      */       
/* 6830 */       Iterator it = maximallySpecificIInvocables.iterator();
/* 6831 */       IClass.IMethod m = (IClass.IMethod)it.next();
/* 6832 */       IClass[] parameterTypesOfFirstMethod = m.getParameterTypes();
/*      */       for (;;) {
/* 6834 */         if (!m.isAbstract()) {
/* 6835 */           if (theNonAbstractMethod == null) {
/* 6836 */             theNonAbstractMethod = m;
/*      */           } else {
/* 6838 */             IClass declaringIClass = m.getDeclaringIClass();
/* 6839 */             IClass theNonAbstractMethodDeclaringIClass = theNonAbstractMethod.getDeclaringIClass();
/* 6840 */             if (declaringIClass == theNonAbstractMethodDeclaringIClass) {
/* 6841 */               if (m.getReturnType() == theNonAbstractMethod.getReturnType()) {
/* 6842 */                 throw new JaninoRuntimeException("Two non-abstract methods '" + m + "' have the same parameter types, " + "declaring type and return type");
/*      */               }
/*      */               
/*      */ 
/*      */ 
/* 6847 */               if (!m.getReturnType().isAssignableFrom(theNonAbstractMethod.getReturnType()))
/*      */               {
/*      */ 
/* 6850 */                 if (theNonAbstractMethod.getReturnType().isAssignableFrom(m.getReturnType())) {
/* 6851 */                   theNonAbstractMethod = m;
/*      */                 }
/*      */                 else {
/* 6854 */                   throw new JaninoRuntimeException("Incompatible return types");
/*      */                 }
/*      */               }
/* 6857 */             } else if (!declaringIClass.isAssignableFrom(theNonAbstractMethodDeclaringIClass))
/*      */             {
/*      */ 
/* 6860 */               if (theNonAbstractMethodDeclaringIClass.isAssignableFrom(declaringIClass)) {
/* 6861 */                 theNonAbstractMethod = m;
/*      */               }
/*      */               else {
/* 6864 */                 throw new JaninoRuntimeException("SNO: Types declaring '" + theNonAbstractMethod + "' are not assignable");
/*      */               }
/*      */             }
/*      */           }
/*      */         }
/*      */         
/*      */ 
/*      */ 
/* 6872 */         if (!it.hasNext())
/*      */           break;
/* 6874 */         m = (IClass.IMethod)it.next();
/* 6875 */         IClass[] pts = m.getParameterTypes();
/* 6876 */         for (int i = 0; i < pts.length; i++) {
/* 6877 */           if (pts[i] != parameterTypesOfFirstMethod[i]) {
/*      */             break label932;
/*      */           }
/*      */         }
/*      */       }
/*      */       
/* 6883 */       if (theNonAbstractMethod != null) { return theNonAbstractMethod;
/*      */       }
/*      */       
/*      */ 
/* 6887 */       Set s = new HashSet();
/*      */       
/* 6889 */       IClass[][] tes = new IClass[maximallySpecificIInvocables.size()][];
/* 6890 */       Iterator it = maximallySpecificIInvocables.iterator();
/* 6891 */       for (int i = 0; i < tes.length; i++) {
/* 6892 */         tes[i] = ((IClass.IMethod)it.next()).getThrownExceptions();
/*      */       }
/* 6894 */       for (int i = 0; i < tes.length; i++) {
/*      */         label843:
/* 6896 */         label859: for (int j = 0; j < tes[i].length; j++) {
/* 6897 */           IClass te1 = tes[i][j];
/*      */           
/* 6899 */           for (int k = 0; k < tes.length; k++)
/* 6900 */             if (k != i) {
/* 6901 */               for (int m = 0; m < tes[k].length; m++) {
/* 6902 */                 IClass te2 = tes[k][m];
/* 6903 */                 if (te2.isAssignableFrom(te1)) break label843;
/*      */               }
/*      */               break label859;
/*      */             }
/* 6907 */           s.add(te1);
/*      */         }
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 6913 */       IClass.IMethod im = (IClass.IMethod)maximallySpecificIInvocables.get(0);
/* 6914 */       IClass[] tes = (IClass[])s.toArray(new IClass[s.size()]); IClass 
/* 6915 */         tmp919_916 = im.getDeclaringIClass();tmp919_916.getClass();new IClass.IMethod(tmp919_916) {
/* 6916 */         public String getName() { return this.val$im.getName(); }
/* 6917 */         public IClass getReturnType() throws CompileException { return this.val$im.getReturnType(); }
/* 6918 */         public boolean isAbstract() { return this.val$im.isAbstract(); }
/* 6919 */         public boolean isStatic() { return this.val$im.isStatic(); }
/* 6920 */         public Access getAccess() { return this.val$im.getAccess(); }
/* 6921 */         public IClass[] getParameterTypes() throws CompileException { return this.val$im.getParameterTypes(); }
/* 6922 */         public IClass[] getThrownExceptions() { return this.val$tes; }
/*      */       };
/*      */     }
/*      */     
/*      */     label932:
/*      */     
/* 6928 */     StringBuffer sb = new StringBuffer("Invocation of constructor/method with actual parameter type(s) \"");
/* 6929 */     for (int i = 0; i < argumentTypes.length; i++) {
/* 6930 */       if (i > 0) sb.append(", ");
/* 6931 */       sb.append(Descriptor.toString(argumentTypes[i].getDescriptor()));
/*      */     }
/* 6933 */     sb.append("\" is ambiguous: ");
/* 6934 */     for (int i = 0; i < maximallySpecificIInvocables.size(); i++) {
/* 6935 */       if (i > 0) sb.append(" vs. ");
/* 6936 */       sb.append("\"" + maximallySpecificIInvocables.get(i) + "\"");
/*      */     }
/* 6938 */     compileError(sb.toString(), l.getLocation());
/*      */     
/*      */ 
/* 6941 */     return (IClass.IMethod)iInvocables[0];
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean isMethodInvocationConvertible(IClass sourceType, IClass targetType, boolean boxingPermitted)
/*      */     throws CompileException
/*      */   {
/* 6954 */     if (sourceType == targetType) { return true;
/*      */     }
/*      */     
/* 6957 */     if (isWideningPrimitiveConvertible(sourceType, targetType)) { return true;
/*      */     }
/*      */     
/* 6960 */     if (isWideningReferenceConvertible(sourceType, targetType)) { return true;
/*      */     }
/*      */     
/* 6963 */     if (boxingPermitted) {
/* 6964 */       IClass boxedType = isBoxingConvertible(sourceType);
/* 6965 */       if (boxedType != null) {
/* 6966 */         return (isIdentityConvertible(boxedType, targetType)) || (isWideningReferenceConvertible(boxedType, targetType));
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6974 */     if (boxingPermitted) {
/* 6975 */       IClass unboxedType = isUnboxingConvertible(sourceType);
/* 6976 */       if (unboxedType != null) {
/* 6977 */         return (isIdentityConvertible(unboxedType, targetType)) || (isWideningPrimitiveConvertible(unboxedType, targetType));
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 6986 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */   private void checkThrownExceptions(Java.Invocation in, IClass.IMethod iMethod)
/*      */     throws CompileException
/*      */   {
/* 6993 */     IClass[] thrownExceptions = iMethod.getThrownExceptions();
/* 6994 */     for (int i = 0; i < thrownExceptions.length; i++) {
/* 6995 */       checkThrownException(in, thrownExceptions[i], in.getEnclosingBlockStatement());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private final IClass.IMethod val$im;
/*      */   
/*      */ 
/*      */ 
/*      */   private final IClass[] val$tes;
/*      */   
/*      */ 
/*      */ 
/*      */   private void checkThrownException(Java.Locatable l, IClass type, Java.Scope scope)
/*      */     throws CompileException
/*      */   {
/* 7013 */     if (!this.iClassLoader.THROWABLE.isAssignableFrom(type)) {
/* 7014 */       compileError("Thrown object of type \"" + type + "\" is not assignable to \"Throwable\"", l.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 7021 */     if ((this.iClassLoader.RUNTIME_EXCEPTION.isAssignableFrom(type)) || (this.iClassLoader.ERROR.isAssignableFrom(type)))
/*      */     {
/*      */ 
/* 7024 */       return;
/*      */     }
/* 7026 */     for (;; scope = scope.getEnclosingScope())
/*      */     {
/*      */ 
/* 7029 */       if ((scope instanceof Java.TryStatement)) {
/* 7030 */         Java.TryStatement ts = (Java.TryStatement)scope;
/* 7031 */         for (int i = 0; i < ts.catchClauses.size(); i++) {
/* 7032 */           Java.CatchClause cc = (Java.CatchClause)ts.catchClauses.get(i);
/* 7033 */           IClass caughtType = getType(cc.caughtException.type);
/* 7034 */           if (caughtType.isAssignableFrom(type)) { return;
/*      */           }
/*      */           
/*      */         }
/*      */       }
/* 7039 */       else if ((scope instanceof Java.FunctionDeclarator)) {
/* 7040 */         Java.FunctionDeclarator fd = (Java.FunctionDeclarator)scope;
/* 7041 */         for (int i = 0; i < fd.thrownExceptions.length; i++) {
/* 7042 */           IClass te = getType(fd.thrownExceptions[i]);
/* 7043 */           if (te.isAssignableFrom(type)) return;
/*      */         }
/*      */       }
/*      */       else
/*      */       {
/* 7048 */         if ((scope instanceof Java.TypeBodyDeclaration)) {
/*      */           break;
/*      */         }
/*      */       }
/*      */     }
/* 7053 */     compileError("Thrown exception of type \"" + type + "\" is neither caught by a \"try...catch\" block " + "nor declared in the \"throws\" clause of the declaring function", l.getLocation());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass getTargetIClass(Java.QualifiedThisReference qtr)
/*      */     throws CompileException
/*      */   {
/* 7064 */     if (qtr.targetIClass == null) {
/* 7065 */       qtr.targetIClass = getType(qtr.qualification);
/*      */     }
/* 7067 */     return qtr.targetIClass;
/*      */   }
/*      */   
/*      */ 
/*      */   Java.LocalVariable isIntLV(Java.Crement c)
/*      */     throws CompileException
/*      */   {
/* 7074 */     if (!(c.operand instanceof Java.AmbiguousName)) return null;
/* 7075 */     Java.AmbiguousName an = (Java.AmbiguousName)c.operand;
/*      */     
/* 7077 */     Java.Atom rec = reclassify(an);
/* 7078 */     if (!(rec instanceof Java.LocalVariableAccess)) return null;
/* 7079 */     Java.LocalVariableAccess lva = (Java.LocalVariableAccess)rec;
/*      */     
/* 7081 */     Java.LocalVariable lv = lva.localVariable;
/* 7082 */     if (lv.finaL) compileError("Must not increment or decrement \"final\" local variable", lva.getLocation());
/* 7083 */     if ((lv.type == IClass.BYTE) || (lv.type == IClass.SHORT) || (lv.type == IClass.INT) || (lv.type == IClass.CHAR))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/* 7088 */       return lv; }
/* 7089 */     return null;
/*      */   }
/*      */   
/*      */   private IClass resolve(Java.TypeDeclaration td) {
/* 7093 */     Java.AbstractTypeDeclaration atd = (Java.AbstractTypeDeclaration)td;
/* 7094 */     if (atd.resolvedType == null) atd.resolvedType = new IClass() { private IClass[] declaredClasses;
/*      */         
/* 7096 */         protected IClass.IMethod[] getDeclaredIMethods2() { IClass.IMethod[] res = new IClass.IMethod[this.val$atd.getMethodDeclarations().size()];
/* 7097 */           int i = 0;
/* 7098 */           for (Iterator it = this.val$atd.getMethodDeclarations().iterator(); it.hasNext();) {
/* 7099 */             res[(i++)] = UnitCompiler.this.toIMethod((Java.MethodDeclarator)it.next());
/*      */           }
/* 7101 */           return res;
/*      */         }
/*      */         
/*      */         protected IClass[] getDeclaredIClasses2() {
/* 7105 */           if (this.declaredClasses == null) {
/* 7106 */             Collection mtds = this.val$td.getMemberTypeDeclarations();
/* 7107 */             IClass[] mts = new IClass[mtds.size()];
/* 7108 */             int i = 0;
/* 7109 */             for (Iterator it = mtds.iterator(); it.hasNext();) {
/* 7110 */               mts[(i++)] = UnitCompiler.this.resolve((Java.AbstractTypeDeclaration)it.next());
/*      */             }
/* 7112 */             this.declaredClasses = mts;
/*      */           }
/* 7114 */           return this.declaredClasses;
/*      */         }
/*      */         
/* 7117 */         protected IClass getDeclaringIClass2() { for (Java.Scope s = this.val$atd; 
/* 7118 */               !(s instanceof Java.TypeBodyDeclaration); s = s.getEnclosingScope()) {
/* 7119 */             if ((s instanceof Java.CompilationUnit)) return null;
/*      */           }
/* 7121 */           return UnitCompiler.this.resolve((Java.AbstractTypeDeclaration)s.getEnclosingScope());
/*      */         }
/*      */         
/* 7124 */         protected IClass getOuterIClass2() throws CompileException { Java.AbstractTypeDeclaration oc = (Java.AbstractTypeDeclaration)UnitCompiler.getOuterClass(this.val$atd);
/* 7125 */           if (oc == null) return null;
/* 7126 */           return UnitCompiler.this.resolve(oc);
/*      */         }
/*      */         
/* 7129 */         protected final String getDescriptor2() { return Descriptor.fromClassName(this.val$atd.getClassName()); }
/*      */         
/*      */ 
/* 7132 */         public boolean isArray() { return false; }
/*      */         
/* 7134 */         protected IClass getComponentType2() { throw new JaninoRuntimeException("SNO: Non-array type has no component type"); }
/*      */         
/* 7136 */         public boolean isPrimitive() { return false; }
/* 7137 */         public boolean isPrimitiveNumeric() { return false; }
/*      */         
/* 7139 */         protected IClass.IConstructor[] getDeclaredIConstructors2() { if ((this.val$atd instanceof Java.ClassDeclaration)) {
/* 7140 */             Java.ConstructorDeclarator[] cs = ((Java.ClassDeclaration)this.val$atd).getConstructors();
/*      */             
/* 7142 */             IClass.IConstructor[] res = new IClass.IConstructor[cs.length];
/* 7143 */             for (int i = 0; i < cs.length; i++) res[i] = UnitCompiler.this.toIConstructor(cs[i]);
/* 7144 */             return res;
/*      */           }
/* 7146 */           return new IClass.IConstructor[0];
/*      */         }
/*      */         
/* 7149 */         protected IClass.IField[] getDeclaredIFields2() { if ((this.val$atd instanceof Java.ClassDeclaration)) {
/* 7150 */             Java.ClassDeclaration cd = (Java.ClassDeclaration)this.val$atd;
/* 7151 */             List l = new ArrayList();
/*      */             
/*      */ 
/* 7154 */             for (int i = 0; i < cd.variableDeclaratorsAndInitializers.size(); i++) {
/* 7155 */               Java.BlockStatement vdoi = (Java.BlockStatement)cd.variableDeclaratorsAndInitializers.get(i);
/* 7156 */               if ((vdoi instanceof Java.FieldDeclaration)) {
/* 7157 */                 Java.FieldDeclaration fd = (Java.FieldDeclaration)vdoi;
/* 7158 */                 IClass.IField[] flds = UnitCompiler.this.getIFields(fd);
/* 7159 */                 for (int j = 0; j < flds.length; j++) l.add(flds[j]);
/*      */               }
/*      */             }
/* 7162 */             return (IClass.IField[])l.toArray(new IClass.IField[l.size()]);
/*      */           }
/* 7164 */           if ((this.val$atd instanceof Java.InterfaceDeclaration)) {
/* 7165 */             Java.InterfaceDeclaration id = (Java.InterfaceDeclaration)this.val$atd;
/* 7166 */             List l = new ArrayList();
/*      */             
/*      */ 
/* 7169 */             for (int i = 0; i < id.constantDeclarations.size(); i++) {
/* 7170 */               Java.BlockStatement bs = (Java.BlockStatement)id.constantDeclarations.get(i);
/* 7171 */               if ((bs instanceof Java.FieldDeclaration)) {
/* 7172 */                 Java.FieldDeclaration fd = (Java.FieldDeclaration)bs;
/* 7173 */                 IClass.IField[] flds = UnitCompiler.this.getIFields(fd);
/* 7174 */                 for (int j = 0; j < flds.length; j++) l.add(flds[j]);
/*      */               }
/*      */             }
/* 7177 */             return (IClass.IField[])l.toArray(new IClass.IField[l.size()]);
/*      */           }
/* 7179 */           throw new JaninoRuntimeException("SNO: AbstractTypeDeclaration is neither ClassDeclaration nor InterfaceDeclaration");
/*      */         }
/*      */         
/*      */ 
/*      */         public IClass.IField[] getSyntheticIFields()
/*      */         {
/* 7185 */           if ((this.val$atd instanceof Java.ClassDeclaration)) {
/* 7186 */             Collection c = ((Java.ClassDeclaration)this.val$atd).syntheticFields.values();
/* 7187 */             return (IClass.IField[])c.toArray(new IClass.IField[c.size()]);
/*      */           }
/* 7189 */           return new IClass.IField[0];
/*      */         }
/*      */         
/* 7192 */         protected IClass getSuperclass2() throws CompileException { if ((this.val$atd instanceof Java.AnonymousClassDeclaration)) {
/* 7193 */             IClass bt = UnitCompiler.this.getType(((Java.AnonymousClassDeclaration)this.val$atd).baseType);
/* 7194 */             return bt.isInterface() ? UnitCompiler.this.iClassLoader.OBJECT : bt;
/*      */           }
/* 7196 */           if ((this.val$atd instanceof Java.NamedClassDeclaration)) {
/* 7197 */             Java.NamedClassDeclaration ncd = (Java.NamedClassDeclaration)this.val$atd;
/* 7198 */             if (ncd.optionalExtendedType == null) return UnitCompiler.this.iClassLoader.OBJECT;
/* 7199 */             IClass superclass = UnitCompiler.this.getType(ncd.optionalExtendedType);
/* 7200 */             if (superclass.isInterface()) {
/* 7201 */               UnitCompiler.this.compileError("\"" + superclass.toString() + "\" is an interface; classes can only extend a class", this.val$td.getLocation());
/*      */             }
/*      */             
/*      */ 
/*      */ 
/* 7206 */             return superclass;
/*      */           }
/* 7208 */           return null; }
/*      */         
/* 7210 */         public Access getAccess() { return UnitCompiler.modifiers2Access(this.val$atd.getModifiers()); }
/* 7211 */         public boolean isFinal() { return (this.val$atd.getModifiers() & 0x10) != 0; }
/*      */         
/* 7213 */         protected IClass[] getInterfaces2() throws CompileException { if ((this.val$atd instanceof Java.AnonymousClassDeclaration)) {
/* 7214 */             IClass bt = UnitCompiler.this.getType(((Java.AnonymousClassDeclaration)this.val$atd).baseType);
/* 7215 */             return bt.isInterface() ? new IClass[] { bt } : new IClass[0];
/*      */           }
/* 7217 */           if ((this.val$atd instanceof Java.NamedClassDeclaration)) {
/* 7218 */             Java.NamedClassDeclaration ncd = (Java.NamedClassDeclaration)this.val$atd;
/* 7219 */             IClass[] res = new IClass[ncd.implementedTypes.length];
/* 7220 */             for (int i = 0; i < res.length; i++) {
/* 7221 */               res[i] = UnitCompiler.this.getType(ncd.implementedTypes[i]);
/* 7222 */               if (!res[i].isInterface()) {
/* 7223 */                 UnitCompiler.this.compileError("\"" + res[i].toString() + "\" is not an interface; classes can only implement interfaces", this.val$td.getLocation());
/*      */               }
/*      */             }
/*      */             
/*      */ 
/*      */ 
/*      */ 
/* 7230 */             return res;
/*      */           }
/* 7232 */           if ((this.val$atd instanceof Java.InterfaceDeclaration)) {
/* 7233 */             Java.InterfaceDeclaration id = (Java.InterfaceDeclaration)this.val$atd;
/* 7234 */             IClass[] res = new IClass[id.extendedTypes.length];
/* 7235 */             for (int i = 0; i < res.length; i++) {
/* 7236 */               res[i] = UnitCompiler.this.getType(id.extendedTypes[i]);
/* 7237 */               if (!res[i].isInterface()) {
/* 7238 */                 UnitCompiler.this.compileError("\"" + res[i].toString() + "\" is not an interface; interfaces can only extend interfaces", this.val$td.getLocation());
/*      */               }
/*      */             }
/*      */             
/*      */ 
/*      */ 
/*      */ 
/* 7245 */             return res;
/*      */           }
/* 7247 */           throw new JaninoRuntimeException("SNO: AbstractTypeDeclaration is neither ClassDeclaration nor InterfaceDeclaration");
/*      */         }
/*      */         
/*      */ 
/*      */         public boolean isAbstract()
/*      */         {
/* 7253 */           return ((this.val$atd instanceof Java.InterfaceDeclaration)) || ((this.val$atd.getModifiers() & 0x400) != 0);
/*      */         }
/*      */         
/*      */         public boolean isInterface()
/*      */         {
/* 7258 */           return this.val$atd instanceof Java.InterfaceDeclaration;
/*      */         }
/*      */       };
/* 7261 */     return atd.resolvedType;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void referenceThis(Java.Locatable l, Java.ClassDeclaration declaringClass, Java.TypeBodyDeclaration declaringTypeBodyDeclaration, IClass targetIClass)
/*      */     throws CompileException
/*      */   {
/* 7270 */     List path = getOuterClasses(declaringClass);
/*      */     
/* 7272 */     if (declaringTypeBodyDeclaration.isStatic()) {
/* 7273 */       compileError("No current instance available in static context", l.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 7278 */     for (int j = 0; j < path.size(); j++)
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 7291 */       if (targetIClass.isAssignableFrom(resolve((Java.AbstractTypeDeclaration)path.get(j)))) {
/*      */         break label121;
/*      */       }
/*      */     }
/* 7295 */     compileError("\"" + declaringClass + "\" is not enclosed by \"" + targetIClass + "\"", l.getLocation());
/*      */     
/*      */ 
/*      */     label121:
/*      */     
/*      */     int i;
/*      */     
/* 7302 */     if ((declaringTypeBodyDeclaration instanceof Java.ConstructorDeclarator)) {
/* 7303 */       if (j == 0) {
/* 7304 */         writeOpcode(l, 42);
/* 7305 */         return;
/*      */       }
/*      */       
/* 7308 */       Java.ConstructorDeclarator constructorDeclarator = (Java.ConstructorDeclarator)declaringTypeBodyDeclaration;
/*      */       
/*      */ 
/* 7311 */       String spn = "this$" + (path.size() - 2);
/* 7312 */       Java.LocalVariable syntheticParameter = (Java.LocalVariable)constructorDeclarator.syntheticParameters.get(spn);
/*      */       
/*      */ 
/* 7315 */       if (syntheticParameter == null) {
/* 7316 */         throw new JaninoRuntimeException("SNO: Synthetic parameter \"" + spn + "\" not found");
/*      */       }
/* 7318 */       load(l, syntheticParameter);
/* 7319 */       i = 1;
/*      */     } else {
/* 7321 */       writeOpcode(l, 42); }
/* 7322 */     for (int i = 0; 
/*      */         
/* 7324 */         i < j; i++) {
/* 7325 */       String fieldName = "this$" + (path.size() - i - 2);
/* 7326 */       Java.InnerClassDeclaration inner = (Java.InnerClassDeclaration)path.get(i);
/* 7327 */       IClass iic = resolve((Java.AbstractTypeDeclaration)inner);
/* 7328 */       Java.TypeDeclaration outer = (Java.TypeDeclaration)path.get(i + 1);
/* 7329 */       IClass oic = resolve((Java.AbstractTypeDeclaration)outer);
/* 7330 */       inner.defineSyntheticField(new SimpleIField(iic, fieldName, oic));
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 7335 */       writeOpcode(l, -76);
/* 7336 */       writeConstantFieldrefInfo(iic.getDescriptor(), fieldName, oic.getDescriptor());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static List getOuterClasses(Java.TypeDeclaration inner)
/*      */   {
/* 7349 */     List path = new ArrayList();
/* 7350 */     for (Java.TypeDeclaration ic = inner; ic != null; ic = getOuterClass(ic)) path.add(ic);
/* 7351 */     return path;
/*      */   }
/*      */   
/*      */ 
/*      */   static Java.TypeDeclaration getOuterClass(Java.TypeDeclaration atd)
/*      */   {
/* 7357 */     if ((atd instanceof Java.PackageMemberClassDeclaration)) { return null;
/*      */     }
/*      */     
/* 7360 */     if ((atd instanceof Java.LocalClassDeclaration)) {
/* 7361 */       Java.Scope s = atd.getEnclosingScope();
/* 7362 */       while (!(s instanceof Java.FunctionDeclarator)) s = s.getEnclosingScope();
/* 7363 */       if (((s instanceof Java.MethodDeclarator)) && ((((Java.FunctionDeclarator)s).modifiers & 0x8) != 0))
/*      */       {
/*      */ 
/* 7366 */         return null; }
/* 7367 */       while (!(s instanceof Java.TypeDeclaration)) s = s.getEnclosingScope();
/* 7368 */       Java.TypeDeclaration immediatelyEnclosingTypeDeclaration = (Java.TypeDeclaration)s;
/* 7369 */       return (immediatelyEnclosingTypeDeclaration instanceof Java.ClassDeclaration) ? immediatelyEnclosingTypeDeclaration : null;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 7375 */     if (((atd instanceof Java.MemberClassDeclaration)) && ((((Java.MemberClassDeclaration)atd).getModifiers() & 0x8) != 0))
/*      */     {
/*      */ 
/* 7378 */       return null;
/*      */     }
/*      */     
/* 7381 */     for (Java.Scope s = atd; 
/* 7382 */         !(s instanceof Java.TypeBodyDeclaration); s = s.getEnclosingScope()) {
/* 7383 */       if ((s instanceof Java.ConstructorInvocation)) return null;
/* 7384 */       if ((s instanceof Java.CompilationUnit)) { return null;
/*      */       }
/*      */     }
/* 7387 */     if (((Java.TypeBodyDeclaration)s).isStatic()) return null;
/* 7388 */     return (Java.AbstractTypeDeclaration)s.getEnclosingScope();
/*      */   }
/*      */   
/*      */   private IClass getIClass(Java.ThisReference tr) throws CompileException {
/* 7392 */     if (tr.iClass == null)
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/* 7397 */       Java.Scope s = tr.getEnclosingBlockStatement();
/* 7398 */       while (((s instanceof Java.Statement)) || ((s instanceof Java.CatchClause))) {
/* 7399 */         s = s.getEnclosingScope();
/*      */       }
/* 7401 */       if ((s instanceof Java.FunctionDeclarator)) {
/* 7402 */         Java.FunctionDeclarator function = (Java.FunctionDeclarator)s;
/* 7403 */         if ((function.modifiers & 0x8) != 0) {
/* 7404 */           compileError("No current instance available in static method", tr.getLocation());
/*      */         }
/*      */       }
/*      */       
/*      */ 
/* 7409 */       while (!(s instanceof Java.TypeDeclaration)) {
/* 7410 */         s = s.getEnclosingScope();
/*      */       }
/* 7412 */       if (!(s instanceof Java.ClassDeclaration)) {
/* 7413 */         compileError("Only methods of classes can have a current instance", tr.getLocation());
/*      */       }
/* 7415 */       tr.iClass = resolve((Java.ClassDeclaration)s);
/*      */     }
/* 7417 */     return tr.iClass;
/*      */   }
/*      */   
/*      */   private IClass getReturnType(Java.FunctionDeclarator fd) throws CompileException {
/* 7421 */     if (fd.returnType == null) {
/* 7422 */       fd.returnType = getType(fd.type);
/*      */     }
/* 7424 */     return fd.returnType;
/*      */   }
/*      */   
/*      */   IClass.IConstructor toIConstructor(Java.ConstructorDeclarator cd) {
/* 7428 */     if (cd.iConstructor != null) { return cd.iConstructor;
/*      */     }
/* 7430 */     IClass tmp29_26 = resolve((Java.AbstractTypeDeclaration)cd.getDeclaringType());tmp29_26.getClass();cd.iConstructor = new IClass.IConstructor(tmp29_26) {
/*      */       private final Java.ConstructorDeclarator val$cd;
/*      */       
/*      */       public Access getAccess() {
/* 7434 */         switch (this.val$cd.modifiers & 0x7) {
/*      */         case 2: 
/* 7436 */           return Access.PRIVATE;
/*      */         case 4: 
/* 7438 */           return Access.PROTECTED;
/*      */         case 0: 
/* 7440 */           return Access.DEFAULT;
/*      */         case 1: 
/* 7442 */           return Access.PUBLIC;
/*      */         }
/* 7444 */         throw new JaninoRuntimeException("Invalid access");
/*      */       }
/*      */       
/*      */       public String getDescriptor()
/*      */         throws CompileException
/*      */       {
/* 7450 */         if (!(this.val$cd.getDeclaringClass() instanceof Java.InnerClassDeclaration)) { return super.getDescriptor();
/*      */         }
/* 7452 */         List l = new ArrayList();
/*      */         
/*      */ 
/* 7455 */         IClass outerClass = UnitCompiler.this.resolve(this.val$cd.getDeclaringClass()).getOuterIClass();
/* 7456 */         if (outerClass != null) { l.add(outerClass.getDescriptor());
/*      */         }
/*      */         
/* 7459 */         for (Iterator it = this.val$cd.getDeclaringClass().syntheticFields.values().iterator(); it.hasNext();) {
/* 7460 */           IClass.IField sf = (IClass.IField)it.next();
/* 7461 */           if (sf.getName().startsWith("val$")) l.add(sf.getType().getDescriptor());
/*      */         }
/* 7463 */         Java.FunctionDeclarator.FormalParameter[] fps = this.val$cd.formalParameters;
/* 7464 */         for (int i = 0; i < fps.length; i++) {
/* 7465 */           l.add(UnitCompiler.this.getType(fps[i].type).getDescriptor());
/*      */         }
/* 7467 */         String[] apd = (String[])l.toArray(new String[l.size()]);
/* 7468 */         return new MethodDescriptor(apd, "V").toString();
/*      */       }
/*      */       
/*      */       public IClass[] getParameterTypes() throws CompileException {
/* 7472 */         Java.FunctionDeclarator.FormalParameter[] fps = this.val$cd.formalParameters;
/* 7473 */         IClass[] res = new IClass[fps.length];
/* 7474 */         for (int i = 0; i < fps.length; i++) {
/* 7475 */           res[i] = UnitCompiler.this.getType(fps[i].type);
/*      */         }
/* 7477 */         return res;
/*      */       }
/*      */       
/* 7480 */       public IClass[] getThrownExceptions() throws CompileException { IClass[] res = new IClass[this.val$cd.thrownExceptions.length];
/* 7481 */         for (int i = 0; i < res.length; i++) {
/* 7482 */           res[i] = UnitCompiler.this.getType(this.val$cd.thrownExceptions[i]);
/*      */         }
/* 7484 */         return res;
/*      */       }
/*      */       
/*      */       public String toString() {
/* 7488 */         StringBuffer sb = new StringBuffer();
/* 7489 */         sb.append(this.val$cd.getDeclaringType().getClassName());
/* 7490 */         sb.append('(');
/* 7491 */         Java.FunctionDeclarator.FormalParameter[] fps = this.val$cd.formalParameters;
/* 7492 */         for (int i = 0; i < fps.length; i++) {
/* 7493 */           if (i != 0) sb.append(", ");
/*      */           try {
/* 7495 */             sb.append(UnitCompiler.this.getType(fps[i].type).toString());
/*      */           } catch (CompileException ex) {
/* 7497 */             sb.append("???");
/*      */           }
/*      */         }
/* 7500 */         return ')';
/*      */       }
/*      */       
/* 7503 */     };return cd.iConstructor;
/*      */   }
/*      */   
/*      */   public IClass.IMethod toIMethod(Java.MethodDeclarator md) {
/* 7507 */     if (md.iMethod != null) return md.iMethod;
/* 7508 */     IClass tmp29_26 = resolve((Java.AbstractTypeDeclaration)md.getDeclaringType());tmp29_26.getClass();md.iMethod = new IClass.IMethod(tmp29_26) {
/*      */       private final Java.MethodDeclarator val$md;
/*      */       
/*      */       public Access getAccess() {
/* 7512 */         switch (this.val$md.modifiers & 0x7) {
/*      */         case 2: 
/* 7514 */           return Access.PRIVATE;
/*      */         case 4: 
/* 7516 */           return Access.PROTECTED;
/*      */         case 0: 
/* 7518 */           return Access.DEFAULT;
/*      */         case 1: 
/* 7520 */           return Access.PUBLIC;
/*      */         }
/* 7522 */         throw new JaninoRuntimeException("Invalid access");
/*      */       }
/*      */       
/*      */       public IClass[] getParameterTypes()
/*      */         throws CompileException
/*      */       {
/* 7528 */         Java.FunctionDeclarator.FormalParameter[] fps = this.val$md.formalParameters;
/* 7529 */         IClass[] res = new IClass[fps.length];
/* 7530 */         for (int i = 0; i < fps.length; i++) {
/* 7531 */           res[i] = UnitCompiler.this.getType(fps[i].type);
/*      */         }
/* 7533 */         return res;
/*      */       }
/*      */       
/* 7536 */       public IClass[] getThrownExceptions() throws CompileException { IClass[] res = new IClass[this.val$md.thrownExceptions.length];
/* 7537 */         for (int i = 0; i < res.length; i++) {
/* 7538 */           res[i] = UnitCompiler.this.getType(this.val$md.thrownExceptions[i]);
/*      */         }
/* 7540 */         return res;
/*      */       }
/*      */       
/*      */ 
/* 7544 */       public boolean isStatic() { return (this.val$md.modifiers & 0x8) != 0; }
/*      */       
/* 7546 */       public boolean isAbstract() { return ((this.val$md.getDeclaringType() instanceof Java.InterfaceDeclaration)) || ((this.val$md.modifiers & 0x400) != 0); }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 7552 */       public IClass getReturnType()
/* 7552 */         throws CompileException { return UnitCompiler.this.getReturnType(this.val$md); }
/*      */       
/* 7554 */       public String getName() { return this.val$md.name;
/*      */       }
/* 7556 */     };return md.iMethod;
/*      */   }
/*      */   
/*      */   private IClass.IInvocable toIInvocable(Java.FunctionDeclarator fd) {
/* 7560 */     if ((fd instanceof Java.ConstructorDeclarator)) {
/* 7561 */       return toIConstructor((Java.ConstructorDeclarator)fd);
/*      */     }
/* 7563 */     if ((fd instanceof Java.MethodDeclarator)) {
/* 7564 */       return toIMethod((Java.MethodDeclarator)fd);
/*      */     }
/*      */     
/* 7567 */     throw new JaninoRuntimeException("FunctionDeclarator is neither ConstructorDeclarator nor MethodDeclarator");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass importSingleType(String simpleTypeName, Location location)
/*      */     throws CompileException
/*      */   {
/* 7577 */     String[] ss = getSingleTypeImport(simpleTypeName);
/* 7578 */     if (ss == null) { return null;
/*      */     }
/* 7580 */     IClass iClass = loadFullyQualifiedClass(ss);
/* 7581 */     if (iClass == null) {
/* 7582 */       compileError("Imported class \"" + Java.join(ss, ".") + "\" could not be loaded", location);
/* 7583 */       return this.iClassLoader.OBJECT;
/*      */     }
/* 7585 */     return iClass;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public String[] getSingleTypeImport(String name)
/*      */   {
/* 7595 */     return (String[])this.singleTypeImports.get(name);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private final Java.AbstractTypeDeclaration val$atd;
/*      */   
/*      */ 
/*      */   private final Java.TypeDeclaration val$td;
/*      */   
/*      */ 
/*      */   public IClass importTypeOnDemand(String simpleTypeName, Location location)
/*      */     throws CompileException
/*      */   {
/* 7609 */     IClass importedClass = (IClass)this.onDemandImportableTypes.get(simpleTypeName);
/* 7610 */     if (importedClass != null) { return importedClass;
/*      */     }
/*      */     
/* 7613 */     for (Iterator i = this.typeImportsOnDemand.iterator(); i.hasNext();) {
/* 7614 */       String[] ss = (String[])i.next();
/* 7615 */       String[] ss2 = concat(ss, simpleTypeName);
/* 7616 */       IClass iClass = loadFullyQualifiedClass(ss2);
/* 7617 */       if (iClass != null) {
/* 7618 */         if ((importedClass != null) && (importedClass != iClass)) {
/* 7619 */           compileError("Ambiguous class name: \"" + importedClass + "\" vs. \"" + iClass + "\"", location);
/*      */         }
/*      */         
/*      */ 
/*      */ 
/* 7624 */         importedClass = iClass;
/*      */       }
/*      */     }
/* 7627 */     if (importedClass == null) { return null;
/*      */     }
/*      */     
/* 7630 */     this.onDemandImportableTypes.put(simpleTypeName, importedClass);
/* 7631 */     return importedClass; }
/*      */   
/* 7633 */   private final Map onDemandImportableTypes = new HashMap();
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void declareClassDollarMethod(Java.ClassLiteral cl)
/*      */   {
/* 7646 */     Location loc = cl.getLocation();
/*      */     
/* 7648 */     for (Java.Scope s = cl.getEnclosingBlockStatement();; s = s.getEnclosingScope()) {
/* 7649 */       if ((s instanceof Java.AbstractTypeDeclaration)) {
/* 7650 */         Java.AbstractTypeDeclaration declaringType = (Java.AbstractTypeDeclaration)s;
/* 7651 */         break;
/*      */       }
/*      */     }
/*      */     
/*      */     Java.AbstractTypeDeclaration declaringType;
/*      */     
/* 7657 */     Java.MethodInvocation mi = new Java.MethodInvocation(loc, new Java.SimpleType(loc, this.iClassLoader.CLASS), "forName", new Java.Rvalue[] { new Java.AmbiguousName(loc, new String[] { "className" }) });
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     IClass classNotFoundExceptionIClass;
/*      */     
/*      */ 
/*      */ 
/*      */     try
/*      */     {
/* 7668 */       classNotFoundExceptionIClass = this.iClassLoader.loadIClass("Ljava/lang/ClassNotFoundException;");
/*      */     } catch (ClassNotFoundException ex) {
/* 7670 */       throw new JaninoRuntimeException("Loading class \"ClassNotFoundException\": " + ex.getMessage());
/*      */     }
/* 7672 */     if (classNotFoundExceptionIClass == null) {
/* 7673 */       throw new JaninoRuntimeException("SNO: Cannot load \"ClassNotFoundException\"");
/*      */     }
/*      */     IClass noClassDefFoundErrorIClass;
/*      */     try
/*      */     {
/* 7678 */       noClassDefFoundErrorIClass = this.iClassLoader.loadIClass("Ljava/lang/NoClassDefFoundError;");
/*      */     } catch (ClassNotFoundException ex) {
/* 7680 */       throw new JaninoRuntimeException("Loading class \"NoClassDefFoundError\": " + ex.getMessage());
/*      */     }
/* 7682 */     if (noClassDefFoundErrorIClass == null) {
/* 7683 */       throw new JaninoRuntimeException("SNO: Cannot load \"NoClassFoundError\"");
/*      */     }
/*      */     
/*      */ 
/* 7687 */     Java.Block b = new Java.Block(loc);
/*      */     
/* 7689 */     b.addStatement(new Java.ThrowStatement(loc, new Java.NewClassInstance(loc, (Java.Rvalue)null, new Java.SimpleType(loc, noClassDefFoundErrorIClass), new Java.Rvalue[] { new Java.MethodInvocation(loc, new Java.AmbiguousName(loc, new String[] { "ex" }), "getMessage", new Java.Rvalue[0]) })));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 7703 */     List l = new ArrayList();
/* 7704 */     l.add(new Java.CatchClause(loc, new Java.FunctionDeclarator.FormalParameter(loc, true, new Java.SimpleType(loc, classNotFoundExceptionIClass), "ex"), b));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 7714 */     Java.TryStatement ts = new Java.TryStatement(loc, new Java.ReturnStatement(loc, mi), l, null);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 7721 */     List statements = new ArrayList();
/* 7722 */     statements.add(ts);
/*      */     
/*      */ 
/* 7725 */     Java.FunctionDeclarator.FormalParameter fp = new Java.FunctionDeclarator.FormalParameter(loc, false, new Java.SimpleType(loc, this.iClassLoader.STRING), "className");
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 7731 */     Java.MethodDeclarator cdmd = new Java.MethodDeclarator(loc, null, (short)8, new Java.SimpleType(loc, this.iClassLoader.CLASS), "class$", new Java.FunctionDeclarator.FormalParameter[] { fp }, new Java.Type[0], statements);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 7742 */     declaringType.addDeclaredMethod(cdmd);
/* 7743 */     declaringType.invalidateMethodCaches();
/*      */   }
/*      */   
/*      */   private IClass pushConstant(Java.Locatable l, Object value) {
/* 7747 */     if (((value instanceof Integer)) || ((value instanceof Short)) || ((value instanceof Character)) || ((value instanceof Byte)))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 7753 */       int i = (value instanceof Character) ? ((Character)value).charValue() : ((Number)value).intValue();
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 7758 */       if ((i >= -1) && (i <= 5)) {
/* 7759 */         writeOpcode(l, 3 + i);
/*      */       }
/* 7761 */       else if ((i >= -128) && (i <= 127)) {
/* 7762 */         writeOpcode(l, 16);
/* 7763 */         writeByte((byte)i);
/*      */       } else {
/* 7765 */         writeLDC(l, addConstantIntegerInfo(i));
/*      */       }
/* 7767 */       return IClass.INT;
/*      */     }
/* 7769 */     if ((value instanceof Long)) {
/* 7770 */       long lv = ((Long)value).longValue();
/* 7771 */       if ((lv >= 0L) && (lv <= 1L)) {
/* 7772 */         writeOpcode(l, 9 + (int)lv);
/*      */       } else {
/* 7774 */         writeOpcode(l, 20);
/* 7775 */         writeConstantLongInfo(lv);
/*      */       }
/* 7777 */       return IClass.LONG;
/*      */     }
/* 7779 */     if ((value instanceof Float)) {
/* 7780 */       float fv = ((Float)value).floatValue();
/* 7781 */       if ((Float.floatToIntBits(fv) == Float.floatToIntBits(0.0F)) || (fv == 1.0F) || (fv == 2.0F))
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/* 7786 */         writeOpcode(l, 11 + (int)fv);
/*      */       }
/*      */       else {
/* 7789 */         writeLDC(l, addConstantFloatInfo(fv));
/*      */       }
/* 7791 */       return IClass.FLOAT;
/*      */     }
/* 7793 */     if ((value instanceof Double)) {
/* 7794 */       double dv = ((Double)value).doubleValue();
/* 7795 */       if ((Double.doubleToLongBits(dv) == Double.doubleToLongBits(0.0D)) || (dv == 1.0D))
/*      */       {
/*      */ 
/*      */ 
/* 7799 */         writeOpcode(l, 14 + (int)dv);
/*      */       }
/*      */       else {
/* 7802 */         writeOpcode(l, 20);
/* 7803 */         writeConstantDoubleInfo(dv);
/*      */       }
/* 7805 */       return IClass.DOUBLE;
/*      */     }
/* 7807 */     if ((value instanceof String)) {
/* 7808 */       String s = (String)value;
/* 7809 */       String[] ss = makeUTF8Able(s);
/* 7810 */       writeLDC(l, addConstantStringInfo(ss[0]));
/* 7811 */       for (int i = 1; i < ss.length; i++) {
/* 7812 */         writeLDC(l, addConstantStringInfo(ss[i]));
/* 7813 */         writeOpcode(l, -74);
/* 7814 */         writeConstantMethodrefInfo("Ljava/lang/String;", "concat", "(Ljava/lang/String;)Ljava/lang/String;");
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 7820 */       return this.iClassLoader.STRING;
/*      */     }
/* 7822 */     if ((value instanceof Boolean)) {
/* 7823 */       writeOpcode(l, ((Boolean)value).booleanValue() ? 4 : 3);
/* 7824 */       return IClass.BOOLEAN;
/*      */     }
/* 7826 */     if (value == Java.Rvalue.CONSTANT_VALUE_NULL) {
/* 7827 */       writeOpcode(l, 1);
/* 7828 */       return IClass.VOID;
/*      */     }
/* 7830 */     throw new JaninoRuntimeException("Unknown literal type \"" + value.getClass().getName() + "\"");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static String[] makeUTF8Able(String s)
/*      */   {
/* 7840 */     if (s.length() < 21845) { return new String[] { s };
/*      */     }
/* 7842 */     int sLength = s.length();int uTFLength = 0;
/* 7843 */     int from = 0;
/* 7844 */     List l = new ArrayList();
/* 7845 */     for (int i = 0;; i++) {
/* 7846 */       if (i == sLength) {
/* 7847 */         l.add(s.substring(from));
/* 7848 */         break;
/*      */       }
/* 7850 */       if (uTFLength >= 65532) {
/* 7851 */         l.add(s.substring(from, i));
/* 7852 */         if (i + 21845 > sLength) {
/* 7853 */           l.add(s.substring(i));
/* 7854 */           break;
/*      */         }
/* 7856 */         from = i;
/* 7857 */         uTFLength = 0;
/*      */       }
/* 7859 */       int c = s.charAt(i);
/* 7860 */       if ((c >= 1) && (c <= 127)) {
/* 7861 */         uTFLength++;
/*      */       }
/* 7863 */       else if (c > 2047) {
/* 7864 */         uTFLength += 3;
/*      */       }
/*      */       else {
/* 7867 */         uTFLength += 2;
/*      */       }
/*      */     }
/* 7870 */     return (String[])l.toArray(new String[l.size()]);
/*      */   }
/*      */   
/*      */   private void writeLDC(Java.Locatable l, short index) {
/* 7874 */     if ((0 <= index) && (index <= 255)) {
/* 7875 */       writeOpcode(l, 18);
/* 7876 */       writeByte((byte)index);
/*      */     } else {
/* 7878 */       writeOpcode(l, 19);
/* 7879 */       writeShort(index);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void assignmentConversion(Java.Locatable l, IClass sourceType, IClass targetType, Object optionalConstantValue)
/*      */     throws CompileException
/*      */   {
/* 7905 */     if (tryIdentityConversion(sourceType, targetType)) { return;
/*      */     }
/*      */     
/* 7908 */     if (tryWideningPrimitiveConversion(l, sourceType, targetType)) { return;
/*      */     }
/*      */     
/* 7911 */     if (isWideningReferenceConvertible(sourceType, targetType)) { return;
/*      */     }
/*      */     
/*      */ 
/* 7915 */     IClass boxedType = isBoxingConvertible(sourceType);
/* 7916 */     if (boxedType != null) {
/* 7917 */       if (tryIdentityConversion(boxedType, targetType)) {
/* 7918 */         boxingConversion(l, sourceType, boxedType);
/* 7919 */         return;
/*      */       }
/* 7921 */       if (isWideningReferenceConvertible(boxedType, targetType)) {
/* 7922 */         boxingConversion(l, sourceType, boxedType);
/* 7923 */         return;
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 7930 */     IClass unboxedType = isUnboxingConvertible(sourceType);
/* 7931 */     if (unboxedType != null) {
/* 7932 */       if (tryIdentityConversion(unboxedType, targetType)) {
/* 7933 */         unboxingConversion(l, sourceType, unboxedType);
/* 7934 */         return;
/*      */       }
/* 7936 */       if (isWideningPrimitiveConvertible(unboxedType, targetType)) {
/* 7937 */         unboxingConversion(l, sourceType, unboxedType);
/* 7938 */         tryWideningPrimitiveConversion(l, unboxedType, targetType);
/* 7939 */         return;
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 7945 */     if ((optionalConstantValue != null) && 
/* 7946 */       (tryConstantAssignmentConversion(l, optionalConstantValue, targetType)))
/*      */     {
/*      */ 
/*      */ 
/* 7950 */       return;
/*      */     }
/*      */     
/* 7953 */     compileError("Assignment conversion not possible from type \"" + sourceType + "\" to type \"" + targetType + "\"", l.getLocation());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private Object assignmentConversion(Java.Locatable l, Object value, IClass targetType)
/*      */     throws CompileException
/*      */   {
/* 7967 */     Object result = null;
/*      */     
/* 7969 */     if (targetType == IClass.BOOLEAN) {
/* 7970 */       if ((value instanceof Boolean)) result = value;
/*      */     }
/* 7972 */     else if (targetType == this.iClassLoader.STRING) {
/* 7973 */       if ((value instanceof String)) result = value;
/*      */     }
/* 7975 */     else if (targetType == IClass.BYTE) {
/* 7976 */       if ((value instanceof Byte)) {
/* 7977 */         result = value;
/*      */       }
/* 7979 */       else if (((value instanceof Short)) || ((value instanceof Integer))) {
/* 7980 */         int x = ((Number)value).intValue();
/* 7981 */         if ((x >= -128) && (x <= 127)) result = new Byte((byte)x);
/*      */       }
/* 7983 */       else if ((value instanceof Character)) {
/* 7984 */         int x = ((Character)value).charValue();
/* 7985 */         if ((x >= -128) && (x <= 127)) result = new Byte((byte)x);
/*      */       }
/*      */     }
/* 7988 */     else if (targetType == IClass.SHORT) {
/* 7989 */       if ((value instanceof Byte)) {
/* 7990 */         result = new Short(((Number)value).shortValue());
/*      */       }
/* 7992 */       else if ((value instanceof Short)) {
/* 7993 */         result = value;
/*      */       }
/* 7995 */       else if ((value instanceof Character)) {
/* 7996 */         int x = ((Character)value).charValue();
/* 7997 */         if ((x >= 32768) && (x <= 32767)) result = new Short((short)x);
/*      */       }
/* 7999 */       else if ((value instanceof Integer)) {
/* 8000 */         int x = ((Integer)value).intValue();
/* 8001 */         if ((x >= 32768) && (x <= 32767)) result = new Short((short)x);
/*      */       }
/*      */     }
/* 8004 */     else if (targetType == IClass.CHAR) {
/* 8005 */       if ((value instanceof Short)) {
/* 8006 */         result = value;
/*      */       }
/* 8008 */       else if (((value instanceof Byte)) || ((value instanceof Short)) || ((value instanceof Integer))) {
/* 8009 */         int x = ((Number)value).intValue();
/* 8010 */         if ((x >= 0) && (x <= 65535)) result = new Character((char)x);
/*      */       }
/*      */     }
/* 8013 */     else if (targetType == IClass.INT) {
/* 8014 */       if ((value instanceof Integer)) {
/* 8015 */         result = value;
/*      */       }
/* 8017 */       else if (((value instanceof Byte)) || ((value instanceof Short))) {
/* 8018 */         result = new Integer(((Number)value).intValue());
/*      */       }
/* 8020 */       else if ((value instanceof Character)) {
/* 8021 */         result = new Integer(((Character)value).charValue());
/*      */       }
/*      */     }
/* 8024 */     else if (targetType == IClass.LONG) {
/* 8025 */       if ((value instanceof Long)) {
/* 8026 */         result = value;
/*      */       }
/* 8028 */       else if (((value instanceof Byte)) || ((value instanceof Short)) || ((value instanceof Integer))) {
/* 8029 */         result = new Long(((Number)value).longValue());
/*      */       }
/* 8031 */       else if ((value instanceof Character)) {
/* 8032 */         result = new Long(((Character)value).charValue());
/*      */       }
/*      */     }
/* 8035 */     else if (targetType == IClass.FLOAT) {
/* 8036 */       if ((value instanceof Float)) {
/* 8037 */         result = value;
/*      */       }
/* 8039 */       else if (((value instanceof Byte)) || ((value instanceof Short)) || ((value instanceof Integer)) || ((value instanceof Long))) {
/* 8040 */         result = new Float(((Number)value).floatValue());
/*      */       }
/* 8042 */       else if ((value instanceof Character)) {
/* 8043 */         result = new Float(((Character)value).charValue());
/*      */       }
/*      */     }
/* 8046 */     else if (targetType == IClass.DOUBLE) {
/* 8047 */       if ((value instanceof Double)) {
/* 8048 */         result = value;
/*      */       }
/* 8050 */       else if (((value instanceof Byte)) || ((value instanceof Short)) || ((value instanceof Integer)) || ((value instanceof Long)) || ((value instanceof Float)))
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 8057 */         result = new Double(((Number)value).doubleValue());
/*      */       }
/* 8059 */       else if ((value instanceof Character)) {
/* 8060 */         result = new Double(((Character)value).charValue());
/*      */       }
/*      */     }
/* 8063 */     else if ((value == Java.Rvalue.CONSTANT_VALUE_NULL) && (!targetType.isPrimitive())) {
/* 8064 */       result = value;
/*      */     }
/* 8066 */     if (result == null) {
/* 8067 */       compileError("Cannot convert constant of type \"" + value.getClass().getName() + "\" to type \"" + targetType.toString() + "\"", l.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 8075 */     return result;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass unaryNumericPromotion(Java.Locatable l, IClass type)
/*      */     throws CompileException
/*      */   {
/* 8084 */     type = convertToPrimitiveNumericType(l, type);
/*      */     
/* 8086 */     IClass promotedType = unaryNumericPromotionType(l, type);
/*      */     
/* 8088 */     numericPromotion(l, type, promotedType);
/* 8089 */     return promotedType;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void reverseUnaryNumericPromotion(Java.Locatable l, IClass sourceType, IClass targetType)
/*      */     throws CompileException
/*      */   {
/* 8097 */     IClass unboxedType = isUnboxingConvertible(targetType);
/* 8098 */     IClass pt = unboxedType != null ? unboxedType : targetType;
/* 8099 */     if ((!tryIdentityConversion(sourceType, pt)) && (!tryNarrowingPrimitiveConversion(l, sourceType, pt)))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 8106 */       throw new JaninoRuntimeException("SNO: reverse unary numeric promotion failed"); }
/* 8107 */     if (unboxedType != null) { boxingConversion(l, unboxedType, targetType);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass convertToPrimitiveNumericType(Java.Locatable l, IClass type)
/*      */     throws CompileException
/*      */   {
/* 8117 */     if (type.isPrimitiveNumeric()) return type;
/* 8118 */     IClass unboxedType = isUnboxingConvertible(type);
/* 8119 */     if (unboxedType != null) {
/* 8120 */       unboxingConversion(l, type, unboxedType);
/* 8121 */       return unboxedType;
/*      */     }
/* 8123 */     compileError("Object of type \"" + type.toString() + "\" cannot be converted to a numeric type", l.getLocation());
/*      */     
/*      */ 
/*      */ 
/* 8127 */     return type;
/*      */   }
/*      */   
/*      */   private void numericPromotion(Java.Locatable l, IClass sourceType, IClass targetType) {
/* 8131 */     if ((!tryIdentityConversion(sourceType, targetType)) && (!tryWideningPrimitiveConversion(l, sourceType, targetType)))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 8138 */       throw new JaninoRuntimeException("SNO: Conversion failed"); }
/*      */   }
/*      */   
/*      */   private IClass unaryNumericPromotionType(Java.Locatable l, IClass type) throws CompileException {
/* 8142 */     if (!type.isPrimitiveNumeric()) {
/* 8143 */       compileError("Unary numeric promotion not possible on non-numeric-primitive type \"" + type + "\"", l.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 8149 */     return type == IClass.LONG ? IClass.LONG : type == IClass.FLOAT ? IClass.FLOAT : type == IClass.DOUBLE ? IClass.DOUBLE : IClass.INT;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass binaryNumericPromotion(Java.Locatable locatable, IClass type1, CodeContext.Inserter convertInserter1, IClass type2)
/*      */     throws CompileException
/*      */   {
/* 8168 */     return binaryNumericPromotion(locatable, type1, convertInserter1, type2, this.codeContext.currentInserter());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass binaryNumericPromotion(Java.Locatable l, IClass type1, CodeContext.Inserter convertInserter1, IClass type2, CodeContext.Inserter convertInserter2)
/*      */     throws CompileException
/*      */   {
/* 8190 */     IClass c1 = isUnboxingConvertible(type1);
/* 8191 */     IClass c2 = isUnboxingConvertible(type2);
/* 8192 */     IClass promotedType = binaryNumericPromotionType(l, c1 != null ? c1 : type1, c2 != null ? c2 : type2);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 8199 */     if (convertInserter1 != null) {
/* 8200 */       this.codeContext.pushInserter(convertInserter1);
/*      */       try {
/* 8202 */         numericPromotion(l, convertToPrimitiveNumericType(l, type1), promotedType);
/*      */       } finally {
/* 8204 */         this.codeContext.popInserter();
/*      */       }
/*      */     }
/*      */     
/* 8208 */     if (convertInserter2 != null) {
/* 8209 */       this.codeContext.pushInserter(convertInserter2);
/*      */       try {
/* 8211 */         numericPromotion(l, convertToPrimitiveNumericType(l, type2), promotedType);
/*      */       } finally {
/* 8213 */         this.codeContext.popInserter();
/*      */       }
/*      */     }
/*      */     
/* 8217 */     return promotedType;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private IClass binaryNumericPromotionType(Java.Locatable locatable, IClass type1, IClass type2)
/*      */     throws CompileException
/*      */   {
/* 8225 */     if ((!type1.isPrimitiveNumeric()) || (!type2.isPrimitiveNumeric()))
/*      */     {
/*      */ 
/* 8228 */       compileError("Binary numeric promotion not possible on types \"" + type1 + "\" and \"" + type2 + "\"", locatable.getLocation());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 8233 */     return (type1 == IClass.LONG) || (type2 == IClass.LONG) ? IClass.LONG : (type1 == IClass.FLOAT) || (type2 == IClass.FLOAT) ? IClass.FLOAT : (type1 == IClass.DOUBLE) || (type2 == IClass.DOUBLE) ? IClass.DOUBLE : IClass.INT;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean isIdentityConvertible(IClass sourceType, IClass targetType)
/*      */   {
/* 8250 */     return sourceType == targetType;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean tryIdentityConversion(IClass sourceType, IClass targetType)
/*      */   {
/* 8262 */     return sourceType == targetType;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private boolean isWideningPrimitiveConvertible(IClass sourceType, IClass targetType)
/*      */   {
/* 8269 */     return PRIMITIVE_WIDENING_CONVERSIONS.get(sourceType.getDescriptor() + targetType.getDescriptor()) != null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean tryWideningPrimitiveConversion(Java.Locatable l, IClass sourceType, IClass targetType)
/*      */   {
/* 8284 */     byte[] opcodes = (byte[])PRIMITIVE_WIDENING_CONVERSIONS.get(sourceType.getDescriptor() + targetType.getDescriptor());
/*      */     
/*      */ 
/* 8287 */     if (opcodes != null) {
/* 8288 */       writeOpcodes(l, opcodes);
/* 8289 */       return true;
/*      */     }
/* 8291 */     return false; }
/*      */   
/* 8293 */   private static final HashMap PRIMITIVE_WIDENING_CONVERSIONS = new HashMap();
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static final HashMap PRIMITIVE_NARROWING_CONVERSIONS;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static void fillConversionMap(Object[] array, HashMap map)
/*      */   {
/* 8330 */     byte[] opcodes = null;
/* 8331 */     for (int i = 0; i < array.length; i++) {
/* 8332 */       Object o = array[i];
/* 8333 */       if ((o instanceof byte[])) {
/* 8334 */         opcodes = (byte[])o;
/*      */       } else {
/* 8336 */         map.put(o, opcodes);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean isWideningReferenceConvertible(IClass sourceType, IClass targetType)
/*      */     throws CompileException
/*      */   {
/* 8350 */     if ((targetType.isPrimitive()) || (sourceType == targetType))
/*      */     {
/*      */ 
/* 8353 */       return false;
/*      */     }
/* 8355 */     return targetType.isAssignableFrom(sourceType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean tryWideningReferenceConversion(IClass sourceType, IClass targetType)
/*      */     throws CompileException
/*      */   {
/* 8367 */     if ((targetType.isPrimitive()) || (sourceType == targetType))
/*      */     {
/*      */ 
/* 8370 */       return false;
/*      */     }
/* 8372 */     return targetType.isAssignableFrom(sourceType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean isNarrowingPrimitiveConvertible(IClass sourceType, IClass targetType)
/*      */   {
/* 8382 */     return PRIMITIVE_NARROWING_CONVERSIONS.containsKey(sourceType.getDescriptor() + targetType.getDescriptor());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean tryNarrowingPrimitiveConversion(Java.Locatable l, IClass sourceType, IClass targetType)
/*      */   {
/* 8397 */     byte[] opcodes = (byte[])PRIMITIVE_NARROWING_CONVERSIONS.get(sourceType.getDescriptor() + targetType.getDescriptor());
/*      */     
/*      */ 
/* 8400 */     if (opcodes != null) {
/* 8401 */       writeOpcodes(l, opcodes);
/* 8402 */       return true;
/*      */     }
/* 8404 */     return false;
/*      */   }
/*      */   
/*      */   static
/*      */   {
/* 8294 */     fillConversionMap(new Object[] { new byte[0], "BS", "BI", "SI", "CI", { -123 }, "BJ", "SJ", "CJ", "IJ", { -122 }, "BF", "SF", "CF", "IF", { -119 }, "JF", { -121 }, "BD", "SD", "CD", "ID", { -118 }, "JD", { -115 }, "FD" }, PRIMITIVE_WIDENING_CONVERSIONS);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 8407 */     PRIMITIVE_NARROWING_CONVERSIONS = new HashMap();
/* 8408 */     fillConversionMap(new Object[] { new byte[0], "BC", "SC", "CS", { -111 }, "SB", "CB", "IB", { -109 }, "IS", "IC", { -120, -111 }, "JB", { -120, -109 }, "JS", "JC", { -120 }, "JI", { -117, -111 }, "FB", { -117, -109 }, "FS", "FC", { -117 }, "FI", { -116 }, "FJ", { -114, -111 }, "DB", { -114, -109 }, "DS", "DC", { -114 }, "DI", { -113 }, "DJ", { -112 }, "DF" }, PRIMITIVE_NARROWING_CONVERSIONS);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean tryConstantAssignmentConversion(Java.Locatable l, Object constantValue, IClass targetType)
/*      */     throws CompileException
/*      */   {
/*      */     int cv;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 8478 */     if ((constantValue instanceof Byte)) {
/* 8479 */       cv = ((Byte)constantValue).byteValue();
/*      */     } else { int cv;
/* 8481 */       if ((constantValue instanceof Short)) {
/* 8482 */         cv = ((Short)constantValue).shortValue();
/*      */       } else { int cv;
/* 8484 */         if ((constantValue instanceof Integer)) {
/* 8485 */           cv = ((Integer)constantValue).intValue();
/*      */         } else { int cv;
/* 8487 */           if ((constantValue instanceof Character)) {
/* 8488 */             cv = ((Character)constantValue).charValue();
/*      */           }
/*      */           else
/* 8491 */             return false;
/*      */         } } }
/*      */     int cv;
/* 8494 */     if (targetType == IClass.BYTE) return (cv >= -128) && (cv <= 127);
/* 8495 */     if (targetType == IClass.SHORT) return (cv >= 32768) && (cv <= 32767);
/* 8496 */     if (targetType == IClass.CHAR) { return (cv >= 0) && (cv <= 65535);
/*      */     }
/* 8498 */     IClassLoader icl = this.iClassLoader;
/* 8499 */     if ((targetType == icl.BYTE) && (cv >= -128) && (cv <= 127)) {
/* 8500 */       boxingConversion(l, IClass.BYTE, targetType);
/* 8501 */       return true;
/*      */     }
/* 8503 */     if ((targetType == icl.SHORT) && (cv >= 32768) && (cv <= 32767)) {
/* 8504 */       boxingConversion(l, IClass.SHORT, targetType);
/* 8505 */       return true;
/*      */     }
/* 8507 */     if ((targetType == icl.CHARACTER) && (cv >= 0) && (cv <= 65535)) {
/* 8508 */       boxingConversion(l, IClass.CHAR, targetType);
/* 8509 */       return true;
/*      */     }
/*      */     
/* 8512 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean isNarrowingReferenceConvertible(IClass sourceType, IClass targetType)
/*      */     throws CompileException
/*      */   {
/* 8522 */     if (sourceType.isPrimitive()) return false;
/* 8523 */     if (sourceType == targetType) { return false;
/*      */     }
/*      */     
/* 8526 */     if (sourceType.isAssignableFrom(targetType)) { return true;
/*      */     }
/*      */     
/* 8529 */     if ((targetType.isInterface()) && (!sourceType.isFinal()) && (!targetType.isAssignableFrom(sourceType)))
/*      */     {
/*      */ 
/*      */ 
/* 8533 */       return true;
/*      */     }
/*      */     
/* 8536 */     if ((sourceType == this.iClassLoader.OBJECT) && (targetType.isArray()))
/*      */     {
/*      */ 
/* 8539 */       return true;
/*      */     }
/*      */     
/* 8542 */     if ((sourceType == this.iClassLoader.OBJECT) && (targetType.isInterface()))
/*      */     {
/*      */ 
/* 8545 */       return true;
/*      */     }
/*      */     
/* 8548 */     if ((sourceType.isInterface()) && (!targetType.isFinal()))
/*      */     {
/*      */ 
/* 8551 */       return true;
/*      */     }
/*      */     
/* 8554 */     if ((sourceType.isInterface()) && (targetType.isFinal()) && (sourceType.isAssignableFrom(targetType)))
/*      */     {
/*      */ 
/*      */ 
/* 8558 */       return true;
/*      */     }
/*      */     
/*      */ 
/* 8562 */     if ((sourceType.isInterface()) && (targetType.isInterface()) && (!targetType.isAssignableFrom(sourceType)))
/*      */     {
/*      */ 
/*      */ 
/* 8566 */       return true;
/*      */     }
/*      */     
/* 8569 */     if ((sourceType.isArray()) && (targetType.isArray())) {
/* 8570 */       IClass st = sourceType.getComponentType();
/* 8571 */       IClass tt = targetType.getComponentType();
/* 8572 */       if ((isNarrowingPrimitiveConvertible(st, tt)) || (isNarrowingReferenceConvertible(st, tt)))
/*      */       {
/*      */ 
/* 8575 */         return true;
/*      */       }
/*      */     }
/* 8578 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean tryNarrowingReferenceConversion(Java.Locatable l, IClass sourceType, IClass targetType)
/*      */     throws CompileException
/*      */   {
/* 8591 */     if (!isNarrowingReferenceConvertible(sourceType, targetType)) { return false;
/*      */     }
/* 8593 */     writeOpcode(l, -64);
/* 8594 */     writeConstantClassInfo(targetType.getDescriptor());
/* 8595 */     return true;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private IClass isBoxingConvertible(IClass sourceType)
/*      */   {
/* 8602 */     IClassLoader icl = this.iClassLoader;
/* 8603 */     if (sourceType == IClass.BOOLEAN) return icl.BOOLEAN;
/* 8604 */     if (sourceType == IClass.BYTE) return icl.BYTE;
/* 8605 */     if (sourceType == IClass.CHAR) return icl.CHARACTER;
/* 8606 */     if (sourceType == IClass.SHORT) return icl.SHORT;
/* 8607 */     if (sourceType == IClass.INT) return icl.INTEGER;
/* 8608 */     if (sourceType == IClass.LONG) return icl.LONG;
/* 8609 */     if (sourceType == IClass.FLOAT) return icl.FLOAT;
/* 8610 */     if (sourceType == IClass.DOUBLE) return icl.DOUBLE;
/* 8611 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private boolean tryBoxingConversion(Java.Locatable l, IClass sourceType, IClass targetType)
/*      */     throws CompileException
/*      */   {
/* 8619 */     if (isBoxingConvertible(sourceType) == targetType) {
/* 8620 */       boxingConversion(l, sourceType, targetType);
/* 8621 */       return true;
/*      */     }
/* 8623 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void boxingConversion(Java.Locatable l, IClass sourceType, IClass targetType)
/*      */     throws CompileException
/*      */   {
/* 8633 */     if (targetType.hasIMethod("valueOf", new IClass[] { sourceType })) {
/* 8634 */       writeOpcode(l, -72);
/* 8635 */       writeConstantMethodrefInfo(targetType.getDescriptor(), "valueOf", '(' + sourceType.getDescriptor() + ')' + targetType.getDescriptor());
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 8640 */       return;
/*      */     }
/*      */     
/* 8643 */     writeOpcode(l, -69);
/* 8644 */     writeConstantClassInfo(targetType.getDescriptor());
/* 8645 */     if (Descriptor.hasSize2(sourceType.getDescriptor())) {
/* 8646 */       writeOpcode(l, 91);
/* 8647 */       writeOpcode(l, 91);
/* 8648 */       writeOpcode(l, 87);
/*      */     }
/*      */     else {
/* 8651 */       writeOpcode(l, 90);
/* 8652 */       writeOpcode(l, 95);
/*      */     }
/* 8654 */     writeOpcode(l, -73);
/* 8655 */     writeConstantMethodrefInfo(targetType.getDescriptor(), "<init>", '(' + sourceType.getDescriptor() + ')' + "V");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass isUnboxingConvertible(IClass sourceType)
/*      */   {
/* 8666 */     IClassLoader icl = this.iClassLoader;
/* 8667 */     if (sourceType == icl.BOOLEAN) return IClass.BOOLEAN;
/* 8668 */     if (sourceType == icl.BYTE) return IClass.BYTE;
/* 8669 */     if (sourceType == icl.CHARACTER) return IClass.CHAR;
/* 8670 */     if (sourceType == icl.SHORT) return IClass.SHORT;
/* 8671 */     if (sourceType == icl.INTEGER) return IClass.INT;
/* 8672 */     if (sourceType == icl.LONG) return IClass.LONG;
/* 8673 */     if (sourceType == icl.FLOAT) return IClass.FLOAT;
/* 8674 */     if (sourceType == icl.DOUBLE) return IClass.DOUBLE;
/* 8675 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean tryUnboxingConversion(Java.Locatable l, IClass sourceType, IClass targetType)
/*      */   {
/* 8683 */     if (isUnboxingConvertible(sourceType) == targetType) {
/* 8684 */       unboxingConversion(l, sourceType, targetType);
/* 8685 */       return true;
/*      */     }
/* 8687 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void unboxingConversion(Java.Locatable l, IClass sourceType, IClass targetType)
/*      */   {
/* 8697 */     writeOpcode(l, -74);
/* 8698 */     writeConstantMethodrefInfo(sourceType.getDescriptor(), targetType.toString() + "Value", "()" + targetType.getDescriptor());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass loadFullyQualifiedClass(String[] identifiers)
/*      */     throws CompileException
/*      */   {
/* 8714 */     int[] slashes = new int[identifiers.length - 1];
/* 8715 */     StringBuffer sb = new StringBuffer("L");
/* 8716 */     for (int i = 0;; i++) {
/* 8717 */       sb.append(identifiers[i]);
/* 8718 */       if (i == identifiers.length - 1) break;
/* 8719 */       slashes[i] = sb.length();
/* 8720 */       sb.append('/');
/*      */     }
/* 8722 */     sb.append(';');
/*      */     
/*      */ 
/*      */ 
/* 8726 */     for (int j = slashes.length - 1;; j--) {
/*      */       IClass result;
/*      */       try {
/* 8729 */         result = this.iClassLoader.loadIClass(sb.toString());
/*      */       } catch (ClassNotFoundException ex) {
/* 8731 */         if ((ex.getException() instanceof CompileException)) throw ((CompileException)ex.getException());
/* 8732 */         throw new CompileException(sb.toString(), null, ex);
/*      */       }
/* 8734 */       if (result != null) return result;
/* 8735 */       if (j < 0) break;
/* 8736 */       sb.setCharAt(slashes[j], '$');
/*      */     }
/* 8738 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass load(Java.Locatable l, Java.LocalVariable localVariable)
/*      */   {
/* 8746 */     load(l, localVariable.type, localVariable.getSlotIndex());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 8751 */     return localVariable.type;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void load(Java.Locatable l, IClass type, int index)
/*      */   {
/* 8758 */     if (index <= 3) {
/* 8759 */       writeOpcode(l, 26 + 4 * ilfda(type) + index);
/*      */     }
/* 8761 */     else if (index <= 255) {
/* 8762 */       writeOpcode(l, 21 + ilfda(type));
/* 8763 */       writeByte(index);
/*      */     }
/*      */     else {
/* 8766 */       writeOpcode(l, -60);
/* 8767 */       writeOpcode(l, 21 + ilfda(type));
/* 8768 */       writeShort(index);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void store(Java.Locatable l, IClass valueType, Java.LocalVariable localVariable)
/*      */   {
/* 8783 */     store(l, localVariable.type, localVariable.getSlotIndex());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void store(Java.Locatable l, IClass lvType, short lvIndex)
/*      */   {
/* 8794 */     if (lvIndex <= 3) {
/* 8795 */       writeOpcode(l, 59 + 4 * ilfda(lvType) + lvIndex);
/*      */     }
/* 8797 */     else if (lvIndex <= 255) {
/* 8798 */       writeOpcode(l, 54 + ilfda(lvType));
/* 8799 */       writeByte(lvIndex);
/*      */     }
/*      */     else {
/* 8802 */       writeOpcode(l, -60);
/* 8803 */       writeOpcode(l, 54 + ilfda(lvType));
/* 8804 */       writeShort(lvIndex);
/*      */     }
/*      */   }
/*      */   
/*      */   private void dup(Java.Locatable l, int n) {
/* 8809 */     switch (n)
/*      */     {
/*      */     case 0: 
/*      */       break;
/*      */     
/*      */ 
/*      */     case 1: 
/* 8816 */       writeOpcode(l, 89);
/* 8817 */       break;
/*      */     
/*      */     case 2: 
/* 8820 */       writeOpcode(l, 92);
/* 8821 */       break;
/*      */     
/*      */     default: 
/* 8824 */       throw new JaninoRuntimeException("dup(" + n + ")");
/*      */     }
/*      */     
/*      */   }
/*      */   
/*      */ 
/*      */   private void dupx(Java.Locatable l, IClass type, int x)
/*      */   {
/* 8832 */     if ((x < 0) || (x > 2)) throw new JaninoRuntimeException("SNO: x has value " + x);
/* 8833 */     int dup = 89 + x;
/* 8834 */     int dup2 = 92 + x;
/* 8835 */     writeOpcode(l, (type == IClass.LONG) || (type == IClass.DOUBLE) ? dup2 : dup);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void pop(Java.Locatable l, IClass type)
/*      */   {
/* 8843 */     if (type == IClass.VOID) return;
/* 8844 */     writeOpcode(l, (type == IClass.LONG) || (type == IClass.DOUBLE) ? 88 : 87);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   static int ilfd(IClass t)
/*      */   {
/* 8852 */     if ((t == IClass.BYTE) || (t == IClass.CHAR) || (t == IClass.INT) || (t == IClass.SHORT) || (t == IClass.BOOLEAN))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 8858 */       return 0; }
/* 8859 */     if (t == IClass.LONG) return 1;
/* 8860 */     if (t == IClass.FLOAT) return 2;
/* 8861 */     if (t == IClass.DOUBLE) return 3;
/* 8862 */     throw new JaninoRuntimeException("Unexpected type \"" + t + "\"");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   static int ilfd(IClass t, int opcodeInt, int opcodeLong, int opcodeFloat, int opcodeDouble)
/*      */   {
/* 8871 */     if ((t == IClass.BYTE) || (t == IClass.CHAR) || (t == IClass.INT) || (t == IClass.SHORT) || (t == IClass.BOOLEAN))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 8877 */       return opcodeInt; }
/* 8878 */     if (t == IClass.LONG) return opcodeLong;
/* 8879 */     if (t == IClass.FLOAT) return opcodeFloat;
/* 8880 */     if (t == IClass.DOUBLE) return opcodeDouble;
/* 8881 */     throw new JaninoRuntimeException("Unexpected type \"" + t + "\"");
/*      */   }
/*      */   
/* 8884 */   private int ilfda(IClass t) { return !t.isPrimitive() ? 4 : ilfd(t); }
/*      */   
/*      */   static int ilfdabcs(IClass t) {
/* 8887 */     if (t == IClass.INT) return 0;
/* 8888 */     if (t == IClass.LONG) return 1;
/* 8889 */     if (t == IClass.FLOAT) return 2;
/* 8890 */     if (t == IClass.DOUBLE) return 3;
/* 8891 */     if (!t.isPrimitive()) return 4;
/* 8892 */     if (t == IClass.BOOLEAN) return 5;
/* 8893 */     if (t == IClass.BYTE) return 5;
/* 8894 */     if (t == IClass.CHAR) return 6;
/* 8895 */     if (t == IClass.SHORT) return 7;
/* 8896 */     throw new JaninoRuntimeException("Unexpected type \"" + t + "\"");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass.IField findIField(IClass iClass, String name, Location location)
/*      */     throws CompileException
/*      */   {
/* 8911 */     IClass.IField f = iClass.getDeclaredIField(name);
/* 8912 */     if (f != null) { return f;
/*      */     }
/*      */     
/*      */ 
/* 8916 */     IClass superclass = iClass.getSuperclass();
/* 8917 */     if (superclass != null) { f = findIField(superclass, name, location);
/*      */     }
/*      */     
/*      */ 
/* 8921 */     IClass[] ifs = iClass.getInterfaces();
/* 8922 */     for (int i = 0; i < ifs.length; i++) {
/* 8923 */       IClass.IField f2 = findIField(ifs[i], name, location);
/* 8924 */       if (f2 != null) {
/* 8925 */         if (f != null) {
/* 8926 */           throw new CompileException("Access to field \"" + name + "\" is ambiguous - both \"" + f.getDeclaringIClass() + "\" and \"" + f2.getDeclaringIClass() + "\" declare it", location);
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 8936 */         f = f2;
/*      */       }
/*      */     }
/* 8939 */     return f;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass findMemberType(IClass iClass, String name, Location location)
/*      */     throws CompileException
/*      */   {
/* 8952 */     IClass[] types = iClass.findMemberType(name);
/* 8953 */     if (types.length == 0) return null;
/* 8954 */     if (types.length == 1) { return types[0];
/*      */     }
/* 8956 */     StringBuffer sb = new StringBuffer("Type \"" + name + "\" is ambiguous: " + types[0].toString());
/* 8957 */     for (int i = 1; i < types.length; i++) sb.append(" vs. ").append(types[i].toString());
/* 8958 */     compileError(sb.toString(), location);
/* 8959 */     return types[0];
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IClass findClass(String className)
/*      */   {
/* 8971 */     String packageName = this.compilationUnit.optionalPackageDeclaration == null ? null : this.compilationUnit.optionalPackageDeclaration.packageName;
/*      */     
/*      */ 
/*      */ 
/* 8975 */     if (packageName != null) {
/* 8976 */       if (!className.startsWith(packageName + '.')) return null;
/* 8977 */       className = className.substring(packageName.length() + 1);
/*      */     }
/*      */     
/* 8980 */     StringTokenizer st = new StringTokenizer(className, "$");
/* 8981 */     Java.TypeDeclaration td = this.compilationUnit.getPackageMemberTypeDeclaration(st.nextToken());
/* 8982 */     if (td == null) return null;
/* 8983 */     while (st.hasMoreTokens()) {
/* 8984 */       td = td.getMemberTypeDeclaration(st.nextToken());
/* 8985 */       if (td == null) return null;
/*      */     }
/* 8987 */     return resolve((Java.AbstractTypeDeclaration)td);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void compileError(String message)
/*      */     throws CompileException
/*      */   {
/* 8995 */     compileError(message, null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void compileError(String message, Location optionalLocation)
/*      */     throws CompileException
/*      */   {
/* 9010 */     this.compileErrorCount += 1;
/* 9011 */     if (this.optionalCompileErrorHandler != null) {
/* 9012 */       this.optionalCompileErrorHandler.handleError(message, optionalLocation);
/*      */     } else {
/* 9014 */       throw new CompileException(message, optionalLocation);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void warning(String handle, String message, Location optionalLocation)
/*      */   {
/* 9026 */     if (this.optionalWarningHandler != null) {
/* 9027 */       this.optionalWarningHandler.handleWarning(handle, message, optionalLocation);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setCompileErrorHandler(ErrorHandler optionalCompileErrorHandler)
/*      */   {
/* 9055 */     this.optionalCompileErrorHandler = optionalCompileErrorHandler;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setWarningHandler(WarningHandler optionalWarningHandler)
/*      */   {
/* 9065 */     this.optionalWarningHandler = optionalWarningHandler;
/*      */   }
/*      */   
/*      */   private CodeContext getCodeContext() {
/* 9069 */     CodeContext res = this.codeContext;
/* 9070 */     if (res == null) throw new JaninoRuntimeException("S.N.O.: Null CodeContext");
/* 9071 */     return res;
/*      */   }
/*      */   
/*      */   private CodeContext replaceCodeContext(CodeContext newCodeContext) {
/* 9075 */     CodeContext oldCodeContext = this.codeContext;
/* 9076 */     this.codeContext = newCodeContext;
/* 9077 */     return oldCodeContext;
/*      */   }
/*      */   
/*      */   private CodeContext createDummyCodeContext() {
/* 9081 */     return new CodeContext(getCodeContext().getClassFile());
/*      */   }
/*      */   
/*      */   private void writeByte(int v) {
/* 9085 */     if (v > 255) {
/* 9086 */       throw new JaninoRuntimeException("Byte value out of legal range");
/*      */     }
/* 9088 */     this.codeContext.write((short)-1, (byte)v);
/*      */   }
/*      */   
/* 9091 */   private void writeShort(int v) { if (v > 65535) {
/* 9092 */       throw new JaninoRuntimeException("Short value out of legal range");
/*      */     }
/* 9094 */     this.codeContext.write((short)-1, (byte)(v >> 8), (byte)v);
/*      */   }
/*      */   
/* 9097 */   private void writeInt(int v) { this.codeContext.write((short)-1, (byte)(v >> 24), (byte)(v >> 16), (byte)(v >> 8), (byte)v); }
/*      */   
/*      */   private void writeOpcode(Java.Locatable l, int opcode)
/*      */   {
/* 9101 */     this.codeContext.write(l.getLocation().getLineNumber(), (byte)opcode);
/*      */   }
/*      */   
/* 9104 */   private void writeOpcodes(Java.Locatable l, byte[] opcodes) { this.codeContext.write(l.getLocation().getLineNumber(), opcodes); }
/*      */   
/*      */   private void writeBranch(Java.Locatable l, int opcode, CodeContext.Offset dst) {
/* 9107 */     this.codeContext.writeBranch(l.getLocation().getLineNumber(), opcode, dst);
/*      */   }
/*      */   
/* 9110 */   private void writeOffset(CodeContext.Offset src, CodeContext.Offset dst) { this.codeContext.writeOffset((short)-1, src, dst); }
/*      */   
/*      */ 
/*      */ 
/*      */   private void writeConstantClassInfo(String descriptor)
/*      */   {
/* 9116 */     CodeContext ca = this.codeContext;
/* 9117 */     ca.writeShort((short)-1, ca.getClassFile().addConstantClassInfo(descriptor));
/*      */   }
/*      */   
/* 9120 */   private void writeConstantFieldrefInfo(String classFD, String fieldName, String fieldFD) { CodeContext ca = this.codeContext;
/* 9121 */     ca.writeShort((short)-1, ca.getClassFile().addConstantFieldrefInfo(classFD, fieldName, fieldFD));
/*      */   }
/*      */   
/* 9124 */   private void writeConstantMethodrefInfo(String classFD, String methodName, String methodMD) { CodeContext ca = this.codeContext;
/* 9125 */     ca.writeShort((short)-1, ca.getClassFile().addConstantMethodrefInfo(classFD, methodName, methodMD));
/*      */   }
/*      */   
/* 9128 */   private void writeConstantInterfaceMethodrefInfo(String classFD, String methodName, String methodMD) { CodeContext ca = this.codeContext;
/* 9129 */     ca.writeShort((short)-1, ca.getClassFile().addConstantInterfaceMethodrefInfo(classFD, methodName, methodMD));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private short addConstantStringInfo(String value)
/*      */   {
/* 9137 */     return this.codeContext.getClassFile().addConstantStringInfo(value);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private short addConstantIntegerInfo(int value)
/*      */   {
/* 9145 */     return this.codeContext.getClassFile().addConstantIntegerInfo(value);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 9153 */   private short addConstantFloatInfo(float value) { return this.codeContext.getClassFile().addConstantFloatInfo(value); }
/*      */   
/*      */   private void writeConstantLongInfo(long value) {
/* 9156 */     CodeContext ca = this.codeContext;
/* 9157 */     ca.writeShort((short)-1, ca.getClassFile().addConstantLongInfo(value));
/*      */   }
/*      */   
/* 9160 */   private void writeConstantDoubleInfo(double value) { CodeContext ca = this.codeContext;
/* 9161 */     ca.writeShort((short)-1, ca.getClassFile().addConstantDoubleInfo(value));
/*      */   }
/*      */   
/*      */   public CodeContext.Offset getWhereToBreak(Java.BreakableStatement bs) {
/* 9165 */     if (bs.whereToBreak == null) {
/* 9166 */       CodeContext tmp16_13 = this.codeContext;tmp16_13.getClass();bs.whereToBreak = new CodeContext.Offset(tmp16_13);
/*      */     }
/* 9168 */     return bs.whereToBreak;
/*      */   }
/*      */   
/*      */   private Java.TypeBodyDeclaration getDeclaringTypeBodyDeclaration(Java.QualifiedThisReference qtr)
/*      */     throws CompileException
/*      */   {
/* 9174 */     if (qtr.declaringTypeBodyDeclaration == null)
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/* 9179 */       Java.Scope s = qtr.getEnclosingBlockStatement();
/* 9180 */       while (!(s instanceof Java.TypeBodyDeclaration)) {
/* 9181 */         s = s.getEnclosingScope();
/*      */       }
/* 9183 */       qtr.declaringTypeBodyDeclaration = ((Java.TypeBodyDeclaration)s);
/* 9184 */       if (qtr.declaringTypeBodyDeclaration.isStatic()) {
/* 9185 */         compileError("No current instance available in static method", qtr.getLocation());
/*      */       }
/*      */       
/*      */ 
/* 9189 */       qtr.declaringClass = ((Java.ClassDeclaration)qtr.declaringTypeBodyDeclaration.getDeclaringType());
/*      */     }
/* 9191 */     return qtr.declaringTypeBodyDeclaration;
/*      */   }
/*      */   
/*      */   private Java.ClassDeclaration getDeclaringClass(Java.QualifiedThisReference qtr) throws CompileException {
/* 9195 */     if (qtr.declaringClass == null) {
/* 9196 */       getDeclaringTypeBodyDeclaration(qtr);
/*      */     }
/* 9198 */     return qtr.declaringClass;
/*      */   }
/*      */   
/*      */   private void referenceThis(Java.Locatable l) {
/* 9202 */     writeOpcode(l, 42);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IClass newArray(Java.Locatable l, int dimExprCount, int dims, IClass componentType)
/*      */   {
/* 9218 */     if ((dimExprCount == 1) && (dims == 0) && (componentType.isPrimitive()))
/*      */     {
/*      */ 
/* 9221 */       writeOpcode(l, -68);
/* 9222 */       writeByte(componentType == IClass.LONG ? 11 : componentType == IClass.INT ? 10 : componentType == IClass.SHORT ? 9 : componentType == IClass.BYTE ? 8 : componentType == IClass.DOUBLE ? 7 : componentType == IClass.FLOAT ? 6 : componentType == IClass.CHAR ? 5 : componentType == IClass.BOOLEAN ? 4 : -1);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 9232 */       return componentType.getArrayIClass(this.iClassLoader.OBJECT);
/*      */     }
/*      */     
/* 9235 */     if (dimExprCount == 1) {
/* 9236 */       IClass at = componentType.getArrayIClass(dims, this.iClassLoader.OBJECT);
/*      */       
/*      */ 
/*      */ 
/* 9240 */       writeOpcode(l, -67);
/* 9241 */       writeConstantClassInfo(at.getDescriptor());
/* 9242 */       return at.getArrayIClass(this.iClassLoader.OBJECT);
/*      */     }
/* 9244 */     IClass at = componentType.getArrayIClass(dimExprCount + dims, this.iClassLoader.OBJECT);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 9249 */     writeOpcode(l, -59);
/* 9250 */     writeConstantClassInfo(at.getDescriptor());
/* 9251 */     writeByte(dimExprCount);
/* 9252 */     return at;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public static class SimpleIField
/*      */     extends IClass.IField
/*      */   {
/*      */     private final String name;
/*      */     
/*      */ 
/*      */     private final IClass type;
/*      */     
/*      */ 
/*      */ 
/*      */     public SimpleIField(IClass declaringIClass, String name, IClass type)
/*      */     {
/* 9269 */       super();
/* 9270 */       this.name = name;
/* 9271 */       this.type = type; }
/*      */     
/* 9273 */     public Object getConstantValue() { return null; }
/* 9274 */     public String getName() { return this.name; }
/* 9275 */     public IClass getType() { return this.type; }
/* 9276 */     public boolean isStatic() { return false; }
/* 9277 */     public Access getAccess() { return Access.DEFAULT; }
/*      */   }
/*      */   
/*      */   private static Access modifiers2Access(short modifiers) {
/* 9281 */     return (modifiers & 0x2) != 0 ? Access.PRIVATE : (modifiers & 0x4) != 0 ? Access.PROTECTED : (modifiers & 0x1) != 0 ? Access.PUBLIC : Access.DEFAULT;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static String last(String[] sa)
/*      */   {
/* 9290 */     if (sa.length == 0) throw new IllegalArgumentException("SNO: Empty string array");
/* 9291 */     return sa[(sa.length - 1)];
/*      */   }
/*      */   
/*      */   private static String[] allButLast(String[] sa) {
/* 9295 */     if (sa.length == 0) throw new IllegalArgumentException("SNO: Empty string array");
/* 9296 */     String[] tmp = new String[sa.length - 1];
/* 9297 */     System.arraycopy(sa, 0, tmp, 0, tmp.length);
/* 9298 */     return tmp;
/*      */   }
/*      */   
/*      */   private static String[] concat(String[] sa, String s) {
/* 9302 */     String[] tmp = new String[sa.length + 1];
/* 9303 */     System.arraycopy(sa, 0, tmp, 0, sa.length);
/* 9304 */     tmp[sa.length] = s;
/* 9305 */     return tmp;
/*      */   }
/*      */   
/*      */ 
/* 9309 */   private CodeContext codeContext = null;
/*      */   
/*      */ 
/* 9312 */   private ErrorHandler optionalCompileErrorHandler = null;
/* 9313 */   private int compileErrorCount = 0;
/*      */   
/*      */ 
/* 9316 */   private WarningHandler optionalWarningHandler = null;
/*      */   
/*      */   public final Java.CompilationUnit compilationUnit;
/*      */   
/*      */   private final IClassLoader iClassLoader;
/*      */   
/*      */   private final boolean isStringBuilderAvailable;
/*      */   
/*      */   private List generatedClassFiles;
/*      */   
/*      */   private boolean debugSource;
/*      */   private boolean debugLines;
/*      */   private boolean debugVars;
/* 9329 */   private final Map singleTypeImports = new HashMap();
/*      */   
/*      */   private final Collection typeImportsOnDemand;
/*      */   
/* 9333 */   private final Map singleStaticImports = new HashMap();
/* 9334 */   private final Collection staticImportsOnDemand = new ArrayList();
/*      */   
/*      */   public void leave2(Java.BlockStatement bs, IClass optionalStackValueType) {}
/*      */   
/*      */   public static abstract interface ErrorHandler
/*      */   {
/*      */     public abstract void handleError(String paramString, Location paramLocation)
/*      */       throws CompileException;
/*      */   }
/*      */   
/*      */   static abstract interface Compilable
/*      */   {
/*      */     public abstract void compile()
/*      */       throws CompileException;
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\UnitCompiler.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */