/*    */ package org.hibernate.annotations.common.reflection.java.generics;
/*    */ 
/*    */ import java.lang.reflect.GenericArrayType;
/*    */ import java.lang.reflect.ParameterizedType;
/*    */ import java.lang.reflect.Type;
/*    */ import java.lang.reflect.TypeVariable;
/*    */ import java.lang.reflect.WildcardType;
/*    */ import java.util.HashMap;
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ class SimpleTypeEnvironment
/*    */   extends HashMap<Type, Type>
/*    */   implements TypeEnvironment
/*    */ {
/*    */   private static final long serialVersionUID = 1L;
/* 20 */   private final TypeSwitch<Type> substitute = new TypeSwitch()
/*    */   {
/*    */     public Type caseClass(Class classType) {
/* 23 */       return classType;
/*    */     }
/*    */     
/*    */     public Type caseGenericArrayType(GenericArrayType genericArrayType)
/*    */     {
/* 28 */       Type originalComponentType = genericArrayType.getGenericComponentType();
/* 29 */       Type boundComponentType = SimpleTypeEnvironment.this.bind(originalComponentType);
/*    */       
/* 31 */       if (originalComponentType == boundComponentType) {
/* 32 */         return genericArrayType;
/*    */       }
/* 34 */       return TypeFactory.createArrayType(boundComponentType);
/*    */     }
/*    */     
/*    */     public Type caseParameterizedType(ParameterizedType parameterizedType)
/*    */     {
/* 39 */       Type[] originalArguments = parameterizedType.getActualTypeArguments();
/* 40 */       Type[] boundArguments = SimpleTypeEnvironment.this.substitute(originalArguments);
/*    */       
/* 42 */       if (areSame(originalArguments, boundArguments)) {
/* 43 */         return parameterizedType;
/*    */       }
/* 45 */       return TypeFactory.createParameterizedType(parameterizedType.getRawType(), boundArguments, parameterizedType.getOwnerType());
/*    */     }
/*    */     
/*    */ 
/*    */     private boolean areSame(Object[] array1, Object[] array2)
/*    */     {
/* 51 */       if (array1.length != array2.length) {
/* 52 */         return false;
/*    */       }
/* 54 */       for (int i = 0; i < array1.length; i++) {
/* 55 */         if (array1[i] != array2[i]) {
/* 56 */           return false;
/*    */         }
/*    */       }
/* 59 */       return true;
/*    */     }
/*    */     
/*    */     public Type caseTypeVariable(TypeVariable typeVariable)
/*    */     {
/* 64 */       if (!SimpleTypeEnvironment.this.containsKey(typeVariable)) {
/* 65 */         return typeVariable;
/*    */       }
/* 67 */       return (Type)SimpleTypeEnvironment.this.get(typeVariable);
/*    */     }
/*    */     
/*    */     public Type caseWildcardType(WildcardType wildcardType)
/*    */     {
/* 72 */       return wildcardType;
/*    */     }
/*    */   };
/*    */   
/*    */   public SimpleTypeEnvironment(Type[] formalTypeArgs, Type[] actualTypeArgs) {
/* 77 */     for (int i = 0; i < formalTypeArgs.length; i++) {
/* 78 */       put(formalTypeArgs[i], actualTypeArgs[i]);
/*    */     }
/*    */   }
/*    */   
/*    */   public Type bind(Type type) {
/* 83 */     return (Type)this.substitute.doSwitch(type);
/*    */   }
/*    */   
/*    */   private Type[] substitute(Type[] types) {
/* 87 */     Type[] substTypes = new Type[types.length];
/* 88 */     for (int i = 0; i < substTypes.length; i++) {
/* 89 */       substTypes[i] = bind(types[i]);
/*    */     }
/* 91 */     return substTypes;
/*    */   }
/*    */ }


/* Location:              /Users/anandkum/Downloads/hibernate-commons-annotations-3.0.0.ga.jar!/org/hibernate/annotations/common/reflection/java/generics/SimpleTypeEnvironment.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */