package prefuse.data.query;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import javax.swing.JComponent;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import prefuse.data.Table;
import prefuse.data.column.ColumnMetadata;
import prefuse.data.expression.ColumnExpression;
import prefuse.data.expression.Literal;
import prefuse.data.expression.RangePredicate;
import prefuse.util.TypeLib;
import prefuse.util.ui.JRangeSlider;
import prefuse.util.ui.ValuedRangeModel;

public class RangeQueryBinding
  extends DynamicQueryBinding
{
  private Class m_type;
  private Listener m_lstnr;
  private ValuedRangeModel m_model;
  private boolean m_ordinal;
  private static FocusListener s_sliderAdj;
  
  public RangeQueryBinding(Table paramTable, String paramString)
  {
    this(paramTable, paramString, false);
  }
  
  public RangeQueryBinding(Table paramTable, String paramString, boolean paramBoolean)
  {
    super(paramTable, paramString);
    this.m_type = paramTable.getColumnType(paramString);
    this.m_ordinal = paramBoolean;
    this.m_lstnr = new Listener(null);
    initPredicate();
    initModel();
  }
  
  private void initPredicate()
  {
    ColumnMetadata localColumnMetadata = this.m_table.getMetadata(this.m_field);
    int i = localColumnMetadata.getMinimumRow();
    int j = localColumnMetadata.getMaximumRow();
    Object localObject1 = this.m_table.get(i, this.m_field);
    Object localObject2 = this.m_table.get(j, this.m_field);
    Literal localLiteral1 = Literal.getLiteral(localObject1, this.m_type);
    Literal localLiteral2 = Literal.getLiteral(localObject2, this.m_type);
    ColumnExpression localColumnExpression = new ColumnExpression(this.m_field);
    RangePredicate localRangePredicate = new RangePredicate(localColumnExpression, localLiteral1, localLiteral2);
    setPredicate(localRangePredicate);
  }
  
  public void initModel()
  {
    if (this.m_model != null) {
      this.m_model.removeChangeListener(this.m_lstnr);
    }
    ColumnMetadata localColumnMetadata = this.m_table.getMetadata(this.m_field);
    Object localObject = null;
    if ((TypeLib.isNumericType(this.m_type)) && (!this.m_ordinal))
    {
      Number localNumber1 = (Number)this.m_table.get(localColumnMetadata.getMinimumRow(), this.m_field);
      Number localNumber2 = (Number)this.m_table.get(localColumnMetadata.getMaximumRow(), this.m_field);
      localObject = new NumberRangeModel(localNumber1, localNumber2, localNumber1, localNumber2);
    }
    else
    {
      localObject = new ObjectRangeModel(localColumnMetadata.getOrdinalArray());
    }
    this.m_model = ((ValuedRangeModel)localObject);
    this.m_model.addChangeListener(this.m_lstnr);
  }
  
  public ValuedRangeModel getModel()
  {
    return this.m_model;
  }
  
  public NumberRangeModel getNumberModel()
  {
    return (this.m_model instanceof NumberRangeModel) ? (NumberRangeModel)this.m_model : null;
  }
  
  public ObjectRangeModel getObjectModel()
  {
    return (this.m_model instanceof ObjectRangeModel) ? (ObjectRangeModel)this.m_model : null;
  }
  
  public JComponent createComponent()
  {
    return createHorizontalRangeSlider();
  }
  
  public JRangeSlider createHorizontalRangeSlider()
  {
    return createRangeSlider(1, 0);
  }
  
  public JRangeSlider createVerticalRangeSlider()
  {
    return createRangeSlider(0, 1);
  }
  
  public JRangeSlider createRangeSlider(int paramInt1, int paramInt2)
  {
    return new JRangeSlider(this.m_model, paramInt1, paramInt2);
  }
  
  public JSlider createSlider()
  {
    JSlider localJSlider = new JSlider(this.m_model);
    localJSlider.addFocusListener(getSliderAdjuster());
    return localJSlider;
  }
  
  private static synchronized FocusListener getSliderAdjuster()
  {
    if (s_sliderAdj == null) {
      s_sliderAdj = new SliderAdjuster(null);
    }
    return s_sliderAdj;
  }
  
  private class Listener
    implements ChangeListener
  {
    private Listener() {}
    
    public void stateChanged(ChangeEvent paramChangeEvent)
    {
      ValuedRangeModel localValuedRangeModel = (ValuedRangeModel)paramChangeEvent.getSource();
      Object localObject1 = localValuedRangeModel.getLowValue();
      Object localObject2 = localValuedRangeModel.getHighValue();
      RangePredicate localRangePredicate = (RangePredicate)RangeQueryBinding.this.m_query;
      localRangePredicate.setLeftExpression(Literal.getLiteral(localObject1, RangeQueryBinding.this.m_type));
      localRangePredicate.setRightExpression(Literal.getLiteral(localObject2, RangeQueryBinding.this.m_type));
    }
  }
  
  private static class SliderAdjuster
    implements FocusListener
  {
    public void focusGained(FocusEvent paramFocusEvent)
    {
      ((JSlider)paramFocusEvent.getSource()).setExtent(0);
    }
    
    public void focusLost(FocusEvent paramFocusEvent) {}
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\query\RangeQueryBinding.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */